# - Find CPPUNIT installation
# This module tries to find the CPPUNIT installation on your system.
# It tries to find the club-config script which gives you all the needed information.

MESSAGE(STATUS "Looking for CPPUNIT...")
SET(CPPUNIT_LIB)
SET(CPPUNIT_VERSION)
SET(CPPUNIT_INCLUDE_DIR)
SET(CPPUNIT_LIBRARY_DIR)

if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
    message(STATUS " **** PKG_CONFIG_FOUND [${PKG_CONFIG_FOUND}]")
    message(STATUS " **** PKG_CONFIG_EXECUTABLE [${PKG_CONFIG_EXECUTABLE}]")
    message(STATUS " **** PKG_CONFIG_VERSION_STRING [${PKG_CONFIG_VERSION_STRING}]")
endif()

IF(WIN32)
    MARK_AS_ADVANCED(CPPUNIT_DLL_DIR)
    FIND_PATH(CPPUNIT_DLL_DIR NAMES cppunit_dll.dll PATHS ENV PATH)
    STRING(REGEX REPLACE "(.*)/bin" "\\1" CPPUNIT_DIR "${CPPUNIT_DLL_DIR}")
    FIND_LIBRARY(CPPUNIT_LIB NAMES cppunit_dll PATHS ${CPPUNIT_DIR}/lib)
    IF(CPPUNIT_LIB)
        SET(CPPUNIT_FOUND TRUE)
    ENDIF(CPPUNIT_LIB)
ENDIF(WIN32)

IF(UNIX)
  IF(PKG_CONFIG_FOUND) ## looking for CPPUNIT from pkg-config (new)
        pkg_check_modules (CPPUNIT cppunit>=1.12.1)

        if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
            message(STATUS " **** CPPUNIT_FOUND  [${CPPUNIT_FOUND}]")
            message(STATUS " **** CPPUNIT_VERSION [${CPPUNIT_VERSION}]")
            message(STATUS " **** CPPUNIT_LIBRARIES [${CPPUNIT_LIBRARIES}]")
            message(STATUS " **** CPPUNIT_LIBRARY_DIRS [${CPPUNIT_LIBRARY_DIRS}]")
            message(STATUS " **** CPPUNIT_LDFLAGS [${CPPUNIT_LDFLAGS}]")
            message(STATUS " **** CPPUNIT_LDFLAGS_OTHER [${CPPUNIT_LDFLAGS_OTHER}]")
            message(STATUS " **** CPPUNIT_INCLUDE_DIRS [${CPPUNIT_INCLUDE_DIRS}]")
            message(STATUS " **** CPPUNIT_CFLAGS [${CPPUNIT_CFLAGS}]")
            message(STATUS " **** CPPUNIT_CFLAGS_OTHER [${CPPUNIT_CFLAGS_OTHER}]")

            message(STATUS " **** CPPUNIT_PREFIX [${CPPUNIT_PREFIX}]")
            message(STATUS " **** CPPUNIT_INCLUDEDIR [${CPPUNIT_INCLUDEDIR}]")
            message(STATUS " **** CPPUNIT_LIBDIR [${CPPUNIT_LIBDIR}]")
        endif()

  ELSE(PKG_CONFIG_FOUND)
      FIND_PROGRAM(CPPUNIT_CONFIG_EXECUTABLE cppunit-config PATHS ENV PATH NO_DEFAULT_PATH)
      SET(CPPUNIT_ADDITIONAL_DIR)

      ## Looking for lib flags
      IF(CPPUNIT_CONFIG_EXECUTABLE) ## with cppunit-config
        EXEC_PROGRAM(${CPPUNIT_CONFIG_EXECUTABLE} ARGS --libs OUTPUT_VARIABLE CPPUNITLIBS)
      ENDIF(CPPUNIT_CONFIG_EXECUTABLE)

      STRING(REGEX MATCH "-L([^ ]*)" CPPUNIT_ADDITIONAL_DIR "${CPPUNITLIBS}")
      STRING(REPLACE "-L" "" CPPUNIT_ADDITIONAL_DIR "${CPPUNIT_ADDITIONAL_DIR}")

      FIND_LIBRARY(CPPUNIT_LIB NAMES cppunit
        HINTS ENV LD_LIBRARY_PATH ${CPPUNIT_ADDITIONAL_DIR}
        PATH_SUFFIXES lib64 lib
        PATHS /usr/local /usr
        NO_DEFAULT_PATH)

      IF(CPPUNIT_LIB)
          SET(CPPUNIT_FOUND TRUE)
      ENDIF(CPPUNIT_LIB)


      IF(NOT CPPUNITLIBS AND CPPUNIT_LIB)
        get_filename_component(CPPUNITLIBS ${CPPUNIT_LIB} NAME_WE)
        STRING(REPLACE "lib" "-l" CPPUNITLIBS "${CPPUNITLIBS}")
      ENDIF(NOT CPPUNITLIBS AND CPPUNIT_LIB)
  ENDIF(PKG_CONFIG_FOUND)

ENDIF(UNIX) 

IF(CPPUNIT_FOUND)
    IF(CPPUNIT_CONFIG_EXECUTABLE)
        EXEC_PROGRAM(${CPPUNIT_CONFIG_EXECUTABLE} ARGS "--version" OUTPUT_VARIABLE CPPUNIT_VERSION)
        EXEC_PROGRAM(${CPPUNIT_CONFIG_EXECUTABLE} ARGS --cflags OUTPUT_VARIABLE CPPUNITFLAGS)
    
        IF(CPPUNITFLAGS)
            STRING(REPLACE "-I" "" CPPUNITFLAGS ${CPPUNITFLAGS})
        ENDIF(CPPUNITFLAGS)
    ENDIF(CPPUNIT_CONFIG_EXECUTABLE)

    IF(WIN32)
        STRING(REGEX REPLACE "(.*)/lib/cppunit_dll.lib" "\\1" CPPUNIT_DIR "${CPPUNIT_LIB}")
        SET(CPPUNIT_VERSION UNKNOWN)
        SET(CPPUNITFLAGS ${CPPUNIT_DIR}/include)
        SET(CPPUNITLIBS cppunit_dll)
        SET(CPPUNIT_LIBRARY_DIR ${CPPUNIT_DIR}/lib)
        ### to verify the pkg_check_modules variables
        SET(CPPUNIT_INCLUDEDIR ${CPPUNIT_DIR}/include)
        SET(CPPUNIT_LIBDIR ${CPPUNIT_DIR}/lib)
        SET(CPPUNIT_LDFLAGS cppunit_dll)
    ENDIF(WIN32)
 
  if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
    message(STATUS " ---- with CPPUNIT_CONFIG_EXECUTABLE")
    message(STATUS "    CPPUNITFLAGS [${CPPUNITFLAGS}]")
    message(STATUS "    CPPUNITLIBS [${CPPUNITLIBS}]")
    message(STATUS "    CPPUNIT_LIBRARY_DIR [${CPPUNIT_LIBRARY_DIR}]")
    message(STATUS " ---- with pkg_check_modules ")
    message(STATUS "    CPPUNIT_INCLUDEDIR [${CPPUNIT_INCLUDEDIR}] with 'pkg-config'")
    message(STATUS "    CPPUNIT_LIBDIR [${CPPUNIT_LIBDIR}] with 'pkg-config'")
    message(STATUS "    CPPUNIT_LDFLAGS [${CPPUNIT_LDFLAGS}] with 'pkg-config'")
    message(STATUS " ---- end of pkg_check_modules ")
  endif()
  MESSAGE(STATUS "Looking for CPPUNIT... - Found")
ELSE(CPPUNIT_FOUND)
  MESSAGE(STATUS "Looking for CPPUNIT... -  Unset! Please ensure that paths are correctly defined.")
ENDIF(CPPUNIT_FOUND)
