###
#[=======================================================================[.rst:
FindFMILibrary
---------

Find ``FMILibrary`` library installed.

The following variables will be defined for your use

::

   FMILibrary_LIB_NAME     - The name of the FMI Library
   FMILibrary_FOUND        - Were all of your specified components found?
   FMILibrary_DEFINITION   - Additional compiler flags
   FMILibrary_LIB_DIR      - The lib directory
   FMILibrary_DEPENDENCIES - The dependencies
   FMILibrary_INCLUDE_DIR  - The include directory

::

#]=======================================================================]


MESSAGE(STATUS "Looking for FMILibrary ...")
SET(FMILibrary_LIB_NAME "fmilib_shared")
SET(FMILibrary_FOUND FALSE)

SET(FMILibrary_DEFINITION)
SET(FMILibrary_LIB_DIR)
SET(FMILibrary_DEPENDENCIES)
SET(FMILibrary_INCLUDE_DIR)

# Activate the verbose mode
##SET(CMAKE_VERBOSE_MAKEFILE "OFF")
if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
  message(STATUS "==============================")
  message(STATUS " >> FMILibrary_LIB_NAME      [${FMILibrary_LIB_NAME}]")
endif(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")

# Temporary variables
SET(FMILibrary_ROOT_DIR)
SET(FMILibrary_INCLUDE_FILE)

# Search task : the lib file
FIND_LIBRARY(theFMILibrary
  NAMES ${FMILibrary_LIB_NAME}
  PATHS ENV LD_LIBRARY_PATH
  DOC "The FMILibrary library")

# Main task
if (theFMILibrary)
  if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
    message(STATUS " >> theFMILibrary            [${theFMILibrary}]")
  endif(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")

  get_filename_component(FMILibrary_LIB_DIR ${theFMILibrary} DIRECTORY)
  get_filename_component(FMILibrary_ROOT_DIR ${FMILibrary_LIB_DIR} DIRECTORY)
  if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
    message(STATUS " >> FMILibrary_LIB_DIR       [${FMILibrary_LIB_DIR}]")
    message(STATUS " >> FMILibrary_ROOT_DIR      [${FMILibrary_ROOT_DIR}]")
  endif(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")

  SET(FMILibrary_INCLUDE_DIR ${FMILibrary_ROOT_DIR}/include)
  if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
    message(STATUS " >> FMILibrary_INCLUDE_DIR   [${FMILibrary_INCLUDE_DIR}]")
  endif(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")

  find_file(FMILibrary_INCLUDE_FILE fmilib.h PATHS ${FMILibrary_INCLUDE_DIR} NO_DEFAULT_PATH)
  if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
    message(STATUS " >> FMILibrary_INCLUDE_FILE  [${FMILibrary_INCLUDE_FILE}]")
  endif(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")

  if (FMILibrary_INCLUDE_FILE)
    MESSAGE(STATUS "Looking for fmilib.h ... - Found  ${FMILibrary_INCLUDE_FILE}")
    SET(FMILibrary_DEPENDENCIES "-L${FMILibrary_LIB_DIR}" "-l${FMILibrary_LIB_NAME}")
  else(FMILibrary_INCLUDE_FILE)
    MESSAGE(STATUS "Looking for fmilib.h ... - Not Found in ${FMILibrary_INCLUDE_DIR}")
  endif()

  if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
    message(STATUS " >> FMILibrary_DEPENDENCIES  [${FMILibrary_DEPENDENCIES}]")
    message(STATUS "==============================")
  endif(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")

endif(theFMILibrary)


# Final task
# handle the QUIETLY and REQUIRED arguments and set FMILibrary_FOUND to TRUE
# if all listed variables are TRUE, hide their existence from configuration view
include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(FMILibrary
  DEFAULT_MSG
  FMILibrary_INCLUDE_FILE FMILibrary_LIB_DIR FMILibrary_INCLUDE_DIR )


if (FMILibrary_FOUND)
  set(FMILibrary_DEFINITION "-DWITH_FMI")
  MESSAGE(STATUS "Looking for FMILibrary... - Found")
else(FMILibrary_FOUND)
  MESSAGE(STATUS "Looking for FMILibrary... - Not Found")
endif(FMILibrary_FOUND)

##SET(CMAKE_VERBOSE_MAKEFILE "OFF")

#-----------------------------------------------------------------------
# End of File
#-----------------------------------------------------------------------
