# Find jsoncpp
#
# Find the jsoncpp includes and library
# 
# if you nee to add a custom library search path, do it via via CMAKE_PREFIX_PATH 
# 
# This module defines
#  JSONCPP_INCLUDE_DIRS, where to find header, etc.
#  JSONCPP_LIBRARIES, the libraries needed to use jsoncpp.
#  JSONCPP_FOUND, If false, do not try to use jsoncpp.
#  JSONCPP_INCLUDE_PREFIX, include prefix for jsoncpp



MESSAGE(STATUS "Looking for JSONCPP...")
SET(JSONCPP_LIB)
SET(JSONCPP_ROOT_DIR)
SET(JSONCPP_INCLUDE_DIR)
SET(JSONCPP_LIBRARY)
SET(JSONCPP_INCLUDE_FILE)

if(WIN32)
	SET(CMAKE_FIND_LIBRARY_PREFIXES "")
	SET(CMAKE_FIND_LIBRARY_SUFFIXES ".lib")
	set(JSONCPP_DEFINITION "-DJSON_DLL")
	add_definitions("${JSONCPP_DEFINITION}")
endif(WIN32)

find_library(
	JSONCPP_LIB
	NAMES jsoncpp
	PATHS ENV LD_LIBRARY_PATH
	      ENV PATH
	      /usr/lib
	      /usr/lib64
		  ${JSONCPP_LIB_DIR}
	DOC "jsoncpp library"
)


# Main task

if (JSONCPP_LIB)
    if(UNIX)
        STRING(REGEX REPLACE "(.*)/libjsoncpp.so" "\\1" JSONCPP_LIBRARY "${JSONCPP_LIB}")
        STRING(REGEX REPLACE "(.*)/lib[64]*(.*)" "\\1" JSONCPP_ROOT_DIR "${JSONCPP_LIBRARY}")
        if ("${JSONCPP_ROOT_DIR}" STREQUAL "/usr")
            SET(JSONCPP_INCLUDE_DIR /usr/include/jsoncpp /usr/include)
        else("${JSONCPP_ROOT_DIR}" STREQUAL "/usr")
            SET(JSONCPP_INCLUDE_DIR ${JSONCPP_ROOT_DIR}/include)
        endif("${JSONCPP_ROOT_DIR}" STREQUAL "/usr")
    else(UNIX)
        STRING(REGEX REPLACE "(.*)/jsoncpp.lib" "\\1" JSONCPP_LIBRARY "${JSONCPP_LIB}")
        STRING(REGEX REPLACE "(.*)/lib" "\\1" JSONCPP_ROOT_DIR "${JSONCPP_LIBRARY}")
        SET(JSONCPP_INCLUDE_DIR ${JSONCPP_ROOT_DIR}/include)
        SET(JSONCPP_LIBRARIES ${JSONCPP_LIBRARY}/jsoncpp.lib)
    endif(UNIX)
    find_file(JSONCPP_INCLUDE_FILE json/json.h PATHS ${JSONCPP_INCLUDE_DIR} NO_DEFAULT_PATH)
    if (JSONCPP_INCLUDE_FILE)
        MESSAGE(STATUS "Looking for json/json.h... - Found  ${JSONCPP_INCLUDE_FILE}")
    else(JSONCPP_INCLUDE_FILE)
        MESSAGE(STATUS "Looking for json/json.h... - Not Found in ${JSONCPP_INCLUDE_DIR}")
    endif()
endif(JSONCPP_LIB)


# handle the QUIETLY and REQUIRED arguments and set JSONCPP_FOUND to TRUE
# if all listed variables are TRUE, hide their existence from configuration view
include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(JSONCPP
				  DEFAULT_MSG
				  JSONCPP_LIBRARY JSONCPP_INCLUDE_FILE)

mark_as_advanced (JSONCPP_INCLUDE_FILE JSONCPP_LIBRARY)


if (JSONCPP_FOUND)
    MESSAGE(STATUS "Looking for JSONCPP... - Found")
else(JSONCPP_FOUND)
    MESSAGE(STATUS "Looking for JSONCPP... - Not Found")
endif(JSONCPP_FOUND)

#-----------------------------------------------------------------------
# End of File
#-----------------------------------------------------------------------
