#-------------------------------------------------------------------------
# Initialize variables
#-------------------------------------------------------------------------
set(LIBXML2_DEFINITION)
set(LIBXSLT_DEFINITION)
set(ICONV_INCLUDE_DIR_WIN)
set(LIBXML2_DEPENDENCIES)
set(LIBXML2_LINK_DLL)
set(LIBXML2_LIBRARIES_DIR)
set(MATHMORE_DEFINITION)
set(DATAFRAME_DEFINITION)
set(FFTW_DEFINITION)
set(FFTW_DEPENDENCIES)
set(OPT_DEPENDENCIES)
set(OPT_DEFINITION)
set(JSONCPP_DEFINITION)
set(JSONCPP_DEPENDENCIES)
set(LIBSSH_DEFINITION)
set(LIBSSH_DEPENDENCIES)
set(MPI_DEFINITION)
set(MPIOLD_DEFINITION)
set(NLOPT_DEFINITION)
set(PCL_DEFINITION)
set(MINUIT2_DEPENDENCIES)
set(MINUIT2_DEFINITION)
set(PYTHONLIBS_DEFINITION)
set(PYTHON3_DEFINITION)
set(LEVELE_DEFINITION)
set(SUNSET_DEFINITION)
set(ALL_DEFINITION "")

if(WIN32)
    set(ALL_DEFINITION ${ALL_DEFINITION} "-DWIN32")
else(WIN32) 
   if(__unix__)
        set(ALL_DEFINITION ${ALL_DEFINITION} "-D__unix__")
   endif(__unix__)
endif(WIN32)

#------------------------------------------------------------------------- 
# Compiler extra option
#-------------------------------------------------------------------------

# explicit link
if(UNIX)
    set(CMAKE_SHARED_LINKER_FLAGS "-Wl,--no-undefined")
endif(UNIX)

if(UNIX)
    set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -pipe -pthread -W -Wno-deprecated-declarations")
endif(UNIX)

#set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -pipe -W -pthread -Wno-deprecated-declarations")

# fpic
set(POSITION_INDEPENDENT_CODE ON)

# pthread
if(UNIX)
    set(CMAKE_THREAD_PREFER_PTHREAD ON)
    find_package(Threads)
endif()
#------------------------------------------------------------------------- 
# FIND The Uranie Version number
#-------------------------------------------------------------------------
find_package(URANIEversion REQUIRED)


#-------------------------------------------------------------------------
# Find PkgConfig
#-------------------------------------------------------------------------
message(STATUS "Looking for PkgConfig...")
find_package(PkgConfig COMPONENTS)

if(CMAKE_VERBOSE_MAKEFILE STREQUAL "ON")
    message(STATUS " **** PKG_CONFIG_FOUND [${PKG_CONFIG_FOUND}]")
    message(STATUS " **** PKG_CONFIG_EXECUTABLE [${PKG_CONFIG_EXECUTABLE}]")
    message(STATUS " **** PKG_CONFIG_VERSION_STRING [${PKG_CONFIG_VERSION_STRING}]")
endif()

if(PKG_CONFIG_FOUND)
    message(STATUS "Looking for PkgConfig... -  Found")
else(PKG_CONFIG_FOUND)
  message(STATUS "Looking for PkgConfig... - Not Found")
endif(PKG_CONFIG_FOUND)

#------------------------------------------------------------------------- 
# Find CPPUNIT 
#-------------------------------------------------------------------------
find_package(CPPUNIT COMPONENTS)

#-------------------------------------------------------------------------
# Find LibXML2 
# if LibXML2 is found then find ICONV
#-------------------------------------------------------------------------

if(--disable-libxml2 STREQUAL "ON")
    message(STATUS "Libxml2 is disabled")
else(--disable-libxml2 STREQUAL "ON")
    message(STATUS "Looking for LibXml2...")
    find_path(LIBXML2_INCLUDE_DIR NAMES libxml/xpath.h
                                PATHS ENV PATH
                                PATH_SUFFIXES ../include)

    find_package(LibXml2 COMPONENTS)
    if(LIBXML2_FOUND)
        message(STATUS "Looking for LibXml2... -  Found")
    else(LIBXML2_FOUND)
        set(LIBXML2_LIBRARIES "")
        set(LIBXML2_INCLUDE_DIR "")
        message(STATUS "Looking for LibXml2... - Not Found")
    endif(LIBXML2_FOUND)
endif(--disable-libxml2 STREQUAL "ON")

if(LIBXML2_FOUND)
    find_package(ICONV REQUIRED)
endif(LIBXML2_FOUND)

#-------------------------------------------------------------------------
# Find LibXslt
#-------------------------------------------------------------------------
# message(STATUS "Looking for LibXslt... ")
# find_package(LibXslt COMPONENTS)
# if(LIBXSLT_FOUND)
#     message(STATUS "Looking for LibXslt... -  Found")
# 	set(LIBXSLT_DEFINITION "-DWITH_LIBXSLT")
# else(LIBXSLT_FOUND)
#    message(STATUS "Looking for LibXslt... -  Not Found")
# endif(LIBXSLT_FOUND)

#-------------------------------------------------------------------------
# Find the executable xsltproc
#-------------------------------------------------------------------------

message(STATUS "Looking for xsltproc... ")
if(--disable-libxslt STREQUAL "ON" )
    message(STATUS "LIBXSLT is disabled")
else(--disable-libxslt STREQUAL "ON" )
    # find_package(LibXslt COMPONENTS)
    find_program(LIBXSLT_XSLTPROC_EXECUTABLE xsltproc)
    if(LIBXSLT_XSLTPROC_EXECUTABLE)
        message(STATUS "Looking for xsltproc... -  Found")
        set(LIBXSLT_DEFINITION "-DWITH_LIBXSLT")
        set(ALL_DEFINITION ${ALL_DEFINITION} ${LIBXSLT_DEFINITION})
    else(LIBXSLT_XSLTPROC_EXECUTABLE)
        message(STATUS "Looking for xsltproc... - Not Found")
    endif(LIBXSLT_XSLTPROC_EXECUTABLE)
endif(--disable-libxslt STREQUAL "ON" )

#-------------------------------------------------------------------------
# Find ROOT
#-------------------------------------------------------------------------

find_package(ROOT 6.16 REQUIRED)

#verify that root is compiled with Minuit2
if(HAS_MINUIT2)
    message(STATUS "   Library Minuit2 in ROOT -  Found")
else(HAS_MINUIT2)
    message(STATUS "   Library Minuit2 in ROOT - Not Found")
endif(HAS_MINUIT2)

if(LIBXML2_FOUND)
    set(LIBXML2_DEFINITION "-DWITH_LIBXML2")   
    set(ALL_DEFINITION ${ALL_DEFINITION} ${LIBXML2_DEFINITION})   
    if(WIN32)
        set(LIBXML2_DEPENDENCIES "xml2")
        string(REGEX REPLACE "libxml2.lib" "" tmp_str_lib ${LIBXML2_LIBRARIES})
        set(LIBXML2_LIBRARIES_DIR ${tmp_str_lib})
        set(LIBXML2_LINK_DLL  ${LIBXML2_LINK_DLL} ${LIBXML2_DLL_DIR})
        set(ICONV_INCLUDE_DIR_WIN ${ICONV_INCLUDE_DIR})
    else(WIN32)
      set(LIBXML2_DEPENDENCIES ${LIBXML2_LIBRARIES})
#         set(LIBXML2_DEPENDENCIES "xml2")        
        string(REGEX REPLACE "libxml2.so" "" tmp_str_lib ${LIBXML2_LIBRARIES})
        set(LIBXML2_LIBRARIES_DIR ${tmp_str_lib})
    endif(WIN32)
endif(LIBXML2_FOUND)

if("${ROOT_MATHMORE}" STREQUAL "yes")
  set(MATHMORE_DEFINITION "-DWITH_MATHMORE")
endif("${ROOT_MATHMORE}" STREQUAL "yes")

if("${ROOT_DATAFRAME}" STREQUAL "yes")
  set(DATAFRAME_DEFINITION "-DWITH_DATAFRAME")
endif("${ROOT_DATAFRAME}" STREQUAL "yes")

#-------------------------------------------------------------------------
# Set FFTW variables if FFTW has been found and ROOT has been compiled 
# with FFTW
#-------------------------------------------------------------------------

#verify that root is compiled with FFTW
if(FFTW_FOUND)
    message(STATUS "   Library FFTW in ROOT -  Found")
else(FFTW_FOUND)
  message(STATUS "   Library FFTW in ROOT - Not Found")
endif(FFTW_FOUND)

if(FFTW_FOUND)
    set(FFTW_DEFINITION "-DWITH_FFTW")
    set(ALL_DEFINITION ${ALL_DEFINITION} ${FFTW_DEFINITION})
    if(UNIX)
        set(FFTW_DEPENDENCIES "FFTW")
    else(UNIX)
        set(FFTW_DEPENDENCIES "libFFTW")
    endif(UNIX)
endif(FFTW_FOUND)


#-------------------------------------------------------------------------
# Find OPT
#-------------------------------------------------------------------------

#find_package(OPT COMPONENTS)
find_package(OPT REQUIRED)
if(OPT_FOUND)
    set(OPT_DEPENDENCIES "opt" "newmat")
    if(UNIX)
        set(OPT_DEPENDENCIES ${OPT_DEPENDENCIES} "dl")
    endif(UNIX)
	# IF(WIN32)
		# set(OPT_DEPENDENCIES "libopt" "libnewmat")
	# ENDIF(WIN32)
    set(OPT_DEFINITION "-DWITH_OPT") 
    set(ALL_DEFINITION ${ALL_DEFINITION} ${OPT_DEFINITION}) 
endif(OPT_FOUND)

#-------------------------------------------------------------------------
# Find JSONCPP
#-------------------------------------------------------------------------
if(--disable-jsoncpp STREQUAL "ON")
    message(STATUS "JSONCPP disabled")
else(--disable-jsoncpp STREQUAL "ON")  
    find_package(JSONCPP COMPONENTS)
    if (JSONCPP_FOUND)
        ##	message(STATUS ">> FIND_PACKAGE JSONCPP_INCLUDE_DIR  ${JSONCPP_INCLUDE_DIR}")
        ##	message(STATUS ">> FIND_PACKAGE JSONCPP_LIBRARY      ${JSONCPP_LIBRARY}")
        if(NOT JSONCPP_LIBRARIES)
            set(JSONCPP_DEPENDENCIES "jsoncpp")
        endif(NOT JSONCPP_LIBRARIES)
        set(JSONCPP_DEFINITION "-DWITH_JSONCPP")
        set(ALL_DEFINITION ${ALL_DEFINITION} ${JSONCPP_DEFINITION})
    endif(JSONCPP_FOUND)
endif(--disable-jsoncpp STREQUAL "ON")

#-------------------------------------------------------------------------
# Find FindFMILibrary
#-------------------------------------------------------------------------
if(--disable-libfmi STREQUAL "ON")
  message(STATUS "FMILibrary disabled")
else(--disable-libfmi STREQUAL "ON")
  find_package(FMILibrary COMPONENTS)
  if (FMILibrary_FOUND)
   set(ALL_DEFINITION ${ALL_DEFINITION} ${FMILibrary_DEFINITION})
  endif(FMILibrary_FOUND)
endif(--disable-libfmi STREQUAL "ON")

#-------------------------------------------------------------------------
# Find LIBSHH
#-------------------------------------------------------------------------
if(--disable-libssh STREQUAL "ON")
    message(STATUS "LIBSSH disabled")
else(--disable-libssh STREQUAL "ON")
    find_package(LIBSSH COMPONENTS)
    if (LIBSSH_FOUND)
        ##	message(STATUS ">> FIND_PACKAGE LIBSSH_INCLUDE_DIR  ${LIBSSH_INCLUDE_DIR}")
        ##      message(STATUS ">> FIND_PACKAGE LIBSSH_LIB          [${LIBSSH_LIB}]")
        if("${LIBSSH_LIB}" STREQUAL "")
          set(LIBSSH_DEPENDENCIES "ssh")
        else("${LIBSSH_LIB}" STREQUAL "")
	  set(LIBSSH_DEPENDENCIES "-L${LIBSSH_LIB} -lssh")
        endif("${LIBSSH_LIB}" STREQUAL "")
        set(LIBSSH_DEFINITION "-DWITH_LIBSSH")
	set(ALL_DEFINITION ${ALL_DEFINITION} ${LIBSSH_DEFINITION})
    endif(LIBSSH_FOUND)
endif(--disable-libssh STREQUAL "ON")
#-------------------------------------------------------------------------
# Find MPI
#-------------------------------------------------------------------------

if(NOT IGNORE_MPI STREQUAL "ON")
    message(STATUS "Looking for MPI...")
    find_package(MPI)
    if(MPI_C_FOUND)
        include(CmakePackage/mpi_detection.cmake)
        mpi_type(MPI_IS_SHARED MPI::MPI_C)
        message(STATUS "MPI_IS_SHARED: [${MPI_IS_SHARED}]")
        if(UNIX)
            execute_process(COMMAND ${MPIEXEC} --bind-to none hostname
                RESULT_VARIABLE mpi_test OUTPUT_QUIET)
            if(NOT mpi_test EQUAL "0")
              set(MPIOLD_DEFINITION "-DWITH_MPIOLD")
            endif()
        endif(UNIX)
        set(MPI_DEFINITION "-DWITH_MPI")
        set(ALL_DEFINITION ${ALL_DEFINITION} ${MPI_DEFINITION} ${MPIOLD_DEFINITION})
	get_filename_component(MPIEXEC_PATH ${MPIEXEC} DIRECTORY)
	get_filename_component(MPI_LIBRARY_PATH ${MPI_LIBRARY} DIRECTORY)
        message(STATUS "Looking for MPI... - Found")
    else(MPI_C_FOUND)
        message(STATUS "Looking for MPI... - Not Found")
    endif(MPI_C_FOUND)
endif(NOT IGNORE_MPI STREQUAL "ON")

#-------------------------------------------------------------------------
# Find NLOPT
#-------------------------------------------------------------------------
message(STATUS "Looking for NLOPT...")
if(--disable-nlopt STREQUAL "ON" )
    message(STATUS "NLOPT is disabled")
else(--disable-nlopt STREQUAL "ON" )
    find_package(NLOPT COMPONENTS)
    if(NLOPT_FOUND)
        set(NLOPT_DEFINITION "-DWITH_NLOPT")
        set(ALL_DEFINITION ${ALL_DEFINITION} ${NLOPT_DEFINITION})
        message(STATUS "Looking for NLOPT... - Found")
        if( ${NLOPT_INCLUDE_DIR} STREQUAL "/usr/include")
            set(NLOPT_INCLUDE_DIR "")
        endif( ${NLOPT_INCLUDE_DIR} STREQUAL "/usr/include")
    else(NLOPT_FOUND)
        set(NLOPT_INCLUDE_DIR "")
	set(NLOPT_LIBRARIES "")
        message(STATUS "Looking for NLOPT... - Not Found")
    endif(NLOPT_FOUND)

    #-------------------------------------------------------------------------
    # Find PCL
    #-------------------------------------------------------------------------
    message(STATUS "Looking for PCL...")
    find_package(PCL COMPONENTS)
    if(PCL_FOUND)
        set(PCL_DEFINITION "-DWITH_PCL")
        set(ALL_DEFINITION ${ALL_DEFINITION} ${PCL_DEFINITION})
        message(STATUS "Looking for PCL... - Found")
        if( ${PCL_INCLUDE_DIR} STREQUAL "/usr/include")
            set(PCL_INCLUDE_DIR "")
        endif( ${PCL_INCLUDE_DIR} STREQUAL "/usr/include")
    else(PCL_FOUND)
        set(PCL_INCLUDE_DIR "")
	set(PCL_LIBRARIES "")
        message(STATUS "Looking for PCL... - Not Found")
    endif(PCL_FOUND)

endif(--disable-nlopt STREQUAL "ON" )
#-------------------------------------------------------------------------
# Find pthread for windows
#-------------------------------------------------------------------------

if(WIN32)
    message(STATUS "Looking for pthreads-win32 ...")
    find_package(Pthread-win32)
    if(PTHREAD_FOUND)
        message(STATUS "Looking for pthreads-win32... - Found")
    else(PTHREAD_FOUND)
        message(STATUS "Looking for pthreads-win32... - Not Found")
    endif(PTHREAD_FOUND)    
endif(WIN32)

#-------------------------------------------------------------------------
# Find SUNSET
#-------------------------------------------------------------------------
find_package(SUNSET COMPONENTS)
if(SUNSET_FOUND)
    set(SUNSET_DEFINITION "-DWITH_SUNSET")
    set(ALL_DEFINITION ${ALL_DEFINITION} ${SUNSET_DEFINITION})
endif(SUNSET_FOUND)



#-------------------------------------------------------------------------
# Find MINUIT2
#-------------------------------------------------------------------------

if(HAS_MINUIT2)
	if(UNIX)
		set(MINUIT2_DEPENDENCIES "Minuit2")
    else(UNIX)
		set(MINUIT2_DEPENDENCIES "libMinuit2.dll")
	endif(UNIX)
    set(MINUIT2_DEFINITION "-DWITH_MINUIT2")
    set(ALL_DEFINITION ${ALL_DEFINITION} ${MINUIT2_DEFINITION})
endif(HAS_MINUIT2)




#-------------------------------------------------------------------------
# Find Python if available
#-------------------------------------------------------------------------
if(--disable-python STREQUAL "ON" )
    message(STATUS "Python is disabled")
else(--disable-python STREQUAL "ON" )
    message(STATUS "Looking for Python...")
    #---First look for the python interpreter and fix the version of it for the libraries--
    if(NOT python_version)
        set(python_version ${ROOT_PYTHON_VERSION})
    endif()
    find_package(Python ${python_version} EXACT COMPONENTS Interpreter Development NumPy)
    if(Python_Interpreter_FOUND)
        message(STATUS "Found Python interpreter version ${Python_VERSION}")
        message(STATUS "Found Python interpreter executable ${Python_EXECUTABLE}")
    endif(Python_Interpreter_FOUND)
    
    if(NOT Python_Development_FOUND)
        if(fail-on-missing)
            message(FATAL_ERROR "Python Development package not found and python component required")
        else(fail-on-missing)
            set(python OFF CACHE BOOL "" FORCE)
            message(STATUS "Python not found. Switching off python option")
        endif(fail-on-missing)
    else(NOT Python_Development_FOUND)
      message(STATUS "Looking for Python... - Found")
      set(PYTHONLIBS_DEFINITION "-DWITH_PYTHON")

      #---Checking the version used for ROOT.
      set(COMPARE_PYTHON FALSE)
      if("${Python_VERSION}" STREQUAL "${ROOT_PYTHON_VERSION}")
        message(STATUS " Python version found is [${Python_VERSION}] which is ok with the one used in ROOT [${ROOT_PYTHON_VERSION}]")
        set(COMPARE_PYTHON TRUE)
      endif()

      if("${Python_VERSION}" STREQUAL "${ROOT_SECONDPYTHON_VERSION}")
        message(STATUS " Python version found is [${Python_VERSION}] which is ok with the second version used in ROOT [${ROOT_SECONDPYTHON_VERSION}] (the main one being [${ROOT_PYTHON_VERSION}])")
        set(COMPARE_PYTHON TRUE)
      endif()
        
      if(NOT COMPARE_PYTHON)
        message(STATUS "[91m Python found is [${Python_VERSION}] while the one used in ROOT is [${ROOT_PYTHON_VERSION}][0m")
        if("${ROOT_PYTHON_LIB}" STREQUAL "" )
          message(STATUS "[91m Nothing can be done for python, cause no LIBRARY and INCLUDE path are provided by ROOT. PYTHON IS SWITCHED OFF, SORRY[0m")
          set(Python_Development_FOUND FALSE)
        else("${ROOT_PYTHON_LIB}" STREQUAL "" )
          if(NOT "${Python_LIBRARIES}" STREQUAL "${ROOT_PYTHON_LIB}"  )
            #Investigate quickly the paths
            get_filename_component(POUETPOUET ${ROOT_PYTHON_LIB} NAME)
            string( REPLACE "lib" "" POUETPOUET2 ${POUETPOUET})
            string( REPLACE ".so" "" POUETPOUET3 ${POUETPOUET2})
            find_program(PYTHON_EXECUTABLE_FROMROOT ${POUETPOUET3})
            if(${PYTHON_EXECUTABLE_FROMROOT} STREQUAL "")
              set( PYTHON_EXECUTABLE_FROMROOT "NOTFOUND")
            endif(${PYTHON_EXECUTABLE_FROMROOT} STREQUAL "")
        
            message(STATUS "[93mPython used in ROOT is different : \
\n   Library [root:${ROOT_PYTHON_LIB} / here:${Python_LIBRARIES}] \
\n   include_dir [root:${ROOT_PYTHON_INC} / here:${Python_INCLUDE_DIRS}] \
\n   Executable [root:${PYTHON_EXECUTABLE_FROMROOT} / here:${Python_EXECUTABLE}] [0m")

            message( WARNING  " \
 => Will try to use ROOT_python configuration (not recommended).\n     You might want to re-run the CMAKE command with -DPython_EXECUTABLE=...  -DPython_INCLUDE_DIR=... -DPython_LIBRARY=..." )
            set(Python_LIBRARIES ${ROOT_PYTHON_LIB})
            set(Python_INCLUDE_DIRS ${ROOT_PYTHON_INC})
            set(Python_LIBRARIES ${ROOT_PYTHON_LIB})
            set(Python_INCLUDE_DIRS ${ROOT_PYTHON_INC})
            set(Python_EXECUTABLE ${PYTHON_EXECUTABLE_FROMROOT})
          elseif("${Python_LIBRARIES}" STREQUAL "${ROOT_PYTHON_LIB}"  )
            message(STATUS "[93mPython used in ROOT seems to be the same : \
\n   Library [root:${ROOT_PYTHON_LIB} / here:${Python_LIBRARIES}] \
\n   include_dir [root:${ROOT_PYTHON_INC} / here:${Python_INCLUDE_DIRS}] \
\n   Executable [root:${PYTHON_EXECUTABLE_FROMROOT} / here:${Python_EXECUTABLE}] \
\n => Will do as if everything was fine.[0m")
          endif(NOT "${Python_LIBRARIES}" STREQUAL "${ROOT_PYTHON_LIB}" )          
        endif("${ROOT_PYTHON_LIB}" STREQUAL "" )
      endif(NOT COMPARE_PYTHON)

      if( Python_Development_FOUND )
        if(Python_VERSION VERSION_GREATER_EQUAL "3")
            message(STATUS "[93mPython version requested >= 3[0m")
            set(PYTHON3_DEFINITION "-DWITH_PYTHON3")
            set(PYTHONLIBS_DEFINITION ${PYTHONLIBS_DEFINITION} ${PYTHON3_DEFINITION})
        endif()
        set(ALL_DEFINITION ${ALL_DEFINITION} ${PYTHONLIBS_DEFINITION})
      endif( Python_Development_FOUND )
    endif(NOT Python_Development_FOUND)
endif(--disable-python STREQUAL "ON" )
    

#-------------------------------------------------------------------------
# find levelE library
#-------------------------------------------------------------------------
message(STATUS "Looking for building of Level E code...")
if(--disable-levele STREQUAL "ON" )
    message(STATUS "LEVELE is disabled")
else(--disable-levele STREQUAL "ON" )
    find_package(LevelE COMPONENTS)
    if(LEVELE_BIN_FOUND)
        set(LEVELE_DEFINITION "-DWITH_LEVELE")
        set(ALL_DEFINITION ${ALL_DEFINITION} ${LEVELE_DEFINITION})
        message(STATUS "Building Level E code... - Built")
    else(LEVELE_BIN_FOUND)
        message(STATUS "Building Level E code... - Not built (requires Fortran compiler)")
    endif(LEVELE_BIN_FOUND)
endif(--disable-levele STREQUAL "ON" )
  
#-------------------------------------------------------------------------
# find CUDA library and compiler
#-------------------------------------------------------------------------
if(--disable-cuda STREQUAL "ON" )
    message(STATUS "CUDA is disabled")
else(--disable-cuda STREQUAL "ON" )
    message(STATUS "Looking for CUDA tools ...")
    find_package(CUDA)
    if(CUDA_FOUND)
	message(STATUS "Found CUDA tools -- OK! (version: ${CUDA_VERSION_STRING})")
    else()	
        message(STATUS "CUDA tools: not found!")
    endif(CUDA_FOUND)

    #-------------------------------------------------------------------------
    # find Boost library (used by GPU ANN)
    #-------------------------------------------------------------------------
    message(STATUS "Looking for Boost library ...")
    find_package(Boost 1.47)
    if(Boost_FOUND)
      message(STATUS "Found Boost library -- OK! (version: ${Boost_VERSION})")
    else()
      message(STATUS "Boost library: not found!")
    endif()
endif(--disable-cuda STREQUAL "ON" )
  
#-------------------------------------------------------------------------
# Adds a target doc for doxygen documentation of Uranie'sources
#-------------------------------------------------------------------------
if(--enable-doc STREQUAL "ON" )
  message(STATUS "Looking for Doxygen...")
  find_package(Doxygen REQUIRED)
  if(DOXYGEN_FOUND)
    message(STATUS "Looking for Doxygen... - Found")
    find_path(HTML index.html ${PROJECT_SOURCE_DIR}/html/)
    if(HTML STREQUAL HTML-NOTFOUND)
      file(MAKE_DIRECTORY ${PROJECT_SOURCE_DIR}/html)  
    endif(HTML STREQUAL HTML-NOTFOUND)
    add_custom_target(doc COMMAND "${CMAKE_COMMAND}" -P "${CMAKE_SOURCE_DIR}/CmakePackage/makedoc.cmake" WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR})
  endif(DOXYGEN_FOUND)
endif(--enable-doc STREQUAL "ON")


#-------------------------------------------------------------------------


#-------------------------------------------------------------------------
# Add definitions for pyuranie
#-------------------------------------------------------------------------

if(NOT --disable-vizir STREQUAL "ON")
    set(ALL_DEFINITION ${ALL_DEFINITION} "-DWITH_VIZIR")
endif(NOT --disable-vizir STREQUAL "ON")


if(NOT --disable-NISP STREQUAL "ON")
    set(ALL_DEFINITION ${ALL_DEFINITION} "-DWITH_NISP")
    if(NOT --disable-ANISP STREQUAL "ON")
        set(ALL_DEFINITION ${ALL_DEFINITION} "-DWITH_ANISP")
    endif(NOT --disable-ANISP STREQUAL "ON")
endif(NOT --disable-NISP STREQUAL "ON")

if(NOT --disable-PGAUSS STREQUAL "ON")
    set(ALL_DEFINITION ${ALL_DEFINITION} "-DWITH_PGAUSS")
endif(NOT --disable-PGAUSS STREQUAL "ON")




 message(STATUS "End of UranieFindAllPackages")
