



#-------------------------------------------------------------------------
#-----URANIE_GENERAL COMPILATION FLAG
# * add the common compilation flag for both windows and UNIX
#-------------------------------------------------------------------------

set(URANIE_WARNING_LEVEL) 
set(URANIE_CXX_FLAGS)
set(URANIE_C_FLAGS)

if(UNIX)
  #set(URANIE_WARNING_LEVEL " -Wall -Wno-unused -Wno-strict-aliasing")
  set(URANIE_WARNING_LEVEL " -Wsequence-point -Waddress -Wmissing-braces -Wparentheses -Wreturn-type -Wunused-parameter -Wsign-compare -Wswitch -Warray-bounds") #-Wmaybe-uninitialized -Wshadow
  set(URANIE_CXX_FLAGS " -O2 -pipe -fPIC -pthread ${URANIE_WARNING_LEVEL} -Wreorder")
  set(URANIE_C_FLAGS " -O2 -pipe -fPIC ${URANIE_WARNING_LEVEL}")

  if(CMAKE_BUILD_TYPE STREQUAL "Debug" OR CMAKE_BUILD_TYPE STREQUAL "DEBUG" OR CMAKE_BUILD_TYPE STREQUAL "debug")
    set(URANIE_CXX_FLAGS " -g ${URANIE_CXX_FLAGS}")
    set(URANIE_C_FLAGS " -g ${URANIE_C_FLAGS}")
    message(STATUS "The compilation of URANIE classes will be done in Debug mode")
  else(CMAKE_BUILD_TYPE STREQUAL "Debug" OR CMAKE_BUILD_TYPE STREQUAL "DEBUG" OR CMAKE_BUILD_TYPE STREQUAL "debug")
    message(STATUS "The compilation of URANIE classes will be done in Release mode")
  endif(CMAKE_BUILD_TYPE STREQUAL "Debug" OR CMAKE_BUILD_TYPE STREQUAL "DEBUG" OR CMAKE_BUILD_TYPE STREQUAL "debug")

endif(UNIX)

if(WIN32)
  set(URANIE_CXX_FLAGS " -FIw32pragma.h")
endif(WIN32)



#-------------------------------------------------------------------------
#-------URANIE_WIN32_MODE()
# * Adds an option --enable-WIN32-DEBUG: ON = Debug mode activated, OFF = Release Mode activated. Must set before project command to be able to modify CMAKE_CONFIGURATION_TYPES value in the GUI.
#-------------------------------------------------------------------------
macro(URANIE_WIN32_MODE)
  
if(WIN32)
  set(--enable-WIN32-DEBUG "" CACHE STRING "Mode option.")
  set(CMAKE_INSTALL_PREFIX "" CACHE STRING "Installation path of Uranie.")
  mark_as_advanced(CMAKE_CONFIGURATION_TYPES)
#   set(CMAKE_CONFIGURATION_TYPES Release CACHE STRING "Release mode, default option.")
#   set(MODEDIR Release)
  if( NOT --enable-WIN32-DEBUG STREQUAL OFF AND NOT --enable-WIN32-DEBUG STREQUAL ON )
    message(FATAL_ERROR "Please select a valid value for the option --enable-WIN32-DEBUG, OFF if ROOT is built in release mode, ON if built in debug mode.")
  elseif(--enable-WIN32-DEBUG STREQUAL OFF)
    set(CMAKE_CONFIGURATION_TYPES Release CACHE STRING "Release mode." FORCE)
    set(MODEDIR Release)
  elseif(--enable-WIN32-DEBUG STREQUAL ON)
    set(CMAKE_CONFIGURATION_TYPES Debug CACHE STRING "Debug mode." FORCE)
    set(MODEDIR Debug)  
  endif( NOT --enable-WIN32-DEBUG STREQUAL OFF AND NOT --enable-WIN32-DEBUG STREQUAL ON )
endif(WIN32)


endmacro(URANIE_WIN32_MODE)
#-------------------------------------------------------------------------







#-------------------------------------------------------------------------
#-----URANIE_WRITE_ENVIRONMENT_FILE()
# * Creates files to source UNIX environment variables  
#-------------------------------------------------------------------------
macro(URANIE_WRITE_ENVIRONMENT_FILE)

if(UNIX)
    if(CMAKE_INSTALL_PREFIX EQUAL "")
      set(CMAKE_INSTALL_PREFIX ${CMAKE_BINARY_DIR})
    endif(CMAKE_INSTALL_PREFIX EQUAL "")
  # Création de uranie.cshrc
  configure_file(${URANIE_SOURCE_DIR}/uranie_skeleton.cshrc ${CMAKE_BINARY_DIR}/uranie.cshrc @ONLY)
  install(FILES ${CMAKE_BINARY_DIR}/uranie.cshrc  DESTINATION ${CMAKE_INSTALL_PREFIX} PERMISSIONS OWNER_READ GROUP_READ WORLD_READ OWNER_WRITE GROUP_WRITE OWNER_EXECUTE GROUP_EXECUTE)

  # Création de uranie.bashrc
  configure_file(${URANIE_SOURCE_DIR}/uranie_skeleton.bashrc ${CMAKE_BINARY_DIR}/uranie.bashrc @ONLY)
  install(FILES ${CMAKE_BINARY_DIR}/uranie.bashrc DESTINATION ${CMAKE_INSTALL_PREFIX} PERMISSIONS OWNER_READ GROUP_READ WORLD_READ OWNER_WRITE GROUP_WRITE OWNER_EXECUTE GROUP_EXECUTE)

  exec_program(cp ARGS ${CMAKE_SOURCE_DIR}/valgrind-uranie-root.supp ${CMAKE_BINARY_DIR} OUTPUT_VARIABLE BLA)
endif(UNIX)

if(WIN32)
    configure_file(${CMAKE_SOURCE_DIR}/uranie.ps1 ${CMAKE_BINARY_DIR}/uranie.ps1 @ONLY)
    install(FILES ${CMAKE_BINARY_DIR}/uranie.ps1
        PERMISSIONS OWNER_WRITE OWNER_READ GROUP_READ WORLD_READ
        DESTINATION ${CMAKE_INSTALL_PREFIX})
endif()

endmacro(URANIE_WRITE_ENVIRONMENT_FILE)


#-------------------------------------------------------------------------
# DOCUMENTATION
#-------------------------------------------------------------------------
macro(URANIE_DOCUMENTATION)
    if(EXISTS ${URANIE_SOURCE_DIR}/html)
        message(STATUS "///   Dev documentation will be installed")
        install(DIRECTORY ${URANIE_SOURCE_DIR}/html DESTINATION ${CMAKE_INSTALL_PREFIX}/share/uranie)
    endif(EXISTS ${URANIE_SOURCE_DIR}/html)  
    if(EXISTS ${URANIE_SOURCE_DIR}/Index)
      message(STATUS "///   index documentation will be installed")
      install(DIRECTORY ${URANIE_SOURCE_DIR}/Index/ DESTINATION ${CMAKE_INSTALL_PREFIX}/share/uranie)
    endif(EXISTS ${URANIE_SOURCE_DIR}/Index)
    if(EXISTS ${URANIE_SOURCE_DIR}/docUMENTS)
      message(STATUS "///   PDF documentation will be installed")
      install(DIRECTORY ${URANIE_SOURCE_DIR}/docUMENTS DESTINATION ${CMAKE_INSTALL_PREFIX}/share/uranie)
    endif(EXISTS ${URANIE_SOURCE_DIR}/docUMENTS)
endmacro(URANIE_DOCUMENTATION)


#-------------------------------------------------------------------------
#   MACROS
# Install macros, the if condition stands for compilation from git extract,
# where macros directory doesn't exist
#-------------------------------------------------------------------------
macro(URANIE_MACROS)
    if(EXISTS ${URANIE_SOURCE_DIR}/macros)
        message(STATUS "///   Macros directory will be installed")
        install(DIRECTORY ${URANIE_SOURCE_DIR}/macros/ DESTINATION ${CMAKE_INSTALL_PREFIX}/share/uranie/macros)
#         if(UNIX)
#             install(DIRECTORY ${URANIE_SOURCE_DIR}/macros/ DESTINATION ${CMAKE_INSTALL_PREFIX}/share/uranie/macros)
#         endif(UNIX)
#         if(WIN32)
#             install(DIRECTORY ${URANIE_SOURCE_DIR}/macros/ DESTINATION ${ROOTSYS}/share/uranie/macros)
#         endif(WIN32)
    endif(EXISTS ${URANIE_SOURCE_DIR}/macros)
endmacro(URANIE_MACROS)


#-------------------------------------------------------------------------
# Install .dtd and .xslt files mandatory to use xmlPROBLEM
#-------------------------------------------------------------------------
macro(URANIE_XMLPROBLEM)
  message(STATUS "")
  message(STATUS "///   DTD and XSLT files  will be installed in ${CMAKE_INSTALL_PREFIX}/share/uranie")
  install(FILES xmlPROBLEM/souRCE/teST/uranie.dtd  DESTINATION ${CMAKE_INSTALL_PREFIX}/share/uranie/)
  install(FILES xmlPROBLEM/souRCE/teST/uranie.xslt DESTINATION ${CMAKE_INSTALL_PREFIX}/share/uranie/)  
endmacro(URANIE_XMLPROBLEM)


#-------------------------------------------------------------------------
#----URANIE_SHOW_OPTION()
# * Shows the options of the installation of Uranie
#-------------------------------------------------------------------------

macro(URANIE_SHOW_OPTION)

    message(STATUS "")
    message(STATUS "///////////////////////////////////////////////////")
    message(STATUS "///   URANIE")
    message(STATUS "///      VERSION:         ${URANIE_VERSION}")
    message(STATUS "///      INSTALL PATH:    ${CMAKE_INSTALL_PREFIX}")
    message(STATUS "///")


    ## Case of Vizir
    if(--disable-vizir STREQUAL "ON")
        message(STATUS "///   LIBRARY VIZIR WILL NOT BE COMPILED !!!")
    else(--disable-vizir STREQUAL "ON")
        message(STATUS "///   LIBRARY VIZIR WILL BE COMPILED!")
    endif(--disable-vizir STREQUAL "ON")

    ## Case of NISP
    if(--disable-NISP STREQUAL "ON")
        message(STATUS "///   LIBRARY NISP WILL NOT BE COMPILED !!!")
    else(--disable-NISP STREQUAL "ON")
        message(STATUS "///   LIBRARY NISP WILL BE COMPILED!")
    endif(--disable-NISP STREQUAL "ON")
    
    ## Case of ANISP
    if(--disable-ANISP STREQUAL "ON")
        message(STATUS "///   LIBRARY ANISP WILL NOT BE COMPILED !!!")
    else(--disable-ANISP STREQUAL "ON")
        message(STATUS "///   LIBRARY ANISP WILL BE COMPILED!")
    endif(--disable-ANISP STREQUAL "ON")
    
    # -- ngi --
    ## Case of PGauss
    if(--disable-PGAUSS STREQUAL "ON")
        message(STATUS "///   LIBRARY PGauss WILL NOT BE COMPILED !!!")
    else(--disable-PGAUSS STREQUAL "ON")
        message(STATUS "///   LIBRARY PGauss WILL BE COMPILED!")
    endif(--disable-PGAUSS STREQUAL "ON")
    # -- ngi --
    

    message(STATUS "///")
    if(LIBXML2_FOUND)
        message(STATUS "///   FIND LibXml2 :  Version ${LIBXML2_VERSION_STRING}")
    else(LIBXML2_FOUND)
        message(STATUS "///   LIBRARY LibXml2 NOT FOUND !!!")
    endif(LIBXML2_FOUND)
    
    message(STATUS "///   FIND ROOT :  Version ${ROOTVERSION}")
    if(HAS_MINUIT2)
        message(STATUS "///     - LIBRARY Minuit2 installed with ROOT")
    else(HAS_MINUIT2)
        message(STATUS "///     - LIBRARY Minuit2 NOT FOUND in ROOT: Install it with configure --enable-minuit2!")
    endif(HAS_MINUIT2)

    if(HAS_FFTW)
        message(STATUS "///     - LIBRARY FFTW3 installed with ROOT")
    else(HAS_FFTW)
        message(STATUS "///     - LIBRARY FFTW  NOT FOUND in ROOT")
    endif(HAS_FFTW)
    
    if(HAS_XML)
        message(STATUS "///     - XML interface installed with ROOT")
    else(HAS_XML)
        message(STATUS "///     - XML interface NOT FOUND in ROOT")
    endif(HAS_XML)

    if(CLUB_FOUND)
        message(STATUS "///   FIND    CLUB :  Version ${CLUBVERSION}")
    endif(CLUB_FOUND)

    if(OPT_FOUND)
        message(STATUS "///   FIND   OPT++ : ")
    else(OPT_FOUND)
        message(STATUS "///   LIBRARY OPT++ NOT FOUND !!!")
    endif(OPT_FOUND) 

    if(JSONCPP_FOUND)
        message(STATUS "///   FIND   JSONCPP : ")
    else(JSONCPP_FOUND)
        message(STATUS "///   LIBRARY JSONCPP NOT FOUND !!!")
    endif(JSONCPP_FOUND)

    if(FMILibrary_FOUND)
        message(STATUS "///   FIND   FMILibrary : ")
    else(FMILibrary_FOUND)
        message(STATUS "///   LIBRARY FMILibrary NOT FOUND !!!")
    endif(FMILibrary_FOUND)

    if(LIBSSH_FOUND)
        message(STATUS "///   FIND   LIBSSH : ")
    else(LIBSSH_FOUND)
        message(STATUS "///   LIBRARY LIBSSH NOT FOUND !!!")
    endif(LIBSSH_FOUND)

    if(CPPUNIT_FOUND)
        message(STATUS "///   FIND CPPUNIT :  Version ${CPPUNIT_VERSION}")
    else(CPPUNIT_FOUND)
        message(STATUS "///   LIBRARY CPPUNIT NOT FOUND !!!")
    endif(CPPUNIT_FOUND) 

    if(MPI_C_FOUND)
        message(STATUS "///   FIND   MPI_C :")
    else(MPI_C_FOUND)
        message(STATUS "///   LIBRARY MPI_C NOT FOUND !!!")
    endif(MPI_C_FOUND) 

    if(SUNSET_FOUND)
        message(STATUS "///   FIND SUNSET (IRSN) :")
    else(SUNSET_FOUND)
        message(STATUS "///   TOOL SUNSET (IRSN) NOT FOUND !!!")
    endif(SUNSET_FOUND)
    
    if(CUDA_FOUND)
        message(STATUS "///   FIND CUDA : Version ${CUDA_VERSION_STRING} - GPU capabilities will be used!")
    else()
        message(STATUS "///   LIBRARY CUDA NOT FOUND!!!")
    endif()

    message(STATUS "///")
    message(STATUS "///////////////////////////////////////////////////")
#-------------------------------------------------------------------------
endmacro(URANIE_SHOW_OPTION)


#-------------------------------------------------------------------------








