//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TAttributeFormula.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date    Fri Oct  1 13:37:23 2004
 
 $Revision$
 $Date$

 \brief  Fichier de d�finition de la classe TAttribut
 
 */

#include <iostream>

#include "TAttributeFormula.h"
using std::cout;
using std::endl;

ClassImp (URANIE::DataServer::TAttributeFormula)

namespace URANIE
{
namespace DataServer
{

// -----------------------------------------------------------------------
//  Constructors
// -----------------------------------------------------------------------
//       TAttributeFormula::TAttributeFormula() :
// 	TAttribute(),
// 	sformula(0)
//       {
// 	if ( _blog )
// 	  cout << "TAttributeFormula::TAttributeFormula" << endl;
//       }

TAttributeFormula::TAttributeFormula(TString str, TString formula) :
        TAttribute(str), sformula(formula)
{
    try
    {
        if (formula == TString(""))
        {
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    "TAttributeFormula constructor: No formula were entered");
        }
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        throw ue;
        exit(-1);
    }

    if (_blog)
        cout << "TAttributeFormula::TAttributeFormula[" << str << "]["
                << formula << "]" << endl;
}

TAttributeFormula::~TAttributeFormula()
{

}
// -----------------------------------------------------------------------
//  Getter and setter methods
// -----------------------------------------------------------------------
TString TAttributeFormula::getFormula()
{
    return sformula;
}

void TAttributeFormula::setFormula(TString str)
{
    this->sformula = str;
}

void TAttributeFormula::printLog()
{
    TAttribute::printLog();
    cout << "*******************************" << endl;
    cout << "** TAttributeFormula::printLog  ******" << endl;
    cout << "**   formula[" << sformula << "]" << endl;
    cout << "*******************************" << endl;

}
} // Fin du namespace DataServer
} // Fin du namespace URANIE

