//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef TAttributeFormula_H
#define TAttributeFormula_H
#include "TString.h"
#include <string>

/**
 
 * @class URANIE::DataServer::TAttributeFormula

 * @brief Description of a variable (field) by a formula.

 * Available informations are:
 * \li formula : the mathematical formula used to define the variable.

 */

// Uranie
#include "DataServer.h"
#include "TAttribute.h"
#include "UExceptions.h"

namespace URANIE
{
namespace DataServer
{
class TAttributeFormula: public TAttribute
{
protected:
    TString sformula;          ///< Formula

public:
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default Constructor
    /*! 
     
     */
    //TAttributeFormula();
    //! 
    /*! 
     
     \param name (TString) Name of the attribut
     \param formula (TString) Formula of the attribut
     */
    TAttributeFormula(TString name, TString formula = TString(""));
    virtual ~TAttributeFormula();
    //@}

    //---------------------------------------------
    /** @name Setting and Getting formula
     **/
    //@{
    //! Sets the formula to the attribute
    /*!
     
     @param name (TString) the formula to set 
     */
    void setFormula(TString name);
    //! Returns the formula of the attribute
    TString getFormula();
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void printLog();
    //@}
    ClassDef(URANIE::DataServer::TAttributeFormula, ID_DATASERVER)
    //Definition of an variable by a formula

};
} // Fin du namespace DataServer
} // Fin du namespace URANIE

#endif
