//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TGumbelDistribution.h
 \author Nicolas Gilardi <nicolas.gilardi@cea.fr>
 \date   Mon Jan 19 12:00:00 CET 2009

 \brief Interface of the class URANIE::DataServer::TGumbelDistribution

 */
/**

 * @class URANIE::DataServer::TGumbelDistribution
 *
 * @brief Description of the class TGumbelDistribution.
 *
 * To be written by the developper.
 *
 */
#ifndef TGUMBELDISTRIBUTION_H
#define TGUMBELDISTRIBUTION_H

//Uranie
#include "TStochAttribut.h"

namespace URANIE
{
namespace DataServer
{
class TGumbelDistribution: public TInfiniteDistribution
{
private:
protected:
    Double_t dscale; // Gumbel distribution scale (strictly positive)
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TGumbelDistribution();

    //! Constructor by name only
    /*! 
     
     \param name (TString) Name of the attribute
     */
    TGumbelDistribution(TString name);

    //! Constructor by name and statistics
    /*! 
     
     \param name (TString) Name of the attribute
     \param mu (Double_t) location (mode) of the attribute
     \param beta (Double_t) scale of the attribute (beta > 0)
     */
    TGumbelDistribution(TString name, Double_t mu, Double_t beta);

    //! Constructor by attribute and statistics
    /*! 
     
     \param att (URANIE::DataServer::TAttribute*) pointer to attribute
     \param mu (Double_t) location (mode) of the attribute
     \param beta (Double_t) scale of the attribute (beta > 0)
     */
    TGumbelDistribution(URANIE::DataServer::TAttribute *att, Double_t mu,
            Double_t beta);

    //! Default destructor
    virtual ~TGumbelDistribution();
    //@}
    //---------------------------------------------
    /** @name Getters and Setters methods
     Getting and setting attribute values.
     **/
    //@{
    //! Sets the location
    virtual void setLocation(Double_t mu);
    //! Sets the scale
    virtual void setScale(Double_t beta);
    //! Gets the location
    virtual Double_t getLocation();
    //! Gets the scale
    virtual Double_t getScale();
    //@}
    /** @name Statistical methods
     The statistical methods ( pdf, cdf, quantile, ...).
     **/
    //@{
    //! Computes the mean
    virtual void computeMean();
    //! Gives the PDF at a value
    virtual Double_t getPDF(Double_t x);
    //! Returns the inverse cumulative Density Function
    virtual Double_t getInvCDF(Double_t x);
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::DataServer::TGumbelDistribution, ID_DATASERVER)
    //Classe de
};
} // Fin du namespace DataServer
} // Fin du namespace URANIE
#endif
