//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TPatternsEventList.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   jeu avr 26 14:14:11 CEST 2007
 
 $Revision$
 $Date$
 
 \brief Interface of the class URANIE::DataServer::TPatternsEventList
 
 */
/*!
 @class URANIE::DataServer::TPatternsEventList

 @brief Description of the class TPatternsEventList.

 This classe hrits from the ROOT object TEventList.
 It permits to have several times the same element entry needed by a \b Bootstrap \b analysis.
 
 */

#ifndef TPATTERNSEVENTLIST
#define TPATTERNSEVENTLIST

// ROOT
#include "TNamed.h"
#include "TEventList.h"

// Uranie
#include "DataServer.h"

class TDirectory;
class TCollection;

namespace URANIE
{
namespace DataServer
{
class TPatternsEventList: public TEventList
{
    // Attributes
protected:
    Int_t _nSeed;                  ///< The seed parameter
public:
    Bool_t _blog;          /// log printing

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TPatternsEventList();
    //! Default constructor with a name, a title and the init size
    TPatternsEventList(const char *name, const char *title, Int_t initsize);

    //! Default destructor
    virtual ~TPatternsEventList();
    //@}

    //---------------------------------------------
    /** @name Manipulate the entries
     **/
    //@{
    //!  Enter element entry into the list
    virtual void Enter(Long64_t entry);
    //!  Enter element entry into the list
    /*!
     \see Enter
     */
    virtual void enter(Long64_t entry)
    {
        Enter(entry);
    }

    //!  Mix all the elements entry into the list
    void Mix();
    //!  Mix all the elements entry into the list
    /*!
     \see Mix
     */
    void mix()
    {
        Mix();
    }
    //@}

    //---------------------------------------------
    /** @name The Seed
     **/
    //@{
    //! Init the seed
    /*!
     If the value is zero, the seed is initialized with the hour in milliseconds
     
     \param ind (Int_t) the seed value
     */
    void setSeed(Int_t ind = 0);
    //! Returns the seed value
    Int_t getSeed()
    {
        return _nSeed;
    }
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    virtual void Print(Option_t *option = "") const;
    void printLog(Option_t *option = "")
    {
	(void)option;
        Print("all");
    }
    //@}

    ClassDef(TPatternsEventList, ID_DATASERVER)
    //A list of selected entries in a TTree.
};

} // Fin du namespace DataServer
} // Fin du namespace URANIE
#endif
