//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   UExceptions.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   jeu ao� 9 11:58:09 CEST 2007
 
 $Revision$
 $Date$
 
 \brief Implementation of the class URANIE::DataServer::UExceptions
 \par Modifications
 F. Gaudier (2010-08-06 10:11:32)
 - Add the \b Deprecated item between kINFO and kWARNIG

 */

#include "UExceptions.h"

ClassImp (URANIE::Exceptions::UExceptions)
ClassImp (URANIE::Exceptions::UWarningExceptions)
ClassImp (URANIE::Exceptions::UDeprecatedExceptions)
ClassImp (URANIE::Exceptions::UErrorExceptions)
ClassImp (URANIE::Exceptions::UInfoExceptions)

namespace URANIE
{
namespace Exceptions
{
//GA URANIE::DataServer::UMessageLogger * UExceptions::_fLogger = 0; // message logger

UExceptions::UExceptions() :
        _fLogger("Unknown"), _msg("No Message"), _msgtype(kERROR), _file("Unknown"), _line(-1)
{
}
UExceptions::UExceptions(TString str) :
        _fLogger(str.Data()), _msg(str), _msgtype(kERROR), _file("Unknown"), _line(-1)
{
}
UExceptions::UExceptions(TString file, Int_t line, TString msg) :
        _fLogger(file.Data()), _msg(msg), _msgtype(kERROR), _file(file), _line(line)
{
}

UExceptions::~UExceptions()
{
//     printLog();
//     printMessage();
//     cout << "detruit UExceptions" << endl;
}

void UExceptions::printLog(Option_t *option)
{
    cout << "UExceptions::printLog[" << option << "]" << endl;
    cout << "File[" << _file << "] Line[" << _line << "]" << endl;
    cout << "Message[" << _msg << "]" << endl;
}

void UExceptions::printMessage()
{
    cout << "*** URANIE ERROR ***" << endl << "*** File[" << _file << "] Line["
            << _line << "]" << endl << "Message[" << _msg << "]" << endl
            << "*** END of URANIE ERROR ***" << endl;
}

void UExceptions::setMessageType(UMsgType activeType, UMsgType minType)
{
    _msgtype = activeType;

    // The active type
    switch (_msgtype)
    {
    case kVERBOSE:
    {
        _fLogger << URANIE::DataServer::UMessageLogger::kVERBOSE;
        break;
    }
    case kDEBUG:
    {
        _fLogger << URANIE::DataServer::UMessageLogger::kDEBUG;
        break;
    }
    case kINFO:
    {
        _fLogger << URANIE::DataServer::UMessageLogger::kINFO;
        break;
    }
    case kDEPRECATED:
    {
        _fLogger << URANIE::DataServer::UMessageLogger::kDEPRECATED;
        break;
    }
    case kWARNING:
    {
        _fLogger << URANIE::DataServer::UMessageLogger::kWARNING;
        break;
    }
    case kERROR:
    {
        _fLogger << URANIE::DataServer::UMessageLogger::kERROR;
        break;
    }
    case kFATAL:
    {
        _fLogger << URANIE::DataServer::UMessageLogger::kFATAL;
        break;
    }
    case kALWAYS:
    {
        _fLogger << URANIE::DataServer::UMessageLogger::kALWAYS;
        break;
    }
    }

    // The minType (inside, no information is written in the log)
    switch (minType)
    {
    case kVERBOSE:
    {
        _fLogger.SetMinType(URANIE::DataServer::UMessageLogger::kVERBOSE);
        break;
    }
    case kDEBUG:
    {
        _fLogger.SetMinType(URANIE::DataServer::UMessageLogger::kDEBUG);
        break;
    }
    case kINFO:
    {
        _fLogger.SetMinType(URANIE::DataServer::UMessageLogger::kINFO);
        break;
    }
    case kDEPRECATED:
    {
        _fLogger.SetMinType(
                URANIE::DataServer::UMessageLogger::kDEPRECATED);
        break;
    }
    case kWARNING:
    {
        _fLogger.SetMinType(URANIE::DataServer::UMessageLogger::kWARNING);
        break;
    }
    case kERROR:
    {
        _fLogger.SetMinType(URANIE::DataServer::UMessageLogger::kERROR);
        break;
    }
    case kFATAL:
    {
        _fLogger.SetMinType(URANIE::DataServer::UMessageLogger::kFATAL);
        break;
    }
    case kALWAYS:
    {
        _fLogger.SetMinType(URANIE::DataServer::UMessageLogger::kALWAYS);
        break;
    }
    }
}




UWarningExceptions::~UWarningExceptions()
{
//     printLog();
//     cout << "detruit UWarningExceptions" << endl; 
}

void UWarningExceptions::printMessage()
{
      _fLogger << Endl<< "*** URANIE WARNING ***" << Endl
		  << "*** File[" << getFile() << "] Line[" << getLine() << "]" << Endl;
      if ( getMessage().Length() > 0 )
	_fLogger << getMessage() << Endl;
      _fLogger << "*** END of URANIE WARNING ***" << Endl<< Endl;
}

UDeprecatedExceptions::~UDeprecatedExceptions()
{
}
void UDeprecatedExceptions::printMessage()
{
	_fLogger << "*** URANIE Deprecated Method ***" << Endl<< "*** Deprecated function [" <<_sobject <<"::" << _smethod << "]" << Endl
		    << "*** From version v" << _sfromVersion << " to v" << _slastVersion << Endl;
	if ( getMessage().Length() > 0 )
	  _fLogger << "*** " << getMessage() << Endl;
	_fLogger << "*** END of Deprecated Method ***" << Endl;
}

UErrorExceptions::~UErrorExceptions()
{
}
void UErrorExceptions::printMessage()
{
      _fLogger << Endl<< "*** URANIE ERROR ***" << Endl
		  << "*** File[" << getFile() << "] Line[" << getLine() << "]" << Endl;
      if ( getMessage().Length() > 0 )
	_fLogger << getMessage() << Endl;
      _fLogger << "*** END of URANIE ERROR ***" << Endl<< Endl;
}

UInfoExceptions::~UInfoExceptions()
{
}
void UInfoExceptions::printMessage()
{
      _fLogger << Endl<<"*** URANIE INFORMATION ***" << Endl
		  << "*** File[" << getFile() << "] Line[" << getLine() << "]" << Endl;
      if ( getMessage().Length() > 0 )
	_fLogger << getMessage() << Endl;
      _fLogger << "*** END of URANIE INFORMATION ***" << Endl<< Endl;
}



} // Fin du namespace Exceptions
} // Fin du namespace URANIE

