//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestQuantilesEstimation.cxx
 \author Maud Salvoldi <Areva/OCC>
 \date   jeu jui  7 10:56:35 2011
 
 \brief   Fichier de test des nouvelles methodes d'estimation du quantile
 implement�es dans la classe URANIE::DataServer::TDataServer
 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-5

// Uranie
#include "TestQuantilesEstimation.h"
using namespace std;

QuantilesEstimationTest::QuantilesEstimationTest() :
        _tds(NULL), _tdsza(NULL), _dZa(1.0), _dYa(1.0), _dRho(0.1)
{
}

QuantilesEstimationTest::~QuantilesEstimationTest()
{
}

void QuantilesEstimationTest::setUp()
{
    _tds = new URANIE::DataServer::TDataServer("tds_flowrate", "Ex. Flowrate");
    _tds->fileDataRead(TString("TDS_Flowrate_100Zj.dat"));
}

void QuantilesEstimationTest::tearDown()
{
    if (_tds)
        delete _tds;
    _tds = NULL;
    if (_tdsza)
        delete _tdsza;
    _tdsza = NULL;

}

void QuantilesEstimationTest::throwExceptioncomputeQuantileBadAlpha()
{
    CPPUNIT_ASSERT_THROW(
            _tds->computeQuantileCV("Y", -1.5, "Zj", _dZa, _dYa, _dRho),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            _tds->computeQuantileCV("Y", 0.0, "Zj", _dZa, _dYa, _dRho),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            _tds->computeQuantileCV("Y", 1.0, "Zj", _dZa, _dYa, _dRho),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            _tds->computeQuantileCV("Y", 1.5, "Zj", _dZa, _dYa, _dRho),
            URANIE::Exceptions::UErrorExceptions);
}

void QuantilesEstimationTest::throwExceptioncomputeQuantileFalseYname()
{
    CPPUNIT_ASSERT_THROW(
            _tds->computeQuantileCV("toto", 0.5, "Zj", _dZa, _dYa, _dRho),
            URANIE::Exceptions::UErrorExceptions);
}

void QuantilesEstimationTest::throwExceptioncomputeQuantileFalseZname()
{
    CPPUNIT_ASSERT_THROW(
            _tds->computeQuantileCV("Y", 0.5, "toto", _dZa, _dYa, _dRho),
            URANIE::Exceptions::UErrorExceptions);
}

void QuantilesEstimationTest::computeQuantileCV()
{

    _tdsza = new URANIE::DataServer::TDataServer("tds_Z",
            "tds de la variable de controle");
    _tdsza->fileDataRead(TString("TDS_Flowrate_zaZ.dat"));

    Double_t dAlpha = 0.95;

    _tdsza->computeQuantile("Z", dAlpha, _dZa);
    _tds->computeQuantileCV("Y", dAlpha, "Zj", _dZa, _dYa, _dRho);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(162.1347738, _dYa, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.05263157894736, _dRho,
            DBL_PRECISION_COMPARE);

    dAlpha = 0.98;
    _tdsza->computeQuantile("Z", dAlpha, _dZa);
    _tds->computeQuantileCV("Y", dAlpha, "Zj", _dZa, _dYa, _dRho);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(177.8582192, _dYa, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.02040816326532, _dRho,
            DBL_PRECISION_COMPARE);

    dAlpha = 0.05;
    _tdsza->computeQuantile("Z", dAlpha, _dZa);
    _tds->computeQuantileCV("Y", dAlpha, "Zj", _dZa, _dYa, _dRho);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(19.22124903, _dYa, DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(17.7661088204382, _dRho,
            DBL_PRECISION_COMPARE);

}

// void QuantilesEstimationTest::throwExceptioncomputeQuantileSRBadAlpha()
// {
//   _Alpha=0.0;
//   TString funct;
//   funct=TString("SR_LR");
//   CPPUNIT_ASSERT_THROW(_tds->computeQuantileCV("Y", _Alpha,funct, _dYa, _rho),URANIE::Exceptions::UErrorExceptions);
// }

//void QuantilesEstimationTest::throwExceptioncomputeQuantileSRFalseYname()
// {
//   TString funct;
//   funct=TString("SR_LR");
//   CPPUNIT_ASSERT_THROW(_tds->computeQuantileCV("toto", _Alpha, funct,  _dYa, _rho),URANIE::Exceptions::UErrorExceptions);
// }

// void QuantilesEstimationTest::throwExceptioncomputeQuantileFalseSR()
// {
//   TString funct;
//   funct=TString("toto");

//   CPPUNIT_ASSERT_THROW(_tds->computeQuantileCV("Y", _Alpha,funct,  _dYa, _rho),URANIE::Exceptions::UErrorExceptions); 
// }

//   void QuantilesEstimationTest::computeQuantileCVwithSR()
// {
//   TString funct;
//   funct=TString("SR_LR");

//   _tdstmp = new  URANIE::DataServer::TDataServer("tds_flowrate","test de validation Flowrate - 100 patterns");
//   _tdstmp->addAttribute( new  URANIE::DataServer::TUniformDistribution("rw", 0.05, 0.15));
//   _tdstmp->addAttribute( new  URANIE::DataServer::TUniformDistribution("r",  100.0,  50000.0));
//   _tdstmp->addAttribute( new  URANIE::DataServer::TUniformDistribution("tu", 63070.0,115600.0));
//   _tdstmp->addAttribute( new  URANIE::DataServer::TUniformDistribution("tl", 63.1,   116.0));
//   _tdstmp->addAttribute( new  URANIE::DataServer::TUniformDistribution("hu", 990.0,  1110.0));
//   _tdstmp->addAttribute( new  URANIE::DataServer::TUniformDistribution("hl", 700.0,  820.0));
//   _tdstmp->addAttribute( new  URANIE::DataServer::TUniformDistribution("l",  1120.0, 1680.0));
//   _tdstmp->addAttribute( new  URANIE::DataServer::TUniformDistribution("kw", 9855.0, 12045.0));
//   _tdstmp->setTuple(_tuple);

//   _Alpha=0.95;
//   _tlfy = new  URANIE::Launcher::TLauncherFunction(_tdstmp, TString("FctFlowrate"),"rw:r:tu:tl:hu:hl:l:kw","Y");
//   _tlfy->run();
//   _tdstmp->computeQuantileCV("Y", _Alpha, funct, _dYa2, _rho2);
//    CPPUNIT_ASSERT_DOUBLES_EQUAL(_dYa2 , 163.50, 6.0);
//    CPPUNIT_ASSERT_DOUBLES_EQUAL(_rho2 , 1.50, 2.0);

//   _Alpha=0.98;
//   _tlfyy = new  URANIE::Launcher::TLauncherFunction(_tdstmp, TString("FctFlowrate"),"rw:r:tu:tl:hu:hl:l:kw","YY");
//   _tlfyy->run();
//   _tdstmp->computeQuantileCV("YY", _Alpha, funct, _dYa2, _rho2);
//   CPPUNIT_ASSERT_DOUBLES_EQUAL(_dYa2 , 179.50, 6.0);
//     CPPUNIT_ASSERT_DOUBLES_EQUAL(_rho2 , 2.20, 2.0);

//   _Alpha=0.05;
//   _tlfyyy = new  URANIE::Launcher::TLauncherFunction(_tdstmp, TString("FctFlowrate"),"rw:r:tu:tl:hu:hl:l:kw","YYY");
//   _tlfyyy->run();
//   _tdstmp->computeQuantileCV("YYY", _Alpha, funct, _dYa2, _rho2);
//   CPPUNIT_ASSERT_DOUBLES_EQUAL(_dYa2 , 19.20, 2.0);
//   CPPUNIT_ASSERT_DOUBLES_EQUAL(_rho2 , 17.20, 1.0);
// }

