//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTBetaDistribution.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TBetaDistribution
 
 Created: 2009-12-08 09:14:10
 
 */
#ifndef CPPUNITEST_TBETADISTRIBUTION_H
#define CPPUNITEST_TBETADISTRIBUTION_H

//CPPUNIT
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

//URANIE
#include "TBetaDistribution.h"
#include "TStochAttribut.h"
#include "UExceptions.h"

class TBetaDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TBetaDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (testOtherLaw);
    CPPUNIT_TEST (testSetParameters);
    CPPUNIT_TEST (testSetParameterBeta);
    CPPUNIT_TEST (testSetParameterAlpha);
    CPPUNIT_TEST (throwExceptionConstructorWithBadBoundaries);
    CPPUNIT_TEST (throwExceptionConstructorWithNegativeAlpha);
    CPPUNIT_TEST (throwExceptionConstructorWithNegativeBeta);
    CPPUNIT_TEST (throwExceptionSetParametersWithNegativeAlpha);
    CPPUNIT_TEST (throwExceptionSetParametersWithNegativeBeta);
    CPPUNIT_TEST (throwExceptionSetParameterBetaWithNegativeBeta);
    CPPUNIT_TEST (throwExceptionSetParameterAlphaWithNegativeAlpha);CPPUNIT_TEST_SUITE_END();

public:
    TBetaDistributionTest();

    /// Destructor.
    virtual ~TBetaDistributionTest();

    void setUp();
    void tearDown();
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Test the default constructor with three beta laws : Beta(0.5, 0.5) on [0,1], on [1,2] and [-1, 4]
    void testDefaultLaw();
    //! Test the constructor with a specif law parameter 3.0, 2.0 with three beta laws : Beta(0.5, 0.5) on [0,1], on [1,2] and [-1, 4]
    void testOtherLaw();
    //! Tests setParameters() method.
    void testSetParameters();
    //! Tests setParameterBeta() method
    void testSetParameterBeta();
    //! Tests setParameterAlpha() method
    void testSetParameterAlpha();

    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Throws an exception when the given upperbound is inferior to the given lowerboundreading twice a ASCII File and the dataserver isn't not empty
    void throwExceptionConstructorWithBadBoundaries();
    //! Throws an exception when given alpha is negative
    void throwExceptionConstructorWithNegativeAlpha();
    //! Throws an exception when given beta is negative
    void throwExceptionConstructorWithNegativeBeta();
    //! Throws an exception when boundries are not set in setParameters() method
    void throwExceptionSetParametersWithBadBoundaries();
    //! Throws an exception when alpha is negative in setParameters() method  
    void throwExceptionSetParametersWithNegativeAlpha();
    //! Throws an exception when beta is negative in setParameters() method    
    void throwExceptionSetParametersWithNegativeBeta();
    //! Throws an exception when beta is negative in setBeta() method    
    void throwExceptionSetParameterBetaWithNegativeBeta();
    //! Throws an exception when Alpha is negative in setAlpha() method    
    void throwExceptionSetParameterAlphaWithNegativeAlpha();

    //@}
private:
    URANIE::DataServer::TBetaDistribution *_dproba;
};

#endif  // CPPUNITEST_TBETADISTRIBUTION_H
