//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTDSNtupleD.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TDSNtupleDTest
 
 Cr�e le : mer june 16 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TDSNTUPLED_H
#define CPPUNITEST_TDSNTUPLED_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TDSNtupleD.h"
#include "TAttribute.h"
#include "UExceptions.h"
#include "TNtupleD.h"
#include "TLeaf.h"

/*! \class TDSNtupleDTest
 * \brief Unit tests for TDSNtupleDTest.
 */

class TDSNtupleDTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TDSNtupleDTest);
    CPPUNIT_TEST (testTDSNtupleDConstructorFromTuple);
    CPPUNIT_TEST (testTDSNtupleDConstructorFromHomemadeTuple);
    CPPUNIT_TEST (testTDSNtupleDConstructorWithVarlist);
    CPPUNIT_TEST (testTDSNtupleDConstructorWithMatrix);
    CPPUNIT_TEST (testTDSNtupleDFillWithString);
    CPPUNIT_TEST (testTDSNtupleDFillDoubleMethod);
    CPPUNIT_TEST (testGetBranchData);
    CPPUNIT_TEST (testCopyBranchData);
    CPPUNIT_TEST (testExtractData);
    CPPUNIT_TEST (testReadFile);
    CPPUNIT_TEST (throwExceptionGetBranchData);
    CPPUNIT_TEST (throwExceptionCopyBranchData);
    CPPUNIT_TEST (throwExceptionExtractData);
    CPPUNIT_TEST (throwExceptionConstructWithMatrix);
    CPPUNIT_TEST (throwExceptionConstructNameAndTypeNotMatching);
    CPPUNIT_TEST (throwExceptionFillDoublesWhenStringOrVectorInTree);
    CPPUNIT_TEST (throwExceptionFillDoubleArrayWhenStringOrVectorInTree);
    CPPUNIT_TEST (throwExceptionMergerNotSameNumberOfEntries);
    CPPUNIT_TEST (throwExceptionFillWithStringArgumentProblem);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TDSNtupleDTest object.
     */
    TDSNtupleDTest();

    /// Destructor.
    virtual ~TDSNtupleDTest();

    void setUp();
    void tearDown();

    URANIE::DataServer::TDSNtupleD *_tdsntuple;
    
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests the constructor built from a NTupleD.
    void testTDSNtupleDConstructorFromTuple();

    //! Tests the constructor built from a TDSNTupleD with strings and vectors, saved with saveTuple.
    void testTDSNtupleDConstructorFromHomemadeTuple();

    //! Tests the constructor built with a list of variables.
    void testTDSNtupleDConstructorWithVarlist();

    //! Tests the constructor built with a list of variables and a matrix of data.
    void testTDSNtupleDConstructorWithMatrix();

    //! Tests the fillingmethod <ith a string
    void testTDSNtupleDFillWithString();

    //! Tests all the filing methods with doubles
    void testTDSNtupleDFillDoubleMethod();

    //! Tests the getBranchData() method.
    void testGetBranchData();

    //! Tests the copyBranchData() method.
    void testCopyBranchData();

    //! Tests the extractData() method.
    void testExtractData();

    //! Tests the readFile() method.
    void testReadFile();

    //! Throws an exception if we try to get an inexisting data branch or a string/vector branch
    void throwExceptionGetBranchData();

    //! Throws an exception if we try to get an inexisting data branch, or a string/vector branch, or if the array can't store all the data.  
    void throwExceptionCopyBranchData();

    //! Throws an exception if we try to construct a tdsntupleD with a matrix whose number of branch is larger than the one provided in the string
    void throwExceptionConstructWithMatrix();

    //! Throws an exception if we try to construct a tdnstupleD using a list of name and types with not same amount of caracters
    void throwExceptionConstructNameAndTypeNotMatching();

    //! Throws an exception if we try to fill tree with list of doubles and the tree contains either vector or string
    void throwExceptionFillDoublesWhenStringOrVectorInTree();

    //! Throws an exception if we try to fill tree with an array of double and the tree contains either vector or string
    void throwExceptionFillDoubleArrayWhenStringOrVectorInTree();

    //! Throws an exception if we try to get an inexisting data branch, or if the array can't store all the data. 
    void throwExceptionExtractData();

    //! Throws an exception if we try to fill with a string whose number of field not equal to number of branches
    void throwExceptionFillWithStringArgumentProblem();

    //! Throw an excepetion when both TDSNtupleD don't have the same number of entries
    void throwExceptionMergerNotSameNumberOfEntries();
    //@}

};
#endif  //CPPUNITEST_TDSNTUPLED_H

