//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTDataSpecification.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TDataSpecification
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2010/04/26
 Ajout des m�thodes
 // *****************************************************************************

 */

#include "TestTDataSpecification.h"
#include "TString.h"

#include <iostream>
using namespace std;

TDataSpecificationTest::TDataSpecificationTest()
{
}

TDataSpecificationTest::~TDataSpecificationTest()
{
}

void TDataSpecificationTest::setUp()
{
    tdspecBase = new URANIE::DataServer::TDataSpecification();
    tdspec = new URANIE::DataServer::TDataSpecification("tdspec",
            "Title of TDataSpecification");
    x = new URANIE::DataServer::TAttribute("x");
    y = new URANIE::DataServer::TAttribute("y");
    z = new URANIE::DataServer::TAttribute("z");
}

void TDataSpecificationTest::tearDown()
{
    delete tdspec;
    tdspec = NULL;
    delete tdspecBase;
    tdspecBase = NULL;
    delete x;
    x = NULL;
    delete y;
    y = NULL;
    delete z;
    z = NULL;
}

////////////////////////////////
///                          ///
///         Tests            ///
///                          ///
////////////////////////////////

void TDataSpecificationTest::testTDataSpecificationAttributes()
{
    CPPUNIT_ASSERT(tdspecBase->GetName() == TString("_header_"));
    CPPUNIT_ASSERT(tdspecBase->GetTitle() == TString("_title_"));
    CPPUNIT_ASSERT(tdspec->GetName() == TString("tdspec"));
    CPPUNIT_ASSERT(
            tdspec->GetTitle() == TString("Title of TDataSpecification"));
}

void TDataSpecificationTest::testGetClassName()
{
    CPPUNIT_ASSERT(tdspec->GetClassName() == TString("TDataSpecification"));
}

void TDataSpecificationTest::testGetSetRelationName()
{
    CPPUNIT_ASSERT(tdspecBase->getRelationName() == TString("noname"));
    tdspecBase->setRelationName(TString("New Relation Name"));
    CPPUNIT_ASSERT(
            tdspecBase->getRelationName() == TString("New Relation Name"));

    CPPUNIT_ASSERT(
            tdspec->getRelationName()
                    == TString("Title of TDataSpecification"));
    tdspec->setRelationName(TString("New Relation Name"));
    CPPUNIT_ASSERT(tdspec->getRelationName() == TString("New Relation Name"));
}

void TDataSpecificationTest::testGetVariablesNames()
{
    CPPUNIT_ASSERT(tdspec->getVariablesNames() == "");
    URANIE::DataServer::TAttribute *att = new URANIE::DataServer::TAttribute("");
    att->delShare();
    tdspec->addAttribute(att);
    CPPUNIT_ASSERT(tdspec->getVariablesNames() == "");
    tdspec->delAttribute("");
    tdspec->addAttribute(x);
    CPPUNIT_ASSERT(tdspec->getVariablesNames() == "x");
    tdspec->addAttribute(y);
    tdspec->addAttribute(z);
    CPPUNIT_ASSERT(tdspec->getVariablesNames() == TString("x:y:z"));
    CPPUNIT_ASSERT(tdspec->getVariablesNames("|") == TString("x|y|z"));
}

void TDataSpecificationTest::testGetNAttributes()
{
    CPPUNIT_ASSERT_EQUAL(0, tdspec->getNAttributes());
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT_EQUAL(2, tdspec->getNAttributes());
}

void TDataSpecificationTest::testIsAttribute()
{
    tdspec->addAttribute(x);
    CPPUNIT_ASSERT(tdspec->isAttribute("x") == kTRUE);
    CPPUNIT_ASSERT(tdspec->isAttribute("y") == kFALSE);
    CPPUNIT_ASSERT(tdspec->isAttribute("") == kFALSE);
}

void TDataSpecificationTest::testAddAndDelAttribute()
{
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT(tdspec->getNAttributes() == 2);
    tdspec->delAttribute("x");
    CPPUNIT_ASSERT(tdspec->getNAttributes() == 1);
    CPPUNIT_ASSERT(tdspec->isAttribute("x") == kFALSE);
    tdspec->delAttribute("y");
    CPPUNIT_ASSERT(tdspec->getNAttributes() == 0);
    CPPUNIT_ASSERT(tdspec->isAttribute("y") == kFALSE);
}

void TDataSpecificationTest::testGetAttribute()
{
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT(x == tdspec->getAttribute(TString("x")));
    CPPUNIT_ASSERT(y == tdspec->getAttribute(TString("y")));
    CPPUNIT_ASSERT(x == tdspec->getAttribute(0));
    CPPUNIT_ASSERT(y == tdspec->getAttribute(1));
}

void TDataSpecificationTest::testGetAttributeIndex()
{
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT_EQUAL(tdspec->getAttributeIndex(x), 0);
    CPPUNIT_ASSERT_EQUAL(tdspec->getAttributeIndex(y), 1);
    CPPUNIT_ASSERT_EQUAL(tdspec->getAttributeIndex("x"), 0);
    CPPUNIT_ASSERT_EQUAL(tdspec->getAttributeIndex("y"), 1);
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TDataSpecificationTest::throwExceptionAddAttribute()
{
    tdspec->addAttribute(x);
    CPPUNIT_ASSERT_THROW(tdspec->addAttribute(x),
            URANIE::Exceptions::UErrorExceptions);
}

void TDataSpecificationTest::throwExceptionDelAttribute()
{
    CPPUNIT_ASSERT_THROW(tdspec->delAttribute(TString("x")),
            URANIE::Exceptions::UErrorExceptions);
}

void TDataSpecificationTest::throwExceptionGetAttributeWithBigIndex()
{
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT_THROW(tdspec->getAttribute(3),
            URANIE::Exceptions::UErrorExceptions);
}

void TDataSpecificationTest::throwExceptionGetAttributeWithNegativeIndex()
{
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT_THROW(tdspec->getAttribute(-5),
            URANIE::Exceptions::UErrorExceptions);
}

void TDataSpecificationTest::throwExceptionGetAttributeWithName()
{
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT_THROW(tdspec->getAttribute(TString("z")),
            URANIE::Exceptions::UErrorExceptions);
}

void TDataSpecificationTest::throwExceptionGetAttributeIndexWithName()
{
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT_THROW(tdspec->getAttributeIndex(TString("z")),
            URANIE::Exceptions::UErrorExceptions);
}

void TDataSpecificationTest::throwExceptionGetAttributeIndexWithAttribute()
{
    tdspec->addAttribute(x);
    tdspec->addAttribute(y);
    CPPUNIT_ASSERT_THROW(
            tdspec->getAttributeIndex(z),
            URANIE::Exceptions::UErrorExceptions);
}
