//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTDataSpecification.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TDataSpecification
 
 Cr�e le : mar avri 26 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TDATASPECIFICATION_H
#define CPPUNITEST_TDATASPECIFICATION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TDataSpecification.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TDataSpecificationTest
 * \brief Unit tests for TDataSpecification.
 */
class TDataSpecificationTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TDataSpecificationTest);
    CPPUNIT_TEST (testTDataSpecificationAttributes);
    CPPUNIT_TEST (testGetClassName);
    CPPUNIT_TEST (testGetSetRelationName);
    CPPUNIT_TEST (testGetVariablesNames);
    CPPUNIT_TEST (testGetNAttributes);
    CPPUNIT_TEST (testIsAttribute);
    CPPUNIT_TEST (testAddAndDelAttribute);
    CPPUNIT_TEST (testGetAttribute);
    CPPUNIT_TEST (testGetAttributeIndex);
    CPPUNIT_TEST (throwExceptionAddAttribute);
    CPPUNIT_TEST (throwExceptionDelAttribute);
    CPPUNIT_TEST (throwExceptionGetAttributeWithBigIndex);
    CPPUNIT_TEST (throwExceptionGetAttributeWithNegativeIndex);
    CPPUNIT_TEST (throwExceptionGetAttributeWithName);
    CPPUNIT_TEST (throwExceptionGetAttributeIndexWithName);
    CPPUNIT_TEST (throwExceptionGetAttributeIndexWithAttribute);CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TDataSpecificationTest object.
     */
    TDataSpecificationTest();

    /// Destructor.
    virtual ~TDataSpecificationTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests basic attributes for constructors.
    void testTDataSpecificationAttributes();

    //! Tests that the name returned by the GetClassName() method is "TDataSpecification".
    void testGetClassName();

    //! Tests the good behaviour of Get(Set)RelationName() methods. 
    void testGetSetRelationName();

    //! Tests the good behaviour of the addAttribute() and delAttributes() methods for a TDataSpecification.
    void testAddAndDelAttribute();

    //! Tests the good behaviour of the getNAttributes() method for a TDataSpecification.
    void testGetNAttributes();

    //! Tests the getVariablesNames() method to retrieve the expected string of variables.
    void testGetVariablesNames();

    //! Tests the isAttribute() method.
    void testIsAttribute();

    //! Tests the getAttribute() methods.
    void testGetAttribute();

    //! Tests the getAttributeIndex() methods (attribute or TString as argument).
    void testGetAttributeIndex();

    //! Tests that throw exception is returned by addAttribute method when a same attribute is addes more than one time.
    void throwExceptionAddAttribute();

    //! Tests that throw exception is returned by delAttribute() method when trying to delete a non existing TAttribute.
    void throwExceptionDelAttribute();

    //! Tests that throw exception is returned by getAttribute(Int_t) method when trying to get a non existing TAttribute in the TDataSpecification.
    void throwExceptionGetAttributeWithBigIndex();

    //! Tests that throw exception is returned by getAttribute(Int_t) method when trying to get an TAttribute through a negative index in the TDataSpecification.
    void throwExceptionGetAttributeWithNegativeIndex();

    //! Tests that throw exception is returned by getAttribute(TString) method when trying to get a non existing TAttribute in the TDataSpecification.
    void throwExceptionGetAttributeWithName();

    //! Tests that throw exception is returned by getAttributeIndex() method when trying the index of a non existing TAttribute in the TDataSpecification.
    void throwExceptionGetAttributeIndex();

    //! Tests that throw exception is returned by getAttributeIndex() method when trying to get the index of a non existing TAttribute in the TDataSpecification by its name.
    void throwExceptionGetAttributeIndexWithName();

    //! Tests that throw exception is returned by getAttributeIndex() method when trying to get the index of a non existing TAttribute in the TDataSpecification.
    void throwExceptionGetAttributeIndexWithAttribute();

    //@}

private:
    URANIE::DataServer::TDataSpecification * tdspec, *tdspecBase;
    URANIE::DataServer::TAttribute * x;
    URANIE::DataServer::TAttribute * y;
    URANIE::DataServer::TAttribute * z;
};

#endif  //CPPUNITEST_TDATASPECIFICATION_H
