//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTExponentialDistribution.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TExponentialDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.  
 \li Cedric Forgues (C.S.) 2010-11-19
 Tests testExponentialSetParameterLambdaRecomputing(), testExponentialSetParameterModeRecomputing() and testExponentialSetParametersRecomputing() were added.
 \li Cedric Forgues (C.S.) 2010-06-16
 Ajout des m�thodes de la DI 49
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

#include "TestTInfiniteDistribution.h"
#include "TestTExponentialDistribution.h"
#include "TString.h"

#include <iostream>
using namespace std;

TExponentialDistributionTest::TExponentialDistributionTest()
{
}

TExponentialDistributionTest::~TExponentialDistributionTest()
{
}

void TExponentialDistributionTest::setUp()
{
    _att = new URANIE::DataServer::TAttribute("exponential", 0.0, 10.0);
    _att2 = new URANIE::DataServer::TAttribute("exponential");
    _dproba = new URANIE::DataServer::TExponentialDistribution("exponential");
}

void TExponentialDistributionTest::tearDown()
{
    delete _att;
    _att = NULL;
    delete _att2;
    _att2 = NULL;
    delete _dproba;
    _dproba = NULL;
}

void TExponentialDistributionTest::testDefaultLaw()
{
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kExponential);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLaguerre);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1, _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0975309912028333, _dproba->getPDF(0.25),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.090483741803596, _dproba->getPDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0367879441171442, _dproba->getPDF(10.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0246900879716674, _dproba->getCDF(0.25),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0951625819640405, _dproba->getCDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.632120558828558, _dproba->getCDF(10.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.05360515657826, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.10825623765991, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.93147180559945, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.16290731874155, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(23.0258509299405, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TExponentialDistribution("exponential",
            1.0);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kExponential);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLaguerre);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.778800783071405, _dproba->getPDF(0.25),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.367879441171442, _dproba->getPDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.53999297624849e-05, _dproba->getPDF(10.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.221199216928595, _dproba->getCDF(0.25),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.632120558828558, _dproba->getCDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999954600070238, _dproba->getCDF(10.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.105360515657826, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.510825623765991, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.693147180559945, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.916290731874155, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.30258509299405, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TExponentialDistribution(_att, 1.0);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kExponential);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLaguerre);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.778800783071405, _dproba->getPDF(0.25),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.367879441171442, _dproba->getPDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.53999297624849e-05, _dproba->getPDF(10.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.221199216928595, _dproba->getCDF(0.25),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.632120558828558, _dproba->getCDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999954600070238, _dproba->getCDF(10.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.105360515657826, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.510825623765991, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.693147180559945, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.916290731874155, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.30258509299405, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
}

void TExponentialDistributionTest::testOtherLaw()
{
  delete _dproba;
  _dproba = new URANIE::DataServer::TExponentialDistribution("exponential",
            2.0, -2.);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kExponential);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLaguerre);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2., _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2., _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-2.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getPDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.735758882342885, _dproba->getPDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00495750435333272, _dproba->getPDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(7.5502690885582e-11, _dproba->getPDF(10.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.632120558828558, _dproba->getCDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.997521247823334, _dproba->getCDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999999999962249, _dproba->getCDF(10.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.94731974217109, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.744587188117, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.65342640972003, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.54185463406292, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.848707453502977, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TExponentialDistribution(_att2, 2.0, -2.);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kExponential);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLaguerre);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2., _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2., _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2., _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-2.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getPDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.735758882342885, _dproba->getPDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00495750435333272, _dproba->getPDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(7.5502690885582e-11, _dproba->getPDF(10.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.632120558828558, _dproba->getCDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.997521247823334, _dproba->getCDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999999999962249, _dproba->getCDF(10.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.94731974217109, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.744587188117, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.65342640972003, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.54185463406292, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.848707453502977, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TExponentialDistribution(_att, 2.0, -2.);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kExponential);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kLaguerre);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2., _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2., _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-2.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00495750435333272, _dproba->getPDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(7.5502690885582e-11, _dproba->getPDF(10.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.997521247823334, _dproba->getCDF(1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999999999962249, _dproba->getCDF(10.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.94731974217109, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.744587188117, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.65342640972003, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.54185463406292, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.848707453502977, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
}

void TExponentialDistributionTest::testExponentialSetParameterLambdaRecomputing()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1, _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterLambda(2.0, kTRUE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
}

void TExponentialDistributionTest::testExponentialSetParameterModeRecomputing()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameterMode(-2.0, kTRUE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(8., _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
}

void TExponentialDistributionTest::testExponentialSetParametersRecomputing()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1, _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(10., _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    _dproba->setParameters(2.0, -2.0, kTRUE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterLambda(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getParameterMode(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
}

static void truncation_set(unordered_map<string, double>& references) {
    references["rate"] = 1.;

    references["x_min_default"] = 0.;
    references["x_min_default_prev"] = nextafter(references.at("x_min_default"), -HUGE_VAL);
    references["x_min"] = 0.4;
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = 6.;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // ROOT::Math::exponential_cdf()
    references["pmin_default"] = 0.;
    references["pmin"] = 3.2967995396436073e-01;
    references["pmax"] = 9.9752124782333362e-01;
}

void TExponentialDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TExponentialDistribution distribution("exponential", references.at("rate"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}

void TExponentialDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TExponentialDistribution distribution("exponential", references.at("rate"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}

void TExponentialDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TExponentialDistribution distribution("exponential", references.at("rate"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}

void TExponentialDistributionTest::throwExceptionWithNegativeLambda()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TExponentialDistribution("exponential",
                    -1.0), URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TExponentialDistribution(_att, -1.0),
            URANIE::Exceptions::UErrorExceptions);
}

