//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTExponentialDistribution.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TExponentialDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TEXPONENTIALDISTRIBUTION_H
#define CPPUNITEST_TEXPONENTIALDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TExponentialDistribution.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TExponentialDistributionTest
 * \brief Unit tests for TExponentialDistribution.
 */
class TExponentialDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TExponentialDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (testOtherLaw);
    CPPUNIT_TEST (testExponentialSetParameterLambdaRecomputing);
    CPPUNIT_TEST (testExponentialSetParameterModeRecomputing);
    CPPUNIT_TEST (testExponentialSetParametersRecomputing);
    CPPUNIT_TEST (testTruncationBounds);
    CPPUNIT_TEST (testTruncationLowerBound);
    CPPUNIT_TEST (testTruncationUpperBound);
    CPPUNIT_TEST (throwExceptionWithNegativeLambda);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TExponentialDistributionTest object.
     */
    TExponentialDistributionTest();

    /// Destructor.
    virtual ~TExponentialDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests the default parameters for an exponential distribution.
    void testDefaultLaw();

    //! Tests Exponential Distribution with set parameters (set lambda and min).
    void testOtherLaw();

    //! Tests if mean and standard deviation are recomputed if specified in setParameterLambda() method.
    void testExponentialSetParameterLambdaRecomputing();

    //! Tests if mean and standard deviation are recomputed if specified in setParameterMode() method.
    void testExponentialSetParameterModeRecomputing();

    //! Tests if mean and standard deviation are recomputed if specified in setParameters() method.
    void testExponentialSetParametersRecomputing();

    //! Tests truncation on expnentila distribution.
    void testTruncationBounds();
    void testTruncationLowerBound();
    void testTruncationUpperBound();

    //! Throws an exception when lambda is negative. 
    void throwExceptionWithNegativeLambda();
    //@}

private:
    URANIE::DataServer::TExponentialDistribution *_dproba;
    URANIE::DataServer::TAttribute * _att;
    URANIE::DataServer::TAttribute * _att2;
};
#endif  // CPPUNITEST_TEXPONENTIALDISTRIBUTION_H
