//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTGenParetoDistribution.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TGenParetoDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.
 \li Cedric Forgues (C.S.) 2010/04/26
 Ajout des m�thodes
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-7

#include "TestTInfiniteDistribution.h"
#include "TestTGenParetoDistribution.h"
#include "TString.h"
#include "UDisableTest.h"


#include <iostream>
using namespace std;

TGenParetoDistributionTest::TGenParetoDistributionTest()
{
}

TGenParetoDistributionTest::~TGenParetoDistributionTest()
{
}

void TGenParetoDistributionTest::setUp()
{
    _att = new URANIE::DataServer::TAttribute("genpareto", 0.0, 1.0);
    _dproba = new URANIE::DataServer::TGenParetoDistribution("genpareto");
}

void TGenParetoDistributionTest::tearDown()
{
    delete _att;
    _att = NULL;
    delete _dproba;
    _dproba = NULL;
}

void TGenParetoDistributionTest::testDefaultLaw()
{
    CPPUNIT_ASSERT(_dproba->getLawType()
		   == URANIE::DataServer::TStochasticAttribute::kGenPareto);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getLowerBound(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getParameterMu(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1, _dproba->getParameterSigma(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25, _dproba->getParameterXi(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.88385428761, _dproba->getPDF(0.1), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.554928957307, _dproba->getPDF(0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.423479028908, _dproba->getPDF(0.75), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.131687242798, _dproba->getPDF(2.0), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0940493552002, _dproba->getCDF(0.1), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.37570492303, _dproba->getCDF(0.50), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.497118653172, _dproba->getCDF(0.75), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.802469135802, _dproba->getCDF(2.0), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.106760384321, _dproba->getInvCDF(0.10), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.54487746587, _dproba->getInvCDF(0.40), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.756828460011, _dproba->getInvCDF(0.50), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.02973371873, _dproba->getInvCDF(0.60), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.11311764016, _dproba->getInvCDF(0.90), DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;
    
    _dproba = new URANIE::DataServer::TGenParetoDistribution("genpareto", 0.0, 1.0, 0.45);

    CPPUNIT_ASSERT(_dproba->getLawType()
		   == URANIE::DataServer::TStochasticAttribute::kGenPareto);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getLowerBound(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getParameterMu(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1, _dproba->getParameterSigma(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.45, _dproba->getParameterXi(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.867766867399, _dproba->getPDF(0.1), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.520003104453, _dproba->getPDF(0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.391789994748, _dproba->getPDF(0.75), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.126413644583, _dproba->getPDF(2.0), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0931836235677, _dproba->getCDF(0.1), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.362996197045, _dproba->getCDF(0.50), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.475980882024, _dproba->getCDF(0.75), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.759814075292, _dproba->getCDF(2.0), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.107898150286, _dproba->getInvCDF(0.10), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.574307397436, _dproba->getInvCDF(0.40), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.813422792788, _dproba->getInvCDF(0.50), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.13407559222, _dproba->getInvCDF(0.60), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.04085095837, _dproba->getInvCDF(0.90), DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;
    
    _dproba = new URANIE::DataServer::TGenParetoDistribution(_att, 0.0, 1.0, 0.45);

    CPPUNIT_ASSERT(_dproba->getLawType()
		   == URANIE::DataServer::TStochasticAttribute::kGenPareto);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getLowerBound(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1, _dproba->getUpperBound(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getParameterMu(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1, _dproba->getParameterSigma(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.45, _dproba->getParameterXi(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getPDF(-0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.867766867399, _dproba->getPDF(0.1), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.520003104453, _dproba->getPDF(0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.391789994748, _dproba->getPDF(0.75), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getPDF(2.0), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0, _dproba->getCDF(-0.5), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0931836235677, _dproba->getCDF(0.1), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.362996197045, _dproba->getCDF(0.50), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.475980882024, _dproba->getCDF(0.75), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1, _dproba->getCDF(2.0), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.107898150286, _dproba->getInvCDF(0.10), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.574307397436, _dproba->getInvCDF(0.40), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.813422792788, _dproba->getInvCDF(0.50), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.13407559222, _dproba->getInvCDF(0.60), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.04085095837, _dproba->getInvCDF(0.90), DBL_PRECISION_COMPARE);
    
}

static void truncation_set(unordered_map<string, double>& references) {
    references["location"] = 1.;
    references["scale"] = 1.;
    references["shape"] = 0.15;

    references["x_min_default"] = 1.;
    references["x_min_default_prev"] = nextafter(references.at("x_min_default"), -HUGE_VAL);
    references["x_min"] = 1.4;
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = 4.;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // scipy.stats.genpareto.cdf()
    references["pmin_default"] = 0.;
    references["pmin"] = 3.2189923930755693e-01;
    references["pmax"] = 9.1601275830153683e-01;
}

void TGenParetoDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGenParetoDistribution distribution("gen_pareto",
        references.at("location"), references.at("scale"), references.at("shape"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}

void TGenParetoDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGenParetoDistribution distribution("gen_pareto",
        references.at("location"), references.at("scale"), references.at("shape"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}

void TGenParetoDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGenParetoDistribution distribution("gen_pareto",
        references.at("location"), references.at("scale"), references.at("shape"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TGenParetoDistributionTest::throwExceptionSetWrongParameters()
{
    cout << endl;
    delete _dproba;
    _dproba = new URANIE::DataServer::TGenParetoDistribution("genpareto", 0.0, 1.0, 0.3);

    CPPUNIT_ASSERT_THROW(_dproba->setParameterSigma(-1.0), URANIE::Exceptions::UErrorExceptions);


    CPPUNIT_ASSERT_THROW(new URANIE::DataServer::TGenParetoDistribution("genpareto", 0.0, -1.0, 0.45), URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(new URANIE::DataServer::TGenParetoDistribution(_att, 0.0, -1.0, 0.45), URANIE::Exceptions::UErrorExceptions);
    
}

