//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTGeneralizedNormalV2Distribution.cxx
 \author Rudy Chocat <rudy.chocat@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TGeneralizedNormalV2Distribution

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.  
 Created: 2009-12-08 09:14:10
 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTInfiniteDistribution.h"
#include "TestTGeneralizedNormalV2Distribution.h"

using namespace std;

TGeneralizedNormalV2DistributionTest::TGeneralizedNormalV2DistributionTest()
{
}

TGeneralizedNormalV2DistributionTest::~TGeneralizedNormalV2DistributionTest()
{
}

void TGeneralizedNormalV2DistributionTest::setUp()
{
    _dproba = new URANIE::DataServer::TGeneralizedNormalV2Distribution("gennormalv2");
}

void TGeneralizedNormalV2DistributionTest::tearDown()
{
    delete _dproba;
    _dproba = NULL;
}

////////////////////////////////
///                          ///                            
///         Tests            ///
///                          ///
////////////////////////////////

void TGeneralizedNormalV2DistributionTest::testDefaultLaw()
{
   
    CPPUNIT_ASSERT(
    _dproba->getLawType()
    == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormalV2);
CPPUNIT_ASSERT(
    _dproba->getStochasticBasis()
    == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
//     
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterKappa(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterXi(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.391042693975,  _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0539909665132, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000133830225765, _dproba->getPDF(4.0),
                                 DBL_PRECISION_COMPARE);
        

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.674489750196, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);
    
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGeneralizedNormalV2Distribution("gennormalv2", 0.0, 1.0, 0.0);
    CPPUNIT_ASSERT(
    _dproba->getLawType()
    == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormalV2);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
//     
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterKappa(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterXi(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.391042693975,  _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0539909665132, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000133830225765, _dproba->getPDF(4.0),
                                 DBL_PRECISION_COMPARE);
        

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.674489750196, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;
    
    
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
        "Att");
    _dproba = new URANIE::DataServer::TGeneralizedNormalV2Distribution(att, 0.0, 1.0, 0.0);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormalV2);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterKappa(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getParameterXi(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.391042693975,  _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0539909665132, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000133830225765, _dproba->getPDF(4.0),
                                 DBL_PRECISION_COMPARE);
        
     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.674489750196, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);
//     delete _dproba;
//     _dproba = NULL;
// 
    delete att;
    att = NULL;
}

void TGeneralizedNormalV2DistributionTest::testOtherLaw()
{

    delete _dproba;
    _dproba = new URANIE::DataServer::TGeneralizedNormalV2Distribution("gennormalv2", 2.0, 2.0, 1.0);
    

    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormalV2);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.99999999999, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.702557458599744, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.32239483179018, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getParameterKappa(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterXi(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0854411191206077, _dproba->getPDF(0.2),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.199471140200716, _dproba->getPDF(2.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.140795094507634, _dproba->getPDF(3.8),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.98116743227345, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
        "Att");
    _dproba = new URANIE::DataServer::TGeneralizedNormalV2Distribution(att, -1.0, 2.0, -0.5);
    CPPUNIT_ASSERT(
        _dproba->getLawType()
        == URANIE::DataServer::TStochasticAttribute::kGeneralizedNormalV2);
    CPPUNIT_ASSERT(
        _dproba->getStochasticBasis()
        == URANIE::DataServer::TStochasticAttribute::kUnknownBasis);
    CPPUNIT_ASSERT_DOUBLES_EQUAL( -4.99999999999, _dproba->getLowerBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.467406187732695, _dproba->getTheoreticalMean(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.41560213284352, _dproba->getTheoreticalStdDev(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.0, _dproba->getTheoreticalMode(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterAlpha(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.5, _dproba->getParameterKappa(),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.0, _dproba->getParameterXi(),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.188026772521566, _dproba->getPDF(-2.7),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.144450938639847, _dproba->getPDF(0.0),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0261508628626985, _dproba->getPDF(3.8),
                                 DBL_PRECISION_COMPARE);
    
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1, _dproba->getInvCDF(0.5),
                                 DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.604328447417417, _dproba->getInvCDF(0.75),
                                 DBL_PRECISION_COMPARE);

   
    delete att;
    att = NULL;
}

static void truncation_set(unordered_map<string, double>& references, bool farAway=false) {
    references["location"] = 0.;
    references["shape"] = 0.5;
    references["scale"] = 1.;

    references["x_min"] = ((farAway) ? -140 : -1.);
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = ((farAway) ? 200 : 1.0);;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);
;

    // ROOT::Math::generalizedNormal_cdf()
    references["pmin"] = ((farAway) ? 0 : 0.208702873384 );
    references["pmax"] = ((farAway) ? 1 : 0.917171480998  );
}

void TGeneralizedNormalV2DistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGeneralizedNormalV2Distribution distribution("gennormalv2", references.at("location"), references.at("scale"),
        references.at("shape"));
    TestTInfiniteDistribution::test_bounds(distribution, references);
}

void TGeneralizedNormalV2DistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGeneralizedNormalV2Distribution distribution("gennormalv2",references.at("location"),  references.at("scale"),
        references.at("shape"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);
}

void TGeneralizedNormalV2DistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TGeneralizedNormalV2Distribution distribution("gennormalv2",references.at("location"),  references.at("scale"),
        references.at("shape"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///   
                       ///
////////////////////////////////

void TGeneralizedNormalV2DistributionTest::throwExceptionWithNegativeAlpha()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGeneralizedNormalV2Distribution("gennormalv2", 1.0, -1.0,  1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGeneralizedNormalV2Distribution(att, 1.0, -1.0,  1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TGeneralizedNormalV2DistributionTest::throwExceptionWithNullAlpha()
{
    cout << endl;
    URANIE::DataServer::TAttribute * att = new URANIE::DataServer::TAttribute(
            "Att");
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGeneralizedNormalV2Distribution("gennormalv2", 1.0, 0.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGeneralizedNormalV2Distribution(att, 1.0, 0.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    delete att;
    att = NULL;
}

void TGeneralizedNormalV2DistributionTest::throwExceptionSetParametersWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(-1.0, -2.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGeneralizedNormalV2DistributionTest::throwExceptionSetParametersWithNullAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(1.0, 0.0, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGeneralizedNormalV2DistributionTest::throwExceptionSetParametersWithTooLowAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(1.0, 1.0e-7, 0.0),
            URANIE::Exceptions::UErrorExceptions);
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameters(0.0, 1.0e-7, 0.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGeneralizedNormalV2DistributionTest::throwExceptionSetParameterAlphaWithNegativeAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterAlpha(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TGeneralizedNormalV2DistributionTest::throwExceptionSetParameterAlphaWithNullAlpha()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(_dproba->setParameterAlpha(0.0),
            URANIE::Exceptions::UErrorExceptions);
}


