//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTGumbelDistribution.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TGumbelDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.
 \li Cedric Forgues (C.S.) 2010/06/16
 Ajout des m�thodes pour la DI 49
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

#include "TestTGumbelDistribution.h"
#include "TString.h"

#include <iostream>
using namespace std;

TGumbelDistributionTest::TGumbelDistributionTest()
{
}

TGumbelDistributionTest::~TGumbelDistributionTest()
{
}

void TGumbelDistributionTest::setUp()
{
    _att = new URANIE::DataServer::TAttribute("gumbel", 0.0, 2.0);
    _dproba = new URANIE::DataServer::TGumbelDistribution("gumbel");
}

void TGumbelDistributionTest::tearDown()
{
    delete _att;
    _att = NULL;
    delete _dproba;
    _dproba = NULL;
}

void TGumbelDistributionTest::testLaw()
{

    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kGumbel);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getMean(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353553390593274, _dproba->getSigma(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPDF(0.), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.63661977236758, _dproba->getPDF(1.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29516723530087, _dproba->getCDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(1.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29516723530087, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.70483276469913, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelDistribution("gumbel", 1., 2.);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kGumbel);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getMean(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353553390593274, _dproba->getSigma(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPDF(0.), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.63661977236758, _dproba->getPDF(1.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29516723530087, _dproba->getCDF(1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(2.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29516723530087, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.70483276469913, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);

    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelDistribution(_att, 0.5, 1.);

    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kGumbel);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getMean(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.353553390593274, _dproba->getSigma(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPDF(1.), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.63661977236758, _dproba->getPDF(2.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29516723530087, _dproba->getCDF(1.),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(2.),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.29516723530087, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.70483276469913, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
}

void TGumbelDistributionTest::testGetSetScale()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getScale(),
            DBL_PRECISION_COMPARE);
    _dproba->setScale(2.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getScale(),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelDistribution("gumbel", 0.5, 1.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getScale(),
            DBL_PRECISION_COMPARE);
    _dproba->setScale(2.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getScale(),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelDistribution(_att, 0.5, 1.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getScale(),
            DBL_PRECISION_COMPARE);
    _dproba->setScale(2.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getScale(),
            DBL_PRECISION_COMPARE);
}

void TGumbelDistributionTest::testGetSetLocation()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getLocation(),
            DBL_PRECISION_COMPARE);
    _dproba->setLocation(0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getLocation(),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelDistribution("gumbel", 0.0, 1.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLocation(),
            DBL_PRECISION_COMPARE);
    _dproba->setLocation(0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getLocation(),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TGumbelDistribution(_att, 0.0, 1.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLocation(),
            DBL_PRECISION_COMPARE);
    _dproba->setLocation(0.5);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getLocation(),
            DBL_PRECISION_COMPARE);
}

void TGumbelDistributionTest::testGumbelTruncation()
{
    _dproba->setBounds(-2., 2.);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2., _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2., _dproba->getPMax(), DBL_PRECISION_COMPARE);
}

void TGumbelDistributionTest::throwExceptionWithNegativeBeta()
{
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGumbelDistribution("gumbel", 0.5, 1.),
            URANIE::Exceptions::UWarningExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TGumbelDistribution(_att, 0.5, 1.),
            URANIE::Exceptions::UWarningExceptions);
}

