//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTNormalDistribution.cxx
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TNormalDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 \par Modifications
 \li Cedric Forgues (C.S.) 2011/01/10: liberation correcte des dproba.
 \li Cedric Forgues (C.S.) 2010/04/26
 Ajout des m�thodes
 // *****************************************************************************

 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-7

#include "TestTInfiniteDistribution.h"
#include "TestTNormalDistribution.h"
#include "TString.h"

#include <iostream>
using namespace std;

TNormalDistributionTest::TNormalDistributionTest()
{
}

TNormalDistributionTest::~TNormalDistributionTest()
{
}

void TNormalDistributionTest::setUp()
{
    _att = new URANIE::DataServer::TAttribute("normal", 0.0, 0.5);
    _att2 = new URANIE::DataServer::TAttribute("normal");
    _dproba = new URANIE::DataServer::TNormalDistribution("normal");
}

void TNormalDistributionTest::tearDown()
{
    delete _att;
    _att = NULL;
    delete _att2;
    _att2 = NULL;
    delete _dproba;
    _dproba = NULL;
}

void TNormalDistributionTest::testDefaultLaw()
{
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.129517595665892, _dproba->getPDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3520653267643, _dproba->getPDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.398942280401433, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.3520653267643, _dproba->getPDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.129517595665892, _dproba->getPDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0668072012688581, _dproba->getCDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.308537538725987, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.691462461274013, _dproba->getCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.93319279873114, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.2815515655446, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.2533471031358, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.2533471031358, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.2815515655446, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TNormalDistribution("normal", -2.0, 0.5);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.21517656996466e-08, _dproba->getPDF(-5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.483941449038287, _dproba->getPDF(-2.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.797884560802865, _dproba->getPDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.483941449038287, _dproba->getPDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.21517656996466e-08, _dproba->getPDF(1.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.86587645021748e-10, _dproba->getCDF(-5.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.158655253931457, _dproba->getCDF(-2.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(-2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.841344746068543, _dproba->getCDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999999999013412, _dproba->getCDF(1.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.6407757827723, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.1266735515679, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-2.0, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.8733264484321, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.3592242172277, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TNormalDistribution(_att2, 2.0, 0.5);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.42387e+64, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.42387e+64, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.21517656996466e-08, _dproba->getPDF(-1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.483941449038287, _dproba->getPDF(1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.797884560802865, _dproba->getPDF(2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.483941449038287, _dproba->getPDF(2.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.21517656996466e-08, _dproba->getPDF(5.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(9.86587645021748e-10, _dproba->getCDF(-1.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.158655253931457, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(2.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.841344746068543, _dproba->getCDF(2.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.999999999013412, _dproba->getCDF(5.0),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.3592242172277, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.8733264484321, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.1266735515679, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.6407757827723, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TNormalDistribution(_att, 0.0, 0.5);
    CPPUNIT_ASSERT(
            _dproba->getLawType()
                    == URANIE::DataServer::TStochasticAttribute::kNormal);
    CPPUNIT_ASSERT(
            _dproba->getStochasticBasis()
                    == URANIE::DataServer::TStochasticAttribute::kHermite);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalStdDev(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPMin(), DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1., _dproba->getPMax(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.797884560802865, _dproba->getPDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.483941449038287, _dproba->getPDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-1.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getCDF(0.0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.841344746068543, _dproba->getCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(1.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.6407757827723, _dproba->getInvCDF(0.1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.1266735515679, _dproba->getInvCDF(0.4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getInvCDF(0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1266735515679, _dproba->getInvCDF(0.6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.6407757827723, _dproba->getInvCDF(0.9),
            DBL_PRECISION_COMPARE);
}

static void truncation_set(unordered_map<string, double>& references, bool farAway=false) {
    references["mean"] = 0.5;
    references["sigma"] = 1.;

    references["x_min"] = ((farAway) ? -14 : -1.4);
    references["x_min_prev"] = nextafter(references.at("x_min"), -HUGE_VAL);
    references["x_max"] = ((farAway) ? 20 : 2.);;
    references["x_max_next"] = nextafter(references.at("x_max"), HUGE_VAL);

    // ROOT::Math::normal_cdf()
    references["pmin"] = ((farAway) ? 0 : 2.8716559816001817e-02 );
    references["pmax"] = ((farAway) ? 1 : 9.3319279873114191e-01);
}

void TNormalDistributionTest::testTruncationBounds() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TNormalDistribution distribution("normal", references.at("mean"),
        references.at("sigma"));
    TestTInfiniteDistribution::test_bounds(distribution, references);

    truncation_set(references,true);
    URANIE::DataServer::TNormalDistribution distribution2("normal2", references.at("mean"),
        references.at("sigma"));
    TestTInfiniteDistribution::test_bounds(distribution2, references);
}

void TNormalDistributionTest::testTruncationLowerBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TNormalDistribution distribution("normal", references.at("mean"),
        references.at("sigma"));
    TestTInfiniteDistribution::test_lower_bound(distribution, references);

    truncation_set(references,true);
    URANIE::DataServer::TNormalDistribution distribution2("normal2", references.at("mean"),
        references.at("sigma"));
    TestTInfiniteDistribution::test_lower_bound(distribution2, references);
}

void TNormalDistributionTest::testTruncationUpperBound() {
    unordered_map<string, double> references;
    truncation_set(references);

    URANIE::DataServer::TNormalDistribution distribution("normal", references.at("mean"),
        references.at("sigma"));
    TestTInfiniteDistribution::test_upper_bound(distribution, references);

    truncation_set(references,true);
    URANIE::DataServer::TNormalDistribution distribution2("normal2", references.at("mean"),
        references.at("sigma"));
    TestTInfiniteDistribution::test_upper_bound(distribution2, references);
}

////////////////////////////////
///                          ///
///       Exceptions         ///
///                          ///
////////////////////////////////

void TNormalDistributionTest::throwExceptionWithNegativeSigma()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TNormalDistribution("normal", 0.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TNormalDistribution(_att2, 0.0, -1.0),
            URANIE::Exceptions::UErrorExceptions);
}

void TNormalDistributionTest::throwExceptionSetParameterSigma()
{
    cout << endl;
    delete _dproba;
    _dproba = new URANIE::DataServer::TNormalDistribution("normal", 0.0, 1.0);
    CPPUNIT_ASSERT_THROW(_dproba->setParameterSigma(-1.0),
            URANIE::Exceptions::UErrorExceptions);
}

