//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTNormalDistribution.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TNormalDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TNORMALDISTRIBUTION_H
#define CPPUNITEST_TNORMALDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TNormalDistribution.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TNormalDistributionTest
 * \brief Unit tests for TNormalDistribution.
 */
class TNormalDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TNormalDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (testTruncationBounds);
    CPPUNIT_TEST (testTruncationLowerBound);
    CPPUNIT_TEST (testTruncationUpperBound);
    CPPUNIT_TEST (throwExceptionWithNegativeSigma);
    CPPUNIT_TEST (throwExceptionSetParameterSigma);
    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TNormalDistributionTest object.
     */
    TNormalDistributionTest();

    /// Destructor.
    virtual ~TNormalDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests normal distribution with set parameters (set mean and sigma).
    void testDefaultLaw();

    //! Tests truncation on normal distribution.
    void testTruncationBounds();
    void testTruncationLowerBound();
    void testTruncationUpperBound();

    //! Throws exception when giving a negative sigma.
    void throwExceptionWithNegativeSigma();

    //! Throws exception when giving a negative sigma to the setParameterSigma() method.
    void throwExceptionSetParameterSigma();
    //@}

private:
    URANIE::DataServer::TNormalDistribution *_dproba;
    URANIE::DataServer::TAttribute * _att, *_att2;
};
#endif  // CPPUNITEST_TNORMALDISTRIBUTION_H
