//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTTemporalTree.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe TestTTemporalTree.

 // *****************************************************************************

 */
#include <iostream>
using namespace std;

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTTemporalTree.h"
#include "uranie_commands.h"

// ROOT
#include "TString.h"

#define LEVELEFILENAME1000        "levelE1000-1.dat"
#define LEVELESOBOLFILENAME1000   "levelESobol1000-1.dat"


TTemporalTreeTest::TTemporalTreeTest():
  _tdsLevelE(NULL), _tttLevelE(NULL)
{
}

TTemporalTreeTest::~TTemporalTreeTest()
{
}

void TTemporalTreeTest::setUp()
{
}

void TTemporalTreeTest::tearDown()
{

    UranieTests::removeFile(_workingTestPath.Data(),"levelE*.root");

  if ( _tttLevelE ) {
      delete _tttLevelE; _tttLevelE = NULL;
    }
  if ( _tdsLevelE ) {
      delete _tdsLevelE; _tdsLevelE = NULL;
    }
}

////////////////////////////////
///                          ///
///         Tests            ///
///                          ///
////////////////////////////////
void TTemporalTreeTest::testNPatterns()
{
  cout << endl;
  _tdsLevelE = new URANIE::DataServer::TDataServer("_tds_", "My title");
  _tdsLevelE->fileDataRead(LEVELEFILENAME1000);

  TString sY="dose";
  TString sTime = "log10tps";

  _tttLevelE = new URANIE::DataServer::TTemporalTree(_tdsLevelE, sTime, sY);
  _tttLevelE->unsetLog ();
  _tttLevelE->setDrawProgressBar(kFALSE);
  _tttLevelE->buildTree();

  CPPUNIT_ASSERT_EQUAL(26624, _tdsLevelE->getNPatterns());
  CPPUNIT_ASSERT_EQUAL(_tttLevelE->getNMinTime(), _tttLevelE->getNMaxTime());

  CPPUNIT_ASSERT(_tttLevelE->getTTree());
  CPPUNIT_ASSERT_EQUAL(1024, (Int_t)(_tttLevelE->getTTree()->GetEntries()));
  CPPUNIT_ASSERT( _tttLevelE->getStatTree() != NULL);

  CPPUNIT_ASSERT_EQUAL(_tttLevelE->getNMaxTime(), (Int_t)(_tttLevelE->getStatTree()->GetEntries()));


}


void TTemporalTreeTest::testWithXAttributes()
{
  cout << endl;
  _tdsLevelE = new URANIE::DataServer::TDataServer("_tds_", "My title");
  _tdsLevelE->fileDataRead(LEVELESOBOLFILENAME1000);
  TString sTime = "log10tps";

  TString sY="dose";
  TString sXs = "t:kl:kc:v1:l1:r1:rc1:v2:l2:r2:rc2:w";
  TString sAddAtt="sobol__n__iter___tds_";

  _tttLevelE = new URANIE::DataServer::TTemporalTree(_tdsLevelE, sTime, sY, sXs);
  _tttLevelE->unsetLog ();
  _tttLevelE->setDrawProgressBar(kFALSE);
  _tttLevelE->setAdditinalAttribute(sAddAtt);
  _tttLevelE->buildTree();

  CPPUNIT_ASSERT_EQUAL(26364, _tdsLevelE->getNPatterns());
  CPPUNIT_ASSERT_EQUAL(_tttLevelE->getNMinTime(), _tttLevelE->getNMaxTime());
  CPPUNIT_ASSERT(_tttLevelE->getTTree());
  CPPUNIT_ASSERT_EQUAL(1014, (Int_t)(_tttLevelE->getTTree()->GetEntries()));

  Int_t nLeaves = 2;                          // nT and nS
  nLeaves += 2;                               // sTime and sY
  nLeaves += sXs.CountChar (':')+1;           // sXs
  nLeaves += sAddAtt.CountChar (':') +1;      // sAddAttribute
  CPPUNIT_ASSERT_EQUAL( nLeaves, _tttLevelE->getTTree()->GetListOfLeaves()->GetEntries ());

  CPPUNIT_ASSERT( _tttLevelE->getStatTree() != NULL );
}
