//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTTriangularDistribution.h
 \author C�dric Forgues (C.S.) <uranietm@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TTriangularDistribution
 
 Cr�e le : mar avri 26 10:00:00 2010

 // *****************************************************************************

 */
#ifndef CPPUNITEST_TTRIANGULARDISTRIBUTION_H
#define CPPUNITEST_TTRIANGULARDISTRIBUTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TTriangularDistribution.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TTriangularDistributionTest
 * \brief Unit tests for TTriangularDistribution.
 */
class TTriangularDistributionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TTriangularDistributionTest);
    CPPUNIT_TEST (testDefaultLaw);
    CPPUNIT_TEST (throwExceptionWithBadMode);
    CPPUNIT_TEST (throwExceptionWithBadBoundaries);CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TTriangularDistributionTest object.
     */
    TTriangularDistributionTest();

    /// Destructor.
    virtual ~TTriangularDistributionTest();

    void setUp();
    void tearDown();

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Tests the default parameters for triangular distribution.
    void testDefaultLaw();

    //! Throws an exception when mode is between lower and upepr bounds.
    void throwExceptionWithBadMode();

    //! Throws an exception when upper bound is inferior to lower bound.
    void throwExceptionWithBadBoundaries();

    //@}

private:
    URANIE::DataServer::TTriangularDistribution *_dproba;
    URANIE::DataServer::TAttribute * _att;
};
#endif  // CPPUNITEST_TTRIANGULARDISTRIBUTION_H
