//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTUniformByPartsDistribution.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test file of the class URANIE::DataServer::TUniformByPartsDistribution
 \par Modifications
 \li Cedric Forgues (C.S.) 2010/11/17
 testOtherLaw() has been mixed with testDefaultLaw() method.
 Created: 2009-12-08 09:14:10
 
 */

// CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-12

// Uranie
#include "TestTUniformByPartsDistribution.h"

using namespace std;

TUniformByPartsDistributionTest::TUniformByPartsDistributionTest()
{
}

TUniformByPartsDistributionTest::~TUniformByPartsDistributionTest()
{
}

void TUniformByPartsDistributionTest::setUp()
{
    _dproba = new URANIE::DataServer::TUniformByPartsDistribution("unifparts");
}

void TUniformByPartsDistributionTest::tearDown()
{
  if(_dproba) delete _dproba;
    _dproba = NULL;
}

////////////////////////////////
///                          ///                            
///         Tests            ///
///                          ///
////////////////////////////////

void TUniformByPartsDistributionTest::testDefaultLaw()
{

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getParameterMedian(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.288675134594813,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPDF(0.25),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getPDF(0.75),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(-0.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25, _dproba->getCDF(0.25),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.50, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.75, _dproba->getCDF(0.75),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(1.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.10, _dproba->getInvCDF(0.10),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.40, _dproba->getInvCDF(0.40),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.50, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.60, _dproba->getInvCDF(0.60),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.90, _dproba->getInvCDF(0.90),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TUniformByPartsDistribution("unifparts",
            1.0, 4.0);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.5, _dproba->getParameterMedian(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.5, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.866025403784439,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.333333333333333, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.333333333333333, _dproba->getPDF(3.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(4.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.166666666666667, _dproba->getCDF(1.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.833333333333333, _dproba->getCDF(3.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(4.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.3, _dproba->getInvCDF(0.10),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.2, _dproba->getInvCDF(0.40),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.5, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.8, _dproba->getInvCDF(0.60),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.7, _dproba->getInvCDF(0.90),
            DBL_PRECISION_COMPARE);
    delete _dproba;
    _dproba = NULL;

    _dproba = new URANIE::DataServer::TUniformByPartsDistribution("unifparts",
            1.0, 4.0, 2.0);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getLowerBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(4.0, _dproba->getUpperBound(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getParameterMedian(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.25, _dproba->getTheoreticalMean(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.877971146071061,
            _dproba->getTheoreticalStdDev(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.5, _dproba->getPDF(1.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25, _dproba->getPDF(3.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getPDF(4.50),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, _dproba->getCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.25, _dproba->getCDF(1.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.875, _dproba->getCDF(3.5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.0, _dproba->getCDF(4.5),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.2, _dproba->getInvCDF(0.10),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.8, _dproba->getInvCDF(0.40),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.0, _dproba->getInvCDF(0.50),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(2.4, _dproba->getInvCDF(0.60),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(3.6, _dproba->getInvCDF(0.90),
            DBL_PRECISION_COMPARE);
}

void TUniformByPartsDistributionTest::throwExceptionWithBadMode()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TUniformByPartsDistribution("unifparts",
                    1.0, 4.0, 5.0), URANIE::Exceptions::UErrorExceptions);
}

void TUniformByPartsDistributionTest::throwExceptionWithBadBoundaries()
{
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TUniformByPartsDistribution("unifparts",
                    4.0, 1.0), URANIE::Exceptions::UErrorExceptions);
    cout << endl;
    CPPUNIT_ASSERT_THROW(
            new URANIE::DataServer::TUniformByPartsDistribution("unifparts",
                    4.0, 1.0, 2.0), URANIE::Exceptions::UErrorExceptions);
}

