//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCodeFile.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   tue dec 5 13:24:01 CET 2006
 
 \brief Implementation of the class URANIE::Launcher::TCodeFile
 
 */

#include "TCodeFile.h"
#include "UExceptions.h"

ClassImp (URANIE::Launcher::TCodeFile)

namespace URANIE
{
namespace Launcher
{

TCodeFile::TCodeFile(TString str) :
    _sFileName(str), _blog(kFALSE)
{
    if (_blog)
        cout << " *** Method TCodeFile::" << __func__ << " with str[" << str << "]" << endl;
    _cComments = "!";
    _cSeparators = " =;\n";
    _cFieldSeparator = " ";

    SetName(str);
    SetTitle(Form("TCodeFile with name[%s]", str.Data()));

    if (_blog)
        cout << " *** End of Method TCodeFile::" << __func__ << " with str[" << str << "]" << endl;
}

TCodeFile::~TCodeFile()
{
    if (_blog)
        cout << " ** deletes TCodeFile [" << getFileName() << "]" << endl;

    Int_t natt = (Int_t)(_attributes.size());
    if (_blog)
        cout << " ** N attribute[" << natt << "]" << endl;
    for (Int_t i = 0; i < natt; i++)
    {
        URANIE::DataServer::TAttribute *att = _attributes[i];
        if (_blog)
            cout << "    **  DECREASE SHARE [" << att->GetName()
		 << "] FileName[" << GetName() << "] .....";
        att->delShare();
        if (_blog)
            cout << " Share[" << att->getShare() << "]" << endl;
        if (att->getShare() == 0)
        {
            if (_blog)
            {
                att->setLog();
                cout << " ** delete att[" << att->GetName() << "] ...";
            }
            delete att;
            att = NULL;
            if (_blog)
                cout << " Done" << endl;
        }
        
    }
    if (_blog)
        cout << " ** End Of deletes TCodeFile [" << getFileName() << "]"
	     << endl;
}

/////////////////////
//// Add attribut to the Output File
/////////////////////
void TCodeFile::addAttribute(URANIE::DataServer::TAttribute* att)
{
    att->addShare();
    _attributes.push_back(att);
    if (_blog)
        cout << " TCodeFile::addAttribute file[" << GetName() << "] Att name["
	     << att->GetName() << "] getShare[" << att->getShare()
	     << "] -- size(" << _attributes.size() << ")" << endl;
}

Bool_t TCodeFile::isAttribute(TString sAttributeName)
{
    if (_blog)
        cout << " ** TCodeFile::isAttribute for TCodeFile(" << GetName() << ") and attribute(" << sAttributeName << ")" << endl;
    Bool_t bexist = kFALSE;

    Int_t natt = (Int_t)(_attributes.size());
    if (_blog)
        cout << " ** N attribute[" << natt << "]" << endl;
    for (Int_t i = 0; i < natt; i++)
    {
        URANIE::DataServer::TAttribute *att = _attributes[i];
        if (_blog)
            cout << "    **  attribute i(" << i << "/" << natt << ") Name [" << att->GetName()<< "] FileName[" << GetName() << "]" << endl;
        if( sAttributeName.EqualTo(att->GetName())) {
            bexist = kTRUE;
            break;
        }
    }

    if (_blog)
        cout << " ** Attribute (" << sAttributeName << ") exist[" << bexist << "]" << endl;


    return bexist;
}


void TCodeFile::setSeparatorCharacter(const char* str)
{
    TString sTmp(str);
    try{
	if (sTmp.Contains("."))
	    throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
						       "TCodeFile::setSeparatorCharacter ERROR : '.' is not authorized as separator");
    }
    catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
	ue.printMessage();
	throw ue;
	exit(-1);
    }
    _cSeparators = str;
}

void TCodeFile::setFieldSeparatorCharacter(const char* str)
{
    TString sTmp(str);
    try{
	if (sTmp.Contains("."))
	    throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
						       "TCodeFile::setFieldSeparatorCharacter ERROR : '.' is not authorized as separator");
    }
    catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
	ue.printMessage();
	throw ue;
	exit(-1);
    }
    _cFieldSeparator = str;
}

URANIE::DataServer::TAttribute * TCodeFile::getAttribute(Int_t ind)
{
    URANIE::DataServer::TAttribute *att = 0;
    if (ind < (Int_t)(_attributes.size()))
        att = _attributes[ind];
    return att;
}

Int_t TCodeFile::getAttributesNumber() const
{
    return (Int_t) _attributes.size();
}

void TCodeFile::printLog(Option_t *option)
{
    cout << "TCodeFile::printLog[" << option << "]" << endl;
    cout << "_sFileName[" << _sFileName << "]" << endl;
    cout << "_cComments[" << _cComments << "]" << endl;
    cout << "_cSeparators[" << _cSeparators << "]" << endl;
    cout << "_cFieldSeparator[" << _cFieldSeparator << "]" << endl;
    // Les attributs
    Int_t natt = (Int_t)(_attributes.size());
    cout << "natt[" << natt << "]" << endl;
    for (Int_t indx = 0; indx < natt; indx++)
    {
        cout << "Attributes index[" << indx + 1 << "/" << natt << "]" << endl;
        _attributes[indx]->printLog();
    }
    cout << "End of TCodeFile::printLog[" << option << "]" << endl;

}

} // Fin du namespace Launcher
} // Fin du namespace URANIE

