//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCodeFile.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   thu dec 5 13:24:01 CET 2006
 
 \brief Interface de la classe URANIE::Launcher::TCodeFile
 
 */
/**
 
 * @class URANIE::Launcher::TCodeFile
 *
 * @brief Description of the class TCodeFile.
 *
 * To be written by the developper.
 *
 */
#ifndef TCODEFILE_H
#define TCODEFILE_H

#include <stdlib.h>
#include <iostream>
#include <vector>
#include <algorithm>    // std::find
using std::cout;
using std::endl;
using std::vector;

// ROOT
#include "Rtypes.h"
#include "TString.h"
#include "TSystem.h"

// URANIE
#include "Launcher.h"
#include "TAttribute.h"
#include "UEntry.h"

//STL
#include <vector>

namespace URANIE
{
namespace Launcher
{

class TCodeFile: public TNamed
{
public:
    vector<URANIE::DataServer::TAttribute *> _attributes; ///< Vector of attributes
    TString _svecbeg, _svecend, _svecdel, _sstrbeg, _sstrend;  ///< string used for the beginning, end and delimitation of vectors and strings
    TString _sFileName;                       ///< The name of file
    Bool_t _blog;                        ///< Boolean for edit the log

    TString _cComments;                   ///< The set of comment character
    TString _cSeparators;                ///< The set of separator character
    TString _cFieldSeparator;           ///< the separator to be used in between fields
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{

    //! Default constructor
    TCodeFile(TString str);
    //! Default destructor
    virtual ~TCodeFile();
    //@}

    //---------------------------------------------
    /** @name Add attribut to the File
     **/
    //@{
    //!   Add the attribute in the file
    /*!
     */
    virtual void addAttribute(URANIE::DataServer::TAttribute*);
    //@}

    //---------------------------------------------
    /** @name Other  Setting and Getting 
     **/
    //@{
    TString getFileName()
	{
	    return _sFileName;
	}

    void setCommentCharacter(const char* str)
	{
	    _cComments = str;
	}
    const char* getCommentCharacter()
	{
	    return _cComments;
	}
    /*!
      Changes the separator character in the \em key \em = \em value \em ;
      The separator character is the symbol "=".
    */
    void setSeparatorCharacter(const char* str);

    const char* getSeparatorCharacter()
	{
	    return _cSeparators;
	}

    void setFieldSeparatorCharacter(const char* str);

    const char* getFieldSeparatorCharacter()
	{
	    return _cFieldSeparator;
	}

    URANIE::DataServer::TAttribute * getAttribute(Int_t ind);

    Int_t getAttributesNumber() const;

    Int_t getNAttributes() const
    {
        return getAttributesNumber();
    }

    inline void setVectorProperties( TString beg, TString delim, TString end)
	{
	    _svecbeg = beg;
	    _svecdel = delim;
	    _svecend = end;
	}
    inline void setStringProperties( TString beg, TString end)
	{
	    _sstrbeg = beg;
	    _sstrend = end;
	}

    inline void getBegEndProperties( URANIE::DataServer::TAttribute *att, TString &beg, TString &end)
	{
	    if( att->getDataType()==URANIE::DataServer::TAttribute::kVector)
	    {
		beg = _svecbeg;
		end = _svecend;
	    }
	    else if( att->getDataType()==URANIE::DataServer::TAttribute::kString)
	    {
		beg = _sstrbeg;
		end = _sstrend;
	    }
	    else
	    {
		beg = "";
		end = "";
	    }
	}

    /*!
     * \brief Verify is the attribute given by the name is is the list of attribute
     * \param sAttributeName (TString) the name of the attribute to test if exist
     * \return true if the attribute exists in the list of attributes else false
     */
    Bool_t isAttribute(TString sAttributeName);

    //@}

    //---------------------------------------------
    /** @name The  Log
     **/
    //@{
    void setLog()
	{
	    _blog = kTRUE;
	}
    void unsetLog()
	{
	    _blog = kFALSE;
	}
    void changeLog()
	{
	    _blog = _blog ? kFALSE : kTRUE;
	}
    Bool_t getLog()
	{
	    return _blog;
	}
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Launcher::TCodeFile, ID_LAUNCHER)
    //Classe de
};

} // Fin du namespace Launcher
} // Fin du namespace URANIE
#endif
