//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TPlatineCluster.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   Mon August 4 14:19:19 CEST 2008
 
 \brief Interface of the class URANIE::Launcher::TEcCluster
 
 */
/**
 
 * @class URANIE::Launcher::TEcCluster
 *
 * @brief Description of the class TEcCluster.
 *
 * To be written by the developper.
 *
 */
#ifndef TECCLUSTER_H
#define TECCLUSTER_H

#include <iostream>
#include <sstream>
#include <TPRegexp.h>

using std::cout;
using std::endl;
using std::istringstream;
using std::string;
#define BUF_SIZE 128
//ROOT
#include "Rtypes.h"

// Uranie
#include "TCluster.h"
#include "TCode.h"

namespace URANIE
{
namespace Launcher
{
//#pragma clang diagnostic push
//#pragma clang diagnostic ignored "-Winconsistent-missing-override"
class TEcCluster: public TCluster
{
private:
    static const char *nodeList();

public:
#ifdef WITH_LIBSSH
    EDistrib _edistrib=EDistrib::kEc;
#endif
    static bool eligible();

    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TEcCluster();
#ifdef WITH_LIBSSH        
    TEcCluster(URANIE::DataServer::TDataServer *tds, TCode *code);
    TEcCluster(TCode *code);
#endif
    
    //! Default destructor
     ~TEcCluster();
    //@}

    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Construct and return the list of nodes
    void init() override;
    void getCommandPrefix(char *, char *) override;
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    virtual void printLog(Option_t *option = "") override;
    //@}
    // These functions , will always return 0 due ecmwf plafform being connected in every command send.
    //! Sets the Cluster name
    /*!
     * \param cluster_name (TString) name of the cluster
     */

#ifdef WITH_LIBSSH    
    virtual void setCluster(TString cluster_name) override
    {
        _sClusterName = cluster_name;
        _sClusterIP="ecaccess.ecmwf.int";
        _eType="kEc";
        _sConnectionProtocol="ecaccess";
        _sConnected=kTRUE;
        cluster_name.ToLower();
        if ( cluster_name.Contains("cca") )
        {
            _sGateWay="cca";
        }
        else if ( cluster_name.Contains("ccb") )
        {
            _sGateWay="ccb";
        }
        else if ( cluster_name.Contains("ecgate") )
        {
            _sGateWay="ecgate";
        }
        else
        {
            _sGateWay="cca";
        }

    }
    Int_t sendHeader() ; 
    Int_t sendCompiledBinary(list<TString> filenames) ;

    Int_t retrieveInputFiles(const char * localFolder) ;

    TString getJobStatusCommand(list<pair<TString, pair<int, int> > > JobList);
    TString checkJobStatus(TString JobId, TString Command) ;

    Int_t closeSFTPChannel()
    {
        return 0;
    }
    
    Int_t sendInputFiles(Int_t start, Int_t end);
    TString submitJob(TString command) override;
    virtual void selectHeader(TString SkeletonHeaderName="header_skeleton.in",TString localHeaderFolderPath=gSystem->pwd() ) override;
    virtual void generateHeader(Option_t *option = "")  override;
    virtual void generateHeaderSplit(Option_t *option = "") override;
#endif
    TString GetFromPipe(const char *command, bool output = false);
public:
};
//#pragma clang diagnostic pop

} // Fin du namespace Launcher
} // Fin du namespace URANIE
#endif
