//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TInputFileFlag.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mon oct 9th 14:52:50 CEST 2006
 
 \brief Interface of the class URANIE::Launcher::TInputFileFlag
 
 */
/**
 
 * @class URANIE::Launcher::TInputFileFlag
 *
 * @brief Description of the class TInputFileFlag.
 *
 * To be written by the developper.
 *
 */
#ifndef TINPUTFILEFLAG_H
#define TINPUTFILEFLAG_H

// ROOT
#include "Rtypes.h"
#include "TString.h"
// URANIE
#include "TAttribute.h"
#include "TInputFile.h"
#include "UExceptions.h"

namespace URANIE
{
namespace Launcher
{

//!  A class for input file of a code with a flag. 
/*!
 The method is used to give \em flags to the numerical values you want to modify, in your data set.
 
 It must be noticed that the flag can appear several times in the same file of the data.
 \warning It's up to the user to put a flag wherever this variable appears.

 We can naturally use this type of object with the formalism <b>key = value ; </b>, but the user
 must take the initiative to put a flag to the value of this variable in his data set, whereas there is nothing to do 
 with the object \ref TInputFile.
 */
class TInputFileFlag: public TInputFile
{
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TInputFileFlag(TString str);
    //! Default destructor
    virtual ~TInputFileFlag();
    //@}

    //---------------------------------------------
    /** @name Implement the abstract pure methods
     **/
    //@{
    //! search the indexes of the attribute in the input file
    /*!
     \param att (URANIE::DataServer::TAttribute*) the attribute to search the indexes
     */
    virtual void searchIndexeAttribute(URANIE::DataServer::TAttribute* att);
    //! Substitute the values of the attributes linked in the input file and write it
    /*!
     The first value is defined par the parameter \em n and the values are stored in the parameter \em val.
     After the method, the parameter is increased by the number of attributes contained in the input file.
     \param n (Int_t &) the index of the first value in the vector of data \e val
     \param val (Double_t *) the pointer of the data
     \param index (Int_t *)the pointer of the index of data
     */
    void substitue(Int_t &n, Double_t *val, Int_t *index = NULL);
    void substitue(Int_t &n, vector<URANIE::DataServer::UEntry*> *ent, Int_t *index = NULL);
    void substitue();
    //@}

    ClassDef(URANIE::Launcher::TInputFileFlag, ID_LAUNCHER)
    //Classe de
};

} // Fin du namespace Launcher
} // Fin du namespace URANIE
#endif
