//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TInputFileXML.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   Tue Sep  6 10:46:46 2011
 
 \brief Interface of the class URANIE::Launcher::TInputFileXML
 
 */

/**
 
 * @class URANIE::Launcher::TInputFileXML
 *
 * @brief Description of the class TInputFileXML.
 *
 * To be written by the developper.
 *
 */
#ifndef TINPUTFILEXML_H
#define TINPUTFILEXML_H

// ROOT
#include "Rtypes.h"
#include "TString.h"
// URANIE
#include "TAttribute.h"
#include "TInputFile.h"

namespace URANIE
{
namespace Launcher
{

class TInputFileXML: public TInputFile
{
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TInputFileXML(TString str);
    //! Default destructor
    virtual ~TInputFileXML();
    //@}

    //---------------------------------------------
    /** @name File properties
     **/
    //@{
    //! Gets the type of the file. This method is used to find InputFile of type 'Recreate'
    Bool_t isFileToCopy()
	{
            return kFALSE;
	}
    //! Return the name of the attribute for the key
    /*!
      The name of the attribute is the string after the last :"@" character
    */
    TString getAttributeInKey(TString skey);
    //! Return the last field in the key
    TString getFieldInKey(TString skey);

    //@}

    //---------------------------------------------
    /** @name Implement the abstract pure methods
     **/
    //@{
    //! search the indexes of the attribute in the input file
    /*!
      \param att (URANIE::DataServer::TAttribute*) the attribute to search the indexes
    */
    virtual void searchIndexeAttribute(URANIE::DataServer::TAttribute* att);
    //! Substitute the values of the attributes linked in the input file and write it
    /*!
      The first value is defined par the parameter \em n and the values are stored in the parameter \em val.
      After the method, the parameter is increased by the number of attributes contained in the input file.
      \param n (Int_t &) the index of the first value in the vector of data \e val
      \param val (Double_t *) the pointer of the data
      \param index (Int_t *)the pointer of the index of data
    */
    void substitue(Int_t &n, Double_t *val, Int_t *index = NULL);
    void substitue(Int_t &n, vector<URANIE::DataServer::UEntry*> *ent, Int_t *index = NULL);
    //@}

    //---------------------------------------------
    /** @name The Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Launcher::TInputFileXML, ID_LAUNCHER)
    //Classe de
};

} // Fin du namespace Launcher
} // Fin du namespace URANIE
#endif
