//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
// TLoadLevelerCluster lun aoû 4 14:19:19 CEST 2008
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TLoadLevelerCluster.cxx
 \author Thierry Crestaux <thierry.crestaux@c-s.fr>
 \date   2011-05-20 10:15:42
 
 \brief Implementation of the class URANIE::Launcher::TLoadLevelerCluster
 
 This class implement the wrapper to submit parralel launch in the poincare cluster
 
 */

#include "TLoadLevelerCluster.h"

#include "TSystem.h"
#include <sstream>
#include <fstream>

ClassImp (URANIE::Launcher::TLoadLevelerCluster)

namespace URANIE
{
namespace Launcher
{
// class method
bool TLoadLevelerCluster::eligible()
{
    const char *tmp = gSystem->Getenv("LOADL_PROCESSOR_LIST");
    return (tmp != NULL && tmp[0]);
}

const char *TLoadLevelerCluster::nodeList()
{
    // 
    return gSystem->Getenv("LOADL_PROCESSOR_LIST");
}

// Constructor
TLoadLevelerCluster::TLoadLevelerCluster() :
        TCluster("LOADLEVELER", TLoadLevelerCluster::nodeList(), 1)
{
    if (_blog)
        cout << " ** Constructor TLoadLevelerCluster::TLoadLevelerCluster [" << GetName()
                << "] Title[" << GetTitle() << "]" << endl;
    init();
}
TLoadLevelerCluster::~TLoadLevelerCluster()
{
    if (_blog)
        cout << " ** Destroy TLoadLevelerCluster [" << GetName() << "] Title["
                << GetTitle() << "]" << endl;
}

///////////////////////////////////////////////////////////
//  Construct and return the list of node
///////////////////////////////////////////////////////////
void TLoadLevelerCluster::init()
{
    TString snodes = GetTitle();

    if (_blog)
        cout << " ** TCluster::getListOfNodes Node[" << snodes << "]" << endl;

    Int_t nbegin = 0;
//     Int_t nbproc_heart;
//     TString nbproc_heart_alpha = "";
    TString node = "";
//     TString ssnodes = "";
//     addNodeName(node, 1);
//     setNNodes(1);
    while (not snodes.IsNull())
    {
        nbegin = snodes.Index(' ');
        node = snodes(0, nbegin);
        snodes.Remove(0,node.Sizeof());
//         snodes.ReplaceAll(node + " ", "");
//         nbegin = snodes.Index(' ');
//         nbproc_heart_alpha = snodes(0, nbegin);
//         nbproc_heart = atoi(nbproc_heart_alpha.Data());
//         nbegin = snodes.Index(":");
//         snodes = snodes(nbegin + 1, snodes.Sizeof() - 1);
        addNodeName(node, 1);
        setNNodes(1);

        std::fstream nodefile;
        nodefile.open(
                gSystem->Getenv("TMPDIR") + TString("/") + node
                        + TString(".txt"), std::ios::out | std::ios::trunc);
        if (nodefile.is_open())
            nodefile << node << endl;
        nodefile.close();
    }

    TCluster::getListOfNodes();
}

void TLoadLevelerCluster::getCommandPrefix(char *mpiexe, char* procs)
{
    cout << "MPIRUN FOR LOADLEVELER" << endl;
    cout << "procs = " << procs << endl;

    strcpy(mpiexe, "mpirun ");
    strcat(mpiexe, " --machinefile ");
    TString machine = gSystem->Getenv("TMPDIR") + TString("/") + procs
            + TString(".txt");
    strcat(mpiexe, machine);
    strcat(mpiexe, " -np 1 ");
}


///////////////////////////////////////////////////////////
//  Log
///////////////////////////////////////////////////////////
void TLoadLevelerCluster::printLog(Option_t *option)
{
    cout << "TLoadLevelerCluster::printLog[" << option << "]" << endl;
    TCluster::printLog(option);
}

} // Fin du namespace Launcher
} // Fin du namespace URANIE

 
