//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOutputFile.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mon oct 9th 14:52:50 CEST 2006
 
 \brief Interface of the class URANIE::Launcher::TOutputFile
 
 */
/**
 
 @class URANIE::Launcher::TOutputFile

 @brief Description of the class TOutputFile.

 This classe enables to raise information from an ASCII file with the "Clef = valeur ;" formalism

 \verbatim
 coutMoy = 3.741194e-01 ;
 coutSig = 1.385948e-02 ;
 coutInc = 1.000000e+02 ;
 coutMax = 5.293728e-01 ;
 \endverbatim

 Thus, to raise the 2 informations \em coutSig and \em coutMax from file \em _coutwithkey_.dat exposed above,
 the code of \uranie is as follows :

 \verbatim
 TOutputFile *fout = new TOutputFile("outputfile.dat");
 
 fout->addAttribute(new TAttribute("coutSig"));
 fout->addAttribute(new TAttribute("coutMax"));
 \endverbatim

 \see TOutputFileRow, TOutputFileColumn, TOutputFileKey and TOutputFileDataServer


 */
#ifndef TOUTPUTFILE_H
#define TOUTPUTFILE_H

// ROOT
#include "Rtypes.h"
#include "TString.h"
#include "TList.h"
// URANIE
#include "Launcher.h"
#include "TCodeFile.h"

#include <vector>
#include <map>
#include <sstream>
#include <string>

using std::string;
using std::map;

namespace URANIE
{
namespace Launcher
{
class TOutputFile: public TCodeFile
{
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TOutputFile(TString str);
    //! Default destructor
    virtual ~TOutputFile();
    //@}

    //---------------------------------------------
    /** @name Add attribut to the File
     **/
    //@{
    //!   Add the attribute in the file
    /*!
     */
    virtual void addAttribute(URANIE::DataServer::TAttribute*);
    //! Add the attribute by this name in the file
    /*!
     */
    virtual void addAttribute(TString name);
    //@}

    //---------------------------------------------
    /** @name Computes Output
     **/
    //@{
//       virtual Bool_t searchOutput(Int_t &n, Double_t *val, TList *lstOut = NULL);
    virtual Bool_t searchOutput(Int_t &n, Double_t *val,
            TList *lstOut = NULL) = 0;

    virtual Bool_t searchOutput(vector<URANIE::DataServer::UEntry*> *out)=0;//{ return 0; }
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Launcher::TOutputFile, ID_LAUNCHER)
    //Classe de
};

} // Fin du namespace Launcher
} // Fin du namespace URANIE
#endif
