//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TamponTexte.h
 \author Vincent Bergeaud <vincent.bergeaud@cea.fr>
 \date   wed may 29th 14:21:10 CEST 2013
 
 \brief Tool for input file management
 
 */
/**
 
 * @class TamponTexte
 *
 * @brief TamponTExte contains basic tools for handling strings in data file manipulations
 *
 * TamponTexte comes as a replacement for the CLUB equivalent class
 *
 */
#ifndef TAMPONTEXTE_H
#define TAMPONTEXTE_H

#include <stdlib.h>
#include <string>
#include <iostream>
#include <vector>

#include "Launcher.h"

#ifndef WIN32
#define USE_COLORED_CONSOLE
#endif

struct TamponTexteError
{
    std::string _message;
    TamponTexteError(const std::string& st)
    {
        _message = st;
    }
    void printMessage()
    {
#ifdef USE_COLORED_CONSOLE        
        std::cout <<"\033[31m"<< _message << "\033[0m"  << std::endl;
#else
        std::cout << _message << std::endl; 
#endif        
    }
};

class TamponTexte
{
public:
    TamponTexte(const char*, int dummy = 0);
    void ecrit(const char* nom = 0);
    const char* champLigne(int ifield, int iline);
    const char* champ(int ifield);
    std::string ligne(int iline);
    void initSeparateurs(const char*);
    void initFieldSeparateur(const char*);
    void initCarCommentaires(const char*);
    void elimineCommentaires();
    void elimineBlancs();
    bool estVide() const;
    int nombreLignes();
    int nombreChamps();
    int nombreChampsLigne(int);
    void remplaceChamp(int, const char*);
    const char*nom() const;

private:
    bool _blog;
    std::vector<std::string> _separators;
    std::string _cFieldSeparator;
    std::vector<std::string> _comments;
    std::string _buffer;
    std::string _name;
    void replaceAll(std::string& buffer, const std::string& str1,
            const std::string& str2);
};

#endif
