//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   jeu jan 24 15:56:01 2008
 
 $Revision$
 $Date$
 
 \brief Programme principal pour les tests unitaires sur la librairie DataServer.
 
 */
#include "cppunit/CompilerOutputter.h"
#include "cppunit/TestResult.h"
#include "cppunit/TestResultCollector.h"
#include "cppunit/TextTestProgressListener.h"
#include "cppunit/BriefTestProgressListener.h"
#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/TestRunner.h"
#include "cppunit/XmlOutputter.h"

#include <stdexcept>

#include <iostream>
#include <fstream>

//uranie
#include "UVersion.h"
// Liste des Tests Unitaires d'uranie
#include "TestTLauncher.h"
#include "TestTLauncherFunction.h"
#include "TestTCluster.h"
#include "TestTamponTexte.h"
#include "TestTCode.h"
#ifdef WITH_FMI
#include "TestTCodeFMU.h"
#endif
#include "uranie_commands.h"

CPPUNIT_TEST_SUITE_REGISTRATION (TestTamponTexte);
CPPUNIT_TEST_SUITE_REGISTRATION (TLauncherTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TLauncherFunctionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TCodeTest);
#ifdef WITH_FMI
CPPUNIT_TEST_SUITE_REGISTRATION (TCodeFMUTest);
#endif
CPPUNIT_TEST_SUITE_REGISTRATION (TClusterTest);

bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{

    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("Launcher",
                                       "testLauncherResults.xml", "..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("Launcher",
                                       "testLauncherResults.xml", "../../../testReport.xsl");
#endif

    ////////////////
    // Clean
    ////////////////

    UranieTests::removeFolder(gSystem->pwd(),"tmpLanceurUranie");
    UranieTests::removeFolder(gSystem->pwd(),"URANIE");

    // Clean
    gDirectory->ls();
    
    // ---  Return error code 1 if the one of test failed.
    return wasSuccessful ? 0 : 1;
}
