//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTCluster.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Launcher::TLauncher
 
 Cr�e le : mer nov 21 07:59:24 2007

 \par Modifications
 \li M. Salvoldi (Areva/OCC) 2011/09/28
 - Authorize  the acces to testSlurm() and  testLocal() only on Unix platform.
 \li C. Forgues (2011-01-06 19:02:59)
 - Getenv("HOSTNAME") is replaced by HostName().  
 // ***************************************************************
 
 */
//CPPUNIT
#define DBL_PRECISION_COMPARE 1.e-2

// Uranie
#include "TestTCluster.h"
#include "TAttribute.h"
#include "TUniformDistribution.h"
#include "TCluster.h"
#include "TSlurmCluster.h"
#include "TLSFCluster.h"
#include "TLocalHostCluster.h"

#define ASCIIDATAFILE  "flowrateborhole.dat"
#define ASCIIDATAFILE2 "flowrateborhole2.dat"
#define ASCIIDATAFILE3 "flowrateUniformDesign.dat"
#define ASCIIDATAFILE4 "flowrate1200Samples.dat"
// ROOT
#include "TString.h"
#include "TMath.h"

#include <iostream>
using namespace std;

TClusterTest::TClusterTest()
{
}

TClusterTest::~TClusterTest()
{
}

void TClusterTest::setUp()
{
    cout << endl;
    //cout<<"*******************************************************************************"<<endl;
    //cout<<"*******************************************************************************"<<endl;  
}

void TClusterTest::tearDown()
{
    //cout << endl << endl << " *** Debut du tearDown ****" << endl << endl;
    // Delete

    cout
            << "*******************************************************************************"
            << endl;
    cout
            << "*******************************************************************************"
            << endl;

}

////////////////////////////////
///                          ///
///         Tests            ///
///                          ///
////////////////////////////////

#ifdef UNIX
void TClusterTest::testSlurm()
{
    TString splatine1="platine257";
    TString splatine1t="8(x1)";
    TString splatine2="platine[312-313]";
    TString splatine2t="8(x2)";
    TString splatine3="platine[267,325,343,428,431,537,620,696]";
    TString splatine3t="8(x7),7(x1)";
    TString splatine4="platine[1068-1069,1085,1088,1091-1093,1112]";
    TString splatine4t="8(x8)";
    TString splatine5="platine[001-008]";
    TString splatine5t="8(x8)";

    CPPUNIT_ASSERT(! URANIE::Launcher::TSlurmCluster::eligible());

    gSystem->Setenv("SLURM_NODELIST", splatine1);
    gSystem->Setenv("SLURM_TASKS_PER_NODE", splatine1t);
    CPPUNIT_ASSERT(URANIE::Launcher::TSlurmCluster::eligible());

    URANIE::Launcher::TSlurmCluster tcl1;
    TString listOfNodes1 = tcl1.getListOfNodes();
    CPPUNIT_ASSERT( !strcmp(listOfNodes1, "platine257 platine257 platine257 platine257 platine257 platine257 platine257 platine257 "));

    gSystem->Setenv("SLURM_NODELIST", splatine2);
    gSystem->Setenv("SLURM_TASKS_PER_NODE", splatine2t);
    URANIE::Launcher::TSlurmCluster tcl2;
    TString listOfNodes2 = tcl2.getListOfNodes();
    CPPUNIT_ASSERT( !strcmp(listOfNodes2, "platine312 platine312 platine312 platine312 platine312 platine312 platine312 platine312 platine313 platine313 platine313 platine313 platine313 platine313 platine313 platine313 "));

    gSystem->Setenv("SLURM_NODELIST", splatine3);
    gSystem->Setenv("SLURM_TASKS_PER_NODE", splatine3t);
    URANIE::Launcher::TSlurmCluster tcl3;
    TString listOfNodes3 = tcl3.getListOfNodes();
    CPPUNIT_ASSERT( !strcmp(listOfNodes3, "platine267 platine267 platine267 platine267 platine267 platine267 platine267 platine267 platine325 platine325 platine325 platine325 platine325 platine325 platine325 platine325 platine343 platine343 platine343 platine343 platine343 platine343 platine343 platine343 platine428 platine428 platine428 platine428 platine428 platine428 platine428 platine428 platine431 platine431 platine431 platine431 platine431 platine431 platine431 platine431 platine537 platine537 platine537 platine537 platine537 platine537 platine537 platine537 platine620 platine620 platine620 platine620 platine620 platine620 platine620 platine620 platine696 platine696 platine696 platine696 platine696 platine696 platine696 "));

    gSystem->Setenv("SLURM_NODELIST", splatine4);
    gSystem->Setenv("SLURM_TASKS_PER_NODE", splatine4t);
    URANIE::Launcher::TSlurmCluster tcl4;
    TString listOfNodes4 = tcl4.getListOfNodes();
    CPPUNIT_ASSERT( !strcmp(listOfNodes4, "platine1068 platine1068 platine1068 platine1068 platine1068 platine1068 platine1068 platine1068 platine1069 platine1069 platine1069 platine1069 platine1069 platine1069 platine1069 platine1069 platine1085 platine1085 platine1085 platine1085 platine1085 platine1085 platine1085 platine1085 platine1088 platine1088 platine1088 platine1088 platine1088 platine1088 platine1088 platine1088 platine1091 platine1091 platine1091 platine1091 platine1091 platine1091 platine1091 platine1091 platine1092 platine1092 platine1092 platine1092 platine1092 platine1092 platine1092 platine1092 platine1093 platine1093 platine1093 platine1093 platine1093 platine1093 platine1093 platine1093 platine1112 platine1112 platine1112 platine1112 platine1112 platine1112 platine1112 platine1112 "));

    gSystem->Unsetenv("SLURM_NODELIST");
    CPPUNIT_ASSERT(! URANIE::Launcher::TSlurmCluster::eligible());
    
    gSystem->Setenv("SLURM_NODELIST", splatine5);
    gSystem->Setenv("SLURM_TASKS_PER_NODE", splatine5t);
    URANIE::Launcher::TSlurmCluster tcl5;
    TString listOfNodes5 = tcl5.getListOfNodes();
    CPPUNIT_ASSERT( !strcmp(listOfNodes5, "platine001 platine001 platine001 platine001 platine001 platine001 platine001 platine001 platine002 platine002 platine002 platine002 platine002 platine002 platine002 platine002 platine003 platine003 platine003 platine003 platine003 platine003 platine003 platine003 platine004 platine004 platine004 platine004 platine004 platine004 platine004 platine004 platine005 platine005 platine005 platine005 platine005 platine005 platine005 platine005 platine006 platine006 platine006 platine006 platine006 platine006 platine006 platine006 platine007 platine007 platine007 platine007 platine007 platine007 platine007 platine007 platine008 platine008 platine008 platine008 platine008 platine008 platine008 platine008 "));

    gSystem->Unsetenv("SLURM_NODELIST");
    CPPUNIT_ASSERT(! URANIE::Launcher::TSlurmCluster::eligible());

}
#endif

#ifdef UNIX
void TClusterTest::testLSF()
{
    TString sListOfHosts="titane1516 8 titane1517 2";
    const char* tmpEnv=gSystem->Getenv("LSB_MCPU_HOSTS");
    gSystem->Setenv("LSB_MCPU_HOSTS", sListOfHosts);
    URANIE::Launcher::TLSFCluster tcl1;
    TString listOfNodes1 = tcl1.getListOfNodes();
    gSystem->Setenv("LSB_MCPU_HOSTS","");
    CPPUNIT_ASSERT( !strcmp(listOfNodes1.Data(), "titane1516 titane1516 titane1516 titane1516 titane1516 titane1516 titane1516 titane1516 titane1517 titane1517 "));

    if (tmpEnv !=0) gSystem->Setenv("LSB_MCPU_HOSTS",tmpEnv);
    else gSystem->Unsetenv("LSB_MCPU_HOSTS");
}
#endif

#ifdef UNIX
void TClusterTest::testLocal()
{
    const char* hn = gSystem->HostName();
    size_t hsz = strlen(hn);
    char mpiexe[32];
    char hote[32] = "hote";

    CPPUNIT_ASSERT(! URANIE::Launcher::TLocalHostCluster::eligible(""));
    CPPUNIT_ASSERT( URANIE::Launcher::TLocalHostCluster::eligible("localhost=3"));

    URANIE::Launcher::TLocalHostCluster ls;
    TString listOfNode = ls.getListOfNodes();
    CPPUNIT_ASSERT( !strncmp(listOfNode, hn, hsz));
    CPPUNIT_ASSERT( strlen(listOfNode) == 3*(hsz+1) );

    ls.getCommandPrefix(mpiexe, hote);
    CPPUNIT_ASSERT(! strcmp(mpiexe, ""));

}
#endif
//! This test is meant to be run on the Titane CCRT machine
//! On other machines it will pass doing nothing
#ifdef UNIX
void TClusterTest::testTitaneRun()
{
    // checking the machine 
    const char* hosts = gSystem->HostName();
    const char* usys= gSystem->Getenv("URANIESYS");
    if (strncmp(hosts,"irene",7)!=0) return;
    //on titane, perform a test by creating a batch
    ofstream bsubfile("test_lsf.bsub");
    bsubfile <<"#BSUB -n 16"<<endl;
    bsubfile <<"export OMPI_MCA_orte_process_binding=none"<<endl;
    bsubfile <<"source $"<<usys<<"/souRCE/uranie.bashrc"<<endl;
    bsubfile <<" root -b -l -q testTitane.C"<<endl;
    bsubfile.close();
    gSystem->Exec("ccc_msub test_lsf.bsub");
    gSystem->Sleep(100000);
    string dir("./tmpTestUranieTitane/UranieResults");
    TString file("uranielauncher.root");
    const char* f=gSystem->FindFile(dir.c_str(),file,kFileExists);
    CPPUNIT_ASSERT(f!=0);
}
#endif
//! End Of File
