//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TLAUNCHERFUNCTION_H
#define CPPUNITEST_TLAUNCHERFUNCTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TDataServer.h"
#include "UExceptions.h"
#include "TLauncherFunction.h"

class TLauncherFunctionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TLauncherFunctionTest);

    CPPUNIT_TEST (testNames);
    CPPUNIT_TEST (testTitles);
    CPPUNIT_TEST (testLauncherFunctionWithoutInputNames);
    CPPUNIT_TEST (testLauncherFunctionWithInputNames);
    CPPUNIT_TEST (testLauncherFunctionRun);
    CPPUNIT_TEST (testLauncherFunctionSetNOutput);
    CPPUNIT_TEST (testLauncherFunctionSetNOutputAndTupleGestion);
    CPPUNIT_TEST (testMultipleLauncherFunction);
    CPPUNIT_TEST (testMultipleLauncherFunctionWithMultidimensionalOutput);
    CPPUNIT_TEST (throwExceptionLauncherFunctionConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionLauncherFunctionConstructorFctUndefined);
    CPPUNIT_TEST (throwExceptionOutputExists);
    CPPUNIT_TEST (throwExceptionInputNotExists);
    CPPUNIT_TEST (testFunctionComposition);
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TLauncherFunctionTest();
    //! Default destructor
    virtual ~TLauncherFunctionTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    //! Original SetUp
    void setUp();
    //! Original tearDown
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of the names
    void testNames();
    //! Test of the titles
    /*!
     The title given by the constructor, the default title, and the title given with the setter method
     \em setTitle.
     */
    void testTitles();
    //@}

    //---------------------------------------------
    /** @name Unitary Tests of run
     The list of unitary test of the TLauncherFunction class
     **/
    //@{
    //! Tests the correct behaviour of TLauncherFunction constructor with a string given as the name of the function and "empty" inputs + run().
    void testLauncherFunctionWithoutInputNames();
    //! Tests the correct behaviour of TLauncherFunction constructor with a string given as the name of the function and explicit inputs + run().
    void testLauncherFunctionWithInputNames();
    //! Tests the correct behaviour of run method with given inputs and outputs.
    void testLauncherFunctionRun();
    //! Tests the setNOutput method
    void testLauncherFunctionSetNOutput();
    //! Tests the setNOutput method and the way dataserver deals memory
    void testLauncherFunctionSetNOutputAndTupleGestion();
    //! Throws an exception when the TDS is NULL
    void throwExceptionLauncherFunctionConstructorTdsNull();
    //! Throws an exception when the function wan not found
    void throwExceptionLauncherFunctionConstructorFctUndefined();
    //! Throws an exception when add output attribute which already exists
    void throwExceptionOutputExists();
    //! Throws an exception when using input attributes no exist
    void throwExceptionInputNotExists();
    //! Tests the correct behaviour of TLauncherFunction when several TLauncherFunction objects are defined
    void testMultipleLauncherFunction();
    //! Tests the correct behaviour of TLauncherFunction when several TLauncherFunction objects are defined with multidimensional outputs
    void testMultipleLauncherFunctionWithMultidimensionalOutput();
    
     //!Tests DI285: verify that we can use the results of a first calcul as the the data for a second calcul using the same TDataServer.
    void testFunctionComposition();
    
    //@}

private:
    TString _workingTestPath;
    URANIE::DataServer::TDataServer *tds1, *tds2, *tds3;
    URANIE::Launcher::TLauncherFunction * tlf1;
    Bool_t _bLog;

#ifdef ROOT5
    void * _p2fFlowrate;        ///< Pointer to the Flowrate Analytical function
    void * _p2fHoXuSurrogateModel; ///< Pointer to the HoXuSurrogateModel Analytical function
    void * _p2fFlowrateSquared; ///< Pointer to the FlowrateSquared Analytical function
    void * _p2fSum;                 ///< Pointer to the Sum  Analytical function
    void * _p2fProd;               ///< Pointer to the Prod  Analytical function
    void * _p2fUndefined;          ///< Pointer to the Prod  Analytical function
    void * _p2FctIshigami;          ///< Pointer to the FctIshigami  Analytical function
    void * _p2FctIshigamiPlus10;          ///< Pointer to the FctIshigamiPlus10  Analytical function
    void * _p2FctIshigamiPlus20;          ///< Pointer to the FctIshigamiPlus20  Analytical function
    void * _p2FctIshigamiPlus30;          ///< Pointer to the FctIshigamiPlus30  Analytical function
#endif
};
#endif  // CPPUNITEST_TDATASERVER_H
