{
  // Load the function flowrateCalib1D
  gROOT->LoadMacro("UserFunctions.C");
  
  // Input reference file   
  TString ExpData="Ex2DoE_n100_sd1.75.dat";
  
  // define the reference
  TDataServer *tdsRef = new TDataServer("tdsRef","doe_exp_Re_Pr");
  tdsRef->fileDataRead(ExpData.Data());
  tdsRef->addAttribute("wei_exp","1./(sd_eps*sd_eps)");
  
  // define the parameters
  TDataServer *tdsPar = new TDataServer("tdsPar","pouet");
  tdsPar->addAttribute( new TUniformDistribution("hl", 700.0, 760.0) );
  
  // Create the output attribute
  TAttribute *out = new TAttribute("out");
  
  // Create interface to assessors
  TCIntEval *Model = new TCIntEval("flowrateCalib1D");
  Model->addInput(tdsPar->getAttribute("hl"));
  Model->addInput(tdsRef->getAttribute("rw"));
  Model->addInput(tdsRef->getAttribute("l"));
  Model->addOutput(out);
  
  // Set the runner
  TSequentialRun *runner = new TSequentialRun(Model);    
  
  // Set the calibration object
  TMetropHasting *cal = new TMetropHasting(tdsPar,runner,2000,"");
  cal->setDistanceAndReference("weightedLS",tdsRef,"rw:l","Qexp","wei_exp");
  cal->setNbDump(400);
  cal->setAcceptationRatioRange(0.4, 0.45);
  cal->estimateParameters();
  
  // Quality assessment :  Draw the trace the MCMC
  TCanvas *canTr = new TCanvas("CanTr","CanTr",1200,800);
  TPad *padTr = new TPad("padTr","padTr",0, 0.03, 1, 1);  padTr->Draw();  padTr->cd();
  cal->drawTrace("Trace title","*","","nonewcanvas");
  
  // Draw the parameters
  TCanvas *canPar = new TCanvas("CanPar","CanPar",1200,800);
  TPad *padPar = new TPad("padPar","padPar",0, 0.03, 1, 1);  padPar->Draw();  padPar->cd();
  cal->drawParameters("Parameter title","*","","nonewcanvas");
  
  // Draw the residuals
  TCanvas *canRes = new TCanvas("CanRes","CanRes",1200,800);
  TPad *padRes = new TPad("padRes","padRes",0, 0.03, 1, 1);  padRes->Draw();  padRes->cd();
  cal->drawResidues("Residual title","*","","nonewcanvas");
  
  // Compute the auto-correlation
  int burn=20; // Remove first 20 elements
  vector<int> lag={1,5,10,20};
  vector<double> autoCorr;
  cal->getAutoCorrelation(lag, &autoCorr, burn);
  
  cout<<"Autocorrelation are "<<autoCorr.size()<<":"<<endl;
  for(unsigned il=0; il<lag.size(); il++)
  {
    cout<<"*** for lag="<<lag.at(il)<<": ";
    for(unsigned ip=0; ip<cal->getNPar(); ip++)            
    cout<<autoCorr.at(ip*lag.size()+il)<<";  ";
    cout<<endl;
  } 
}