{
  // Load the function flowrateCalib1D
  gROOT->LoadMacro("UserFunctions.C");
  
  // Input reference file   
  TString ExpData="Ex2DoE_n100_sd1.75.dat";
  
  // define the reference
  TDataServer *tdsRef = new TDataServer("tdsRef","doe_exp_Re_Pr");
  tdsRef->fileDataRead(ExpData.Data());
  
  // define the parameters
  TDataServer *tdsPar = new TDataServer("tdsPar","pouet");
  tdsPar->addAttribute( new TAttribute("hl", 700.0, 760.0) );
  tdsPar->getAttribute("hl")->setDefaultValue(728.0);
  
  // Create the output attribute
  TAttribute *out = new TAttribute("out");
  
  // Create interface to assessors
  TCIntEval *Model = new TCIntEval("flowrateCalib1D");
  Model->addInput(tdsPar->getAttribute("hl"));
  Model->addInput(tdsRef->getAttribute("rw"));
  Model->addInput(tdsRef->getAttribute("l"));
  Model->addOutput(out);
  
  // Set the runner
  TSequentialRun *runner = new TSequentialRun(Model);    
  
  // Set the calibration object
  TMinimisation *cal = new TMinimisation(tdsPar,runner,1);
  cal->setDistanceAndReference("LS",tdsRef,"rw:l","Qexp");
  TNloptSubplexe solv;
  cal->setOptimProperties(&solv); 
  cal->estimateParameters();
  
  // Draw the residuals
  TCanvas *canRes = new TCanvas("CanRes","CanRes",1200,800);
  TPad *apad = new TPad("apad","apad",0, 0.03, 1, 1);  apad->Draw();  apad->cd();
  cal->drawResidues("Residual title","*","","nonewcanvas");
  
}