{
  // Load the function flowrateCalib1D
  gROOT->LoadMacro("UserFunctions.C");
  
  // Input reference file   
  TString ExpData="Ex2DoE_n100_sd1.75.dat";
  
  // define the reference
  TDataServer *tdsRef = new TDataServer("tdsRef","doe_exp_Re_Pr");
  tdsRef->fileDataRead(ExpData.Data());
  
  // define the parameters
  TDataServer *tdsPar = new TDataServer("tdsPar","pouet");
  tdsPar->addAttribute( new TUniformDistribution("hl", 700.0, 760.0) );
  
  // Create the output attribute
  TAttribute *out = new TAttribute("out");
  
  // Create interface to assessors
  TCIntEval *Model = new TCIntEval("flowrateCalib1D");
  Model->addInput(tdsPar->getAttribute("hl"));
  Model->addInput(tdsRef->getAttribute("rw"));
  Model->addInput(tdsRef->getAttribute("l"));
  Model->addOutput(out);
  
  // Set the runner
  TSequentialRun *runner = new TSequentialRun(Model);    
  
  // Set the calibration object
  Int_t nABC = 100; Double_t eps = 0.05;
  TRejectionABC *cal = new TRejectionABC(tdsPar, runner, nABC, ""); 
  cal->setDistanceAndReference("LS",tdsRef,"rw:l","Qexp");
  cal->setGaussianNoise("sd_eps");
  cal->setPercentile(eps);
  cal->estimateParameters();
    
  // Compute statistics
  tdsPar->computeStatistic();    
  cout << "The mean of hl is " << tdsPar->getAttribute("hl")->getMean() << endl;
  cout << "The std of hl is " << tdsPar->getAttribute("hl")->getStd() << endl;
  
  // Draw the parameters
  TCanvas *canPar = new TCanvas("CanPar","CanPar",1200,800);
  TPad *padPar = new TPad("padPar","padPar",0, 0.03, 1, 1);  padPar->Draw();  padPar->cd();
  cal->drawParameters("Parameter title","*","","nonewcanvas");
  
  // Draw the residuals
  TCanvas *canRes = new TCanvas("CanRes","CanRes",1200,800);
  TPad *padRes = new TPad("padRes","padRes",0, 0.03, 1, 1);  padRes->Draw();  padRes->cd();
  cal->drawResidues("Residual title","*","","nonewcanvas"); 
}