"""
Example of flowrate code launching with flag file in failure mode
"""
from rootlogon import ROOT, DataServer, Launcher, Sampler

nS = 400
# Define the DataServer
tds = DataServer.TDataServer("tdsflowrate", "DataBase flowrate")
# Add the eight attributes of the study with uniform law
tds.addAttribute(DataServer.TUniformDistribution("rw", 0.05, 0.15))
tds.addAttribute(DataServer.TUniformDistribution("r", 100.0, 50000.0))
tds.addAttribute(DataServer.TUniformDistribution("tu", 63070.0, 115600.0))
tds.addAttribute(DataServer.TUniformDistribution("tl", 63.1, 116.0))
tds.addAttribute(DataServer.TUniformDistribution("hu", 990.0, 1110.0))
tds.addAttribute(DataServer.TUniformDistribution("hl", 700.0, 820.0))
tds.addAttribute(DataServer.TUniformDistribution("l", 1120.0, 1680.0))
tds.addAttribute(DataServer.TUniformDistribution("kw", 9855.0, 12045.0))

# The input file
sFileName = "flowrate_input_with_flags.in"

# Set the input file with flag format and the flag for each input attributes
tds.getAttribute("rw").setFileFlag(sFileName, "@Rw@")
tds.getAttribute("r").setFileFlag(sFileName, "@R@")
tds.getAttribute("tu").setFileFlag(sFileName, "@Tu@")
tds.getAttribute("tl").setFileFlag(sFileName, "@Tl@")
tds.getAttribute("hu").setFileFlag(sFileName, "@Hu@")
tds.getAttribute("hl").setFileFlag(sFileName, "@Hl@")
tds.getAttribute("l").setFileFlag(sFileName, "@L@")
tds.getAttribute("kw").setFileFlag(sFileName, "@Kw@")

# Generate the sample
sampling = Sampler.TSampling(tds, "lhs", nS)
sampling.generateSample()

# The output file of the code where values are stored in row
fout = Launcher.TOutputFileRow("_output_flowrate_withRow_.dat")
# The attribute in the output file
tyhat = DataServer.TAttribute("yhat")
tyhat.setDefaultValue(-200.0)
fout.addAttribute(tyhat)

# Create a Launcher.TCode object
mycode = Launcher.TCode(tds, "flowrate -s -ff")
# Add the output file where to find the output attributes
mycode.addOutputFile(fout)

# Create a TLauncher object from a TDataServer and a TCode objects
tlch = Launcher.TLauncher(tds, mycode)
tlch.setSave()
# tlch.setClean()
tlch.setVarDraw("hu:hl", "yhat>0", "")
Pwd = ROOT.gSystem.Getenv("PWD")
# tlch.setWorkingDirectory(Pwd + "/tmpLanceurUranie/flowrate"))

tlch.run()

# Visualisation
Canvas = ROOT.TCanvas("c1", "Graph launchCodeFlowrateFlagFailure",
                      5, 64, 1270, 667)
tds.draw("hu:hl", "yhat>0", "")

tds.exportData("_flowrate_sampler_launcher_.dat")
