"""
Example of flowrate code launching with key database
"""
from rootlogon import ROOT, DataServer, Launcher

# Create a DataServer.TDataServer
tds = DataServer.TDataServer()
# Load the data base in the DataServer
tds.fileDataRead("flowrateUniformDesign.dat")

# The reference input file
sIn = "flowrate_input_with_keys.in"

# Set the reference input file and the key for each input attributes
tds.getAttribute("rw").setFileKey(sIn, "Rw")
tds.getAttribute("r").setFileKey(sIn, "R")
tds.getAttribute("tu").setFileKey(sIn, "Tu")
tds.getAttribute("tl").setFileKey(sIn, "Tl")
tds.getAttribute("hu").setFileKey(sIn, "Hu")
tds.getAttribute("hl").setFileKey(sIn, "Hl")
tds.getAttribute("l").setFileKey(sIn, "L")
tds.getAttribute("kw").setFileKey(sIn, "Kw")

# The output file of the code
fout = Launcher.TOutputFileRow("_output_flowrate_withRow_.dat")
# The attribute in the output file
fout.addAttribute(DataServer.TAttribute("yhat"))

# Instanciation de mon code
mycode = Launcher.TCode(tds, "flowrate -s -k")
# on ajoute le fichier de sortie du code
mycode.addOutputFile(fout)

# Lancement du code
laun = Launcher.TLauncher(tds, mycode)
laun.setSave()
laun.setClean()
# laun.setWorkingDirectory(ROOT.gSystem.pwd() + "/tmpLaunUranie/flowrate")
laun.setVarDraw("yhat:rw", "", "")
laun.run()


# Graph
Canvas = ROOT.TCanvas("c1", "Graph launchCodeFlowrateKeyDataBase",
                      5, 64, 1270, 667)
pad = ROOT.TPad("pad", "pad", 0, 0.03, 1, 1)
pad.Draw()
pad.Divide(2, 4)

pad.cd(1)
tds.draw("yhat:rw")
pad.cd(2)
tds.draw("yhat:r")
pad.cd(3)
tds.draw("yhat:tu")
pad.cd(4)
tds.draw("yhat:tl")
pad.cd(5)
tds.draw("yhat:hu")
pad.cd(6)
tds.draw("yhat:hl")
pad.cd(7)
tds.draw("yhat:l")
pad.cd(8)
tds.draw("yhat:kw")
