"""
Example of code launching with output dataserver
"""
from rootlogon import ROOT, DataServer, Launcher, Sampler

nS = 100
# Define the DataServer
tds = DataServer.TDataServer("tdsFlowrate", "DoE for Flowrate")

# Add the study attributes ( min, max and nominal values)
tds.addAttribute(DataServer.TUniformDistribution("rw", 0.05, 0.15))
tds.addAttribute(DataServer.TUniformDistribution("r", 100.0, 50000.0))
tds.addAttribute(DataServer.TUniformDistribution("tu", 63070.0, 115600.0))
tds.addAttribute(DataServer.TUniformDistribution("tl", 63.1, 116.0))
tds.addAttribute(DataServer.TUniformDistribution("hu", 990.0, 1110.0))
tds.addAttribute(DataServer.TUniformDistribution("hl", 700.0, 820.0))
tds.addAttribute(DataServer.TUniformDistribution("l", 1120.0, 1680.0))
tds.addAttribute(DataServer.TUniformDistribution("kw", 9855.0, 12045.0))

# The reference input file
sIn = "flowrate_input_with_keys.in"

# Set the reference input file and the key for each input attributes
InType = DataServer.TAttributeFileKey.kNewKey
tds.getAttribute("rw").setFileKey(sIn, "Rw", "", InType)
tds.getAttribute("r").setFileKey(sIn, "R", "", InType)
tds.getAttribute("tu").setFileKey(sIn, "Tu", "", InType)
tds.getAttribute("tl").setFileKey(sIn, "Tl", "", InType)
tds.getAttribute("hu").setFileKey(sIn, "Hu", "", InType)
tds.getAttribute("hl").setFileKey(sIn, "Hl", "", InType)
tds.getAttribute("l").setFileKey(sIn, "L", "", InType)
tds.getAttribute("kw").setFileKey(sIn, "Kw", "", InType)

# Generate the design of experiments
sampling = Sampler.TSampling(tds, "lhs", nS)
sampling.generateSample()

# The output file of the code
fout = Launcher.TOutputFileDataServer("_output_flowrate_withRow_.dat")
# The attribute in the output file
fout.addAttribute("yhat")  # fout.addAttribute(DataServer.TAttribute("y"))

# Create a Launcher.TCode object from a TDS and the command line
mycode = Launcher.TCode(tds, "flowrate -s -k")
# Add the output file
mycode.addOutputFile(fout)

# Create a launcher of code
laun = Launcher.TLauncher(tds, mycode)
laun.setSave()
laun.setClean()
# laun.setWorkingDirectory(ROOT.gSystem.pwd() + "/tmpLaunUranie/flowrate")

laun.setVarDraw("yhat:rw", "", "")

laun.run()

tds.exportData("_flowrate_sampler_launcher_.dat")
# Visualisation
Canvas = ROOT.TCanvas("c1",
                      "launchCodeFlowrateKeyRecreateSamplingOutputDataServer",
                      565, 62, 560, 619)
pad = ROOT.TPad("pad", "pad", 0, 0.03, 1, 1)
pad.Draw()
ROOT.gStyle.SetPalette(1)
pad.Divide(1, 3)

# Scatterplot y versus rw
pad.cd(1)
tds.draw("yhat:rw")

# Histogramm of the output attribute
pad.cd(2)
tds.draw("yhat")

# The parallel plot
pad.cd(3)
tds.draw("rw:r:tu:tl:hu:hl:l:kw:yhat", "", "para")
para = ROOT.gPad.GetListOfPrimitives().FindObject("ParaCoord")

# The output attribute
axis = para.GetVarList().FindObject("yhat")
axis.AddRange(ROOT.TParallelCoordRange(axis, 15.0, 20.0))
para.AddSelection("blue")
para.GetCurrentSelection().SetLineColor(ROOT.kBlue)

axis.AddRange(ROOT.TParallelCoordRange(axis, 155.0, 160.0))
para.AddSelection("blue")
