"""
Example of flowratecode launching with flag inputs
"""
from rootlogon import ROOT, DataServer, Launcher, Sampler

nS = 100
# Define the DataServer
tds = DataServer.TDataServer("tdsFlowrate", "DoE for Flowrate")

# Add the study attributes ( min, max and nominal values)
tds.addAttribute(DataServer.TUniformDistribution("rw", 0.05, 0.15))
tds.addAttribute(DataServer.TUniformDistribution("r", 100.0, 50000.0))
tds.addAttribute(DataServer.TUniformDistribution("tu", 63070.0, 115600.0))
tds.addAttribute(DataServer.TUniformDistribution("tl", 63.1, 116.0))
tds.addAttribute(DataServer.TUniformDistribution("hu", 990.0, 1110.0))
tds.addAttribute(DataServer.TUniformDistribution("hl", 700.0, 820.0))
tds.addAttribute(DataServer.TUniformDistribution("l", 1120.0, 1680.0))
tds.addAttribute(DataServer.TUniformDistribution("kw", 9855.0, 12045.0))

# The reference input file with key
sInKey = "flowrate_input_with_keys.in"

# Set the reference input file and the key for each input attributes
tds.getAttribute("rw").setFileKey(sInKey, "Rw")
tds.getAttribute("r").setFileKey(sInKey, "R")
tds.getAttribute("tu").setFileKey(sInKey, "Tu")
tds.getAttribute("tl").setFileKey(sInKey, "Tl")
tds.getAttribute("hu").setFileKey(sInKey, "Hu")
tds.getAttribute("hl").setFileKey(sInKey, "Hl")
tds.getAttribute("l").setFileKey(sInKey, "L")
tds.getAttribute("kw").setFileKey(sInKey, "Kw")

# Generate the Design of Experiments
sampling = Sampler.TSampling(tds, "lhs", nS)
sampling.generateSample()

# The output file of the code
fout = Launcher.TOutputFileRow("_output_flowrate_withRow_.dat")
# The attribute in the output file
fout.addAttribute(DataServer.TAttribute("yhat"))

# Instanciation de mon code
mycode = Launcher.TCode(tds, "flowrate -s -k")
# on ajoute le fichier de sortie du code
mycode.setReferenceDirectory(ROOT.gSystem.pwd())
mycode.addInputFile("flowrate_input_with_flags.in")

mycode.addOutputFile(fout)

# Lancement du code
laun = Launcher.TLauncher(tds, mycode)
# laun.setWorkingDirectory(ROOT.gSystem.pwd() + "/tmpLaunUranie/flowrate")
laun.setVarDraw("yhat:rw", "", "")

laun.run()

# Visualisation
Canvas = ROOT.TCanvas("c1", "Graph launchCodeFlowrateOATMinMax",
                      5, 64, 1270, 667)
ROOT.gStyle.SetPalette(1)
tds.draw("yhat:rw")

tds.startViewer()
