"""
Example of levelE code launching with column output format
"""
import sys
import numpy as np
from rootlogon import ROOT, DataServer, Launcher, Sampler

# Exit if levele not found
if ROOT.gSystem.Exec("which levele"):
    sys.exit(-1)

# Create DataServer and add input attributes
tds1 = DataServer.TDataServer("tds1", "levelE usecase")
tds1.addAttribute(DataServer.TUniformDistribution("t", 100, 1000))
tds1.addAttribute(DataServer.TLogUniformDistribution("kl", 0.001, 0.01))
tds1.addAttribute(DataServer.TLogUniformDistribution("kc", 0.000001, 0.00001))
tds1.addAttribute(DataServer.TLogUniformDistribution("v1", 0.001, 0.1))
tds1.addAttribute(DataServer.TUniformDistribution("l1", 100, 500))
tds1.addAttribute(DataServer.TUniformDistribution("r1", 1, 5))
tds1.addAttribute(DataServer.TUniformDistribution("rc1", 3, 30))
tds1.addAttribute(DataServer.TLogUniformDistribution("v2", 0.01, 0.1))
tds1.addAttribute(DataServer.TUniformDistribution("l2", 50, 200))
tds1.addAttribute(DataServer.TUniformDistribution("r2", 1, 5))
tds1.addAttribute(DataServer.TUniformDistribution("rc2", 3, 30))
tds1.addAttribute(DataServer.TLogUniformDistribution("w", 100000, 10000000))

# Tell the code where to find attribute value in input file
sJDD = "levelE_input_with_values_rows.in"
InType = DataServer.TAttributeFileKey.kNewRow
tds1.getAttribute("t").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("kl").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("kc").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("v1").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("l1").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("r1").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("rc1").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("v2").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("l2").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("r2").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("rc2").setFileKey(sJDD, "", "%e", InType)
tds1.getAttribute("w").setFileKey(sJDD, "", "%e", InType)

# Create DOE
ns = 100
samp = Sampler.TSampling(tds1, "lhs", ns)
samp.generateSample()

# How to read ouput files
_fout = Launcher.TOutputFileColumn("_output_levelE_withColumn_.dat")
_fout.addAttribute(DataServer.TAttribute("tps", DataServer.TAttribute.kVector))
_fout.addAttribute(DataServer.TAttribute("y", DataServer.TAttribute.kVector))
_fout.addAttribute(DataServer.TAttribute("z", DataServer.TAttribute.kVector))

# Creation of TCode
tc1 = Launcher.TCode(tds1, "levele 2> /dev/null")
tc1.addOutputFile(_fout)

# Run the code
tl1 = Launcher.TLauncher(tds1, tc1)
tl1.run()

# Draw the results
Can = ROOT.TCanvas("Can", "Can", 1)
Can.SetGrid()
Can.SetLogx()

mean = np.zeros([26])
stand = np.zeros([26])
tds1.computeStatistic("y")
for i in range(26):
    mean[i] = tds1.getAttribute("y").getMean(i)
    stand[i] = tds1.getAttribute("y").getStd(i)

tps = np.array([20000, 30000, 40000, 50000, 60000, 70000, 80000, 90000, 100000,
                200000, 300000, 400000, 500000, 600000, 700000, 800000, 900000,
                1e+06, 2e+06, 3e+06, 4e+06, 5e+06, 6e+06, 7e+06, 8e+06, 9e+06])
Zeros = np.zeros([26])
gr = ROOT.TGraphErrors(26, tps, mean, Zeros, stand)
gr.Draw("A*")
gr.SetTitle("")
gr.GetXaxis().SetTitle("Time")
gr.GetYaxis().SetTitle("<y>")
