{
  Int_t nS = 1000;
  // Create a TDataServer
  TDataServer * tds = new TDataServer("tdsFlowrate","TDS for flowrate");
  // Add the eight attributes of the study with uniform law
  tds->addAttribute( new TUniformDistribution("rw", 0.05, 0.15));
  tds->addAttribute( new TUniformDistribution("r", 100.0, 50000.0));
  tds->addAttribute( new TUniformDistribution("tu", 63070.0, 115600.0));
  tds->addAttribute( new TUniformDistribution("tl", 63.1, 116.0));
  tds->addAttribute( new TUniformDistribution("hu", 990.0, 1110.0));
  tds->addAttribute( new TUniformDistribution("hl", 700.0, 820.0));
  tds->addAttribute( new TUniformDistribution("l", 1120.0, 1680.0));
  tds->addAttribute( new TUniformDistribution("kw", 9855.0, 12045.0));
 
  // Generate the sampling from the TDataServer
  TSampling *sampling = new TSampling(tds, "lhs", nS);  
  sampling->generateSample();

  // Load the function in the UserFunction macros file
  gROOT->LoadMacro("UserFunctions.C");
  
  // Create a TLauncherFunction from a TDataServer and an analytical function
  // Rename the outpout attribute "ymod"
  TLauncherFunction * tlf = new TLauncherFunction(tds, "flowrateModel","","ymod");
  // Evaluate the function on all the design of experiments
  tlf->run();
  
  // Graph
  TCanvas  *Canvas = new TCanvas("c1", "Graph for the Macro launchFunctionSampling",5,90,935,614);
  TPad *pad = new TPad("pad","pad",0, 0.03, 1, 1); pad->Draw();
  pad->Divide(2,4);
  pad->cd(1); tds->drawTufte("rw:r","","same");
  pad->cd(2); tds->drawCDF("rw");
  pad->cd(3); tds->draw("ymod");
  pad->cd(4); tds->drawCDF("ymod","","ccdf");
  pad->cd(5); tds->draw("ymod:rw:hu");
  pad->cd(6); tds->drawProfile("ymod:r","","same");
  pad->cd(7); tds->drawProfile("ymod:rw","","same");
  pad->cd(8); tds->draw("ymod:rw","","colz");
    }