void PrintSolutions(TDataServer *tdsObs, TDataServer *tdsEstim, string title) 
{

  const int nbObs = tdsObs->getNPatterns();
  const int nbEst = tdsEstim->getNPatterns();
  vector<double> Rxarr(nbEst,0), x_arr(nbEst,0), y_est(nbEst,0), y_rea(nbEst,0), std_var(nbEst,0);
  
  tdsEstim->computeRank("x1");
  tdsEstim->getTuple()->copyBranchData(&Rxarr[0],nbEst,"Rk_x1");
  tdsEstim->getTuple()->copyBranchData(&x_arr[0],nbEst,"x1");
  tdsEstim->getTuple()->copyBranchData(&y_est[0],nbEst,"yEstim");
  tdsEstim->getTuple()->copyBranchData(&y_rea[0],nbEst,"y");
  tdsEstim->getTuple()->copyBranchData(&std_var[0],nbEst,"vEstim");
  
  vector<double> xarr(nbEst,0), yest(nbEst,0), yrea(nbEst,0), stdvar(nbEst,0);
  int ind;
  for (int i=0; i<nbEst; i++)
  {
    ind = int(Rxarr[i]) - 1;
    xarr[ind] = x_arr[i];
    yest[ind] = y_est[i];
    yrea[ind] = y_rea[i];
    stdvar[ind] = 2*sqrt(fabs(std_var[i]));
  }

  vector<double> xobs(nbObs,0); tdsObs->getTuple()->copyBranchData(&xobs[0],nbObs,"x1");
  vector<double> yobs(nbObs,0); tdsObs->getTuple()->copyBranchData(&yobs[0],nbObs,"y");

  TGraph *gobs = new TGraph(nbObs,&xobs[0],&yobs[0]);
  gobs->SetMarkerColor(1); gobs->SetMarkerSize(1); gobs->SetMarkerStyle(8);
  TGraphErrors *gest = new TGraphErrors(nbEst,&xarr[0],&yest[0],0,&stdvar[0]);
  gest->SetLineColor(2); gest->SetLineWidth(1); gest->SetFillColor(2); gest->SetFillStyle(3002); 
  TGraph *grea = new TGraph(nbEst,&xarr[0],&yrea[0]);
  grea->SetMarkerColor(4); grea->SetMarkerSize(1); grea->SetMarkerStyle(5); grea->SetTitle("Real Values");

  TMultiGraph *mg = new TMultiGraph("toto", title.c_str());
  mg->Add(gest,"l3"); mg->Add(gobs,"P"); mg->Add(grea,"P");
  mg->Draw("A");
  mg->GetXaxis()->SetTitle("x_{1}");

  TLegend *leg= new TLegend(0.4,0.65,0.65,0.8);
  leg->AddEntry(gobs, "Observations", "p");
  leg->AddEntry(grea, "Real values", "p");
  leg->AddEntry(gest, "Estimated values", "lf");
  leg->Draw();
}


void modelerTestKriging()
{
  //Create dataserver and read training database
  TDataServer *tdsObs = new TDataServer("tdsObs","observations"); 
  tdsObs->fileDataRead("utf-1D-train.dat");
  const int nbObs = 6; 

  // Canvas, divided in 2
  TCanvas *Can = new TCanvas("can","can",10,32,1600,700);
  TPad *apad = new TPad("apad","apad",0, 0.03, 1, 1);
  apad->Draw(); apad->Divide(2,1);
  
  //Name of the plot and correlation functions used
  string outplot="GaussAndMatern_1D_AllPoints.png";
  string Correl[2] = {"Gauss","Matern3/2"};

  //Pointer to needed objects, created in the loop
  TGPBuilder *gpb[2];
  TKriging *kg[2];
  TDataServer *tdsEstim[2];
  TLauncher2 *lkrig[2];

  stringstream sstr;
  //Looping over the two correlation function chosen
  for (int imet=0; imet<2; imet++)
  { 
    //Create the TGPBuiler object with chosen option and find optimal parameters
    gpb[imet] = new TGPBuilder(tdsObs, "x1", "y", Correl[imet].c_str(), "");
    gpb[imet]->findOptimalParameters("LOO", 100, "Subplexe", 1000);

    //Get the kriging object
    kg[imet] = gpb[imet]->buildGP();

    //open the dataserver and read the testing basis
    sstr.str(""); sstr << "tdsEstim_" << imet;
    tdsEstim[imet] = new TDataServer(sstr.str().c_str(),"base de test");
    tdsEstim[imet]->fileDataRead("utf-1D-test.dat");
    
    //applied resulting kriging on test basis
    lkrig[imet] = new TLauncher2(tdsEstim[imet], kg[imet], "x1", "yEstim:vEstim");
    lkrig[imet]->solverLoop();
    
    //do the plot
    apad->cd(imet+1);
    PrintSolutions(tdsObs, tdsEstim[imet], Correl[imet]);
    
    stringstream sstr; TLatex *lat=new TLatex(); lat->SetNDC(); lat->SetTextSize(0.025);
    sstr.str(""); sstr << "RMSE (by Loo) " << kg[imet]->getLooRMSE();
    lat->DrawLatex(0.4,0.61,sstr.str().c_str());
    sstr.str(""); sstr << "Q2 " << kg[imet]->getLooQ2();
    lat->DrawLatex(0.4,0.57,sstr.str().c_str());

  }
}