{
  using namespace URANIE::DataServer; // TDataServer, TLogUniformDistribution
  using namespace URANIE::Modeler;    // TGPBuilder, TGPMLCostFunction, TKriging 
  using namespace URANIE::Reoptimizer;// TNlopt, TNloptNelderMead
  
  // Load observations
  TDataServer *tdsObs = new TDataServer("tdsObs","");
  tdsObs->fileDataRead("utf_4D_train.dat");
  
  // Construction of the GPBuilder        
  TGPBuilder *gpb = new TGPBuilder(tdsObs, "x1:x2:x3:x4", "y", "matern3/2");
  
  // Construction of the cost function
  TGPMLCostFunction *costFunc = new TGPMLCostFunction(gpb);
  
  //--------------------------------------------//
  // Construction of the parameters to optimise //
  //--------------------------------------------//
  // construction of the input attributes 
  TAttribute *cl1 = new TAttribute("cl1", 1e-6, 10.0);
  TAttribute *cl2 = new TAttribute("cl2", 1e-6, 10.0);
  TAttribute *cl3 = new TAttribute("cl3", 1e-6, 10.0);
  TAttribute *cl4 = new TAttribute("cl4", 1e-6, 10.0);
  // construction of the output attributes
  TAttribute *cost  = new TAttribute("cost");
  TAttribute *varGP = new TAttribute("varGP");
  
  // Construction of the data server containing the parameters to optimise
  TDataServer *tdsParam = new TDataServer("tdsParam", "");
  tdsParam->addAttribute(cl1);
  tdsParam->addAttribute(cl2);
  tdsParam->addAttribute(cl3);
  tdsParam->addAttribute(cl4);
  
  // Add the parameters to optimise to the cost function
  costFunc->setInputs(4, cl1, cl2, cl3, cl4);
  costFunc->setOutputs(2, cost, varGP);
  
  //-------------------------------//
  // Construction of the optimiser //
  //-------------------------------//       
  // Choice of an optimisation algorithm
  TNloptNelderMead *solver = new TNloptNelderMead();
  
  // Construction of the optimizer, which receives the data server storing 
  // the optimal result, the criterion and the optimisation algorithm.
  TNlopt *opt = new TNlopt(tdsParam, costFunc, solver);
  
  // Optimisation options
  opt->addObjective(cost); // criterion to minimise    
  vector<double> start{1e-2, 1e-2, 1e-2, 1e-2};
  opt->setStartingPoint(start.size(),&start[0]); // starting point for the optimisation
  opt->setMaximumEval(1000); // maximum number of evaluation calls 
  opt->setTolerance(1e-12); // convergence criterion
  
  // Launch the optimisation procedure
  opt->solverLoop();
  
  //-----------------------------------------------------------//
  // Update the GP Builder with the newly found parameters and //
  // create the model                                          //
  //-----------------------------------------------------------//
  // Retrieve the optimal parameters. In this case, the data server contains only
  // one set of parameters, thus we don't have to search for the minimal cost.
  Double_t optimalParams[4];
  optimalParams[0] = tdsParam->getValue("cl1",0);
  optimalParams[1] = tdsParam->getValue("cl2",0);
  optimalParams[2] = tdsParam->getValue("cl3",0);
  optimalParams[3] = tdsParam->getValue("cl4",0);
  // set the parameters of the correlation function
  gpb->getCorrFunction()->setParameters(optimalParams);    
  
  // Retrieve the computed variance
  Double_t optimalVar = tdsParam->getValue("varGP",0);
  // Set the new variance of the GP
  gpb->setVariance(optimalVar);
  
  // Create the model
  TKriging *krig = gpb->buildGP();
  
  // Display model information
  krig->printLog();    

  }