"""
Example of GP model update with new data
"""
import numpy as np
from rootlogon import DataServer, Modeler

# Load observations
tdsObs = DataServer.TDataServer("tdsObs", "")
tdsObs.fileDataRead("utf_4D_train.dat")

# Construct the GPBuilder
gpb = Modeler.TGPBuilder(tdsObs, "x1:x2:x3:x4", "y", "matern3/2")

# Search for the optimal hyper-parameters
gpb.findOptimalParameters("ML", 100, "neldermead", 500)

# Create a new observation
newObs = np.array([tdsObs.getNPatterns()+1, 7.547605e-01, 8.763968e-01,
                   1.255390e-01, 6.370434e-01, 1.189220e+00])

tdsObs.getTuple().Fill(newObs)

# Update the GP Builder
gpb.updateObservations()

# Search for optimal hyper-parameters, starting from their current values
gpb.findOptimalParameters("ML", 0, "bobyqa", 500, False)

# Create the model
krig = gpb.buildGP()

# Display model information
krig.printLog()
