{
   
  // The x attribute of the use case
  TAttribute *x = new TAttribute("x", -1.5, 1.5);
  x->setDefaultValue(-1.2);
  x->setStepValue(0.01);
  x->setFileKey("input_rosenbrock_with_keys.dat");

  // The y attribute of the use case
  TAttribute *y = new TAttribute("y", -1.5,1.5);
  y->setDefaultValue(1.);
  y->setStepValue(0.01);
  y->setFileKey("input_rosenbrock_with_keys.dat");

  // Define the DataServer and add the two attributes
  TDataServer *tdsRosenbrock = new TDataServer("tdsRosenbrock", "Optimize Code externe Rosenbrock via TDataServer");
  tdsRosenbrock->addAttribute(x);
  tdsRosenbrock->addAttribute(y);
    
  // The output file of the code where values are stored in (key = value) format
  TOutputFileKey *fOutputFile = new TOutputFileKey("_output_rosenbrock_with_keys_.dat");
  fOutputFile->addAttribute(new TAttribute("fval"));
  
  // Create an TCode object for my code
  TCode *myRosenbrockCode = new TCode(tdsRosenbrock, "rosenbrock -v -k");
  // The working directory to launch the code
  //myRosenbrockCode->setWorkingDirectory(gSystem->Getenv("PWD") + TString("/tmpLanceurUranie/rosenbrock"));
  // Add the output file
  myRosenbrockCode->addOutputFile( fOutputFile );
  
   // Graph
  TCanvas  *Canvas = new TCanvas("c1", "Graph for the Macro optimizeCodeRosenbrockKey",5,64,1270,667);
  TPad *pad = new TPad("pad","pad",0, 0.03, 1, 1); pad->Draw();
  pad->Divide(2,1);
  pad->cd(1);
  
  // Create an TOptimizer object from TDataServer and TCode objects
  TOptimizer * topt = new TOptimizer(tdsRosenbrock, myRosenbrockCode);  
  topt->setMethod(TOptimizer::kSimplex);
  // topt->setTolerance(1e-5);
  // topt->setPrintLevel(5);
  // topt->setMaxIterations(3);
  // topt->setMaxFunctionCalls(10);

  topt->optimize("same");
  tdsRosenbrock->exportData("_tds_rosenbrock_.dat");
    
  
  pad->cd(2);
  TF2 *frosenbrok = new TF2("fcnRosenbrock","(y-x*x)*(y-x*x)* [0] + ( 1.0 -x ) * ( 1.0 -x )*[1]",-2.0, 2.0, -2.0,2.0);
  frosenbrok->SetParameter(0,10.0);
  frosenbrok->SetParameter(1,1.0);
  frosenbrok->Draw("cont1z");
  tdsRosenbrock->draw("y:x", "", "samel");

  tdsRosenbrock->getTuple()->SetMarkerColor(4);
  tdsRosenbrock->getTuple()->SetMarkerStyle(8);
  tdsRosenbrock->getTuple()->SetMarkerSize(.90);
  tdsRosenbrock->Draw("y:x", Form("%s==1", tdsRosenbrock->getIteratorName()), "psame");
  tdsRosenbrock->getTuple()->SetMarkerColor(50);
  tdsRosenbrock->Draw("y:x", Form("%s==%d", tdsRosenbrock->getIteratorName(), tdsRosenbrock->getNPatterns()), "psame");
    
}
