"""
Example of code optimisation with new input/output attributes
"""
from rootlogon import ROOT, DataServer, Launcher, Optimizer

# The x attribute of the use case
x = DataServer.TAttribute("x", -1.5, 1.5)
x.setDefaultValue(-1.2)
x.setStepValue(0.01)

# The y attribute of the use case
y = DataServer.TAttribute("y", -1.5, 1.5)
y.setDefaultValue(1.)
y.setStepValue(0.01)

# Define the DataServer and add the two attributes
tdsRos = DataServer.TDataServer("tdsRosenbrock", "Optimize Rosenbrock")
tdsRos.addAttribute(x)
tdsRos.addAttribute(y)

tdsRos.addAttribute("xshift", "x-0.1")
tdsRos.getAttribute("xshift").setFileKey("input_rosenbrock_with_keys.dat", "x")
tdsRos.addAttribute("yshift", "y+0.2")
tdsRos.getAttribute("yshift").setFileKey("input_rosenbrock_with_keys.dat", "y")


# The output file of the code where values are stored in (key = value) format
fOutputFile = Launcher.TOutputFileKey("_output_rosenbrock_with_keys_.dat")
fOutputFile.addAttribute(DataServer.TAttribute("fval"))

# Create an Launcher.TCode object for my code
myRosenbrockCode = Launcher.TCode(tdsRos, "rosenbrock -v -k")
# The working directory to launch the code
# myRosenbrockCode.setWorkingDirectory(ROOT.gSystem.Getenv("PWD") +
#                                      TString("/tmpLanceurUranie/rosenbrock"))
# Add the output file
myRosenbrockCode.addOutputFile(fOutputFile)

# Graph
Canvas = ROOT.TCanvas("c1", "Graph optimizeCodeRosenbrockKeyNewInputOutput",
                      5, 64, 1270, 667)
pad = ROOT.TPad("pad", "pad", 0, 0.03, 1, 1)
pad.Draw()
pad.Divide(2, 1)
pad.cd(1)

# Create an TOptimizer object from TDataServer and Launcher.TCode objects
topt = Optimizer.TOptimizer(tdsRos, myRosenbrockCode)
topt.setMethod(Optimizer.TOptimizer.kSimplex)
topt.addOutputVariable("fval+1:fval*fval:fval*3")
topt.selectCost("fval+1")
topt.setTolerance(1e-3)
# topt.setPrintLevel(5)
# topt.setMaxIterations(3)
# topt.setMaxFunctionCalls(10)

topt.optimize("same")
tdsRos.exportData("_tds_rosenbrock_.dat")


pad.cd(2)
frosenbrok = ROOT.TF2("fcnRosenbrock",
                      "(y-x*x)*(y-x*x)* [0] + ( 1.0 -x ) * ( 1.0 -x )*[1]",
                      -2.0, 2.0, -2.0, 2.0)
frosenbrok.SetParameter(0, 10.0)
frosenbrok.SetParameter(1, 1.0)
frosenbrok.Draw("cont1z")
tdsRos.draw("y:x", "", "samel")

tdsRos.getTuple().SetMarkerColor(4)
tdsRos.getTuple().SetMarkerStyle(8)
tdsRos.getTuple().SetMarkerSize(.90)
tdsRos.Draw("y:x", tdsRos.getIteratorName()+"==1", "psame")
tdsRos.getTuple().SetMarkerColor(50)
tdsRos.Draw("y:x", tdsRos.getIteratorName()+"=="+str(tdsRos.getNPatterns()),
            "psame")
