"""
Example of code optimisation multi level
"""
from rootlogon import ROOT, DataServer, Launcher, Optimizer

# Definition of the dataserver
tds = DataServer.TDataServer()

# Definition of the attributes
sJDDReference = "input_rosenbrock_with_keys.dat"
tds.addAttribute(DataServer.TAttribute("a", 0.0, 5.0))
tds.getAttribute("a").setFileKey(sJDDReference, "a")
tds.getAttribute("a").setDefaultValue(1.0)
tds.getAttribute("a").setStepValue(0.1)
tds.addAttribute(DataServer.TAttribute("b", 0.0, 5.0))
tds.getAttribute("b").setFileKey(sJDDReference, "b")
tds.getAttribute("b").setDefaultValue(2.0)
tds.getAttribute("b").setStepValue(0.1)

# Definition of the output file of the code
fout = Launcher.TOutputFileRow("_rosenbrock_multi_.dat")
fout.addAttribute(DataServer.TAttribute("yhat"), 3)

# Definition of the code
mycode = Launcher.TCode(tds, "root -b -l -q rosenbrock_multi.C >> /dev/null")
mycode.addOutputFile(fout)
mycode.addInputFile("rootlogon.C")
mycode.addInputFile("rosenbrock_ref.dat")
mycode.addInputFile("rosenbrock_multi.C")

# Definition of the reference dataserver
tdsref = DataServer.TDataServer("tdsref", "Objectives values")
tdsref.fileDataRead("rosenbrock_ref.dat")

# Definition of the optimizer
topt = Optimizer.TOptimizer(tds, mycode)
topt.addObjective("obj1", tdsref, "y", fout, "yhat")
topt.optimize()

c = ROOT.TCanvas()
pad = ROOT.TPad("pad", "pad", 0, 0.03, 1, 1)
pad.Draw()
tds.draw("b:a", "", "lp")
