//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#ifndef CPPUNITEST_TMETROPHASTING_H
#define CPPUNITEST_TMETROPHASTING_H

// CppUnit
#include <cppunit/extensions/HelperMacros.h>
#include <cppunit/Test.h>

//URANIE
#include "TDataServer.h"
#include "TMetropHasting.h"
#include "TSequentialRun.h"
#include "TThreadedRun.h"
#include "TNormalDistribution.h"
#include "TCJitEval.h"
#include "TCodeEval.h"
#include "TCode.h"
#include "TOutputFileRow.h"
#include "TAttribute.h"
#include "TKeyScript.h"
#include "TKeyResult.h"

using namespace URANIE::DataServer;
using namespace URANIE::Calibration;
using namespace URANIE::Relauncher;
using namespace URANIE::Launcher;

class TMetropHastingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TMetropHastingTest);

    CPPUNIT_TEST( runFlow1DCJitNoPlots );
    CPPUNIT_TEST( runFlow1DCJitNoPlotsVerySmall );
    CPPUNIT_TEST( runFlow1DNoPlotsVerySmallCodeEval );
    CPPUNIT_TEST( runFlow1DNoPlotsVerySmallCodeEvalThreaded );
    CPPUNIT_TEST( runFlow1DNoPlotsVerySmallCode );
    CPPUNIT_TEST( runFlow1DFunctionNoPlots );
    CPPUNIT_TEST( runFlow1DPointerFunctionNoPlots );
    
    CPPUNIT_TEST ( throwExceptionConstruction );
    CPPUNIT_TEST ( throwExceptionInitialisation );
    CPPUNIT_TEST ( throwExceptionReference );
    CPPUNIT_TEST ( throwExceptionAccRange );

    CPPUNIT_TEST_SUITE_END();

public:
    
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TMetropHastingTest();

    //! Default destructor
    virtual ~TMetropHastingTest();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void setUpException();
    void setUpFlow1D(string type="TCJitEval");
    void tearDown();

    void cleanWorkingDirectories();
    //@}
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    void runFlow1DCJitNoPlots();
    void runFlow1DCJitNoPlotsVerySmall();
    void runFlow1DNoPlotsVerySmallCodeEval();
    void runFlow1DNoPlotsVerySmallCodeEvalThreaded();
    void runFlow1DNoPlotsVerySmallCode();
    void runFlow1DPointerFunctionNoPlots();
    void runFlow1DFunctionNoPlots();
    
    void throwExceptionConstruction();
    void throwExceptionInitialisation();
    void throwExceptionReference();
    void throwExceptionAccRange();
    //@}

private:
    TString _workingTestPath;
   
    TDataServer *_tds;
    TDataServer *_tdsRef;
    TMetropHasting *_tmh;
    TSequentialRun *_seqRun;
    TThreadedRun *_thrRun;
    TCJitEval *_evalJit;
    TCodeEval *_evalCode;
    TCode *_code;
    TOutputFileRow *_outRow;
    TKeyScript *_inp;
    TKeyResult *_outp;
    TNormalDistribution *_norm;
    TAttribute *_out;

    bool _threaded;
};

#endif

