/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2019 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TEgoKBModeler.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2019-11-12
 
 \brief  kriging believe ego modeler
*/


#ifndef __EGOKBMODELER__
#define __EGOKBMODELER__

// ROOT
#include "Rtypes.h"

// Uranie
#include "TDataServer.h"
#include "TAttribute.h"
#include "TKriging.h"
#include "TGPBuilder.h"

#include "TEGO.h"

namespace URANIE {
namespace MetaModelOptim {

class TEgoKBModeler : public TEgoModeler
{
protected:
    const char *_algo, *_obj, *_correlat, *_tendance, *_outdescr;
    int _maxeval, _NinDat, _NC, _doesize;
    double *_fmin, _regul;
    TString _indescr;
    URANIE::DataServer::TDataServer *_tds, *_tds2;
    URANIE::DataServer::TAttribute *_y, *_yvar;
    vector<URANIE::DataServer::TAttribute*> _in;
    URANIE::Modeler::TGPBuilder *_mm1, *_mm2;
    URANIE::Reoptimizer::TFitGetter *_crit;
    
    double fillWithExpectedValue(vector<char> &mark, 
        vector<vector<URANIE::DataServer::UEntry*>> &sampl,
        URANIE::DataServer::TDataServer &tds2, URANIE::Modeler::TKriging *krig);

public:
    TEgoKBModeler();
    virtual ~TEgoKBModeler();
    
    // setup
    void setModel(const char *model, const char *trend, double regularisation=0.0);
    void setSolver(const char *objective, const char *algo, int screening=0, int optsize=0);


    // protocole
    virtual void build(URANIE::DataServer::TDataServer *, URANIE::Reoptimizer::TFitGetter *,
            vector<URANIE::DataServer::TAttribute*>&,
            URANIE::DataServer::TAttribute *, URANIE::DataServer::TAttribute *, 
            double *, int
        );
    virtual URANIE::Modeler::TKriging *next(vector<char> &, 
        vector<vector<URANIE::DataServer::UEntry*>> &, int);
    
    ClassDef(URANIE::MetaModelOptim::TEgoKBModeler, ID_EGO);
};

} } //namespace

#endif
