/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2019 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   TEgoShareSolver.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2019-11-06
 
 \brief  recherche du point minimisant l'EI
*/


#ifndef __EGOSHARESOLVER__
#define __EGOSHARESOLVER__

// ROOT

// Uranie
#include "TDataServer.h"
#include "TRun.h"
#ifdef WITH_VIZIR
#include "TVizirGenetic.h"
#include "TVizir2.h"
#endif
#include "TOptimFit.h"

#include "TEGO.h"
#include "THJMA.h"

namespace URANIE {
namespace MetaModelOptim {

class TEgoShareSolver : public TEgoSolver
{
protected:
    URANIE::DataServer::TDataServer *_tds;
    vector<URANIE::DataServer::TAttribute*> _in;
    URANIE::DataServer::TAttribute *_ego;
    URANIE::Relauncher::TRun *_run;
    URANIE::Reoptimizer::TMaximizeFit _maxim;
    int _NinDat, _first, _many;
    double _valid_ei;

    virtual void fitnessChanged() = 0;
    virtual void extend() = 0;
    virtual void solverLoop() = 0;

public:
    TEgoShareSolver();
    virtual ~TEgoShareSolver();

    virtual void significantEI(double);
    void setManyNewItem(int);
        
    virtual void build(URANIE::Relauncher::TRun *,
        vector<URANIE::DataServer::TAttribute*>&, URANIE::DataServer::TAttribute *);
    virtual void next();
    virtual void more();

    ClassDef(URANIE::MetaModelOptim::TEgoShareSolver, ID_EGO)
};

#ifdef WITH_VIZIR
class TEgoVzrSolver : public TEgoShareSolver
{
protected:
    URANIE::Reoptimizer::TVizirSolverShare *_algo;
    URANIE::Reoptimizer::TVizir2 *_viz;
    URANIE::Reoptimizer::TGreaterFit _signif;
    int _NG;
    
public:
    TEgoVzrSolver();
    virtual ~TEgoVzrSolver();

    void setSolver(URANIE::Reoptimizer::TVizirSolverShare *);
    void setStepSize(int);

    void solverLoop();

    ClassDef(URANIE::MetaModelOptim::TEgoVzrSolver, ID_EGO)
};
#endif

class TEgoHjSolver : public TEgoShareSolver
{
protected:
    int _Nfirst, _Nstep;
    THJMA *_algo;

public:
    TEgoHjSolver();
    virtual ~TEgoHjSolver();
    
    void setSize(int, int);
    virtual int getNext(long ev, int nb, 
        vector<vector<URANIE::DataServer::UEntry*>> &sampl, vector<char> &mark);
    void solverLoop();    

    ClassDef(URANIE::MetaModelOptim::TEgoHjSolver, ID_EGO)
};


} } //namespace

#endif
