/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2019 CEA/DEN
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   THJMA.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2019-11-21
 
 \brief  HJMA search algorithm
*/

#ifndef __HJMA__
#define __HJMA__

// ROOT
#include "Rtypes.h"
#include "TRandom3.h"

// URANIE
#include "TOptimShare.h"

#include "TDataServer.h"
#include "TAttribute.h"
#include "TRun.h"
#include "TEval.h"
#include "TOptimFit.h"
#include "TFitGetter.h"

#include "TEGO.h"

struct domain {
    double mini, maxi, it;
};
struct hjma_pt {
    double *xy;
    vector<URANIE::DataServer::UEntry*> *codi, *codo;
};
struct hjma_work {
    struct hjma_pt *best, *cur, one, two;
};
struct hjma_mem {
    double crit;
    vector<URANIE::DataServer::UEntry*> data;
};

namespace URANIE {
namespace MetaModelOptim {

class THJMA : public URANIE::Reoptimizer::TOptimShare
{
private:
    inline double evaluate_pt(struct hjma_pt &pt);
    inline void evaluate_best(struct hjma_work &self);
    inline bool evaluate_current(struct hjma_work &self);


protected:
    int _Ncall, _Ncut, _Nfirst, _Nstep;
    int _first, _update;
    URANIE::DataServer::TAttribute *_y;
    URANIE::Reoptimizer::TFitGetter *_crit, *_valid;
    TRandom3 _rnd;
    double **_global;
    std::vector<double *> *_mode;
    struct domain *_dom;

    void build(); 
    void globalsearch(struct hjma_work &self);
    int hjonestep(struct hjma_work &self, double *dep, double step);
    void hjtwostep(struct hjma_work &self, double *dep);
    int hjsearch(struct hjma_work &self, double prox, int loop, int dup);
    int duplicate(struct hjma_work &self, double it);
    void addSolution(struct hjma_work &self);
    int getBest(struct hjma_work &self);

    
public:
    THJMA(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TRun *run);
    virtual ~THJMA();
    
    // initialisation
    virtual void addObjective(URANIE::DataServer::TAttribute *,
            URANIE::Reoptimizer::TOptimFit *f=NULL);
    void setUsable(URANIE::Reoptimizer::TOptimFit *);
    void setSize(int first, int step);

    // solve
    void fitnessChanged();
    virtual void solverLoop();
    
    // post
    double *item(int);
    
    ClassDef(URANIE::MetaModelOptim::THJMA, ID_EGO)
};

} } //namespace

#endif
