#include <stdlib.h>
#include <math.h>
#include "linpack_d.h"
#ifdef WIN32
#include "libpgauss_Export.h"
#else
#define libpgauss_EXPORT
#endif

/* Calcul des erreurs par LOO			*/
/* Methode gpLoo0				*/
/* Input Kin : matrice de variance		*/
/* Input yin : vecteur des observations		*/
/* Input n : dimension Kin[n x n], yin[n]	*/
/* Output errorLoo : n erreurs LOO		*/
/* Output varLoo : n variances des erreurs	*/
/* Output eqmLoo   : erreur quadratique moyenne	*/
libpgauss_EXPORT int gpLoo0Error(double *Kin, double *yin, int n, double *errorLoo, double *varLoo, double *eqmLoo) {
  void gpDposl ( double a[], int lda, int n, double b[] );
  int i, j, info;
  double dT, determinant[2], eqm;

  /* Recopie des donnees qui seront modifiees */
  double *K, *pK, *y, *py;
  K = calloc(sizeof(double),n*n);
  y = calloc(sizeof(double),n);
  for(pK = K,i=0;i<n*n;i++) *pK++ = *Kin++;
  for(py = y,i=0;i<n;i++)   *py++ = *yin++;
  /*******************************/

  info = dpofa (K, n, n);        /* decomposition Choleski K = Lt L*/
  if (info == 0) {
    dposl (K,n,n,y);             /* resolution lineaire => e = T * y */
    dtrdi(K,n,n,determinant,11); /* inverse d'une triangulaire superieure */
                                 /* inverse de L */
    for(i=0;i<n;i++) {
      dT=0.;
      for(j=i;j<n;j++) dT += K[i+j*n]*K[i+j*n];
      errorLoo[i] = y[i] / dT;
      varLoo[i]   = 1. / dT;
    }
    for(eqm=0., i=0;i<n;i++) eqm += errorLoo[i] * errorLoo[i];
    eqmLoo[0] = eqm / n;
  }
  else {
    /* printf("gpLoo0 : The leading minor %d is not positive\n", info); */
    for(i=0;i<n;i++) errorLoo[i]=0.;
    for(i=0;i<n;i++) varLoo[i]=0.;
    eqmLoo[0] = -1;
  }
  free(K);  K=NULL;
  free(y);  y=NULL;
  return info;
}

