/*//////////////////////////////////////////////////////////////
 // Copyright (C) 2013-2022 CEA/DES
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Lesser General Public License as published
 // by the Free Software Foundation, either version 3 of the License, or any
 // later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 // $Id$
 // $Author$
 // $Date$
 // $Revision$
 // $State$
 //////////////////////////////////////////////////////////////*/
/*!
 \file   gpMod0.c
 \author Jean-Marc Martinez <jean-marc.martinez@cea.fr>
 \date   2012/07/19 15:38:00

 \brief
 */
#include <stdlib.h>
#include <math.h>
#include "linpack_d.h"
#ifdef WIN32
#include "libpgauss_Export.h"
#else
#define libpgauss_EXPORT
#endif

/* Calcul de gamma et de iL                           */
/* Methode gpMod0                                     */
/* Input Kin : matrice de variance                    */
/* Input yin : vecteur des observations               */
/* Input n : dimension K[n x n], y[n]                 */
/* Output gamma : vecteur de taille n              */
/* Output iL   : inverse de L                         */
libpgauss_EXPORT int gpMod0(double *Kin, double *yin, int n, double *gamma, double *iL)
{
    int i, j, jn, info;
    double determinant[2];

    /* Recopie des donnees qui seront modifiees */
    double *K, *pK, *y, *py;
    K = calloc(sizeof(double), n * n);
    y = calloc(sizeof(double), n);
    for (pK = K, i = 0; i < n * n; i++)
        *pK++ = *Kin++;
    for (py = y, i = 0; i < n; i++)
        *py++ = *yin++;
    /*******************************/

    info = dpofa(K, n, n); /* decomposition Cholesky K = Lt L*/
    if (info == 0)
    {
        for (i = 0; i < n; i++)
            gamma[i] = y[i];
        dposl(K, n, n, gamma); /* resolution lineaire => gamma = inv(K) * y */
        dtrdi(K, n, n, determinant, 11); /* inverse d'une triangulaire superieure */
        /* inverse de L */
        for (i = 0; i < n; i++)
        {
            for (j = 0; j < i; j++)
                iL[i + j * n] = 0.;
            for (j = i; j < n; j++)
            {
                jn = j * n;
                iL[i + jn] = K[i + jn];
            }
        }
    }
    free(K);  K=NULL;
    free(y);  y=NULL;
    return info;
}

