/*//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "linpack_d.h"
#ifdef WIN32
#include "libpgauss_Export.h"
#else
#define libpgauss_EXPORT
#endif
/*
	VERIFIE 26/06/2012
*/
/* Calcul de des predictions - tendance deterministe constante  */
/* Methode gpPred1                                      	*/
/* Input Kp : matrice new x n de covariances new x n		*/
/* Input variance : variance du processus		  	*/
/* Input hbeta : coefficient de la tendance deterministe        */
/* Input vbeta : variance de hbeta				*/
/* Input gamma : vecteur de taille n calcule par gpMOd1         */
/* Input iL : matrice n x n calcule par gpMOd1        		*/
/* Input new : nombre de ligne de Kp                   		*/
/* input n : nombre de colonnes de Kp et taille de gamma	*/
/* Output yp : vecteur des new predictions			*/
/* Output vp : vecteur des new variances			*/
libpgauss_EXPORT void gpPred1(double *Kp, double variance, double hbeta, double vbeta, double *gamma, double *iL, int new, int n, double *yp, double *vp) {
  int i, j, k, jn, in;
  double vt1, vt2, vt3, *pgamma, *v;
 
  /* Calcul de v tel que 1 = Lz v */
  v = calloc(sizeof(double),n);
  for(i=0;i<n;i++) {
    in = i*n;
    for(v[i]=0.,j=0;j<=i;j++) v[i] += iL[j+in];
  }

  /* calcul des predictions */
  for(i=0;i<new;i++) {
    pgamma = gamma;
    for(vt1=0.,j=0;j<n;j++) vt1 += Kp[i+j*new] * *pgamma++;
    yp[i] = hbeta + vt1;
  }
  /* calcul des variances */
  for(i=0;i<new;i++) {
    vp[i] = variance;
    for(vt3=0.,j=0;j<n;j++) {
      jn = j * n;
      for(vt2=0.,k=0;k<=j;k++) vt2 += Kp[i+k*new] * iL[k+jn]; /* iL est triangulaire superieure */
      vt3   += vt2 * v[j];
      vp[i] -= vt2 * vt2;
    }
    vp[i] += (1. - vt3) * (1. - vt3) * vbeta;
    if (vp[i] < 0.) vp[i] = 0.;
  }
  free(v);  v = NULL;
}
