/*//////////////////////////////////////////////////////////////
 // Copyright (C) 2013-2022 CEA/DES
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU Lesser General Public License as published
 // by the Free Software Foundation, either version 3 of the License, or any
 // later version.
 //
 // This program is distributed in the hope that it will be useful,
 // but WITHOUT ANY WARRANTY; without even the implied warranty of
 // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 // GNU Lesser General Public License for more details.
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////
 // $Id$
 // $Author$
 // $Date$
 // $Revision$
 // $State$
 //////////////////////////////////////////////////////////////*/
/*!
 \file   matrice.c
 \author Jean-Marc Martinez <jean-marc.martinez@cea.fr>
 \date   2012/07/19 15:38:00

 \brief
 */
#include <stdlib.h>
#include <stdio.h>

# include "linpack_d.h"
# include "matrice.h"

/* Element (i,j) d'une matrice */
double element(struct matrice *a, int i, int j)
{
    return (a->x[i + j * a->n]);
}

/* Allocation d'une matrice */
struct matrice * alloueMatrice(int n, int p)
{
    struct matrice * mat;
    mat = malloc(sizeof(struct matrice));
    mat->n = n;
    mat->p = p;
    mat->x = calloc(sizeof(double), n * p);
    return (mat);
}

/* Produit de 2 matrices A x B */
struct matrice * produitMatriceAB(struct matrice *a, struct matrice *b)
{
    int n1 = a->n;
    int p1 = a->p;
    int n2 = b->n;
    int p2 = b->p;
	struct matrice *c;
    int i, j, k;
    double t;
	
    if (p1 != n2)
    {
        printf("produitMatriceAB - matrices incompatibles\n");
        exit(1);
    }
/*    struct matrice *c = alloueMatrice(a->n, b->p);*/
    c = alloueMatrice(a->n, b->p);

    for (i = 0; i < n1; i++)
    {
        for (j = 0; j < p2; j++)
        {
            for (t = 0., k = 0; k < p1; k++)
                t += a->x[i + k * n1] * b->x[k + j * n2];
            c->x[i + j * n1] = t;
        }
    }
    return (c);
}

/* Produit de 2 matrices At x B */
struct matrice * produitMatriceAtB(struct matrice *a, struct matrice *b)
{
    int n1 = a->n;
    int p1 = a->p;
    int n2 = b->n;
    int p2 = b->p;
	struct matrice *c;
    int i, j, k;
    double t;
    if (n1 != n2)
    {
        printf("produitMatriceAtB - matrices incompatibles\n");
        exit(1);
    }
/*    struct matrice *c = alloueMatrice(a->p, b->p);*/
    c = alloueMatrice(a->p, b->p);

    for (i = 0; i < p1; i++)
    {
        for (j = 0; j < p2; j++)
        {
            for (t = 0., k = 0; k < n1; k++)
                t += element(a, k, i) * element(b, k, j);
            c->x[i + j * p1] = t;
        }
    }
    return (c);
}

void svdDecomposition(struct matrice *H, struct matrice *U, double *S,
        struct matrice *V)
{
    int n = H->n;
    int p = H->p;
    double *e;
    double *work;

    /*
     job = 100 : calcul de S uniquement
     job = 110 : calcul de S et U n x n
     job = 120 : calcul de S et de U n x p
     job = 111 : calcul de S et U n x n et V
     job = 121 : calcul de S et U n x p et V
     */
    int job = 120;
	
	e=(double*) calloc(p,sizeof(int));
	work=(double*) calloc(n,sizeof(int));
	
    dsvdc(H->x, n, n, p, S, e, U->x, n, V->x, p, work, job);
	
    free(e);   e = NULL;
	free(work); work=NULL;

}

/* Edition de la matrice sur la sortie standard */
void printMat(struct matrice *a, char *s)
{
    int i, j;
    printf("%s : %d x %d\n", s, a->n, a->p);
    for (i = 0; i < a->n; i++)
    {
        for (j = 0; j < a->p; j++)
            printf("%14.6e", a->x[i + j * a->n]);
        printf("\n");
    }
}

