//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TGPCostFunction.cxx
 \author Nicolas Gilardi <nicolas.gilardi@cea.fr>
 \date   

 \brief Implementation of the class  URANIE::Modeler::TGPCostFunction

 */

//// ROOT
#include "TObjArray.h"

//// URANIE
#include "TGPCostFunction.h"
#include "TAttribute.h"

using std::cout;
using std::endl;
using namespace URANIE::DataServer;

ClassImp (URANIE::Modeler::TGPCostFunction)

namespace URANIE
{
namespace Modeler
{
// -----------------------------------------------------------------------
//  Constructors
// -----------------------------------------------------------------------
TGPCostFunction::TGPCostFunction(TGPBuilder* gpb) :
_blog(kFALSE)
{
    // Information transfer
    gpBuilder = gpb;
    _H = gpBuilder->H;
    _K = gpBuilder->K;
    _C = gpBuilder->C;
    _yObs = gpBuilder->yObs;
    _n = gpBuilder->_nS;
    _p = gpBuilder->_nP;
}

// -----------------------------------------------------------------------
//  Destructor
// -----------------------------------------------------------------------

TGPCostFunction::~TGPCostFunction()
{
    if (_blog)
        cout << "detruit TGPCostFunction" << endl;
}

// -----------------------------------------------------------------------
//  Log
// -----------------------------------------------------------------------
void TGPCostFunction::printLog(Option_t *option)
{
    cout << "*******************************" << endl;
    cout << " ** TGPCostFunction::printLog[" << option << "]" << endl;
    cout << "*******************************" << endl;
}

void TGPCostFunction::duplicateGPB()
{
    gpBuilder = gpBuilder->buildWithDuplicateKandCandCorrFunc();
    _K = gpBuilder->K;
    _C = gpBuilder->C;
}

} // Fin du namespace Modeler
} // Fin du namespace URANIE

