//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TModeler.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mar jui 3 10:38:56 CEST 2007

 \brief Implementation of the class URANIE::Modeler::TModeler

 */
#include <iostream>
using std::cout;
using std::endl;
#include <ctime>
// Uranie
#include "TModeler.h"
#include "TDataServer.h"

ClassImp (URANIE::Modeler::TModeler)

namespace URANIE
{
namespace Modeler
{

TModeler::TModeler(URANIE::DataServer::TDataServer *tds, TString architecture,
        Option_t *option) :
  _blog(kFALSE), _bdrawProgressBar(kTRUE), _bStoreYHat(kTRUE), _tds(tds), _sIntermediateArchitecture(""),
  _listOfInputAttributes(NULL), _listOfOutputAttributes(NULL), _sFileName("model"),
  _sFunctionName("myFunction"), _dr2(-1.), _bIntercept(kTRUE)

{
    try
    {
        if (_tds == NULL)
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    "TModeler::constructor ERROR : the pointer of \"tds\" is empty");
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }

    _nS = _tds->getNPatterns();

    // Gestion des options
    TString soption = TString(option);
    soption.ToLower();
    if (soption.Contains("nointercept"))
        _bIntercept = kFALSE;
    if (soption.Contains("nostoreyhat"))
        _bStoreYHat = kFALSE;

    // Treat the architecture string to extract input ans output attributes
    TString svar = architecture;
    Int_t nparts = svar.CountChar(',') + 1;
    if (_blog)
    {
        cout << " ** svar[" << svar << "]" << endl;
        cout << " ** nparts[" << nparts << "]" << endl;
    }
    svar.ReplaceAll(",", " ");
    istringstream inStream((char *) svar.Data());
    string inName;
    Int_t ncrtpart = 0;
    while (inStream >> inName)
    {
        TString satt(inName.c_str());
        if (_blog)
            cout << " ** satt[" << satt << "]" << endl;
        if (ncrtpart == 0)
            _sInputAttributes = TString(inName.c_str());
        ncrtpart++;
        if (ncrtpart == nparts)
            _sOutputAttributes = TString(inName.c_str());
        if (_sOutputAttributes.BeginsWith('@')) {
            _sOutputAttributes.ReplaceAll("@","");
        }
        if ((ncrtpart > 1) && (ncrtpart < nparts))
            _sIntermediateArchitecture += Form("[%s]", inName.c_str());
    }

    try
    {
        //  Get the list of Input attributes
        _listOfInputAttributes = _tds->getListOfAttributes(_sInputAttributes);
        _nX = _listOfInputAttributes->GetEntries();

        if (_blog)
        {
            cout << endl << " **  Input Attributes listing  _sInputAttributes["
                    << _sInputAttributes << "] _nX[" << _nX << "]" << endl;
            _listOfInputAttributes->Print();
        }

        //  Get the list of Output attributes
        _listOfOutputAttributes = _tds->getListOfAttributes(_sOutputAttributes);
        if(_sOutputAttributes=="")
            _nY=0;
        else
            _nY = _listOfOutputAttributes->GetEntries();

        if (_blog)
        {
            cout << endl
                    << " **  Output Attributes listing  _sOutputAttributes["
                    << _sOutputAttributes << "] _nY[" << _nY << "]" << endl;
            _listOfOutputAttributes->Print();

        }
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
  if ( _listOfInputAttributes != NULL )
    delete _listOfInputAttributes;
  _listOfInputAttributes = NULL;
  if ( _listOfOutputAttributes  != NULL)
    delete _listOfOutputAttributes;
  _listOfOutputAttributes=NULL;
        _nY = 0;
        throw ue;
    }

    TString sar = architecture;
    sar.ReplaceAll(",", "_____");
    sar.ReplaceAll(":", "_");
    SetNameTitle(Form("Model_%s_%s", _tds->GetName(), sar.Data()),
            "Classe TModeler");
    if (_blog)
        cout << "construit TModeler [" << GetName() << "]" << endl;

}

TModeler::TModeler(URANIE::DataServer::TDataServer *tds,
        const char *varexpinput, const char *varexpoutput, Option_t * option) :
  _blog(kFALSE), _bdrawProgressBar(kTRUE), _bStoreYHat(kTRUE),
  _tds(tds), _sInputAttributes(TString(varexpinput)), _sOutputAttributes(TString(varexpoutput)),
  _sIntermediateArchitecture(""), _listOfInputAttributes(NULL), _listOfOutputAttributes(NULL),
  _sFileName("model"), _sFunctionName("myFunction"), _dr2(-1.), _bIntercept(kTRUE)

{
    try
    {
        if (_tds == NULL)
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    "TModeler::constructor ERROR : the pointer of \"tds\" is empty");
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
    _nS = _tds->getNPatterns();

    // Gestion des options
    TString soption = TString(option);
    soption.ToLower();
    if (soption.Contains("nointercept"))
        _bIntercept = kFALSE;
    if (soption.Contains("nostoreyhat"))
        _bStoreYHat = kFALSE;

    try
    {
        //  Get the list of Input attributes
        _listOfInputAttributes = _tds->getListOfAttributes(_sInputAttributes);
        _nX = _listOfInputAttributes->GetEntries();

        if (_blog)
        {
            cout << endl << " **  Input Attributes listing  _sInputAttributes["
                    << _sInputAttributes << "] _nX[" << _nX << "]" << endl;
            _listOfInputAttributes->Print();
        }

        //  Get the list of Output attributes
        _listOfOutputAttributes = _tds->getListOfAttributes(_sOutputAttributes);
        if(_sOutputAttributes=="")
            _nY=0;
        else
            _nY = _listOfOutputAttributes->GetEntries();
        
        if (_blog)
        {
            cout << endl << " **  Output Attributes listing _sOutputAttributes["
                    << _sOutputAttributes << "] _nY[" << _nY << "]" << endl;
            _listOfOutputAttributes->Print();

        }
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
  if ( _listOfInputAttributes != NULL )
    delete _listOfInputAttributes;
  _listOfInputAttributes = NULL;
  if ( _listOfOutputAttributes  != NULL)
    delete _listOfOutputAttributes;
  _listOfOutputAttributes=NULL;
        _nY = 0;
        throw ue;
    }
}

TModeler::~TModeler()
{
    if (_blog)
        cout << " ** Destroy TModeler" << endl;

    if ( _listOfInputAttributes != NULL )
      delete _listOfInputAttributes;
    _listOfInputAttributes = NULL;

    if ( _listOfOutputAttributes != NULL  )
      delete _listOfOutputAttributes;
    _listOfOutputAttributes=NULL;

    if (_blog)
        cout << " ** End Of Destroy TModeler" << endl;
}

void TModeler::train(Option_t* option)
{
    cout << "train mlp option[" << option << "]" << endl;
}

// -----------------------------------------------------------------------
//  getInputName Function
// -----------------------------------------------------------------------
const char * TModeler::getInputName(Int_t i) const
{
    if (_blog)
        cout << " ** TModeler::getInputName i[" << i << "] Size["
                << _listOfInputAttributes->GetEntries() << "]" << endl;

    try
    {
        if (i < 0)
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    Form(
                            "TModeler::getInputName method \n The value of index [%d] is negative",
                            i));
        if (i > _listOfInputAttributes->GetEntries())
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    Form(
                            "TModeler::getInputName method \n The value of index [%d] is greater than the number of parameters [%d]",
                            i, _listOfInputAttributes->GetEntries()));

        return ((URANIE::DataServer::TAttribute *) _listOfInputAttributes->At(i))->GetName();

    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
}

// -----------------------------------------------------------------------
//  getNInput Function
// -----------------------------------------------------------------------
Int_t TModeler::getNInput()
{
    Int_t nres = 0;
    if (_listOfInputAttributes)
        nres = _listOfInputAttributes->GetEntries();

    return nres;
}

// -----------------------------------------------------------------------
//  getOutputName Function
// -----------------------------------------------------------------------
const char * TModeler::getOutputName(Int_t i)
{
    try
    {
        if (i < 0)
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    Form(
                            "TModeler::getOutputName method\n The value of index [%d] is negative",
                            i));
        if (i > _listOfOutputAttributes->GetEntries())
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    Form(
                            "TModeler::getOutputName method\n The value of index [%d] is greater than the number of parameters [%d]",
                            i, _listOfOutputAttributes->GetEntries()));

        return ((URANIE::DataServer::TAttribute *) _listOfOutputAttributes->At(
                i))->GetName();

    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
}

// -----------------------------------------------------------------------
//  Export Function
// -----------------------------------------------------------------------
void TModeler::exportFunction(const char* lang, const char* file,
        const char* name, Option_t *option)
{
  TString soption = option;
  soption.ToLower();
  cout << endl << "*******************************" << endl;
  cout << "*** TModeler::exportFunction lang[" << lang << "] file[" << file << "] name[" << name << "] soption[" << soption << "]" << endl;
  cout << endl << "*******************************" << endl;
    enum ELang{kC,kFortran,kPMML,kPython,kUno};
    _tds->computeStatistic();
    TString lg = lang;
    lg.ToLower();
    Int_t nblang = 4;
    TString* sComment = new TString[nblang];

    TString sCommentEnd = " ";
    Bool_t bOK = kFALSE;
    Bool_t ExportC = kFALSE;
    Bool_t ExportFortran = kFALSE;
    Bool_t ExportPMML = kFALSE;
    Bool_t ExportPython = kFALSE;
    TString sFile = TString(file);
    if (sFile.Length() == 0)
        sFile = _sFileName;
    TString* sFileLang = new TString[nblang];
    for(Int_t ilang=0;ilang<nblang;ilang++)
    {
        sComment[ilang] = "**";
        sFileLang[ilang] = sFile;
    }
    try
    {
        if (lg.Contains("c"))
        {
            if (!(sFile.EndsWith(".C") ||
                  sFile.EndsWith(".c") ||
                  sFile.EndsWith(".cpp") ||
                  sFile.EndsWith(".cxx") ||
                  sFile.Contains(".")
                ))
                sFileLang[0] += ".C";
            else if(sFile.Contains("."))
                sFileLang[0] = Form("%s.C",TString(sFile(0,sFile.Last('.'))).Data());

            gSystem->CopyFile(sFileLang[0].Data(), Form("%s.bak", sFileLang[0].Data()),
                    kTRUE);
            sComment[0] = "  //";
            bOK = kTRUE;
            ExportC = kTRUE;
        }
        if (lg.Contains("for"))
        {
            if (!(sFile.EndsWith(".f") ||
                  sFile.Contains(".")
                ))
                sFileLang[1] += ".f";
            else if(sFile.Contains("."))
                sFileLang[1] = Form("%s.f",sFile(0,sFile.Last('.')-2).Data());
            gSystem->CopyFile(sFileLang[1].Data(), Form("%s.bak", sFileLang[1].Data()),
                    kTRUE);
            sComment[1] = "C --- ";
            bOK = kTRUE;
            ExportFortran = kTRUE;
        }
        if (lg.Contains("pmml"))
        {
             if (!(sFile.EndsWith(".pmml") ||
                  sFile.Contains(".")
               ))
                sFileLang[2] += ".pmml";
            else if(sFile.Contains("."))
                sFileLang[2] = Form("%s.pmml",sFile(0,sFile.Last('.')-1).Data());
            sComment[2] = "<--  ";
            sCommentEnd = "  -->";
            bOK = kTRUE;
            ExportPMML = kTRUE;
        }
        if (lg.Contains("py"))
        {
            if (!(sFile.EndsWith(".py") ||
                  sFile.Contains(".")
                ))
                sFileLang[3] += ".py";
            else if(sFile.Contains("."))
                sFileLang[3] = Form("%s.py",sFile(0,sFile.Last('.')-1).Data());
            gSystem->CopyFile(sFileLang[3].Data(), Form("%s.bak", sFileLang[3].Data()),
                    kTRUE);
            sComment[3] = "#  ";
            bOK = kTRUE;
            ExportPython = kTRUE;
        }
        if (bOK == kFALSE)
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    Form(
                            "TModeler::exportFunction method\n The chosen language \"%s\" is not accepted (fortran|c|PMML|Python)",
                            lang));
    } catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
  delete[] sComment; sComment = NULL;
  delete[] sFileLang; sFileLang = NULL;
        throw ue;
    }

    TString sName = TString(name);
    if (sName.Length() == 0)
        sName = _sFunctionName;
    else
      _sFunctionName = sName;
    cout << endl << "*******************************" << endl;
    cout << "*** exportFunction lang[" << lg << "] file[" << sFile << "] name["
            << sName << "]" << endl;

//     ofstream * sourcefile = new ofstream(sFile.Data());
    URANIE::DataServer::TAttribute *att;
    std::ofstream * sourcefileC = NULL;
    std::ofstream * sourcefileFortran = NULL;
    std::ofstream * sourcefilePython = NULL;

    // Prototype of the function
//     if (lg.BeginsWith("c"))
    if(ExportC)
    {
        sourcefileC = new std::ofstream(sFileLang[0].Data());
        *sourcefileC << "void " << sName << "(double *param, double *res)"
                << endl;
        *sourcefileC << "{" << endl;
    }
//     if (lg.BeginsWith("for"))
    if(ExportFortran)
    {
        sourcefileFortran = new std::ofstream(sFileLang[1].Data());
        Int_t nCrtCol = 17;
        nCrtCol += sName.Length()+2;
        *sourcefileFortran << "      SUBROUTINE " << sName << "(";
        if ( soption.Contains("petum") ) {
            if ( nCrtCol < 50 )
                *sourcefileFortran << "Xvar, Yvar, nX, nY)" << endl;
            else
                *sourcefileFortran << endl << "     !Xvar,, Yvar, nX, nY)" << endl;
            // Normalized attibutes
            *sourcefileFortran << "      IMPLICIT DOUBLE PRECISION (v)" << endl;
            *sourcefileFortran << "      INTEGER nX, nY" << endl;
            *sourcefileFortran << "      DOUBLE PRECISION Xvar(nX), Yvar(nY)" << endl;
        } else {
            for (Int_t ivar = 0; ivar < _nX; ivar++) {
                att = (URANIE::DataServer::TAttribute *) _listOfInputAttributes->At(ivar);
                TString sAttName  = att->GetName();
                if ( _blog )
                    cout << " **   Input[" << ivar << "/" << _nX << "] Name[" << sAttName << "]" << endl;
                if ( nCrtCol + sAttName.Length() > 70 ) {
                    cout << " ****** nCrtCol[" << nCrtCol << "]" << endl;
                    *sourcefileFortran << endl << "     !";
                    nCrtCol = 10;
                }
                *sourcefileFortran << att->GetName() << ", ";
                nCrtCol += sAttName.Length()+1;
            }
            for (Int_t ivar = 0; ivar < _nY; ivar++) {
                att = (URANIE::DataServer::TAttribute *) _listOfOutputAttributes->At(ivar);
                TString sAttName  = att->GetName();
                if ( _blog )
                    cout << " **  Output[" << ivar << "/" << _nY << "] Name[" << sAttName << "]" << endl;
                if ( nCrtCol + sAttName.Length() > 70 ) {
                    cout << " ****** nCrtCol[" << nCrtCol << "]" << endl;
                    *sourcefileFortran << endl << "     !";
                    nCrtCol = 10;
                }
                *sourcefileFortran << att->GetName();
                nCrtCol += sAttName.Length();

                if ( ivar < _nY-1 ) {
                    *sourcefileFortran << ", ";
                    nCrtCol += 2;
                }
            }
            *sourcefileFortran << ")" << endl;
            *sourcefileFortran << "      IMPLICIT DOUBLE PRECISION (v)" << endl;
            for (Int_t ivar = 0; ivar < _nX; ivar++) {
                att = (URANIE::DataServer::TAttribute *) _listOfInputAttributes->At(ivar);
                *sourcefileFortran << "      DOUBLE PRECISION " << att->GetName()<< endl;
            }
            for (Int_t ivar = 0; ivar < _nY; ivar++) {
                att = (URANIE::DataServer::TAttribute *) _listOfOutputAttributes->At(ivar);
                *sourcefileFortran << "      DOUBLE PRECISION " << att->GetName()<< endl;
            }
        }
    }

    if(ExportPython)
    {
        sourcefilePython = new std::ofstream(sFileLang[3].Data());
        *sourcefilePython << "def "<< sName << "(param,res):"<<endl;
    }

    // Header
    // product and time
    time_t seconds;
    time(&seconds);

    TString* stringBuffer = new TString[nblang];
    TString str_time = ctime(&seconds);
    for(Int_t ilang = 0; ilang < nblang; ilang++)
    {
        if(ilang==kPMML)
            sCommentEnd = "  -->";
        else
            sCommentEnd = "";

        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "//////////////////////////// ";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     *********************************************";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     ** Uranie v";
        stringBuffer[ilang] += URANIE_RELEASE;
        stringBuffer[ilang] += " -  Date : ";
        stringBuffer[ilang] += URANIE_RELEASE_DATE;
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     ** Export Modeler : ";
        stringBuffer[ilang] += GetName();
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     ** Date : ";
        stringBuffer[ilang] += str_time(0,str_time.Last('\n')-1).Data();
        if(ilang==kPMML)
            stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     *********************************************";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";

        // comment and title
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     *********************************************";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     ** TDataServer ";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     **        Name  : ";
        stringBuffer[ilang] += _tds->GetName();
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     **        Title : ";
        stringBuffer[ilang] += _tds->GetTitle();
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     **     Patterns : ";
        stringBuffer[ilang] += _tds->getNPatterns();
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     **   Attributes : ";
        stringBuffer[ilang] += _tds->getNAttributes();
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     *********************************************";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";

        // input attributes
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     INPUT  : ";
        stringBuffer[ilang] += _nX;
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";

        for (Int_t ivar = 0; ivar < _nX; ivar++)
        {
            att = (URANIE::DataServer::TAttribute *) _listOfInputAttributes->At(ivar);
            stringBuffer[ilang] += sComment[ilang];
            stringBuffer[ilang] += "             ";
            stringBuffer[ilang] += att->GetName();
            stringBuffer[ilang] += " : Min[";
            stringBuffer[ilang] += att->getMinimum();
            stringBuffer[ilang] += "] Max[";
            stringBuffer[ilang] += att->getMaximum();
            stringBuffer[ilang] += "] Unit[";
            stringBuffer[ilang] += att->getUnity();
            stringBuffer[ilang] += "]";
            stringBuffer[ilang] += sCommentEnd;
            stringBuffer[ilang] += "\n";
        }
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";

        // output attribute
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "     OUTPUT : ";
        stringBuffer[ilang] += _nY;
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";

        for (Int_t ivar = 0; ivar < _nY; ivar++)
        {
            att = (URANIE::DataServer::TAttribute *) _listOfOutputAttributes->At(ivar);
            stringBuffer[ilang] += sComment[ilang];
            stringBuffer[ilang] += "             ";
            stringBuffer[ilang] += att->GetName();
            stringBuffer[ilang] += " : Min[";
            stringBuffer[ilang] += att->getMinimum();
            stringBuffer[ilang] += "] Max[";
            stringBuffer[ilang] += att->getMaximum();
            stringBuffer[ilang] += "] Unit[";
            stringBuffer[ilang] += att->getUnity();
            stringBuffer[ilang] += "]";
            stringBuffer[ilang] += sCommentEnd;
            stringBuffer[ilang] += "\n";
        }
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
        stringBuffer[ilang] += sComment[ilang];
        stringBuffer[ilang] += "//////////////////////////// ";
        stringBuffer[ilang] += sCommentEnd;
        stringBuffer[ilang] += "\n";
    }

    try
    {
        if(ExportC)
        {
            *sourcefileC << stringBuffer[0].Data();
            exportModelCplusplus(sourcefileC);
        }
        if(ExportFortran)
        {
            *sourcefileFortran << stringBuffer[1].Data();

            // constantes
            *sourcefileFortran << endl;
            *sourcefileFortran << "C --- constantes" << endl;
            *sourcefileFortran << "      INTEGER nInput" << endl;
            *sourcefileFortran << "      INTEGER nOuput" << endl;

            TString sTitle = GetTitle();
            if ( soption.Contains("petum") == kFALSE ) {
                *sourcefileFortran << "      DOUBLE PRECISION Xvar(" <<  _nX << ")" << endl;
                *sourcefileFortran << "      DOUBLE PRECISION Yvar(" <<  _nY << ")" << endl;

                if ( sTitle.Contains("(WITH_PETUM)"))
                    sTitle.ReplaceAll("(WITH_PETUM)","(WITHOUT_PETUM)");
                else
                    SetTitle(Form("%s (WITHOUT_PETUM)", sTitle.Data()));
            } else {
                sTitle.ReplaceAll("(WITHOUT_PETUM)","(WITH_PETUM)");
                SetTitle(sTitle);
            }

            exportModelFortran(sourcefileFortran);
        }
        if(ExportPMML)
        {
            exportModelPMML(sFileLang[2].Data(), name);
        }
        if(ExportPython)
        {
            *sourcefilePython << stringBuffer[3].Data();
            exportModelPython(sourcefilePython);
        }
    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        // Clean
        if(ExportC) {
    delete sourcefileC; sourcefileC = NULL;
        }

        if(ExportFortran) {
    delete sourcefileFortran; sourcefileFortran = NULL;
        }

        if(ExportPython) {
    delete sourcefilePython; sourcefilePython = NULL;
        }

        delete[] sFileLang;   sFileLang = NULL;
        delete[] sComment;    sComment = NULL;
        delete[] stringBuffer; stringBuffer = NULL;

        throw ue;
    }catch (URANIE::Exceptions::UErrorExceptions& ue)
    {
        ue.printMessage();
        // Clean
        if(ExportC) {
    delete sourcefileC; sourcefileC = NULL;
        }

        if(ExportFortran) {
    delete sourcefileFortran; sourcefileFortran = NULL;
        }

        if(ExportPython) {
    delete sourcefilePython; sourcefilePython = NULL;
        }

        delete[] sFileLang;   sFileLang = NULL;
        delete[] sComment;    sComment = NULL;
        delete[] stringBuffer; stringBuffer = NULL;

        throw ue;
    }
 // Clean and close file
    if(ExportC) {
      *sourcefileC << "}" << endl;
      sourcefileC->close();
      delete sourcefileC; sourcefileC = NULL;
    }

    if(ExportFortran) {


        if ( soption.Contains("petum") == kFALSE ) {
            // Set the output values
            *sourcefileFortran << endl;
            *sourcefileFortran << "C --- Set the output values" << endl;
            for (Int_t ivar = 0; ivar < _nY; ivar++) {
                att = (URANIE::DataServer::TAttribute *) _listOfOutputAttributes->At(ivar);
                *sourcefileFortran << "      " << att->GetName() << " = Yvar(" << ivar+1 << ")" << endl;
            }
        }

        *sourcefileFortran << endl;
        *sourcefileFortran << "C --- End Of exportModelFortran" << endl;

        *sourcefileFortran << "      RETURN" << endl << "      END" << endl;


        if ( soption.Contains("petum") ) {
            cout << " ** PETUM features" << endl;
            *sourcefileFortran << endl;
            *sourcefileFortran << "      SUBROUTINE " << _sFunctionName << "_xytit(Xtitle, Ytitle, nX,  nY)" << endl;
            *sourcefileFortran << "      INTEGER nX, nY" << endl;
            *sourcefileFortran << "      CHARACTER (*) Xtitle, Ytitle" << endl;
            *sourcefileFortran <<  endl;
            *sourcefileFortran << "C --- instructions" << endl;
            *sourcefileFortran << "Cf2py intent(inout) Xtitle,nX" << endl;
            *sourcefileFortran << "Cf2py intent(inout) Ytitle,nY" << endl;
            *sourcefileFortran << "      " << endl;
            *sourcefileFortran << "C --- Titles" << endl;
            *sourcefileFortran << "      Xtitle = \"";
            Int_t nLength = 15;
            TString sStrName;
            Int_t nStrName;
            for(Int_t iatt=0; iatt<_nX; iatt++) {
                att = (URANIE::DataServer::TAttribute *) _listOfInputAttributes->At(iatt);
                sStrName =  att->GetName();
                nStrName =sStrName.Length();
                if ( nLength + nStrName > 75 ) {
                    *sourcefileFortran << endl << "     !";
                    nLength = 7;
                }
                *sourcefileFortran << sStrName;
                nLength += nStrName;
                if ( iatt < _nX -1 ) {
                    *sourcefileFortran << " |";
                    nLength += 3;
                }
            }
            *sourcefileFortran << "\"" << endl;
            TString sOutputTitles = _sOutputAttributes;
            sOutputTitles.ReplaceAll(":","_hat:");
            sOutputTitles += "_hat";
//            *sourcefileFortran << "      Ytitle = \"" << sOutputTitles << "\"" << endl;
            *sourcefileFortran << "      Ytitle = \"";
            nLength = 15;
            for(Int_t iatt=0; iatt<_nY; iatt++) {
                att = (URANIE::DataServer::TAttribute *) _listOfOutputAttributes->At(iatt);
                sStrName =  att->GetName()+ TString("_hat");
                nStrName =sStrName.Length();
                if ( nLength + nStrName > 75 ) {
                    *sourcefileFortran << endl << "     !";
                    nLength = 7;
                }
                *sourcefileFortran << sStrName;
                nLength += nStrName;
                if ( iatt < _nY -1 ) {
                    *sourcefileFortran << " |";
                    nLength += 3;
                }
            }
            *sourcefileFortran << "\"" << endl;
            *sourcefileFortran << endl;

            *sourcefileFortran << "C --- Dimensions" << endl;
            *sourcefileFortran << "      nX = " << _nX << endl;
            *sourcefileFortran << "      nY = " << _nY << endl;
            *sourcefileFortran << "      END" << endl;
        }
      sourcefileFortran->close();
      delete sourcefileFortran; sourcefileFortran = NULL;
    }

    if(ExportPython) {
      sourcefilePython->close();
      delete sourcefilePython; sourcefilePython = NULL;
    }

    delete[] sComment;     sComment = NULL;
    delete[] sFileLang;    sFileLang = NULL;
    delete[] stringBuffer; stringBuffer = NULL;
    cout << "*** End Of exportFunction" << endl;
    cout << "*******************************" << endl;
}

// -----------------------------------------------------------------------
//  Log
// -----------------------------------------------------------------------
void TModeler::printLog(Option_t *option)
{
    cout << " ** TModeler::printLog[" << option << "]" << endl;
    cout << " ** Name[" << GetName() << "] Title[" << GetTitle() << "]" << endl;
    if (_bIntercept)
        cout << " ** _bIntercept[" << _bIntercept << " - YES]]" << endl;
    else
        cout << " ** _bIntercept[" << _bIntercept << " - NO]]" << endl;

    cout << " ** _sInputAttributes[" << _sInputAttributes << "]" << endl;
    cout << " ** _sOutputAttributes[" << _sOutputAttributes << "]" << endl;
    cout << " ** _sIntermediateArchitecture[" << _sIntermediateArchitecture
            << "]" << endl;
}

} // Fin du namespace Modeler
} // Fin du namespace URANIE

