//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   nisp_va.h
 \author Jean-Marc Martinez <jean-marc.martinez@cea.fr>
 \date   2011/04/07 14:33:00
 
 \brief  Class of set of random variables
 \brief  Class of random variables
 */
#ifndef NISP_VA_H
#define NISP_VA_H

#include <string>
#include <vector>
#ifdef WIN32
#include "libnisp_Export.h"
#else
#define libnisp_EXPORT
#endif

using namespace std;

//! Class of random variables
class libnisp_EXPORT RandomVariable
{
public:
    //! Law of the density probability
    string type;
    //! First parameter of the law
    double a;
    //! Second parameter of the law
    double b;
    //! third parameter of the law
    double c;
    //! fourth parameter of the law
    double d;
    //! Function of the law
    double (*function)(double, double, double, double);
public:
    /** @name Constructor and Destructor
     **/
    //@{
    //! Constructor from the law
    RandomVariable(string type);
    //! Constructor from a law with one parameter
    RandomVariable(string type, double a);
    //! Constructor from a law with two parameters
    RandomVariable(string type, double a, double b);
    //! Constructor from a law with three parameters
    RandomVariable(string type, double a, double b, double c);
    //! Constructor from a law with four parameters (as the beta law)
    RandomVariable(string type, double a, double b, double c, double d);
    //! Destructor
    ~RandomVariable();
    //@}
    //! Realisation of the random variable
    double GetValue();
    //! Realisation from a realisation x of another RandomVariable *va
    double GetValue(RandomVariable *va, double x);
    //! Information : type of the law with its parameters
    void GetLog();
};
#endif
