//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   jeu jan 24 15:56:01 2008
 
 $Revision$
 $Date$
 
 \brief Programme principal pour les tests unitiares sur la librairie DataServer.
 
 */
#include "cppunit/CompilerOutputter.h"
#include "cppunit/TestResult.h"
#include "cppunit/TestResultCollector.h"
#include "cppunit/TextTestProgressListener.h"
#include "cppunit/BriefTestProgressListener.h"
#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/TestRunner.h"
#include "cppunit/XmlOutputter.h"

#include <stdexcept>

#include <iostream>
#include <fstream>

//uranie
#include "UVersion.h"

// Liste des Tests Unitaires d'uranie
#include "TestTLinearRegression.h"
#include "TestTPolynomialRegression.h"
#include "TestTGLM.h"
#include "TestTkNN.h"
#include "TestTKernel.h"
#include "TestTPolynomialMultiDimFit.h"
#include "uranie_commands.h"

#ifdef WITH_OPT
#include "TestTANNModeler.h"
#include "TestUMLP.h"
#endif
#ifdef WITH_NISP
#include "TestTNisp.h"
#include "TestTPolynomialChaos.h"
#endif
#ifdef WITH_ANISP
#include "TestTAnisp.h"
#endif
#ifdef WITH_PGAUSS
#include "TestTCorrelationFunction.h"
#include "TestTGPCostFunction.h"
#include "TestTGPBuilder.h"
#include "TestTKriging.h"
#endif

#ifdef  WITH_LIBXML2
#include "PMMLBasicsTest1.hxx"
#endif

CPPUNIT_TEST_SUITE_REGISTRATION (TLinearRegressionTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TPolynomialRegressionTest); //29
CPPUNIT_TEST_SUITE_REGISTRATION (TGLMTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TkNNTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TKernelTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TPolynomialMultiDimFitTest);

#ifdef WITH_OPT
CPPUNIT_TEST_SUITE_REGISTRATION( TANNModelerTest);
CPPUNIT_TEST_SUITE_REGISTRATION( UMLPTest); //2 == Canvas
#endif

#ifdef WITH_NISP // Many links in nisp :D
CPPUNIT_TEST_SUITE_REGISTRATION( TNispTest );
CPPUNIT_TEST_SUITE_REGISTRATION( TPolynomialChaosTest ); //0
#endif

#ifdef WITH_ANISP 
CPPUNIT_TEST_SUITE_REGISTRATION( TAnispTest ); //3
#endif

#ifdef WITH_PGAUSS // 0
CPPUNIT_TEST_SUITE_REGISTRATION( TCorrFunctionTest );
CPPUNIT_TEST_SUITE_REGISTRATION( TGPCostFunctionTest );
#ifdef WITH_NLOPT
CPPUNIT_TEST_SUITE_REGISTRATION( TGPBuilderTest );
CPPUNIT_TEST_SUITE_REGISTRATION( TKrigingTest );
#endif
#endif

#ifdef  WITH_LIBXML2 //0
CPPUNIT_TEST_SUITE_REGISTRATION( PMMLBasicsTest1 );
#endif 

bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{
    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("Modeler",
                                      "testModelerResults.xml", "..\\..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("Modeler",
                                      "testModelerResults.xml", "../../../../testReport.xsl");
#endif

    // --- Suppress the files .root
    UranieTests::removeFile(gSystem->pwd(), "*.root");
    UranieTests::removeFolder(gSystem->pwd(), "tmp");
    UranieTests::removeFolder(gSystem->pwd(), "URANIE");

    gDirectory->Delete("tmpsobol;9999");
    gDirectory->Delete("elist1;9999");
    gDirectory->Delete("htemp;9999");
    gDirectory->ls();
    
    // ---  Return error code 1 if the one of test failed.
    return wasSuccessful ? 0 : 1;
}
