//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTANNModeler.cxx
 \author Maud Salvoldi (OCC)
 \date   18/10/2011

 \brief

 */

// ROOT
#include "TMath.h"
#include "TString.h"

// cppunit
#include <iostream>
using namespace std;
#define DBL_PRECISION_COMPARE 1.e-10

//uranie
#include "TestTANNModeler.h"
#include "TDataServer.h"
#include "UExceptions.h"
#include "TCIntEval.h"
#include "TLauncher2.h"

#include "UMLP.h"
//URANIE::Modeler::UMLP *gUranieUMLP;

TANNModelerTest::TANNModelerTest() :
    _tds(NULL), _archi("rw:r:tu:tl:hu:hl:l:kw,1,yhat"), _tann(NULL)
{
  fstream file("tann_model.C");
  if (!file.fail())
    remove("tann_model.C");
  fstream file10("tann_model0.C");
  if (!file10.fail())
    remove("tann_model0.C");
  fstream file11("tann_model1.C");
  if (!file11.fail())
    remove("tann_model1.C");
  fstream file12("tann_model2.C");
  if (!file12.fail())
    remove("tann_model2.C");
  fstream file20("yhat_nH001.C");
  if (!file10.fail())
    remove("yhat_nH001.C");
  fstream file21("tann_model1_direct.C");
  if (!file21.fail())
    remove("tann_model1_direct.C");
  fstream file22("tann_model2_direct.C");
  if (!file22.fail())
    remove("tann_model2_direct.C");
  fstream file23("tann_model3_direct.C");
  if (!file23.fail())
    remove("tann_model3_direct.C");
  fstream file31("tann_model1_direct.f");
  if (!file31.fail())
    remove("tann_model1_direct.f");
  fstream file32("tann_model2_direct.f");
  if (!file32.fail())
    remove("tann_model2_direct.f");
  fstream file41("tann_model1.f");
  if (!file41.fail())
    remove("tann_model1.f");
  fstream file42("tann_model2.f");
  if (!file42.fail())
    remove("tann_model2.f");
  fstream filebak("tann_model.C.bak");
  if (!filebak.fail())
    remove("tann_model.C.bak");
  fstream filebak2("tann_model2.f.bak");
  if (!filebak2.fail())
      remove("tann_model2.f.bak");
  fstream file3("model_tann.pmml");
  if (!file3.fail())
      remove("model_tann.pmml");
  fstream file4("model_tann_init.pmml");
  if (!file4.fail())
      remove("model_tann_init.pmml");
  fstream file5("totoBis.wei");
  if (!file5.fail())
      remove("totoBis.wei");
  fstream file6("OPT_DEFAULT.out");
  if (!file6.fail())
      remove("OPT_DEFAULT.out");
  fstream file7("_tstOptCalNips__modeler_tdsFlowrate.out");
  if (!file7.fail())
      remove("_tstOptCalNips__modeler_tdsFlowrate.out");
  fstream file8("model.C");
  if (!file8.fail())
      remove("model.C");
}
TANNModelerTest::~TANNModelerTest()
{
    fstream file("tann_model.C");
    if (!file.fail())
      remove("tann_model.C");
    fstream file10("tann_model0.C");
    if (!file10.fail())
      remove("tann_model0.C");
    fstream file11("tann_model1.C");
    if (!file11.fail())
      remove("tann_model1.C");
    fstream file12("tann_model2.C");
    if (!file12.fail())
      remove("tann_model2.C");
    fstream file20("yhat_nH001.C");
    if (!file10.fail())
      remove("yhat_nH001.C");
    fstream file21("tann_model1_direct.C");
    if (!file21.fail())
      remove("tann_model1_direct.C");
    fstream file22("tann_model2_direct.C");
    if (!file22.fail())
      remove("tann_model2_direct.C");
    fstream file23("tann_model3_direct.C");
    if (!file23.fail())
      remove("tann_model3_direct.C");
    fstream file31("tann_model1_direct.f");
    if (!file31.fail())
      remove("tann_model1_direct.f");
    fstream file32("tann_model2_direct.f");
    if (!file32.fail())
      remove("tann_model2_direct.f");
    fstream file41("tann_model1.f");
    if (!file41.fail())
      remove("tann_model1.f");
    fstream file42("tann_model2.f");
    if (!file42.fail())
      remove("tann_model2.f");
    fstream file43("tann_model_multi.C");
    if (!file43.fail())
      remove("tann_model_multi.C");
    fstream filebak("tann_model.C.bak");
    if (!filebak.fail())
      remove("tann_model.C.bak");
    fstream filebak2("tann_model2.f.bak");
    if (!filebak2.fail())
        remove("tann_model2.f.bak");
    fstream file3("model_tann.pmml");
    if (!file3.fail())
        remove("model_tann.pmml");
    fstream file4("model_tann_init.pmml");
    if (!file4.fail())
        remove("model_tann_init.pmml");
    fstream file5("totoBis.wei");
    if (!file5.fail())
        remove("totoBis.wei");
    fstream file6("OPT_DEFAULT.out");
    if (!file6.fail())
        remove("OPT_DEFAULT.out");
    fstream file7("_tstOptCalNips__modeler_tdsFlowrate.out");
    if (!file7.fail())
        remove("_tstOptCalNips__modeler_tdsFlowrate.out");
    fstream file8("model.C");
    if (!file8.fail())
        remove("model.C");
}

void TANNModelerTest::setUp()
{
    _tds = new URANIE::DataServer::TDataServer();
    _tds->unsetLog();
    _tds->fileDataRead("flowrate_sampler_launcher_500.dat");

    _tann = NULL;
}
void TANNModelerTest::tearDown()
{
    if (_tann != NULL)
        delete _tann;
    _tann = NULL;
    delete _tds;
    _tds = NULL;
}
void TANNModelerTest::testTANNModelerFirstConstructor()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();

    CPPUNIT_ASSERT_EQUAL(8, _tann->getNInput());
    CPPUNIT_ASSERT_EQUAL(1, _tann->getNHidden());
    CPPUNIT_ASSERT_EQUAL(400, _tann->getNAPP());
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-numeric_limits<double>::max(), _tann->getWeightDecay(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_EQUAL(0, _tann->getSeed());
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.001, _tann->getFcnTol(),
            DBL_PRECISION_COMPARE);

    ColumnVector tmp;
    tmp = _tann->getWeights();
    CPPUNIT_ASSERT_EQUAL(0, tmp.size());

    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kRegression, _tann->getProblem());

}

#ifdef  WITH_LIBXML2
void TANNModelerTest::testTANNModelerSecondConstructor()
{
    TString sFunctionName("sANNName");
    _tann = new URANIE::Modeler::TANNModeler(_tds,"tann_model_tds.pmml", sFunctionName);
    _tann->unsetLog();

    CPPUNIT_ASSERT_EQUAL(8,_tann->getNInput());
    CPPUNIT_ASSERT_EQUAL(1,_tann->getNHidden());
    CPPUNIT_ASSERT_EQUAL(400, _tann->getNAPP());
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-9999.9, _tann->getWeightDecay(),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_EQUAL(0,_tann->getSeed());
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.001,_tann->getFcnTol(),DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.745483,_tann->getWeight(0),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.965512,_tann->getWeight(1),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.263572,_tann->getWeight(2),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7536629,_tann->getWeight(3),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00165366,_tann->getWeight(4),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.004725001,_tann->getWeight(5),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.009969786,_tann->getWeight(6),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.1787976,_tann->getWeight(7),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.1809809,_tann->getWeight(8),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.1735688,_tann->getWeight(9),DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.08559675,_tann->getWeight(10),DBL_PRECISION_COMPARE);

}
#endif

void TANNModelerTest::testTANNModelergetOutputName()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    TString soutput = "yhat";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(soutput.Data(), _tann->getOutputName()));
}

void TANNModelerTest::testTANNModelergetInputName()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->train(3, 2, "test");

    TString sinput;
    sinput = "rw";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(sinput.Data(), _tann->getInputName(0)));
    sinput = "r";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(sinput.Data(), _tann->getInputName(1)));
    sinput = "tu";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(sinput.Data(), _tann->getInputName(2)));
    sinput = "tl";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(sinput.Data(), _tann->getInputName(3)));
    sinput = "hu";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(sinput.Data(), _tann->getInputName(4)));
    sinput = "hl";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(sinput.Data(), _tann->getInputName(5)));
    sinput = "l";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(sinput.Data(), _tann->getInputName(6)));
    sinput = "kw";
    CPPUNIT_ASSERT_EQUAL(0, strcmp(sinput.Data(), _tann->getInputName(7)));

}

void TANNModelerTest::testtrainANDgetEQM()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->train(3, 2, "test", false);  // explicitly: do not use GPU

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kMinusOneOne, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, _tann->_UMLP->getActivationFunction());

    _tann->setFcnTol(1e-4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1e-4, _tann->getFcnTol(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kRegression, _tann->getProblem());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0090, _tann->getEQM("test"), 0.0050);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0100, _tann->getEQM("learn"), 0.0050);

#ifdef WITH_GPUANN
    // Same thing with GPU now:
    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->train(3, 2, "test", true);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.006222, _tann->getEQM("test"), 0.0040);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.007181, _tann->getEQM("learn"), 0.0030);
#endif
}

void TANNModelerTest::testtrainWithWeightDecay()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->train(3, 2, "test,weight-decay=0.025", false);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.025, _tann->getWeightDecay(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.050, _tann->getEQM("test"), 0.04);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.05, _tann->getEQM("learn"), 0.05);

    _tann->train(3, 2, "+ wd=0.015", false);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.015, _tann->getWeightDecay(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.040, _tann->getEQM("test"), 0.04);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.05, _tann->getEQM("learn"), 0.05);

#ifdef WITH_GPUANN
    // Same thing with GPU now:
    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();

    _tann->train(3, 2, "test,weight-decay=0.025", true);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.025, _tann->getWeightDecay(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0050, _tann->getEQM("test"),  0.003);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0050, _tann->getEQM("learn"), 0.003);

    _tann->train(3, 2, "+ wd=0.015", true);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.015, _tann->getWeightDecay(),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0050, _tann->getEQM("test"),  0.003);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0050, _tann->getEQM("learn"), 0.003);

#endif

}
void TANNModelerTest::testsetNormalization()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();

    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kCR, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kLOGISTIC, _tann->_UMLP->getActivationFunction());

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kMinusOneOne, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, _tann->_UMLP->getActivationFunction());
    _tann->train(3, 2, "test", false);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.100, _tann->getEQM("test"),  0.100);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.100, _tann->getEQM("learn"), 0.100);

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kZeroOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kZeroOne, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kLOGISTIC, _tann->_UMLP->getActivationFunction());

#ifdef WITH_GPUANN
    // Same thing with GPU now:
    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();

    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kCR, _tann->getNormalization());
    //CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kLOGISTIC, gUranieUMLP->getActivationFunction());
  CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kLOGISTIC, _tann->_UMLP->getActivationFunction());

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kMinusOneOne, _tann->getNormalization());
    //CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, gUranieUMLP->getActivationFunction());
  CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, _tann->_UMLP->getActivationFunction());
    _tann->train(3, 2, "test", true);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00300, _tann->getEQM("test"), 0.0040);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00200, _tann->getEQM("learn"), 0.0030);

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kZeroOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kZeroOne, _tann->getNormalization());
    //CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kLOGISTIC, gUranieUMLP->getActivationFunction());
  CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kLOGISTIC, _tann->_UMLP->getActivationFunction());
#endif
}

void TANNModelerTest::testgetWeights()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->train(3, 2, "test", false);
    ColumnVector tmp;
    tmp = _tann->getWeights();

    CPPUNIT_ASSERT_EQUAL(11, tmp.size());

#ifdef WITH_GPUANN
    // Same thing with GPU now:
    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();

    _tann->train(3, 2, "test", true);
    tmp = _tann->getWeights();

    CPPUNIT_ASSERT_EQUAL(11, tmp.size());
#endif

}
void TANNModelerTest::testsetSeed()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->setSeed(45);

    CPPUNIT_ASSERT_EQUAL(45, _tann->getSeed());

    time_t secondes;
    time(&secondes);
    _tann->setSeed(0);

    CPPUNIT_ASSERT_EQUAL((Int_t)(secondes), _tann->getSeed());
}

void TANNModelerTest::testsetFcnTol()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->setFcnTol(0.000001);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000001, _tann->getFcnTol(),
            DBL_PRECISION_COMPARE);

}
void TANNModelerTest::testExportModelFortran()
{
    time_t seconds;
    string line, date;
    Bool_t bStop = kFALSE;
    Int_t test = 0;
    Int_t test2 = 0;
    Int_t test3 = 0;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->init();
    _tann->randomizeWeights();
    _tann->exportFunction("fortran", "tann_model1", "function_tann");
    fstream file("tann_model1.f");
    time(&seconds);
    date = string(ctime(&seconds));
    while (bStop == kFALSE)
    {
        getline(file, line);
        if (line.find("C ---      ** Date : ") != string::npos)
        {
            //verify that the date are the same actually and in the file
            if ((date[0] == line[17]) && (date[1] == line[18])
                    && (date[2] == line[19]) && (date[3] == line[20])
                    && (date[4] == line[21]) && (date[5] == line[22])
                    && (date[6] == line[23]) && (date[7] == line[24])
                    && (date[8] == line[25]) && (date[9] == line[26])
                    && (date[10] == line[27]) && (date[11] == line[28])
                    && (date[12] == line[29]) && (date[13] == line[30])
                    && (date[14] == line[31]) && (date[15] == line[32])
                    && (date[16] == line[33]) && (date[17] == line[34])
                    && (date[18] == line[35]) && (date[19] == line[36])
                    && (date[20] == line[37]) && (date[21] == line[38])
                    && (date[22] == line[39]) && (date[23] == line[40]))
                test = 1;

            if(test==0)
            {
                Int_t pos = date.find_last_of(":");
                Int_t pos2 = line.find_last_of(":");
                Int_t date_seconds, line_seconds;
                date_seconds =  (date[pos-5]*10+date[pos-4])*3600
                            +(date[pos-2]*10+date[pos-1])*60
                            +(date[pos+1]*10+date[pos+2]);

                line_seconds =  (line[pos2-5]*10+line[pos2-4])*3600
                            +(line[pos2-2]*10+line[pos2-1])*60
                            +(line[pos2+1]*10+line[pos2+2]);

                if(fabs(date_seconds - line_seconds + 0.0)<3)
                    test = 1;
            }
        }

        if (line.find(
                "      SUBROUTINE function_tann(rw, r, tu, tl, hu, hl, l, kw, yhat)")
                != string::npos)
            test2 = 1;

        if (line.find("     &   (Xvar(IA) - (val_function_tann_minInp(IA)))")
                != string::npos)
            test3 = 1;
        if (file.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);

    bStop = kFALSE;
    test = 0;
    test2 = 0;
    test3 = 0;

    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    _tann->init();
    _tann->randomizeWeights();
    _tann->exportFunction("fortran", "tann_model2", "function_tann");
    fstream file2("tann_model2.f");
    time(&seconds);
    date = string(ctime(&seconds));
    while (bStop == kFALSE)
    {
        getline(file2, line);
        if (line.find("C ---      ** Date : ") != string::npos)
        {
            //verify that the date are the same actually and in the file
            if ((date[0] == line[17]) && (date[1] == line[18])
                    && (date[2] == line[19]) && (date[3] == line[20])
                    && (date[4] == line[21]) && (date[5] == line[22])
                    && (date[6] == line[23]) && (date[7] == line[24])
                    && (date[8] == line[25]) && (date[9] == line[26])
                    && (date[10] == line[27]) && (date[11] == line[28])
                    && (date[12] == line[29]) && (date[13] == line[30])
                    && (date[14] == line[31]) && (date[15] == line[32])
                    && (date[16] == line[33]) && (date[17] == line[34])
                    && (date[18] == line[35]) && (date[19] == line[36])
                    && (date[20] == line[37]) && (date[21] == line[38])
                    && (date[22] == line[39]) && (date[23] == line[40]))
                test = 1;

            if(test==0)
            {
                Int_t pos = date.find_last_of(":");
                Int_t pos2 = line.find_last_of(":");
                Int_t date_seconds, line_seconds;
                date_seconds =  (date[pos-5]*10+date[pos-4])*3600
                            +(date[pos-2]*10+date[pos-1])*60
                            +(date[pos+1]*10+date[pos+2]);

                line_seconds =  (line[pos2-5]*10+line[pos2-4])*3600
                            +(line[pos2-2]*10+line[pos2-1])*60
                            +(line[pos2+1]*10+line[pos2+2]);


                if(fabs(date_seconds - line_seconds + 0.0)<3)
                    test = 1;
            }
        }
        if (line.find(
                "      SUBROUTINE function_tann(rw, r, tu, tl, hu, hl, l, kw, yhat)")
                != string::npos)
            test2 = 1;
        if (line.find("     &   2.D0 * (Xvar(IA) - val_function_tann_minInp(IA))")
                != string::npos)
            test3 = 1;
        if (file2.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);
}

void TANNModelerTest::testExportModelFortranDirect()
{
    time_t seconds;
    string line, date;
    Bool_t bStop = kFALSE;
    Int_t test = 0;
    Int_t test2 = 0;
    Int_t test3 = 0;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->init();
    _tann->randomizeWeights();
    _tann->exportModelFortran("tann_model1_direct.f", "function_tann");
    fstream file("tann_model1_direct.f");
    time(&seconds);
    date = string(ctime(&seconds));
    while (bStop == kFALSE)
    {
        getline(file, line);
        if (line.find(
                "      SUBROUTINE function_tann(rw, r, tu, tl, hu, hl, l, kw, yhat)")
                != string::npos)
            test = 1;
        if (line.find("     &   (Xvar(IA) - (val_function_tann_minInp(IA)))")
                != string::npos)
            test2 = 1;
        if (line.find("      Yvar(IM) = val_function_tann_minOut(IM)")
                != string::npos)
            test3 = 1;
        if (file.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);

    bStop = kFALSE;
    test = 0;
    test2 = 0;
    test3 = 0;

    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    _tann->init();
    _tann->randomizeWeights();
    _tann->exportModelFortran("tann_model2_direct.f", "function_tann");
    fstream file2("tann_model2_direct.f");
    time(&seconds);
    date = string(ctime(&seconds));
    while (bStop == kFALSE)
    {
        getline(file2, line);
        if (line.find(
                "      SUBROUTINE function_tann(rw, r, tu, tl, hu, hl, l, kw, yhat)")
                != string::npos)
            test = 1;
        if (line.find("     &   2.D0 * (Xvar(IA) - val_function_tann_minInp(IA))")
                != string::npos)
            test2 = 1;
         if (line.find("      Yvar(IM) = val_function_tann_minOut(IM) + 0.5D0")
                != string::npos)
            test3 = 1;
        if (file2.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);
}

void TANNModelerTest::testExportModelCplusplus()
{
    time_t seconds, secondsDefault;
    string line, date;
    Bool_t bStop = kFALSE;
    Int_t test1 = 0;
    Int_t test2 = 0;
    Int_t test3 = 0;

    Int_t natt = _tds->getNAttributes();
    CPPUNIT_ASSERT_EQUAL(9, natt);
    TString sInput = "rw:r:tu:tl:hu:hl:l:kw";
    Int_t nX = sInput.CountChar(':')+1;
    TString sOutput = "yhat";
    Int_t nY = sOutput.CountChar(':')+1;
    Int_t nH = 5;
    Double_t dPrecision = 1e-6;
    Int_t nW = (nX+nY+1)*nH+1;

    _tds->computeStatistic(sOutput);

    CPPUNIT_ASSERT_EQUAL(1,nY);
    Double_t dMeanYhat = _tds->getAttribute(sOutput)->getMean();

    CPPUNIT_ASSERT_DOUBLES_EQUAL(77.8117130600, _tds->getAttribute(sOutput)->getMean(), DBL_PRECISION_COMPARE);

    _tann = new URANIE::Modeler::TANNModeler(_tds, Form("%s,%d,%s", sInput.Data(), nH, sOutput.Data()));
    _tann->unsetLog();
    _tann->setFcnTol(dPrecision);
    _tann->train(3, 2, "test");

    CPPUNIT_ASSERT_EQUAL(nX,_tann->getNInput());
    CPPUNIT_ASSERT_EQUAL(nH,_tann->getNHidden());
    CPPUNIT_ASSERT_EQUAL(nY,_tann->getNOutput());
    CPPUNIT_ASSERT_EQUAL(nW,_tann->_UMLP->getNWeights());
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dPrecision, _tann->getFcnTol(), DBL_PRECISION_COMPARE);

    _tann->exportFunction("c++");
    fstream file2("model.C");
    time(&secondsDefault);
    date = string(ctime(&secondsDefault));

    while (bStop == kFALSE)
    {
        getline(file2, line);
        if (line.find("  //     ** Date : ") != string::npos)
        {
            //verify that the date are the same actually and in the file
            if ((date[0] == line[24]) && (date[1] == line[25])
                    && (date[2] == line[26]) && (date[3] == line[27])
                    && (date[4] == line[28]) && (date[5] == line[29])
                    && (date[6] == line[30]) && (date[7] == line[31])
                    && (date[8] == line[32]) && (date[9] == line[33])
                    && (date[10] == line[34]) && (date[11] == line[35])
                    && (date[12] == line[36]) && (date[13] == line[37])
                    && (date[14] == line[38]) && (date[15] == line[39])
                    && (date[16] == line[40]) && (date[17] == line[41])
                    && (date[18] == line[42]) && (date[19] == line[43])
                    && (date[20] == line[44]) && (date[21] == line[45])
                    && (date[22] == line[46]) && (date[23] == line[47]))
                test1 = 1;

            if(test1==0)
            {
                Int_t pos = date.find_last_of(":");
                Int_t pos2 = line.find_last_of(":");
                Int_t date_seconds, line_seconds;
                date_seconds =  (date[pos-5]*10+date[pos-4])*3600
                            +(date[pos-2]*10+date[pos-1])*60
                            +(date[pos+1]*10+date[pos+2]);

                line_seconds =  (line[pos2-5]*10+line[pos2-4])*3600
                            +(line[pos2-2]*10+line[pos2-1])*60
                            +(line[pos2+1]*10+line[pos2+2]);


                if(fabs(date_seconds - line_seconds + 0.0)<3)
                    test1 = 1;
            }
        }
        if (line.find("  0.099999, 25048.9, 89334.9, 89.5523, 1050,")
                != string::npos)
            test2 = 1;
        if (line.find("void myFunction(double *param, double *res)")
                != string::npos)
            test3 = 1;
        if (file2.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test1);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);

    // Eval the Model
    gROOT->LoadMacro("model.C");
    URANIE::Relauncher::TCIntEval fEval("myFunction");

    TString sOutputANN1 = "yann1";
    URANIE::Relauncher::TLauncher2 *lfct = new URANIE::Relauncher::TLauncher2(_tds, &fEval, sInput, sOutputANN1);
    lfct->solverLoop();
    natt++;
    CPPUNIT_ASSERT_EQUAL(_tds->getNAttributes(), natt);

    _tds->computeStatistic(sOutputANN1);
    Double_t dMeanYANN1 = _tds->getAttribute(sOutputANN1)->getMean();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dMeanYhat, dMeanYANN1, 0.2);



    bStop = kFALSE;
    test1 = 0;
    test2 = 0;
    test3 = 0;
    _tann->exportFunction("c++", "tann_model0");
    fstream file1("tann_model0.C");
    time(&seconds);
    date = string(ctime(&seconds));

    while (bStop == kFALSE)
    {
        getline(file1, line);
        if (line.find("  //     ** Date : ") != string::npos)
        {
            //verify that the date are the same actually and in the file
            if ((date[0] == line[24]) && (date[1] == line[25])
                    && (date[2] == line[26]) && (date[3] == line[27])
                    && (date[4] == line[28]) && (date[5] == line[29])
                    && (date[6] == line[30]) && (date[7] == line[31])
                    && (date[8] == line[32]) && (date[9] == line[33])
                    && (date[10] == line[34]) && (date[11] == line[35])
                    && (date[12] == line[36]) && (date[13] == line[37])
                    && (date[14] == line[38]) && (date[15] == line[39])
                    && (date[16] == line[40]) && (date[17] == line[41])
                    && (date[18] == line[42]) && (date[19] == line[43])
                    && (date[20] == line[44]) && (date[21] == line[45])
                    && (date[22] == line[46]) && (date[23] == line[47]))
                test1 = 1;

            if(test1==0)
            {
                Int_t pos = date.find_last_of(":");
                Int_t pos2 = line.find_last_of(":");
                Int_t date_seconds, line_seconds;
                date_seconds =  (date[pos-5]*10+date[pos-4])*3600
                            +(date[pos-2]*10+date[pos-1])*60
                            +(date[pos+1]*10+date[pos+2]);

                line_seconds =  (line[pos2-5]*10+line[pos2-4])*3600
                            +(line[pos2-2]*10+line[pos2-1])*60
                            +(line[pos2+1]*10+line[pos2+2]);


                if(fabs(date_seconds - line_seconds + 0.0)<3)
                    test1 = 1;
            }
        }
        if (line.find("  0.099999, 25048.9, 89334.9, 89.5523, 1050,")
                != string::npos)
            test2 = 1;
        if (line.find("void myFunction(double *param, double *res)")
                != string::npos)
            test3 = 1;
        if (file1.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test1);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);


    bStop = kFALSE;
    test1 = 0;
    test2 = 0;
    test3 = 0;
    _tann->exportFunction("c++", "tann_model1", "function_tann");
    fstream file("tann_model1.C");
    time(&seconds);
    date = string(ctime(&seconds));

    while (bStop == kFALSE)
    {
        getline(file, line);
        if (line.find("  //     ** Date : ") != string::npos)
        {
            //verify that the date are the same actually and in the file
            if ((date[0] == line[24]) && (date[1] == line[25])
                    && (date[2] == line[26]) && (date[3] == line[27])
                    && (date[4] == line[28]) && (date[5] == line[29])
                    && (date[6] == line[30]) && (date[7] == line[31])
                    && (date[8] == line[32]) && (date[9] == line[33])
                    && (date[10] == line[34]) && (date[11] == line[35])
                    && (date[12] == line[36]) && (date[13] == line[37])
                    && (date[14] == line[38]) && (date[15] == line[39])
                    && (date[16] == line[40]) && (date[17] == line[41])
                    && (date[18] == line[42]) && (date[19] == line[43])
                    && (date[20] == line[44]) && (date[21] == line[45])
                    && (date[22] == line[46]) && (date[23] == line[47]))
                test1 = 1;

            if(test1==0)
            {
                Int_t pos = date.find_last_of(":");
                Int_t pos2 = line.find_last_of(":");
                Int_t date_seconds, line_seconds;
                date_seconds =  (date[pos-5]*10+date[pos-4])*3600
                            +(date[pos-2]*10+date[pos-1])*60
                            +(date[pos+1]*10+date[pos+2]);

                line_seconds =  (line[pos2-5]*10+line[pos2-4])*3600
                            +(line[pos2-2]*10+line[pos2-1])*60
                            +(line[pos2+1]*10+line[pos2+2]);


                if(fabs(date_seconds - line_seconds + 0.0)<3)
                    test1 = 1;
            }
        }
        if (line.find("  0.099999, 25048.9, 89334.9, 89.5523, 1050,")
                != string::npos)
            test2 = 1;
        if (line.find("void function_tann(double *param, double *res)")
                != string::npos)
            test3 = 1;
        if (file.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test1);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);

    // Eval the Model
    gROOT->LoadMacro("tann_model1.C");
    URANIE::Relauncher::TCIntEval fEval2("function_tann");

    TString sOutputANN2 = "yann2";
    URANIE::Relauncher::TLauncher2 *lfct2 = new URANIE::Relauncher::TLauncher2(_tds, &fEval2, sInput, sOutputANN2);
    lfct2->solverLoop();
    natt++;
    CPPUNIT_ASSERT_EQUAL(_tds->getNAttributes(), natt);


    _tds->computeStatistic(sOutputANN2);
    Double_t dMeanYANN2 = _tds->getAttribute(sOutputANN2)->getMean();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dMeanYhat, dMeanYANN2, 0.2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dMeanYANN1, dMeanYANN2, DBL_PRECISION_COMPARE);

    bStop = kFALSE;
    test1 = 0;
    test2 = 0;
    test3 = 0;

    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, Form("%s,%d,%s", sInput.Data(), nH, sOutput.Data()));
    _tann->unsetLog();
    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    _tann->setFcnTol(dPrecision);
    _tann->train(3, 2, "test");


    _tann->exportFunction("c++", "tann_model2", "function_tann2");
    fstream file3("tann_model2.C");
    time(&seconds);
    date = string(ctime(&seconds));
    while (bStop == kFALSE)
    {
        getline(file3, line);
        if (line.find("  //     ** Date : ") != string::npos)
        {
            //verify that the date are the same actually and in the file
            if ((date[0] == line[24]) && (date[1] == line[25])
                    && (date[2] == line[26]) && (date[3] == line[27])
                    && (date[4] == line[28]) && (date[5] == line[29])
                    && (date[6] == line[30]) && (date[7] == line[31])
                    && (date[8] == line[32]) && (date[9] == line[33])
                    && (date[10] == line[34]) && (date[11] == line[35])
                    && (date[12] == line[36]) && (date[13] == line[37])
                    && (date[14] == line[38]) && (date[15] == line[39])
                    && (date[16] == line[40]) && (date[17] == line[41])
                    && (date[18] == line[42]) && (date[19] == line[43])
                    && (date[20] == line[44]) && (date[21] == line[45])
                    && (date[22] == line[46]) && (date[23] == line[47]))
                test1 = 1;

            if(test1==0)
            {
                Int_t pos = date.find_last_of(":");
                Int_t pos2 = line.find_last_of(":");
                Int_t date_seconds, line_seconds;
                date_seconds =  (date[pos-5]*10+date[pos-4])*3600
                            +(date[pos-2]*10+date[pos-1])*60
                            +(date[pos+1]*10+date[pos+2]);

                line_seconds =  (line[pos2-5]*10+line[pos2-4])*3600
                            +(line[pos2-2]*10+line[pos2-1])*60
                            +(line[pos2+1]*10+line[pos2+2]);


                if(fabs(date_seconds - line_seconds + 0.0)<3)
                    test1 = 1;
            }
        }
        if (line.find("  0.0500698, 147.906, 63163.7, 63.1692, 990.01, ")
                != string::npos)
            test2 = 1;
        if (line.find("void function_tann2(double *param, double *res)")
                != string::npos)
            test3 = 1;
        if (file3.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test1);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);


    // Eval the Model
    gROOT->LoadMacro("tann_model2.C");
    URANIE::Relauncher::TCIntEval fEval3("function_tann2");

    TString sOutputANN3 = "yann3";
    URANIE::Relauncher::TLauncher2 *lfct3 = new URANIE::Relauncher::TLauncher2(_tds, &fEval3, sInput, sOutputANN3);
    lfct3->solverLoop();
    natt++;
    CPPUNIT_ASSERT_EQUAL(_tds->getNAttributes(), natt);


    _tds->computeStatistic(sOutputANN3);
    Double_t dMeanYANN3 = _tds->getAttribute(sOutputANN3)->getMean();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dMeanYhat, dMeanYANN3, 0.2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dMeanYANN1, dMeanYANN3, 0.2);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dMeanYANN2, dMeanYANN3, 0.2);

    cout << " *** dPrecision[" << dPrecision << "]" << endl;
    cout << " ***  dMeanYhat[" << dMeanYhat << "]" << endl;
    cout << " *** dMeanYANN1[" << dMeanYANN1 << "]" << endl;
    cout << " *** dMeanYANN2[" << dMeanYANN2 << "]" << endl;
    cout << " *** dMeanYANN3[" << dMeanYANN3 << "]" << endl;

    // Clean
    delete lfct; lfct = NULL;
    delete lfct2; lfct2 = NULL;
    delete lfct3; lfct3 = NULL;

}


void TANNModelerTest::testExportModelCplusplusDirect()
{
    // time_t seconds, secondsDefault;
    string line, date;
    Bool_t bStop = kFALSE;
    Int_t test2 = 0;
    Int_t test3 = 0;

    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->train(3, 2, "test");

    _tann->exportModelCplusplus();
    fstream file2("yhat_nH001.C");
    TString sOutput = "yhat";

    while (bStop == kFALSE)
    {
        getline(file2, line);

        if (line.find("  0.099999, 25048.9, 89334.9, 89.5523, 1050,")
                != string::npos)
            test2 = 1;
        if (line.find(Form("void Fct_%s(double *param, double *res)", sOutput.Data()))
                != string::npos)
            test3 = 1;
        if (file2.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);

    bStop = kFALSE;
    test2 = 0;
    test3 = 0;

    _tann->exportModelCplusplus("tann_model1_direct.C");
    fstream file1("tann_model1_direct.C");

    while (bStop == kFALSE)
    {
        getline(file1, line);

        if (line.find("  0.099999, 25048.9, 89334.9, 89.5523, 1050,")
                != string::npos)
            test2 = 1;
        if (line.find(Form("void Fct_%s(double *param, double *res)", sOutput.Data()))
                != string::npos)
            test3 = 1;
        if (file1.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);

    bStop = kFALSE;
    test2 = 0;
    test3 = 0;


    _tann->exportModelCplusplus("tann_model2_direct.C", "function_tann");
    fstream file("tann_model2_direct.C");

    while (bStop == kFALSE)
    {
        getline(file, line);

        if (line.find("  0.099999, 25048.9, 89334.9, 89.5523, 1050,")
                != string::npos)
            test2 = 1;
        if (line.find("void function_tann(double *param, double *res)")
                != string::npos)
            test3 = 1;
        if (file.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);

    bStop = kFALSE;
    test2 = 0;
    test3 = 0;


    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    _tann->train(3, 2, "test");
    _tann->exportModelCplusplus("tann_model3_direct.C", "function_tann");
    fstream file3("tann_model3_direct.C");

    while (bStop == kFALSE)
    {
        getline(file3, line);

        if (line.find("  0.0500698, 147.906, 63163.7, 63.1692, 990.01, ")
                != string::npos)
            test2 = 1;
        if (line.find("void function_tann(double *param, double *res)")
                != string::npos)
            test3 = 1;
        if (file3.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);


}





#ifdef  WITH_LIBXML2
void TANNModelerTest::testExportModelPMML()
{
    // time_t seconds;
    string line,date;
    Bool_t bStop=kFALSE;
    Int_t test=0;
    Int_t test2=0;
    Int_t test3=0;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->train(3, 2, "test");
    _tann->exportModelPMML("model_tann.pmml", "sANNName","new");
    fstream file("model_tann.pmml");
    while(bStop==kFALSE)
    {
        getline(file,line);
        if (line.find("  <NeuralNetwork modelName=\"sANNName\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
        test=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1,test);
    file.close();

    gSystem->CopyFile("model_tann.pmml","model_tann_init.pmml");
    _tann->train(3, 2,"learn");
    _tann->exportModelPMML("model_tann.pmml", "sANNName", "update");
    fstream file2("model_tann.pmml");
    bStop=kFALSE;
    while(bStop==kFALSE)
    {
        getline(file2,line);
        if (line.find("")!=string::npos)
        test2=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    file2.close();
    CPPUNIT_ASSERT_EQUAL(1,test2);
    Int_t nret = gSystem->Exec("diff model_tann.pmml model_tann_init.pmml");
    CPPUNIT_ASSERT_ASSERTION_FAIL(CPPUNIT_ASSERT(0==nret));

    _tann->train(3, 2,"learn");
    _tann->exportModelPMML("model_tann.pmml", _tann->GetName(), "");
    fstream file3("model_tann.pmml");
    bStop=kFALSE;
    test3=0;
    while(bStop==kFALSE)
    {
        getline(file3,line);
        if (line.find("  <NeuralNetwork modelName=\"sANNName\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
        test3+=1;
        if (line.find("  <NeuralNetwork modelName=\"_modeler_tdsFlowrate\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
        test3+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(2,test3);
}


void TANNModelerTest::testExportModelPMMLBackUp()
{
    // time_t seconds;
    string line,date;
    Bool_t bStop=kFALSE;
    Int_t test=0;
    Int_t test2=0;
    Int_t test3=0;
    Int_t test4=0;
    Int_t test5=0;
    Int_t test6=0;
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->train(3, 2, "test");
    _tann->exportModelPMML("model_tann.pmml", "sANNName");
    fstream file("model_tann.pmml");
    while(bStop==kFALSE)
    {
        getline(file,line);
        if (line.find("  <NeuralNetwork modelName=\"sANNName\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
        test=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1,test);
    file.close();


    _tann->train(3, 2,"learn");
    _tann->exportModelPMML("model_tann.pmml", "sANNName");
    fstream file2("model_tann.pmml");
    bStop=kFALSE;
    while(bStop==kFALSE)
    {
        getline(file2,line);
        if (line.find(" functionName=\"regression\" numberOfLayers=\"2\" modelName=\"sANNName_0\"")!=string::npos)
            test2+=1;
        if(line.find("  <NeuralNetwork modelName=\"sANNName\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
            test2+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    file2.close();
    CPPUNIT_ASSERT_EQUAL(2,test2);


    _tann->train(3, 2,"learn");
    _tann->exportModelPMML("model_tann.pmml", _tann->GetName());
    fstream file3("model_tann.pmml");
    bStop=kFALSE;
    test3=0;
    while(bStop==kFALSE)
    {
        getline(file3,line);
        if (line.find("  <NeuralNetwork modelName=\"sANNName\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
        test3+=1;
        if (line.find("  <NeuralNetwork modelName=\"_modeler_tdsFlowrate\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
        test3+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    file3.close();
    CPPUNIT_ASSERT_EQUAL(2,test3);


    _tann->train(3, 2,"learn");
    _tann->exportModelPMML("model_tann.pmml", "sANNName");
    fstream file4("model_tann.pmml");
    bStop=kFALSE;
    while(bStop==kFALSE)
    {
        getline(file4,line);
        if (line.find(" functionName=\"regression\" numberOfLayers=\"2\" modelName=\"sANNName_0\"")!=string::npos)
            test4+=1;
        if (line.find(" functionName=\"regression\" numberOfLayers=\"2\" modelName=\"sANNName_1\"")!=string::npos)
            test4+=1;
        if(line.find("  <NeuralNetwork modelName=\"sANNName\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
            test4+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    file4.close();
    CPPUNIT_ASSERT_EQUAL(3,test4);


    _tann->train(3, 2,"learn");
    _tann->exportModelPMML("model_tann.pmml", "sANNName");
    fstream file5("model_tann.pmml");
    bStop=kFALSE;
    while(bStop==kFALSE)
    {
        getline(file5,line);
        if (line.find(" functionName=\"regression\" numberOfLayers=\"2\" modelName=\"sANNName_0\"")!=string::npos)
            test5+=1;
        if (line.find(" functionName=\"regression\" numberOfLayers=\"2\" modelName=\"sANNName_1\"")!=string::npos)
            test5+=1;
        if (line.find(" functionName=\"regression\" numberOfLayers=\"2\" modelName=\"sANNName_2\"")!=string::npos)
            test5+=1;
        if(line.find("  <NeuralNetwork modelName=\"sANNName\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
            test5+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    file5.close();
    CPPUNIT_ASSERT_EQUAL(4,test5);

    remove("model_tann.pmml");

    _tann->train(3, 2,"learn");
    _tann->exportModelPMML("model_tann.pmml", _tann->GetName());
    _tann->train(3, 2,"learn");
    _tann->exportModelPMML("model_tann.pmml", "sANNName");
    fstream file6("model_tann.pmml");
    bStop=kFALSE;

    while(bStop==kFALSE)
    {
        getline(file6,line);
        if (line.find("  <NeuralNetwork modelName=\"sANNName\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
        test6+=1;
        if (line.find("  <NeuralNetwork modelName=\"_modeler_tdsFlowrate\" functionName=\"regression\" numberOfLayers=\"2\">")!=string::npos)
        test6+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    file6.close();
    CPPUNIT_ASSERT_EQUAL(2,test6);

}

#endif

void TANNModelerTest::testsetWeights()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    ColumnVector poids(11);
    poids(1) = 5.85862;
    poids(2) = -7.70824;
    poids(3) = 1.30539;
    poids(4) = -0.690922;
    poids(5) = 0.000214278;
    poids(6) = -0.00382477;
    poids(7) = -0.00905621;
    poids(8) = -0.156506;
    poids(9) = 0.162442;
    poids(10) = 0.158448;
    poids(11) = -0.0795347;

    _tann->setWeights(poids);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(5.85862, _tann->getWeight(0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-7.70824, _tann->getWeight(1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.30539, _tann->getWeight(2),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.690922, _tann->getWeight(3),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000214278, _tann->getWeight(4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.00382477, _tann->getWeight(5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.00905621, _tann->getWeight(6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.156506, _tann->getWeight(7),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.162442, _tann->getWeight(8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.158448, _tann->getWeight(9),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.0795347, _tann->getWeight(10),
            DBL_PRECISION_COMPARE);
}

void TANNModelerTest::throwExceptionTANNModelerTdsNull()
{
    URANIE::DataServer::TDataServer * tdsnull =
            new URANIE::DataServer::TDataServer();
    tdsnull = NULL;
    CPPUNIT_ASSERT_THROW(new URANIE::Modeler::TANNModeler(NULL, _archi),
            URANIE::Exceptions::UErrorExceptions);
    delete tdsnull;

}
void TANNModelerTest::testANNModelerMultiOutput()
{
    Int_t natt = _tds->getNAttributes();
    CPPUNIT_ASSERT_EQUAL(9, natt);
    TString sInput = "rw:r:tu:tl:hu:hl:l:kw";
    Int_t nX = sInput.CountChar(':')+1;
    TString sOutput = "yhat:r";
    Int_t nY = sOutput.CountChar(':')+1;
    Int_t nH = 5;
    Double_t dPrecision = 1e-6;
    Int_t nW = (nX+nY+1)*nH+nY;
    CPPUNIT_ASSERT_EQUAL(8,nX);
    CPPUNIT_ASSERT_EQUAL(2,nY);

    _tds->computeStatistic(sOutput);
    Double_t dMeanYhat = _tds->getAttribute("yhat")->getMean();
    Double_t dMeanR = _tds->getAttribute("r")->getMean();
//    cout << " *** dMeanYhat[" << dMeanYhat << "] dMeanR[" << dMeanR << "]" << endl;

    _tann =  new URANIE::Modeler::TANNModeler(_tds, Form("%s,%d,%s", sInput.Data(), nH, sOutput.Data()));
    _tann->unsetLog();

    CPPUNIT_ASSERT_EQUAL(nX,_tann->getNInput());
    CPPUNIT_ASSERT_EQUAL(nH,_tann->getNHidden());
    CPPUNIT_ASSERT_EQUAL(nY,_tann->getNOutput());
    CPPUNIT_ASSERT_EQUAL(nW,_tann->_UMLP->getNWeights());

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kMinusOneOne, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, _tann->_UMLP->getActivationFunction());

    _tann->setFcnTol(dPrecision);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dPrecision, _tann->getFcnTol(), DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kRegression, _tann->getProblem());

    _tann->train(2, 2, "test", kFALSE);
//    cout << " ** App[" << _tann->getEQM("learn") << "] test[" << _tann->getEQM("test") << "]" << endl;
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000228830, _tann->getEQM("test"),  0.0010);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.000317745, _tann->getEQM("learn"), 0.0010);

    _tann->exportFunction("c++", "tann_model_multi", "function_tann_multi");
    // Eval the Model
    gROOT->LoadMacro("tann_model_multi.C");
    URANIE::Relauncher::TCIntEval fEval("function_tann_multi");

    TString sOutputANN1 = "yhat_ann1:r_ann1";
    URANIE::Relauncher::TLauncher2 *lfct = new URANIE::Relauncher::TLauncher2(_tds, &fEval, sInput, sOutputANN1);
    lfct->solverLoop();
    natt += nY;
    CPPUNIT_ASSERT_EQUAL(_tds->getNAttributes(), natt);

    _tds->computeStatistic(sOutputANN1);

    Double_t dMeanYhat_ANN1 = _tds->getAttribute("yhat_ann1")->getMean();
//    cout << " *** dMeanYhat[" << dMeanYhat << "] dMeanYhat_ANN1[" << dMeanYhat_ANN1 << "]" << endl;

    Double_t dMeanR_ANN1 = _tds->getAttribute("r_ann1")->getMean();
//    cout << " *** dMeanR[" << dMeanR << "] dMeanR_ANN1[" << dMeanR_ANN1 << "]" << endl;

    CPPUNIT_ASSERT_DOUBLES_EQUAL(dMeanYhat, dMeanYhat_ANN1, 0.01* dMeanYhat);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dMeanR, dMeanR_ANN1, 0.01 * dMeanR);

    delete lfct;
    
}

void TANNModelerTest::testANNModelerClassification()
{
    delete _tds;    _tds = NULL;

    _tds = new URANIE::DataServer::TDataServer();
    _tds->fileDataRead("problem2Classes_001000.dat");

    Int_t nH1 = 15;
    Int_t nH2 = 10;
    Int_t nH3 = 5;

    TString sX = "x:y";

    Int_t seed=123456;
    Int_t nx = 2;
    Int_t ny = 1;
    Int_t nh = 30;
    Int_t nw = 266;
    Double_t dtol = 1e-4;

    // Case of data with {0., 1.}
    TString sY = "rc01";
    _tann = new URANIE::Modeler::TANNModeler(_tds, Form("%s,%d,%d,%d,@%s", sX.Data(), nH1, nH2, nH3, sY.Data()));

    CPPUNIT_ASSERT_EQUAL(nx,_tann->getNInput());
    CPPUNIT_ASSERT_EQUAL(nh,_tann->getNHidden());
    CPPUNIT_ASSERT_EQUAL(ny,_tann->getNOutput());
    CPPUNIT_ASSERT_EQUAL(nw,_tann->_UMLP->getNWeights());

    _tann->setFcnTol(dtol);
    _tann->setSeed(seed);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dtol, _tann->getFcnTol(), DBL_PRECISION_COMPARE);

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kMinusOneOne, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, _tann->_UMLP->getActivationFunction());

    _tann->train(2, 2, "test", kFALSE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.003, _tann->getEQM("test"),  0.0030);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.003, _tann->getEQM("learn"), 0.0030);

    // Clean
    delete _tann; _tann = NULL;

    // Case of data with {5., 9.}
    sY = "rc59";
    _tann = new URANIE::Modeler::TANNModeler(_tds, Form("%s,%d,%d,%d,@%s", sX.Data(), nH1, nH2, nH3, sY.Data()));
    _tann->setSeed(seed);
    CPPUNIT_ASSERT_EQUAL(nx,_tann->getNInput());
    CPPUNIT_ASSERT_EQUAL(nh,_tann->getNHidden());
    CPPUNIT_ASSERT_EQUAL(ny,_tann->getNOutput());
    CPPUNIT_ASSERT_EQUAL(nw,_tann->_UMLP->getNWeights());

    _tann->setFcnTol(dtol);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dtol, _tann->getFcnTol(), DBL_PRECISION_COMPARE);

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kMinusOneOne, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, _tann->_UMLP->getActivationFunction());

    _tann->train(2, 2, "test", kFALSE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.003, _tann->getEQM("test"),  0.0030);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.003, _tann->getEQM("learn"), 0.0030);

    // Clean
    delete _tann; _tann = NULL;


    // Case of data with {-2., 9.}
    sY = "rcM29";
    _tann = new URANIE::Modeler::TANNModeler(_tds, Form("%s,%d,%d,%d,@%s", sX.Data(), nH1, nH2, nH3, sY.Data()));
    _tann->setSeed(seed);
    CPPUNIT_ASSERT_EQUAL(nx,_tann->getNInput());
    CPPUNIT_ASSERT_EQUAL(nh,_tann->getNHidden());
    CPPUNIT_ASSERT_EQUAL(ny,_tann->getNOutput());
    CPPUNIT_ASSERT_EQUAL(nw,_tann->_UMLP->getNWeights());

    _tann->setFcnTol(dtol);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dtol, _tann->getFcnTol(), DBL_PRECISION_COMPARE);

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kMinusOneOne, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, _tann->_UMLP->getActivationFunction());

    _tann->train(2, 2, "test", kFALSE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.003, _tann->getEQM("test"),  0.0030);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.003, _tann->getEQM("learn"), 0.0030);

    // Clean
    delete _tann; _tann = NULL;


    // Case of data with {-9., -5.}
    sY = "rcM9M5";
    _tann = new URANIE::Modeler::TANNModeler(_tds, Form("%s,%d,%d,%d,@%s", sX.Data(), nH1, nH2, nH3, sY.Data()));
    _tann->setSeed(seed);
    CPPUNIT_ASSERT_EQUAL(nx,_tann->getNInput());
    CPPUNIT_ASSERT_EQUAL(nh,_tann->getNHidden());
    CPPUNIT_ASSERT_EQUAL(ny,_tann->getNOutput());
    CPPUNIT_ASSERT_EQUAL(nw,_tann->_UMLP->getNWeights());

    _tann->setFcnTol(dtol);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(dtol, _tann->getFcnTol(), DBL_PRECISION_COMPARE);

    _tann->setNormalization(URANIE::Modeler::TANNModeler::kMinusOneOne);
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::TANNModeler::kMinusOneOne, _tann->getNormalization());
    CPPUNIT_ASSERT_EQUAL(URANIE::Modeler::UMLP::kTANH, _tann->_UMLP->getActivationFunction());

    _tann->train(2, 2, "test", kFALSE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.003, _tann->getEQM("test"),  0.0030);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.003, _tann->getEQM("learn"), 0.0030);

    // Clean
    delete _tann; _tann = NULL;




}

void TANNModelerTest::throwExceptionTANNModelerAttributeFalse()
{
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TANNModeler(_tds,
                    "rw:r:tu:tl:hu:hl:l:kw,1,toto"),
            URANIE::Exceptions::UErrorExceptions);
}
void TANNModelerTest::throwExceptionTANNexportFunction()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    _tann->init();
    _tann->randomizeWeights();
    CPPUNIT_ASSERT_THROW(
            _tann->exportFunction("toto", "file_toto.toto", "function_toto"),
            URANIE::Exceptions::UErrorExceptions);

    // Case of Multi hidden layers
    TString sNewArchi = "rw:r:tu:tl:hu:hl:l:kw,5,2,yhat";
    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, sNewArchi);
    _tann->unsetLog();
    _tann->init();
    _tann->randomizeWeights();
    CPPUNIT_ASSERT_THROW(
            _tann->exportFunction("pmml", "file_toto.toto", "function_toto"),
            URANIE::Exceptions::UErrorExceptions);

    // Case of multi-ouput
    sNewArchi = "rw:r:tu:tl:hu:hl:l:kw,5,yhat:yhat";
    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, sNewArchi);
    _tann->unsetLog();
    _tann->init();
    _tann->randomizeWeights();
    CPPUNIT_ASSERT_THROW(
            _tann->exportFunction("pmml", "file_toto.toto", "function_toto"),
            URANIE::Exceptions::UErrorExceptions);

    // Case of  classification
    sNewArchi = "rw:r:tu:tl:hu:hl:l:kw,5,@yhat";
    delete _tann;
    _tann = new URANIE::Modeler::TANNModeler(_tds, sNewArchi);
    _tann->unsetLog();
    _tann->init();
    _tann->randomizeWeights();
    CPPUNIT_ASSERT_THROW(
            _tann->exportFunction("pmml", "file_toto.toto", "function_toto"),
            URANIE::Exceptions::UErrorExceptions);
}
void TANNModelerTest::throwExceptiongetEQM()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    CPPUNIT_ASSERT_THROW(_tann->getEQM("toto"),
            URANIE::Exceptions::UWarningExceptions);
}
void TANNModelerTest::throwExceptiontrainFalseniter()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    CPPUNIT_ASSERT_THROW(_tann->train(-1, 2),
            URANIE::Exceptions::UErrorExceptions);
}

void TANNModelerTest::throwExceptiontrainFalseninit()
{
    _tann = new URANIE::Modeler::TANNModeler(_tds, _archi);
    _tann->unsetLog();
    CPPUNIT_ASSERT_THROW(_tann->train(3, -2),
            URANIE::Exceptions::UErrorExceptions);
}

