//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTANNModeler.h
 \author Maud Salvoldi (OCC)
 \date   18/10/2011
 
 \brief Interface of the class TestTANNModeler
 
 */
#ifndef CPPUNITEST_TESTTANNMODELER_H
#define CPPUNITEST_TESTTANNMODELER_H

// cppunit
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// uranie
#include "TANNModeler.h"
#include "TDataServer.h"
#include "UExceptions.h"

/**

 * @class TestTANNModeler
 *
 * @brief Description of the class TestTANNModeler.
 *
 *  The class TestTANNModeler is the unit test class of the class TANNModeler. 
 *  
 *
 */

class TANNModelerTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TANNModelerTest);

    CPPUNIT_TEST (testTANNModelerFirstConstructor);
#ifdef  WITH_LIBXML2
    CPPUNIT_TEST( testTANNModelerSecondConstructor);
#endif
    CPPUNIT_TEST (testTANNModelergetOutputName);
    CPPUNIT_TEST (testTANNModelergetInputName);
    CPPUNIT_TEST (testtrainANDgetEQM);
    CPPUNIT_TEST (testtrainWithWeightDecay);
    CPPUNIT_TEST (testgetWeights);
    CPPUNIT_TEST (testsetSeed);
    CPPUNIT_TEST (testsetFcnTol);
    CPPUNIT_TEST (testExportModelCplusplus);
    CPPUNIT_TEST (testExportModelCplusplusDirect);
    CPPUNIT_TEST (testExportModelFortran);
    CPPUNIT_TEST (testExportModelFortranDirect);
#ifdef  WITH_LIBXML2
    CPPUNIT_TEST( testExportModelPMML);
    CPPUNIT_TEST( testExportModelPMMLBackUp);
#endif
    CPPUNIT_TEST (testsetWeights);
    CPPUNIT_TEST (testsetNormalization);
    CPPUNIT_TEST (testANNModelerMultiOutput);
    CPPUNIT_TEST (testANNModelerClassification);

// //   CPPUNIT_TEST( throwExceptionTANNModelerTdsNull);
    CPPUNIT_TEST (throwExceptionTANNModelerAttributeFalse);
    CPPUNIT_TEST (throwExceptionTANNexportFunction);
    CPPUNIT_TEST (throwExceptiongetEQM);
    CPPUNIT_TEST (throwExceptiontrainFalseniter);
    CPPUNIT_TEST (throwExceptiontrainFalseninit);

    CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructor
     */
    TANNModelerTest();
    /*! Destructor
     */
    virtual ~TANNModelerTest();

    //---------------------------------------------
    /** @name setUp and Tear Down methods
     **/
    //@{
    //! Build a object TNisp from a tds object
    void setUp();
    //! ! Delete the object tds and nips
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name unit test methods
     **/
    //@{
    //! Unit test of constructor with architecture parameter
    void testTANNModelerFirstConstructor();
#ifdef  WITH_LIBXML2
    //! Unit test of constructor with pmml file
    void testTANNModelerSecondConstructor();
#endif
    //! Uit test of getOutputName() method
    void testTANNModelergetOutputName();
    //!Unit test of getInputName() method
    void testTANNModelergetInputName();
    //! Unit test of train() and getEQM()method
    void testtrainANDgetEQM();
    //! Train with weight Decay
    void testtrainWithWeightDecay();
    //!Unit test of getWeights() method
    void testgetWeights();
    //!Unit test of setSeed(Int_t ind=0) method
    void testsetSeed();
    //!Unit test of getFcnTol() and setFcnTol(Double_t dprec) methods
    void testsetFcnTol();
    //! Unit test of ExportFunction in C++ language
    void testExportModelCplusplus();
    //!Unit test of ExportModelCplusplus in C++ language
    void testExportModelCplusplusDirect();
    //! Unit test of ExportFunction in Fortran language
    void testExportModelFortran();
    //! Unit test of ExportFunction in Fortran language
    void testExportModelFortranDirect();
#ifdef  WITH_LIBXML2
    //! Unit test of ExportFunction in PMML format
    void testExportModelPMML();
    //! Unit test of ExportFunction in PMML format with the default "backup" option
    void testExportModelPMMLBackUp();
#endif
    //!Unit test of setWeigth(ColumnVector vec) method
    void testsetWeights();
    //!Unit test of setNormalization(ENorm n) and getNormalization() methods
    void testsetNormalization();
    //! test for multi output
    void testANNModelerMultiOutput();
    //! test with multi hidden layer for classificationn problem
    void testANNModelerClassification();
    //@}

    //---------------------------------------------
    /** @name throwException methods
     **/
    //@{
    //! Verify exception for a tds null
    void throwExceptionTANNModelerTdsNull();
    //! Verify exception for false attrioute name
    void throwExceptionTANNModelerAttributeFalse();
    //! verify exception for false language of export file
    void throwExceptionTANNexportFunction();
    //! verify exception for false option 
    void throwExceptiongetEQM();
    //! Verify exception for false number of iteration
    void throwExceptiontrainFalseniter();
    //!Verify exception for false number of init
    void throwExceptiontrainFalseninit();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds;
    TString _archi;
    URANIE::Modeler::TANNModeler *_tann;

};

#endif  // CPPUNITEST_TESTTANNMODELER_H
