//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTLinearRegression.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TLinearRegression
 
 Cr�e le : mer nov 21 07:59:24 2007
 
 \par Modifications
 \li M. Salvoldi - OCC/Areva group (19-10-2011)
 - add 9 units tests to covert the class Tlinear Regression at 80%
 - add a assertion (Pression variable) in the unit test testLinearRegressionSteamplantDatabaseUnivariate and testLinearRegressionSteamplantDatabaseMultivariate
 */

#define DBL_PRECISION_COMPARE 1.e-10
// Data Header
#include "TestTLinearRegression.h"
#include "TString.h"
#include <iostream>
#include <ctime>
#include <cmath>
#include "uranie_commands.h"
using namespace std;

TLinearRegressionTest::TLinearRegressionTest()
{
    fstream file("Linear_regression.C");
    if (!file.fail())
        remove("Linear_regression.C");
    fstream file2("linearmodel_x1.f");
    if (!file2.fail())
        remove("linearmodel_x1.f");
}

TLinearRegressionTest::~TLinearRegressionTest()
{
    fstream file("Linear_regression.C");
    if (!file.fail())
        remove("Linear_regression.C");
    fstream file2("linearmodel_x1.f");
    if (!file2.fail())
        remove("linearmodel_x1.f");

    UranieTests::removeFile(gSystem->pwd(),"model_tlr.pmml");
}

void TLinearRegressionTest::setUp()
{
    tds1 = new URANIE::DataServer::TDataServer();
    tds1->fileDataRead("steamplant.dat");
    
}

void TLinearRegressionTest::tearDown()
{
    delete tds1;
    tds1 = NULL;

    UranieTests::removeFile(gSystem->pwd(),"CASP.root");
    UranieTests::removeFile(gSystem->pwd(),"_dataserver_.root");
    UranieTests::removeFile(gSystem->pwd(),"steamplant.root");
    UranieTests::removeFile(gSystem->pwd(),"testQ2.root");

}

void TLinearRegressionTest::testLinearRegressionSteamplantDatabaseUnivariate()
{
    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1, "x8", "x1");
    tlin->estimate();

    TVectorD dparam = tlin->getParameters();

    CPPUNIT_ASSERT_EQUAL(2, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(13.6229892681727, dparam(0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.0798286933112, dparam(1),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7144375210132, tlin->getR2(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7020217610573, tlin->getR2Adjusted(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.6631904824084, tlin->getQ2(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(21.4937688127201, tlin->getPress(),
            DBL_PRECISION_COMPARE);
    delete tlin;
}

void TLinearRegressionTest::testLinearRegressionConstructorArchitecture()
{
    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1, "x8,1,x1");
    tlin->estimate();

    TVectorD dparam = tlin->getParameters();

    CPPUNIT_ASSERT_EQUAL(2, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(13.6229892681727, dparam(0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.0798286933112, dparam(1),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7144375210132, tlin->getR2(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7020217610573, tlin->getR2Adjusted(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.6631904824084, tlin->getQ2(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(21.4937688127201, tlin->getPress(),
            DBL_PRECISION_COMPARE);
        delete tlin;
}

void TLinearRegressionTest::testLinearRegressionSteamplantDatabaseMultivariate()
{
    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1,
                    "x2:x3:x4:x5:x6:x7:x8:x9:x10", "x1");
    tlin->estimate();

    TVectorD dparam = tlin->getParameters();

    CPPUNIT_ASSERT_EQUAL(10, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.89421329370215, dparam(0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.70540614502053, dparam(1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-1.89371935292486, dparam(2),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.13422012418993, dparam(3),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.11876299490911, dparam(4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.17934523383018, dparam(5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.01817863768707, dparam(6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.07741747398736, dparam(7),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.08584656065929, dparam(8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.34500532887358, dparam(9),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.9236999190611, tlin->getR2(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8779198704978, tlin->getR2Adjusted(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7023412623609, tlin->getQ2(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(18.9953304694293, tlin->getPress(),
            DBL_PRECISION_COMPARE);
        delete tlin;
}

void TLinearRegressionTest::testLinearRegressionCASPDatabaseMultivariateBigSize()
{
    URANIE::DataServer::TDataServer *tdsC = new URANIE::DataServer::TDataServer();
    tdsC->fileDataRead("CASP.dat"); 

    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tdsC,
                    "F1:F2:F3:F4:F5:F6:F7:F8:F9", "RMSD");
    tlin->estimate();

    TVectorD dparam = tlin->getParameters();

    CPPUNIT_ASSERT_EQUAL(10, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(6.03655151711, dparam(0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00157157913996, dparam(1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00142930297242, dparam(2),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(18.0338792178, dparam(3),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.108211440449, dparam(4),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-4.07500841164e-06, dparam(5),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.0238774289623, dparam(6),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.000138681703084, dparam(7),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0148541790991, dparam(8),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.110121797589, dparam(9),
            DBL_PRECISION_COMPARE);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.282307348047, tlin->getR2(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.282166069966, tlin->getR2Adjusted(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.281829865587, tlin->getQ2(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1229368.9461795995, tlin->getPress(),
            DBL_PRECISION_COMPARE);
    
    delete tlin;
     
    delete tdsC;
    tdsC = NULL;
}

void TLinearRegressionTest::testLinearRegressionDataQ2NoIntercept()
{
    delete tds1;
    tds1 = new URANIE::DataServer::TDataServer();
    tds1->fileDataRead("testQ2.dat");

    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1,
                    " X1:X2:X3:X4:X5:X6:X7:X8:X9:X10:X11", "Densite",
                    "nointercept");
    tlin->unsetLog();
    tlin->estimate();

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.9832, tlin->getR2(), 1e-04);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.9645, tlin->getR2Adjusted(), 1e-04);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.8939, tlin->getQ2(), 1e-04);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.00838, tlin->getPress(), 1e-04);
    delete tlin;
}

void TLinearRegressionTest::testLinearRegressionSteamplantDatabaseMultivariateArchiNoIntercept()
{
//     URANIE::Modeler::TLinearRegression *tlin =
//             new URANIE::Modeler::TLinearRegression(tds1, TString("x6:x8,1,x1"),
//                     "nointercept");
//     tlin->unsetLog();
//     tlin->estimate();
// 
//     TVectorD dparam = tlin->getParameters();
// 
//     CPPUNIT_ASSERT_EQUAL(2, dparam.GetNrows());
// 
//     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.551297418759727, dparam(0),
//             DBL_PRECISION_COMPARE);
//     CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.0354684612639308, dparam(1),
//             DBL_PRECISION_COMPARE);
// 
//     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.982660093559489, tlin->getR2(),
//             DBL_PRECISION_COMPARE);
//     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.981152275608141, tlin->getR2Adjusted(),
//             DBL_PRECISION_COMPARE);
//     CPPUNIT_ASSERT_DOUBLES_EQUAL(0.141736676555745, tlin->getQ2(),
//             DBL_PRECISION_COMPARE);
//     CPPUNIT_ASSERT_DOUBLES_EQUAL(54.7707605962539, tlin->getPress(),
//             DBL_PRECISION_COMPARE);
}

void TLinearRegressionTest::testLinearRegressionEffortDatabase()
{
    URANIE::DataServer::TDataServer * tds =
            new URANIE::DataServer::TDataServer();

    tds->addAttribute("setting");
    tds->addAttribute("effort");
    tds->addAttribute("change");
    tds->createTuple();

    tds->getTuple()->Fill(1, 46, 0, 1);
    tds->getTuple()->Fill(2, 74, 0, 10);
    tds->getTuple()->Fill(3, 89, 16, 29);
    tds->getTuple()->Fill(4, 77, 16, 25);
    tds->getTuple()->Fill(5, 84, 21, 29);
    tds->getTuple()->Fill(6, 89, 15, 40);
    tds->getTuple()->Fill(7, 68, 14, 21);
    tds->getTuple()->Fill(8, 70, 6, 0);
    tds->getTuple()->Fill(9, 60, 13, 13);
    tds->getTuple()->Fill(10, 55, 9, 4);
    tds->getTuple()->Fill(11, 35, 3, 0);
    tds->getTuple()->Fill(12, 51, 7, 7);
    tds->getTuple()->Fill(13, 87, 23, 21);
    tds->getTuple()->Fill(14, 83, 4, 9);
    tds->getTuple()->Fill(15, 68, 0, 7);
    tds->getTuple()->Fill(16, 84, 19, 22);
    tds->getTuple()->Fill(17, 74, 3, 6);
    tds->getTuple()->Fill(18, 73, 0, 2);
    tds->getTuple()->Fill(19, 84, 15, 29);
    tds->getTuple()->Fill(20, 91, 7, 11);

    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds, "setting:effort",
                    "change");
    tlin->estimate();

    TVectorD dparam = tlin->getParameters();

    CPPUNIT_ASSERT_EQUAL(20, tds->getNPatterns());
    CPPUNIT_ASSERT_EQUAL(3, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-14.4511, dparam(0), 1e-4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(.2705885, dparam(1), 1e-7);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(.9677137, dparam(2), 1e-7);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7381, tlin->getR2(), 1e-4);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.7073, tlin->getR2Adjusted(), 1e-4);

    delete tlin;
    
    delete tds;
    tds = NULL;
    
}

void TLinearRegressionTest::testLinearRegressionMileageDatabase()
{

    URANIE::DataServer::TDataServer * tds =
            new URANIE::DataServer::TDataServer();

    tds->addAttribute("mph");
    tds->addAttribute("mpg");
    tds->createTuple();

    tds->getTuple()->Fill(1, 20, 15.4);
    tds->getTuple()->Fill(2, 30, 20.2);
    tds->getTuple()->Fill(3, 40, 25.7);
    tds->getTuple()->Fill(4, 50, 26.2);
    tds->getTuple()->Fill(5, 50, 26.6);
    tds->getTuple()->Fill(6, 50, 27.4);
    tds->getTuple()->Fill(7, 60, 24.8);

    tds->addAttribute("mph2", "mph*mph");

    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds, "mph:mph2", "mpg");
    tlin->estimate();

    TVectorD dparam = tlin->getParameters();

    CPPUNIT_ASSERT_EQUAL(7, tds->getNPatterns());
    CPPUNIT_ASSERT_EQUAL(3, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(-5.985245902, dparam(0), 1e-9);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.305245902, dparam(1), 1e-9);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(-0.013098361, dparam(2), 1e-9);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.974986, tlin->getR2(), 1e-6);

    delete tlin;
	
    delete tds;
    tds = NULL;
}

void TLinearRegressionTest::testexportFunctionC()
{
    time_t seconds;
    string line, date;
    Bool_t bStop = kFALSE;
    Int_t test = 0;
    Int_t test2 = 0;
    Int_t test3 = 0;

    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1, "x6:x8", "x1");
    tlin->unsetLog();
    tlin->estimate();
    tlin->exportFunction("c", "Linear_regression", "Steamplant_LR");

    fstream file("Linear_regression.C");

    time(&seconds);
    date = string(ctime(&seconds));
    while (bStop == kFALSE)
    {
        getline(file, line);
        if (line.find("  //     ** Date : ") != string::npos)
        {
            //verify that the date are the same actually and in the file
            if ((date[0] == line[19]) && (date[1] == line[20])
                    && (date[2] == line[21]) && (date[3] == line[22])
                    && (date[4] == line[23]) && (date[5] == line[24])
                    && (date[6] == line[25]) && (date[7] == line[26])
                    && (date[8] == line[27]) && (date[9] == line[28])
                    && (date[10] == line[29]) && (date[11] == line[30])
                    && (date[12] == line[31]) && (date[13] == line[32])
                    && (date[14] == line[33]) && (date[15] == line[34])
                    && (date[16] == line[35]) && (date[17] == line[36])
                    && (date[18] == line[37]) && (date[19] == line[38])
                    && (date[20] == line[39]) && (date[21] == line[40])
                    && (date[22] == line[41]) && (date[23] == line[42]))
                test = 1;
             
            
            
            if(test==0)
            {
                Int_t pos = date.find_last_of(":");
                Int_t pos2 = line.find_last_of(":");
                Int_t date_seconds, line_seconds;
                date_seconds =  (date[pos-5]*10+date[pos-4])*3600
                            +(date[pos-2]*10+date[pos-1])*60
                            +(date[pos+1]*10+date[pos+2]);
                            
                line_seconds =  (line[pos2-5]*10+line[pos2-4])*3600
                            +(line[pos2-2]*10+line[pos2-1])*60
                            +(line[pos2+1]*10+line[pos2+2]);
                
               
                if(fabs(date_seconds - line_seconds + 0.0)<3)
                    test = 1;
            }
        }
        if (line.find("y += param[1]*-0.0723929") != string::npos)
            test2 = 1;
        if (line.find("void Steamplant_LR(double *param, double *res)")
                != string::npos)
            test3 = 1;
        if (file.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);

    delete tlin;
}
void TLinearRegressionTest::testexportFunctionF()
{
    time_t seconds;
    string line, date;
    Bool_t bStop = kFALSE;
    Int_t test = 0;
    Int_t test2 = 0;
    Int_t test3 = 0;

    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1, "x6:x8", "x1");
    tlin->unsetLog();
    tlin->estimate();
    tlin->exportFunction("for");

    fstream file("linearmodel_x1.f");

    time(&seconds);
    date = string(ctime(&seconds));
    while (bStop == kFALSE)
    {
        getline(file, line);
        if (line.find("C ---      ** Date : ") != string::npos)
        {
            //verify that the date are the same actually and in the file
            if ((date[0] == line[21]) && (date[1] == line[22])
                    && (date[2] == line[23]) && (date[3] == line[24])
                    && (date[4] == line[25]) && (date[5] == line[26])
                    && (date[6] == line[27]) && (date[7] == line[28])
                    && (date[8] == line[29]) && (date[9] == line[30])
                    && (date[10] == line[31]) && (date[11] == line[32])
                    && (date[12] == line[33]) && (date[13] == line[34])
                    && (date[14] == line[35]) && (date[15] == line[36])
                    && (date[16] == line[37]) && (date[17] == line[38])
                    && (date[18] == line[39]) && (date[19] == line[40])
                    && (date[20] == line[41]) && (date[21] == line[42])
                    && (date[22] == line[43]) && (date[23] == line[44]))
                test = 1;
                
            
//             if(test==0)
//             {
//                 Int_t pos = date.find_last_of(":");
//                 Int_t pos2 = line.find_last_of(":");
//                 Int_t date_seconds, line_seconds;
//                 date_seconds =  (date[pos-5]*10+date[pos-4])*3600
//                             +(date[pos-2]*10+date[pos-1])*60
//                             +(date[pos+1]*10+date[pos+2]);
//                             
//                 line_seconds =  (line[pos2-5]*10+line[pos2-4])*3600
//                             +(line[pos2-2]*10+line[pos2-1])*60
//                             +(line[pos2+1]*10+line[pos2+2]);
//                 
//                
//                 if(fabs(date_seconds - line_seconds + 0.0)<3)
//                     test = 1;
//             }
        }
        if (line.find("      x1 = x1 + x8 * -0.0723929") != string::npos)
            test2 = 1;
        if (line.find("      SUBROUTINE linearmodel_Fct_x1(x6, x8, x1)")
                != string::npos)
            test3 = 1;
        if (file.eof())
            bStop = kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1, test);
    CPPUNIT_ASSERT_EQUAL(1, test2);
    CPPUNIT_ASSERT_EQUAL(1, test3);
    delete tlin;
}

void TLinearRegressionTest::testgetInputName()
{

    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1, "x6:x8", "x1");
    tlin->unsetLog();
    CPPUNIT_ASSERT(!strcmp(tlin->getInputName(0), "x6"));
    CPPUNIT_ASSERT(!strcmp(tlin->getInputName(1), "x8"));
        delete tlin;
}

void TLinearRegressionTest::throwExceptionTLinearRegressionTdsNull()
{
    URANIE::DataServer::TDataServer *tdsnull = NULL;
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TLinearRegression(tdsnull, "x6:x8", "x1"),
            URANIE::Exceptions::UErrorExceptions);
}

void TLinearRegressionTest::throwExceptionTLinearRegressionOutputFalse()
{

    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TLinearRegression(tds1, "x6:x8", "toto"),
            URANIE::Exceptions::UErrorExceptions);

}
void TLinearRegressionTest::throwExceptionTLinearRegressionInputFalse()
{
    CPPUNIT_ASSERT_THROW(
            new URANIE::Modeler::TLinearRegression(tds1, "x6:toto:x8", "x1"),
            URANIE::Exceptions::UErrorExceptions);
}

void TLinearRegressionTest::throwExceptionestimateTdsMultiOutput()
{

    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1, "x6:x8", "x1:x2");
    tlin->unsetLog();
    CPPUNIT_ASSERT_THROW(tlin->estimate(),
            URANIE::Exceptions::UErrorExceptions);
        delete tlin;
}

void TLinearRegressionTest::throwExceptiongetInputNameIndiceSup()
{
    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1, "x6:x8", "x1");
    tlin->unsetLog();
    CPPUNIT_ASSERT_THROW(tlin->getInputName(-5),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(tlin->getInputName(10),
            URANIE::Exceptions::UErrorExceptions);
        delete tlin;
}

void TLinearRegressionTest::throwExceptionExportFunction()
{
    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1, "x6:x8", "x1");
    tlin->unsetLog();
    tlin->estimate();

    CPPUNIT_ASSERT_THROW(
            tlin->exportFunction("toto", "filename", "function_name"),
            URANIE::Exceptions::UErrorExceptions);
        delete tlin;
}

#ifdef  WITH_LIBXML2
void TLinearRegressionTest::testExportModelPMML()
{
    _tlr = new URANIE::Modeler::TLinearRegression(tds1, "x6:x8","x1");
    _tlr->unsetLog();
    _tlr->estimate();
    _tlr->exportModelPMML("model_tlr.pmml", "sLRName","new");

    Bool_t bStop=kFALSE;
    string line;
    Int_t test=0;
    fstream file("model_tlr.pmml");
    while(bStop==kFALSE)
    {
        getline(file,line);
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRName\" targetFieldName=\"x1\">")!=string::npos)
        test=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1,test);
    delete _tlr;
}

void TLinearRegressionTest::testExportModelPMMLBackUp()
{
    remove("model_tlr.pmml");
    URANIE::DataServer::TDataServer* tds3 = new URANIE::DataServer::TDataServer();
    tds3->fileDataRead("steamplant.dat");
    
    URANIE::DataServer::TDataServer* tds4 = new URANIE::DataServer::TDataServer();
    tds4->fileDataRead("steamplant.dat");
    
    URANIE::DataServer::TDataServer* tds5 = new URANIE::DataServer::TDataServer();
    tds5->fileDataRead("steamplant.dat");
    
    _tlr = new URANIE::Modeler::TLinearRegression(tds1, "x6:x8","x1");
    _tlr->unsetLog();
    _tlr->estimate();
    _tlr->exportModelPMML("model_tlr.pmml", "sLRName");

    Bool_t bStop;   
    string line;
    Int_t test1=0;
    Int_t test2=0;
    Int_t test3=0;
    Int_t test4=0;
    Int_t test5=0;
    
    bStop=kFALSE;
    fstream file1("model_tlr.pmml");
    while(bStop==kFALSE)
    {
        getline(file1,line);
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRName\" targetFieldName=\"x1\">")!=string::npos)
        test1=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(1,test1);
    file1.close();
    
    URANIE::Modeler::TLinearRegression* tlr2 = new URANIE::Modeler::TLinearRegression(tds1, "x5:x3","x2");
    tlr2->unsetLog();
    tlr2->estimate();
    tlr2->exportModelPMML("model_tlr.pmml", "sLRNameToto");
    bStop=kFALSE;
    fstream file2("model_tlr.pmml");
    while(bStop==kFALSE)
    {
        getline(file2,line);
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRName\" targetFieldName=\"x1\">")!=string::npos)
        test2+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRNameToto\" targetFieldName=\"x2\">")!=string::npos)
        test2+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(2,test2);
    file2.close();
    
    
    URANIE::Modeler::TLinearRegression* tlr3 = new URANIE::Modeler::TLinearRegression(tds3, "x5:x2","x1");
    tlr3->unsetLog();
    tlr3->estimate();
    tlr3->exportModelPMML("model_tlr.pmml", "sLRName");
    bStop=kFALSE;
    fstream file3("model_tlr.pmml");
    while(bStop==kFALSE)
    {
        getline(file3,line);
         if (line.find("  <RegressionModel functionName=\"regression\" targetFieldName=\"x1\" modelName=\"sLRName_0\">")!=string::npos)
        test3+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRName\" targetFieldName=\"x1\">")!=string::npos)
        test3+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRNameToto\" targetFieldName=\"x2\">")!=string::npos)
        test3+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(3,test3);
    file3.close();
    
    
    URANIE::Modeler::TLinearRegression* tlr4 = new URANIE::Modeler::TLinearRegression(tds4, "x3:x2","x1");
    tlr4->unsetLog();
    tlr4->estimate();
    tlr4->exportModelPMML("model_tlr.pmml", "sLRName");
    bStop=kFALSE;
    fstream file4("model_tlr.pmml");
    while(bStop==kFALSE)
    {
        getline(file4,line);
         if (line.find("  <RegressionModel functionName=\"regression\" targetFieldName=\"x1\" modelName=\"sLRName_0\">")!=string::npos)
        test4+=1;
         if (line.find("  <RegressionModel functionName=\"regression\" targetFieldName=\"x1\" modelName=\"sLRName_1\">")!=string::npos)
        test4+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRName\" targetFieldName=\"x1\">")!=string::npos)
        test4+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRNameToto\" targetFieldName=\"x2\">")!=string::npos)
        test4+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(4,test4);
    file4.close();
    
    
    
    URANIE::Modeler::TLinearRegression* tlr5 = new URANIE::Modeler::TLinearRegression(tds5, "x4:x2","x1");
    tlr5->unsetLog();
    tlr5->estimate();
    tlr5->exportModelPMML("model_tlr.pmml", "sLRName");
    bStop=kFALSE;
    fstream file5("model_tlr.pmml");
    while(bStop==kFALSE)
    {
        getline(file5,line);
         if (line.find("  <RegressionModel functionName=\"regression\" targetFieldName=\"x1\" modelName=\"sLRName_0\">")!=string::npos)
        test5+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" targetFieldName=\"x1\" modelName=\"sLRName_1\">")!=string::npos)
        test5+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" targetFieldName=\"x1\" modelName=\"sLRName_2\">")!=string::npos)
        test5+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRName\" targetFieldName=\"x1\">")!=string::npos)
        test5+=1;
        if (line.find("  <RegressionModel functionName=\"regression\" modelName=\"sLRNameToto\" targetFieldName=\"x2\">")!=string::npos)
        test5+=1;
        if (line.find("</PMML>")!=string::npos)
        bStop=kTRUE;
    }
    CPPUNIT_ASSERT_EQUAL(5,test5);
    file5.close();
    

    delete _tlr;
    delete tlr2;
    delete tlr3;
    delete tlr4;
    delete tlr5;
    
    delete tds3;
    delete tds4;
    delete tds5;
    
}

void TLinearRegressionTest::testLinearRegressionConstructorPmml()
{
    URANIE::Modeler::TLinearRegression *tlin =
            new URANIE::Modeler::TLinearRegression(tds1,1,"model_tlr.pmml","sLRName");
    tlin->estimate();

    TVectorD dparam = tlin->getParameters();

    CPPUNIT_ASSERT_EQUAL(3, dparam.GetNrows());

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.39616312988432, dparam(0),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.493699094271995, dparam(1),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.898479609533091, dparam(2),
            DBL_PRECISION_COMPARE);

    delete tlin;
}

#endif
