//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
/*!
 \file   TestTnisp.h
 \author Maud Salvoldi (OCC)
 \date   08/04/2011
 
 \brief Interface of the class TestTnisp
 
 */
#ifndef CPPUNITEST_TESTTNISP_H
#define CPPUNITEST_TESTTNISP_H

// cppunit
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// uranie
#include "TNisp.h"
#include "TDataServer.h"
#include "UExceptions.h"

/**

 * @class TestTNisp
 *
 * @brief Description of the class TestTNisp.
 *
 *  The class TestTNisp is the unit test class of the class TNisp. 
 *  
 *
 */

class TNispTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TNispTest);
  CPPUNIT_TEST(testgetNAttributes);
  CPPUNIT_TEST(testgetNPatterns);
  CPPUNIT_TEST(testgetLevel);
  CPPUNIT_TEST(testsetLevel);
  CPPUNIT_TEST(testGenerateSampleSmolyakClenshawCurtis);
  CPPUNIT_TEST(testGenerateSampleSmolyakGauss);
  CPPUNIT_TEST(testGenerateSampleSmolyakTrapeze);
  CPPUNIT_TEST(testGenerateSampleSmolyakFejer);
  CPPUNIT_TEST(testGenerateSampleMonteCarlo);
  CPPUNIT_TEST(testGenerateSampleLhs);
  CPPUNIT_TEST(testGenerateSampleQuadrature);
  CPPUNIT_TEST(testGenerateSamplePetras);
  CPPUNIT_TEST(testGenerateSampleQmcSobol);
  CPPUNIT_TEST(testTNispLawNormal);
  CPPUNIT_TEST(testTNispLawUniform);
  CPPUNIT_TEST(testTNispLawLogUniform);
  CPPUNIT_TEST(testTNispLawLogNormal);
  CPPUNIT_TEST(testTNispLawBeta);
  CPPUNIT_TEST(throwExceptionGenerateSampleMethod);  
  CPPUNIT_TEST(throwExceptionGenerateSampleLevel);
  CPPUNIT_TEST(throwExceptionGenerateSampleBadOption);
  CPPUNIT_TEST(testGenerateSampleOption);  
  CPPUNIT_TEST(throwExceptionTNispTdsNull);
  CPPUNIT_TEST(throwExceptionGenerateSampleNotEmpty);
  CPPUNIT_TEST(throwExceptionsetSampleEmpty);
  CPPUNIT_TEST(throwExceptionsetSampleBadOption);
  CPPUNIT_TEST(testsetSampleOption);
  CPPUNIT_TEST(testsetSampleOptionWithComputedOutputs);
  CPPUNIT_TEST(testsetSampleOptionWithImportedDatas);
  CPPUNIT_TEST(testsetSample);
  CPPUNIT_TEST(testsetSamplePetras);
  CPPUNIT_TEST(testsetSamplePetrasWithWeight);
  CPPUNIT_TEST(testSetSampleLhsAfterGenerationOfPetrasSample);
  CPPUNIT_TEST(throwExceptionTNispLawUnknown);
  CPPUNIT_TEST(throwExceptionTNispLawLogTriangular);
  CPPUNIT_TEST(throwExceptionTNispTriangular);
  CPPUNIT_TEST(throwExceptionTNispTdsNotAttribute);
  CPPUNIT_TEST(throwExceptionTNispSetLevelNegative);
  CPPUNIT_TEST(throwExceptionTNispReSetLevel);
  CPPUNIT_TEST(throwExceptionTNispSetSamplePetrasWithWeight);
  CPPUNIT_TEST(throwExceptionTNispSetSamplePetrasWithDifferentLeaf);
  CPPUNIT_TEST(throwExceptionTNispSetSamplePetrasWithDifferentLeaf2);
  CPPUNIT_TEST(testPetrasAndNispSampler);
  CPPUNIT_TEST(throwExceptionTNispNormalAttributeUnknownStochasticBasis);
  CPPUNIT_TEST(throwExceptionTNispUniformAttributeUnknownStochasticBasis);
  CPPUNIT_TEST(throwExceptionTNispBetaAttributeUnknownStochasticBasis);
  CPPUNIT_TEST(throwExceptionTNispLogNormalAttributeUnknownStochasticBasis);
  CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TestTNisp object.
     */
    TNispTest();
    /*! Destructor
     */
    virtual ~TNispTest();

    //---------------------------------------------
    /** @name setUp and Tear Down methods
     **/
    //@{
    //! Build a object TNisp from a tds object
    void setUp();
    //! ! Delete the object tds and nips
    void tearDown();
    //@}

    //! Unit test of the method TNisp::getAttributs()
    void testgetNAttributes();
    //! Unit test of the method TNisp::getNPatterns()  
    void testgetNPatterns();
    //! Unit test of the method TNisp::getLevel() 
    void testgetLevel();
    //! Unit test of the method TNisp::setLevel() 
    void testsetLevel();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "SmolyakClenshawCurtis" option
     \li level=3
     \li method=SmolyakClenshawCurtis
     */
    void testGenerateSampleSmolyakClenshawCurtis();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "SmolyakGauss" option
     \li level=3
     \li method=SmolyakGauss
     */
    void testGenerateSampleSmolyakGauss();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "SmolyakTrapeze" option
     \li level=3
     \li method=SmolyakTrapeze
     */
    void testGenerateSampleSmolyakTrapeze();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "SmolyakFejer" option
     \li level=3
     \li method=SmolyakFejer
     */
    void testGenerateSampleSmolyakFejer();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "Petras" option
     \li level=3
     \li method=Petras
     */
    void testGenerateSamplePetras();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "MonteCarlo" option
     \li level=2
     \li method= MonteCarlo
     */
    void testGenerateSampleMonteCarlo();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "Quadrature" option
     \li level=2
     \li method=Quatrature
     */
    void testGenerateSampleLhs();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "Quadrature" option
     \li level=2
     \li method=Quatrature
     */
    void testGenerateSampleQuadrature();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief This test control that the method run correclty with "QmcSobol" option
     \li level=512
     \li method=QmcSobol
     */
    void testGenerateSampleQmcSobol();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief Throw Exception is the parameter method is fault
     */
    void throwExceptionGenerateSampleMethod();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief Throw Exception is the parameter level is <=0
     */
    void throwExceptionGenerateSampleLevel();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief Throw Exception is the sample already exist
     */
    void throwExceptionGenerateSampleNotEmpty();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief Throw Exception is the parameter option is fault
     */
    void throwExceptionGenerateSampleBadOption();
    //! Unit test of the method TNisp::generateSample(level, method, *option)
    /*!
     \brief Test the save of the gvx
     */
    void testGenerateSampleOption();
    //! Unit test of the method TNisp::setSample() 
    /*! 
     \brief test the fucntionning of setSample()
     */
    void testsetSample();
    //! Unit test of the constructor
    /*!
     \brief Test the functionning of Normal law
     */
    void testTNispLawNormal();
    
    //! Unit test of the constructor
    /*!
     \brief Test the functionning of Uniform law
     */
    void testTNispLawUniform();
    
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief Attribute with LogUniform Law
     */
    void testTNispLawLogUniform();

    //! Unit test of the constructor
    /*!
     \brief Test the functionning of LogNormal law
     */
    void testTNispLawLogNormal();
    
    //! Unit test of the constructor
    /*!
     \brief Test the functionning of Beta law
     */
    void testTNispLawBeta();

    //! Unit test of the method setSample(method)
    /*!
     \brief Throw exception if the sample is empty
     */
    void throwExceptionsetSampleEmpty();
    //! Unit test of the method TNisp::setSample(method, *option)
    /*!
     \brief Throw Exception is the parameter option is fault
     */
    void throwExceptionsetSampleBadOption();
    //! Unit test of the method TNisp::setSample(method, *option)
    /*!
     \brief Test the save of the gvx
     */
    void testsetSampleOption();
    //! Unit test of the method TNisp::setSample(method, *option)
    /*!
     \brief Test the save of the gvx with output already computed and stored in the TDS.
     */
    void testsetSampleOptionWithComputedOutputs();
    //! Unit test of the method TNisp::setSample(method, *option)
    /*!
     \brief Test the save of the gvx with datas imported from a files.
     */
    void testsetSampleOptionWithImportedDatas();
    //! Unit test of the method TNisp::setSample(method, *option)
    /*!
     \brief test the fucntionning of setSample() with a TPetras sampling
     */   
    void testsetSamplePetras();
    //! Unit test of the method TNisp::setSample(method, *option)
    /*!
     \brief test the fucntionning of setSample() with a TPetras sampling with name
     of the weight attribute specified
     */
    void testsetSamplePetrasWithWeight();
    //! Unit test of the method TNisp::setSample(method, *option)
    /*!
     \brief test the fucntionning of setSample() with a Lhs sampling when a Petras samping has already been generated.
     */
    void testSetSampleLhsAfterGenerationOfPetrasSample();
    
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief tds is null
     */
    void throwExceptionTNispTdsNull();
    
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief Attribute with Gumbel Law
     */
    void throwExceptionTNispLawUnknown();
    
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief  Attribute with LogNormal Law
     */
    void throwExceptionTNispLawLogNormal();
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief Attribute with LogTriangular Law
     */
    void throwExceptionTNispLawLogTriangular();
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief  Attribute with Triangular Law
     */
    void throwExceptionTNispTriangular();
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief  No Attribute
     */
    void throwExceptionTNispTdsNotAttribute();
    //! Unit test of the method TNisp::setLevel
    /*!
     \brief  test the raise of exception when setting a negative level
     */
    void throwExceptionTNispSetLevelNegative();
     //! Unit test of the method TNisp::setLevel
    /*!
     \brief  test the raise of exception when resetting a level
     */
    void throwExceptionTNispReSetLevel();
    //! Unit test of the method TNisp::setSample
    /*!
     \brief  test the raise of exception the specified weight name doesn't correspond
     to an attribute of the tds
     */
    void throwExceptionTNispSetSamplePetrasWithWeight();
    //! Unit test of the method TNisp::setSample
    /*!
     \brief  test the raise of exception the specified weight name doesn't correspond
     to a leaf of the _datatree
     */
    void throwExceptionTNispSetSamplePetrasWithDifferentLeaf();
    //! Unit test of the method TNisp::setSample
    /*!
     \brief  test the raise of exception the default weight name doesn't correspond
     to a leaf of the _datatree
     */
    void throwExceptionTNispSetSamplePetrasWithDifferentLeaf2();
    
    //! Unit test of the method TNisp::generateSample
    /*!
     \brief  test that the sampling generated with TNisp is identical to the 
     sampling generated with TPetras.
     */
    void testPetrasAndNispSampler();
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief  Attribute with Normal Law and unknown stochastic basis
     */
    void throwExceptionTNispNormalAttributeUnknownStochasticBasis();
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief  Attribute with Uniform Law and unknown stochastic basis
     */
    void throwExceptionTNispUniformAttributeUnknownStochasticBasis();
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief  Attribute with Beta Law and unknown stochastic basis
     */
    void throwExceptionTNispBetaAttributeUnknownStochasticBasis();
    //! Unit test of the contructor TNisp::TNisp
    /*!
     \brief  Attribute with LogNormal Law and unknown stochastic basis
     */
    void throwExceptionTNispLogNormalAttributeUnknownStochasticBasis();


private:
    URANIE::DataServer::TDataServer *_tds;
    URANIE::DataServer::TDataServer *_tds2;
    URANIE::Modeler::TNisp *_nisp;

};

#endif  // CPPUNITEST_TESTTNISP_H
