//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTPolynomialRegression.h
 \author Estelle Deville <estelle.deville@cea.fr>
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::DataServer::TPolynomialRegression
 
 Cree le : jeu juin 30 15:34:06 2011
 
 */
#ifndef CPPUNITEST_TESTTPOLYNOMIALREGRESSION_H
#define CPPUNITEST_TESTTPOLYNOMIALREGRESSION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

#include "TPolynomialRegression.h"

/*! \class TPolynomialRegressionTest
 * \brief Unit tests for TPolynomialRegression.
 */
class TPolynomialRegressionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TPolynomialRegressionTest);

    CPPUNIT_TEST (testPolynomialRegressionPseudoConstituant);
    CPPUNIT_TEST (testPolynomialRegressionPseudoConstituantFewAttribute1);
    CPPUNIT_TEST (testPolynomialRegressionPseudoConstituantFewAttribute1Stop);
    CPPUNIT_TEST (testPolynomialRegressionPseudoConstituantFewAttribute2);
    CPPUNIT_TEST (testPolynomialRegressionPseudoConstituantFewAttribute3);
    CPPUNIT_TEST (testPolynomialRegressionPseudoConstituantFewAttribute3All);
    CPPUNIT_TEST (testPolynomialRegressionPseudoConstituantFewAttribute4);

    CPPUNIT_TEST (testsetMinimalMaximalOrder);
    CPPUNIT_TEST (testPolynomialRegressionArchi);
    CPPUNIT_TEST (testPolynomialRegressionR2A);
    CPPUNIT_TEST (testPolynomialRegressionR2);
    CPPUNIT_TEST (testExportFunction);

    CPPUNIT_TEST (testPolynomialRegressionPseudoConstituantExhaustive);
    CPPUNIT_TEST (testPolynomialRegressionTrace);

    CPPUNIT_TEST (throwExceptionPolynomialRegressionMaxOrder);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionMinOrder);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionCriteriaValue);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionCriteriaNegative);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionEstimateValue);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionNoPolynom);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionMaxOrderNegative);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionMaxOrderInfMinOrder);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionMinOrderSupMaxOrder);
    CPPUNIT_TEST (throwExceptionPolynomialRegressionMultiOutput);
    CPPUNIT_TEST (throwExceptionExportFunction);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor, Destructor, methods with the setUp and tearDown
     **/
    //@{
    //! Constructs a TPolynomialRegressionTest object.
    TPolynomialRegressionTest();
    //! Destructor.
    virtual ~TPolynomialRegressionTest();
    //! the setUp method
    void setUp();
    //! the tearDown method
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Compute the Polynomial Regression for the pseudo-constituant database for a Q2 criteria of 0.9. test the R2, R2Ajusted, Press and Q2 : 
    void testPolynomialRegressionPseudoConstituant();
    //! Compute the Polynomial Regression with few attributes for the pseudo-constituant database for a Q2 criteria of 0.9. test the R2, R2Ajusted, Press and Q2 : 
    void testPolynomialRegressionPseudoConstituantFewAttribute1();
    //! Compute the Polynomial Regression with few attributes for the pseudo-constituant database for a R2, and greater than the simple monomia :
    void testPolynomialRegressionPseudoConstituantFewAttribute1Stop();

    void testPolynomialRegressionPseudoConstituantFewAttribute2();
    void testPolynomialRegressionPseudoConstituantFewAttribute3();
    void testPolynomialRegressionPseudoConstituantFewAttribute3All();
    void testPolynomialRegressionPseudoConstituantFewAttribute4();

    //! Exhaustive Research
    void testPolynomialRegressionPseudoConstituantExhaustive();

    //! the trace
    void testPolynomialRegressionTrace();

    //! Compute the Polynomial Regression for the pseudo-constituant database for a Q2 criteria of 0.9. Use the second contructor
    void testPolynomialRegressionArchi();
    //! Compute the Polynomial Regression for the pseudo-constituant database for a Q2 criteria of 0.9, a minimal order of 1 and a maximal order of 2.
    void testsetMinimalMaximalOrder();
    //!! Compute the Polynomial Regression for the pseudo-constituant database for a R2 criteria of 0.9. test the R2, R2Ajusted, Press and Q2
    void testPolynomialRegressionR2();
    //!! Compute the Polynomial Regression for the pseudo-constituant database for a R2A criteria of 0.9. test the R2, R2Ajusted, Press and Q2
    void testPolynomialRegressionR2A();
    //! Export the data 
    void testExportFunction();
    //@}

    //---------------------------------------------
    /** @name Throw Exceptions Tests
     The list of test to throw an exception
     **/
    //@{
    //! verify the exception Maximal Order >2 :  
    void throwExceptionPolynomialRegressionMaxOrder();
    //! verify the exception Minimal Order <0: 
    void throwExceptionPolynomialRegressionMinOrder();
    //! verify the exception Maximal Order <0: 
    void throwExceptionPolynomialRegressionMaxOrderNegative();
    //! verify the exception MaxOrder < Minimal Order: 
    void throwExceptionPolynomialRegressionMaxOrderInfMinOrder();
    //! verify the exception Minimal Order> Max Order : 
    void throwExceptionPolynomialRegressionMinOrderSupMaxOrder();
    //! verify the exceptions for setCriteria greater than 1: 
    void throwExceptionPolynomialRegressionCriteriaValue();
    //! verify the exceptions for setCriteria negative: 
    void throwExceptionPolynomialRegressionCriteriaNegative();
    //! verify the exceptions for estimate (should be R2, Q2 or R2A): 
    void throwExceptionPolynomialRegressionEstimateValue();
    //! verify the exceptions for no polynom found for the wanted criteria 
    void throwExceptionPolynomialRegressionNoPolynom();
    //!verify the exception for multi output
    void throwExceptionPolynomialRegressionMultiOutput();
    //!verify the exception for false export file language
    void throwExceptionExportFunction();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds;
    TString _sX;
    TString _sY;
};

#endif  // CPPUNITEST_TESTTPOLYNOMIALREGRESSION_H
