//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TBestEstimate.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   thu jul 3 10:38:56 CEST 2007

 \brief Implementation of the class URANIE::Optimizer::TBestEstimate


 */
#include <iostream>
using std::cout;
using std::endl;

// ROOT
#include "TMatrixD.h"

// URANIE
#include "TBestEstimate.h"
#include "UExceptions.h"
#include "TAttribute.h"

using namespace URANIE::DataServer;

ClassImp (URANIE::Optimizer::TBestEstimate)

namespace URANIE
{
namespace Optimizer
{
// ----------------------------------------------------------------------
//  TBestEstimate constructors and destructors
// ----------------------------------------------------------------------

// Empty constructor
TBestEstimate::TBestEstimate()
{
    // Data sets
    _tdsParam = NULL;
    _tdsResp = NULL;
    _tdsMeas = NULL;
    _tdsOAT = NULL;

    // matrix dimensions
    _nParam = 0;
    _nResp = 0;
    _nParamTimeSteps = 0;
    _nRespTimeSteps = 0;

    // chi2
    _chi2 = 0.0;
}

// Constructor with only matrices
TBestEstimate::TBestEstimate(Int_t nParam, TMatrixD a, Int_t nResp, TMatrixD m,
        TMatrixD r, TMatrixD ca, TMatrixD cm, TMatrixD s, TMatrixD car)
{
    // Matrices
    _a.ResizeTo(a);
    _a = a;
    _m.ResizeTo(m);
    _m = m;
    _r.ResizeTo(r);
    _r = r;
    _ca.ResizeTo(ca);
    _ca = ca;
    _cm.ResizeTo(cm);
    _cm = cm;
    _s.ResizeTo(s);
    _s = s;

    // matrix dimensions
    _nParam = nParam;
    _nResp = nResp;
    _nParamTimeSteps = _a.GetNrows() / _nParam;
    _nRespTimeSteps = _r.GetNrows() / _nResp;

    if (car.NonZeros() > 0)
    {
        _car.ResizeTo(car);
        _car = car;
    }
    else
    {
        _car.ResizeTo(_nResp * _nRespTimeSteps, _nParam * _nParamTimeSteps);
        _car.Zero();
    }

    // chi2
    _chi2 = 0.0;

}

// Constructor with a user given sensitivity matrix
TBestEstimate::TBestEstimate(TDataServer *tdsParam, TDataServer *tdsMeasures,
        TDataServer *tdsResponses, TMatrixD sMatrix, TMatrixD carMatrix)
{
    UNUSED(tdsParam,tdsMeasures,tdsResponses,sMatrix,carMatrix);
    try
    {
       
            throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                    TString("TBestEstimate::TBestEstimate constructor\nThis constructor is not yet implemented"));
        
    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "This constructor is not yet implemented" << endl;
}

// Constructor with results of an OAT design instead of a sensitivity matrix
TBestEstimate::TBestEstimate(TDataServer *tdsParam, TDataServer *tdsMeasures,
        TDataServer *tdsResponses, TDataServer *tdsOATSampling,
        TMatrixD carMatrix)
{
    UNUSED(tdsParam,tdsMeasures,tdsResponses,tdsOATSampling,carMatrix);
    try
    {
       
            throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                    TString("TBestEstimate::TBestEstimate constructor\nThis constructor is not yet implemented"));
        
    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "This constructor is not yet implemented" << endl;
}

//! Default destructor
TBestEstimate::~TBestEstimate()
{
}

// ----------------------------------------------------------------------
//  TBestEstimate setters
// ----------------------------------------------------------------------
void TBestEstimate::setParameterData(TDataServer *tds)
{
    UNUSED(tds);
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::setParameterData method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "TBestEstimate::setParameterData is not yet implemented" << endl;
}

void TBestEstimate::setMeasureData(TDataServer *tds)
{
    UNUSED(tds);
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::setMeasureData method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
    
//     cout << "TBestEstimate::setMeasureData is not yet implemented" << endl;
}

void TBestEstimate::setResponseData(TDataServer *tds)
{
    UNUSED(tds);
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::setResponseData method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "TBestEstimate::setResponseData is not yet implemented" << endl;
}

void TBestEstimate::setOATSamplingData(TDataServer *tds)
{
    UNUSED(tds);
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::setOATSamplingData method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "TBestEstimate::setOATSamplingData is not yet implemented" << endl;
}

void TBestEstimate::setSensitivityMatrix(TMatrixD matrix)
{
    UNUSED(matrix);
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::setSensitivityMatrix method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "TBestEstimate::setSensitivityMatrix is not yet implemented"
//             << endl;
}

void TBestEstimate::setParameterResponseCovMatrix(TMatrixD matrix)
{
    UNUSED(matrix);
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::setParameterResponseCovMatrix method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout
//             << "TBestEstimate::setParameterResponseCovMatrix is not yet implemented"
//             << endl;
}

// ----------------------------------------------------------------------
//  TBestEstimate matrix filling
// ----------------------------------------------------------------------
void TBestEstimate::buildSensitivityMatrix()
{
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::buildSensitivityMatrix method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "TBestEstimate::buildSensitivityMatrix() is not yet implemented"
//             << endl;
}

void TBestEstimate::buildMatrices()
{
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::buildMatrices method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "TBestEstimate::buildMatrices() is not yet implemented" << endl;
}

// ----------------------------------------------------------------------
//  TBestEstimate::runBestEstimate
// ----------------------------------------------------------------------

void TBestEstimate::runBestEstimate()
{
    TMatrixD temp1, temp2, temp3;

    temp1.ResizeTo(_s.GetNcols(), _s.GetNrows());
    temp1.Zero();
    temp1.Transpose(_s);
    temp2.ResizeTo(_ca.GetNrows(), temp1.GetNcols());
    temp2.Zero();
    temp2 = _ca * temp1;
    _cr.ResizeTo(_s.GetNrows(), temp2.GetNcols());
    _cr.Zero();
    _cr = _s * temp2;

    TMatrixD AA;
    temp1.ResizeTo(_s.GetNcols(), _s.GetNrows());
    temp1.Zero();
    temp1.Transpose(_s);
    temp2.ResizeTo(_car.GetNrows(), temp1.GetNcols());
    temp2.Zero();
    temp2 = _car * temp1;
    AA.ResizeTo(temp2.GetNrows(), temp2.GetNcols());
    AA.Zero();
    AA = _cm - temp2;

    TMatrixD BB;
    temp1.ResizeTo(_s.GetNcols(), _s.GetNrows());
    temp1.Zero();
    temp1.Transpose(_s);
    temp2.ResizeTo(_ca.GetNrows(), temp1.GetNcols());
    temp2.Zero();
    temp2 = _ca * temp1;
    temp1.ResizeTo(_car.GetNcols(), _car.GetNrows());
    temp1.Zero();
    temp1.Transpose(_car);
    BB.ResizeTo(temp2.GetNrows(), temp2.GetNcols());
    BB.Zero();
    BB = temp2 - temp1;

    _d.ResizeTo(_r.GetNrows(), _r.GetNcols());
    _d.Zero();
    _d = _r - _m;

    temp1.ResizeTo(_s.GetNrows(), BB.GetNcols());
    temp1.Zero();
    temp1 = _s * BB;
    _cd.ResizeTo(AA.GetNrows(), AA.GetNcols());
    _cd.Zero();
    _cd = AA + temp1;

    TMatrixD cdm1;
    cdm1.ResizeTo(_cd.GetNrows(), _cd.GetNcols());
    cdm1.Zero();
    cdm1 = _cd;
    cdm1.Invert();

    temp1.ResizeTo(cdm1.GetNrows(), _d.GetNcols());
    temp1.Zero();
    temp1 = cdm1 * _d;
    temp2.ResizeTo(BB.GetNrows(), temp1.GetNcols());
    temp2.Zero();
    temp2 = BB * temp1;
    _aBE.ResizeTo(_a.GetNrows(), _a.GetNcols());
    _aBE.Zero();
    _aBE = _a - temp2;

    temp1.ResizeTo(BB.GetNcols(), BB.GetNrows());
    temp1.Zero();
    temp1.Transpose(BB);
    temp2.ResizeTo(cdm1.GetNrows(), temp1.GetNcols());
    temp2.Zero();
    temp2 = cdm1 * temp1;
    temp1.ResizeTo(BB.GetNrows(), temp2.GetNcols());
    temp1.Zero();
    temp1 = BB * temp2;
    _caBE.ResizeTo(_ca.GetNrows(), _ca.GetNcols());
    _caBE.Zero();
    _caBE = _ca - temp1;

    temp1.ResizeTo(cdm1.GetNrows(), _d.GetNcols());
    temp1.Zero();
    temp1 = cdm1 * _d;
    temp2.ResizeTo(AA.GetNrows(), temp1.GetNcols());
    temp2.Zero();
    temp2 = AA * temp1;
    _rBE.ResizeTo(_m.GetNrows(), _m.GetNcols());
    _rBE.Zero();
    _rBE = _m + temp2;

    temp1.ResizeTo(AA.GetNcols(), AA.GetNrows());
    temp1.Zero();
    temp1.Transpose(AA);
    temp2.ResizeTo(cdm1.GetNrows(), temp1.GetNcols());
    temp2.Zero();
    temp2 = cdm1 * temp1;
    temp1.ResizeTo(AA.GetNrows(), temp2.GetNcols());
    temp1.Zero();
    temp1 = AA * temp2;
    _crBE.ResizeTo(_cm.GetNrows(), _cm.GetNcols());
    _crBE.Zero();
    _crBE = _cm - temp1;

    TMatrixD chi2;
    temp1.ResizeTo(cdm1.GetNrows(), _d.GetNcols());
    temp1.Zero();
    temp1 = cdm1 * _d;
    temp2.ResizeTo(_d.GetNcols(), _d.GetNrows());
    temp2.Zero();
    temp2.Transpose(_d);
    chi2.ResizeTo(temp2.GetNrows(), temp1.GetNcols());
    chi2.Zero();
    chi2 = temp2 * temp1;
    _chi2 = chi2(0, 0);

    temp1.ResizeTo(BB.GetNcols(), BB.GetNrows());
    temp1.Zero();
    temp1.Transpose(BB);
    temp2.ResizeTo(cdm1.GetNrows(), temp1.GetNcols());
    temp2.Zero();
    temp2 = cdm1 * temp1;
    temp1.ResizeTo(AA.GetNrows(), temp2.GetNcols());
    temp1.Zero();
    temp1 = AA * temp2;
    _carBE.ResizeTo(_car.GetNrows(), _car.GetNcols());
    _carBE.Zero();
    _carBE = _car + temp1;

}

// ----------------------------------------------------------------------
//  TBestEstimate::printLog
// ----------------------------------------------------------------------
void TBestEstimate::printLog(Option_t *option)
{
    TString soption = option;
    soption.ToLower();
    try
    {
       
        throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                TString("TBestEstimate::printLog() method\nThis method is not yet implemented"));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
//     cout << "TBestEstimate::printLog() is not yet implemented" << endl;
}

} // Fin du namespace Optimizer
} // Fin du namespace URANIE

