//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TObjective.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   fri jul 6 13:48:06 CEST 2007
 
 \brief Implementation of the class URANIE::Optimizer::TObjective
 
 */
#include <iostream>
using std::cout;
using std::endl;

// ROOT
#include "TLeaf.h"

// URANIE
#include "TObjective.h"

ClassImp (URANIE::Optimizer::TObjective)

namespace URANIE
{
namespace Optimizer
{
TObjective::TObjective(TString name) :
  _dweight(1.0), _dweightold(1.0), _dvalue(0.0), _blog(kFALSE), _nshare(0), _tds(NULL), _outfile(NULL)
{
    if (_blog)
        cout << "Builds  TObjective [" << name << "]" << endl;
    SetNameTitle(name, Form("Objective function [%s]", name.Data()));
}

TObjective::~TObjective()
{
    if (_blog)
        cout << "detruit TObjective" << endl;
}

//////////////////////////////////////////////////////////
////  compute Objective
//////////////////////////////////////////////////////////

Double_t TObjective::getObjective(Double_t *dvalues, Int_t nrow)
{
    Int_t npat = (Int_t) _tds->getTuple()->GetEntries();

    if (npat != nrow)
    {
        _dvalue = 0.0;
        return 0.0;
    }
//     _tds->getTuple()->Scan("*");
//     Double_t *dvalues = new Double_t[nrow];
//     dvalues = yhat->GetMatrixArray();

//     yhat->Print(Form("ystar [%s]", _syhat.Data()));

    Double_t dres = 0;
    Double_t dvalstar, dvalhat, decart;

    TLeaf *leafstar = _tds->getTuple()->GetLeaf(_systar);
    for (Int_t ipat = 0; ipat < npat; ipat++)
    {
        _tds->getTuple()->GetEntry(ipat);
        dvalstar = leafstar->GetValue(0);
        dvalhat = dvalues[ipat];
        //decart = ( dvalstar - dvalhat ) / dvalstar ;

        decart = (dvalstar - dvalhat);
        dres += decart * decart;

        if (_blog)
        {
            cout << ipat << " - *[" << dvalstar << "] ^[" << dvalhat << "]";
            cout << " e[" << decart << "]";
            cout << " res[" << dres << "] " << endl;
        }

    }
    dres /= (Double_t)(npat);
    _dvalue = dres;
    if (_blog)
        cout << " ** _dvalue[" << _dvalue << "] _dweight[" << _dweight << "]"
                << endl;
    return _dweight * _dvalue;
}

void TObjective::printLog(Option_t *option)
{
    cout << "TObjective::printLog[" << option << "]" << endl;
    cout << "Name[" << GetName() << "] Title[" << GetTitle() << "]" << endl;
    cout << " _dweight[" << _dweight << "] _dweightold[" << _dweightold << "]"
            << endl;
    cout << " _dvalue[" << _dvalue << "] " << endl;
    if (_tds)
        cout << " Star [" << _systar << "] in TDS[" << _tds->GetName()
                << "] title[" << _tds->GetTitle() << "]" << endl;
    if (_outfile)
        cout << " Hat [" << _syhat << "] in OutputFile["
                << _outfile->getFileName() << "]" << endl;
}

} // Fin du namespace Optimizer
} // Fin du namespace URANIE

