//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TObjectiveUncertainty.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   fri jul 6 13:48:06 CEST 2007
 
 \brief Interface of class URANIE::Optimize::TObjectiveUncertainty
 
 */
/**
 
 * @class URANIE::Optimizer::TObjectiveUncertainty
 *
 * @brief Description of the class TObjectiveUncertainty.
 *
 * To be written by the developper.
 *
 */
#ifndef TOBJECTIVEUNCERTAINTY_H
#define TOBJECTIVEUNCERTAINTY_H

#include <iostream>
using namespace std;

//ROOT

// Uranie
#include "TObjective.h"

namespace URANIE
{
namespace Optimizer
{
class TObjectiveUncertainty: public TObjective
{
private:
    TString _systaruncert;                     ///< Name of uncertainty of ystar
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Constructor by name only
    /*! 
     
     \param name Name of the objective
     */
    TObjectiveUncertainty(TString name);
    //! Default destructor
    virtual ~TObjectiveUncertainty();
    //@}
    //---------------------------------------------
    /** @name Value of the Objective
     **/
    //@{
    Double_t getObjective(Double_t *dvalues, Int_t nrow);
    void setYStar(URANIE::DataServer::TDataServer *tds, TString ystar,
            TString ystaruncert)
    {
        TObjective::setYStar(tds, ystar);
        _systaruncert = ystaruncert;
    }
    
    //! return the value of the attribute _systaruncert
    TString getSigmaName()
    {
        return _systaruncert;
    }
    
    //@}
    

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Optimizer::TObjectiveUncertainty, ID_OPTIMIZER)
    //Classe de
};

} // Fin du namespace Optimizer
} // Fin du namespace URANIE
#endif
