//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TOptimizerOpt.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   thu jan 3 14:06:39 CET 2008
 
 \brief Implementation of the class URANIE::Optimizer::TOptimizerOpt
 
 */



//fga 2015/01/12 using namespace std;
// ROOT
//fga 2015/01/12 #include "Api.h"
#include "TSystem.h"

#ifdef WIN32
#pragma warning( disable : 4355 )
#endif

// Uranie
#include "TOptimizerOpt.h"
#include "UExceptions.h"

// Opt++
#include "NLP.h"
#include "NLF.h"
#include "OptNewton.h"
#include "OptNIPS.h"
#include "OptPDS.h"

#include "OptppExceptions.h"

#ifdef WIN32
#include <windows.h>
#else
#include <dlfcn.h>
#endif

using namespace OPTPP;

URANIE::Optimizer::TOptimizerOpt *gOptOptimizer;

ClassImp (URANIE::Optimizer::TOptimizerOpt)

void init_eqm(int ndim, NEWMAT::ColumnVector& x)
{
    cout << endl << endl << "*************************************************"
            << endl;

    cout << "****  init_eqm - Dim[" << ndim << "]" << endl;

    for (int i = 1; i <= ndim; i++)
    {
        //    x(i) = 10. * i / 100.;
        x(i) = -1.0 + 2.0 * rand() / (RAND_MAX + 1.0);
        cout << "** i[" << i << "] x[" << x(i) << "] **" << endl;
    }
    cout << endl << "*************************************************" << endl;
}
void update_model(int, int, NEWMAT::ColumnVector)
{
}

// // Rosenbrock's function
// void myRosen0(int n, const NEWMAT::ColumnVector& x, double& fx, int& result)
// {
//     double f1, f2, x1, x2;
// 
//     if (n != 2)
//         return;
// 
//     x1 = x(1);
//     x2 = x(2);
//     f1 = (x2 - x1 * x1);
//     f2 = 1. - x1;
// 
//     fx = 100. * f1 * f1 + f2 * f2;
//     result = OPTPP::NLPFunction;
// }

void UranieOptimizerOptInteractiveMethod0(int n, const NEWMAT::ColumnVector& x,
        double& fx, int& result)
{

//   cout << endl << "***************" << endl << "UranieOptimizerOptInteractiveMethod0" << endl;
//   cout << "** n[" << n << "]"<<endl;
//   cout << "** param[0]" << x[0] << endl;
//   cout << "** param[1]" << x[1] << endl;
    TMethodCall *m = gOptOptimizer->getMethodCall();
    if (!m)
        return;

    Double_t *valInp = new Double_t[n];
    Double_t *valOut = new Double_t[1];
    for (Int_t i = 0; i < n; i++)
        valInp[i] = x(i + 1);

    Long_t args[3];
    args[0] = (Long_t) n;
    args[1] = (Long_t) valInp;
    args[2] = (Long_t) valOut;

    m->SetParamPtrs(args, 3);
    Double_t dddResult;
    m->Execute(dddResult);

    fx = valOut[0];
    result = NLPFunction;
    gOptOptimizer->incrementIteration();
    cout << "** Iter[" << gOptOptimizer->getIteration() << "] -- fx[" << fx
            << "]" << endl;

    delete[] valInp;
    valInp = NULL;
    delete[] valOut;
    valOut = NULL;
}

//______________________________________________________________________________
void UranieOptimizerOptInteractiveMethod(int n, int mode,
        const NEWMAT::ColumnVector& x, real& fx, NEWMAT::ColumnVector& g,
        NEWMAT::SymmetricMatrix& H, int& result)
{

    UNUSED(x, fx, g, H, result);
    cout << endl << "***************" << endl
            << "UranieOptimizerOptInteractiveMethod" << endl;
//   cout << "param[0]" << x[0] << endl;
//   cout << "param[1]" << x[1] << endl;
    TMethodCall *m = gOptOptimizer->getMethodCall();
    if (!m)
        return;

    Long_t args[7];
    args[0] = (Long_t) n;
    args[1] = (Long_t) mode;
    args[2] = (Long_t)&x;
    args[3] = (Long_t)&fx;
    args[4] = (Long_t)&g;
    args[5] = (Long_t)&H;
    args[6] = (Long_t)&result;
//   m->SetParamPtrs(args, 7);
//   Double_t dddResult;
    //  m->Execute(dddResult);
}

//! Rosenbrock's function (n=2) with first and second order derivatives
/*!
 This function is with first and second derivates the mode parameter.

 */
// void myRosen2(int mode, int n, const NEWMAT::ColumnVector& x, double& fx,
//         NEWMAT::ColumnVector& g, NEWMAT::SymmetricMatrix& H, int &result)
// 
// {
// 
// //   cout << endl << endl <<"*************************************************"<<endl;
// 
// //   cout << "myRosen2 - Mode[" << mode << "]  n["<<n<<"]"<<endl;
// //   cout << "x[0] " << x[0] << endl;
// //   cout << "x[1] " << x[1] << endl;
// //   TMethodCall *m  = gOptOptimizer->getMethodCall();
// //   if (!m) return;
//     // cout << "x[1] " << x[1] << endl;
// 
//     Int_t ninp = n, nout = 1;
//     Double_t *valInp = new Double_t[ninp];
//     Double_t *valOut = new Double_t[nout];
// 
//     valInp[0] = x(0);
//     valInp[1] = x(1);
// //   valInp[0] = x(1);
// //   valInp[1] = x(2);
// 
// //   (*_fMethod)(valInp, valOut);
// 
// //   Long_t args[2];
// //   args[0] = (Long_t)valInp;
// //   args[1] = (Long_t)valOut;
// //   cout << "1 param[1]" << valInp[1] << endl;
// //   m->SetParamPtrs(args, 2);
// //   cout << "2 param[1]" << valInp[1] << endl;
// //   Double_t theResult;
// //   m->Execute(theResult);
// 
// //   cout << "result[[[[[[[[ " << theResult << " ]]]]]]]]" <<endl;
// 
//     double f1, f2, x1, x2;
// 
// //   if (n != 2) return;
// 
//     x1 = x(0);
//     x2 = x(1);
//     f1 = (x2 - x1 * x1);
//     f2 = 1. - x1;
// 
//     if (mode & NLPFunction)
//     {
//         fx = 100. * f1 * f1 + f2 * f2;
//         result = NLPFunction;
//     }
//     if (mode & NLPGradient)
//     {
//         g(1) = -400. * f1 * x1 - 2. * f2;
//         g(2) = 200. * f1;
//         result = NLPGradient;
//     }
// 
//     if (mode & NLPHessian)
//     {
// 
//         f1 = (x2 - 3.0 * x1 * x1);
//         H(1, 1) = -400.0 * f1 + 2.0;
//         H(2, 1) = -400.0 * x1;
//         H(2, 2) = 200.0;
//         result = NLPHessian;
//     }
// 
// //   cout << "  ** End of myrosen2" << endl;
// }

namespace URANIE
{
namespace Optimizer
{

TOptimizerOpt::TOptimizerOpt()
{
    cout << "constructs TOptimizerOpt" << endl;
}

TOptimizerOpt::TOptimizerOpt(URANIE::DataServer::TDataServer *tds,
        TString slibrary, TString sfunction, TString sinit) :
        _sFunctionName(sfunction), _sInitFunctionName(sinit), _niter(0)
{
    _sname = sfunction;

    TString spwd = gSystem->pwd();
    cout << "spwd[" << spwd << "]" << endl;
    _sLibrary = spwd;
    _sLibrary += TString("/");
    _sLibrary += slibrary;
    cout << "spwd[" << _sLibrary << "]" << endl;

    cout << "TOptimizerOpt::TOptimizerOpt with TDS and library[" << _sLibrary
            << "] function[" << _sFunctionName << "]" << endl;

    if (tds)
    {
        //     tds->printLog();
        _tds = tds;
    }

    run("avec DLL");

}
//     TOptimizerOpt::TOptimizerOpt (URANIE::DataServer::TDataServer *tds, void *function)
TOptimizerOpt::TOptimizerOpt(URANIE::DataServer::TDataServer *tds, const char *funcname) :
        _sLibrary(""), _niter(0)

{
//fga 2015/01/12     char *funcname = G__p2f2funcname(function);
    _sname = TString(funcname);

    cout << "TOptimizerOpt::TOptimizerOpt with TDS and function[" << _sname
            << "]" << endl;

    if (tds)
    {
//     tds->printLog();
        _tds = tds;
    }
    //OPTPP::USERFCN0 *_theFunction = function;

    //OPTPP::USERFCN0 *_theFunction = dynamic_cast<OPTPP::USERFCN0 *>(function);
    //OPTPP::USERFCN0 *_theFunction = function;
//       _fMethodOpt = (USERFCN2) function;
//fga 2015/01/12     _function = function;
//fga 2015/01/12     _MyOptFunction = (USERFCN2) function;
//fga 2015/01/12     _MyOptInitFunction = (OPTPPINITFCN) function;

    //run(_MyOptFunction);

//       if ( function != NULL ) {
//     _function = function;

    if (funcname)
    {
        _fMethodCall = new TMethodCall();
//       _fMethodCall->InitWithPrototype(funcname,"int, int, const NEWMAT::ColumnVector&, real&, NEWMAT::ColumnVector&, NEWMAT::SymmetricMatrix&, int&");
//       _fMethodCall->InitWithPrototype(funcname,"int, const NEWMAT::ColumnVector&, real&, int&");
        _fMethodCall->InitWithPrototype(funcname, "int,double*,double*");
    }
    _fMethodOpt = UranieOptimizerOptInteractiveMethod0;

    gOptOptimizer = this;

    Int_t ndim = _tds->getNAttributes();

    cout << "_tds->getNAttributes()[" << _tds->getNAttributes() << "]" << endl;
//     OPTPP::NLF2 *theUranieProblem = new OPTPP::NLF2(2, _fMethodOpt, init_eqm);

    OPTPP::NLF0 *theUranieProblem = new OPTPP::NLF0(ndim, _fMethodOpt,
            init_eqm);

    if (1)
    {
        theUranieProblem->initFcn();
        theUranieProblem->printState(
                Form("%s Problem Interpreted: Initial state ", _sname.Data()));
        theUranieProblem->eval();
        theUranieProblem->printState(
                Form("%s Problem Interpreted: Finished ", _sname.Data()));
    }

    cout << " **************************** en interpreter *********************"
            << endl;

    // M�thode d'optimisation: m�thode Newton avec r�gion de confiance 
    OPTPP::OptPDS *Optpds = new OPTPP::OptPDS(theUranieProblem);
    Optpds->setOutputFile(Form("_tst_opt++_%s_%d.out", _sname.Data(), ndim), 0);

    // Options Optimisation
    Optpds->setFcnTol(1e-6);
    Optpds->setMaxIter(3000);

    Optpds->optimize();
    Optpds->printStatus(Form("end of convergence"));

    //int ret_code = Optpds->getReturnCode();

    theUranieProblem->printState(
            Form("%s Problem : end of convergence ", _sname.Data()));

// //     run(myRosen2);
//     run(function);

//       } else {
//     _function = NULL;
//     _sname = "";
//       }

}

TOptimizerOpt::~TOptimizerOpt()
{
    cout << "destructs TOptimizerOpt" << endl;
}

// ----------------------------------------------------------------------
// Encapsulate a code
// ----------------------------------------------------------------------

void TOptimizerOpt::init(Option_t *option)
{
    cout << "TOptimizerOpt::init option[" << option << "]" << endl;

}
void TOptimizerOpt::optimize(Option_t *option)
{
    cout << "TOptimizerOpt::optimize option[" << option << "]" << endl;

    Int_t ndim = _tds->getNAttributes();

    clean();

    init();

    OPTPP::USERFCN2 userFcnFcn = 0;
//fga 2015/01/12     userFcnFcn = (OPTPP::USERFCN2) (_function);
    //    userFcnFcn = (OPTPP::USERFCN2) (fg);
    ////////////////////////  userFcnFcn = dynamic_cast<OPTPP::USERFCN2>(_function);

    if (userFcnFcn == 0)
    {
        cout << "Error : function " << _sname << " not known" << endl;
        exit(-1);
    }
    else
    {
        cout << "Function known .........." << _sname << endl;
    }
    OPTPP::NLF2 *theUranieProblem = new OPTPP::NLF2(ndim, userFcnFcn, init_eqm);
//     OPTPP::NLF2 *theUranieProblem = new OPTPP::NLF2(ndim, myRosen2, init_eqm);
    if (1)
    {
        theUranieProblem->initFcn();
        theUranieProblem->printState(
                Form("%s Problem : Initial state ", _sname.Data()));
        theUranieProblem->eval();
        theUranieProblem->printState(Form("%s Problem : Eval ", _sname.Data()));
    }
    theUranieProblem->printState(Form("%s Problem : Finished ", _sname.Data()));

    // M�thode d'optimisation: m�thode Newton avec r�gion de confiance 
    OPTPP::OptNewton *OptTrustRegion = new OPTPP::OptNewton(theUranieProblem,
            update_model);
    OptTrustRegion->setOutputFile(
            Form("_tst_opt++_%s_%d.out", _sname.Data(), ndim), 0);

    // Options Optimisation
    OptTrustRegion->setFcnTol(1e-4);
    OptTrustRegion->setMaxIter(1000);

    OptTrustRegion->optimize();

    //int ret_code = OptTrustRegion->getReturnCode();

    theUranieProblem->printState(
            Form("%s Problem : Terminal state ", _sname.Data()));

    delete theUranieProblem;
    theUranieProblem = NULL;
    delete OptTrustRegion;
    OptTrustRegion = NULL;

    terminate();

    clean();

}
void TOptimizerOpt::terminate(Option_t *option)
{
    cout << "TOptimizerOpt::terminate option[" << option << "]" << endl;

}
void TOptimizerOpt::clean(Option_t *option)
{
    cout << "TOptimizerOpt::clean option[" << option << "]" << endl;

}

void TOptimizerOpt::run(Option_t *option)
{
    TString soption = TString(option);
    soption.ToLower();

    if ( !soption.EqualTo("") )
        cout << " ****** run with option[" << option << "] *****" << endl;

//      if ( 1 ) {
    // load in the library function and cast appropriately before returning
#ifdef WIN32
	HMODULE crthandle;
    FARPROC crtfunction;
    DWORD error;

    // Open up the specified library, or look in the global table if 
    // no library has been specified 
    crthandle = LoadLibrary(_sLibrary.Data());

    // if something went wrong, print out the error
    if (!crthandle)
    {
        cerr << GetLastError() << endl;
        return;
    }
    GetLastError();
    // Get the specified Function
    crtfunction = GetProcAddress(crthandle, _sFunctionName.Data());
    if ((error = GetLastError()) != NULL)
    {
        cerr << "Error in read function " << error << endl;
        return;
    }

    FARPROC crtinitfunction;
    OPTPPINITFCN functionInit = init_eqm;

    if (_sInitFunctionName.Length() > 0)
    {
        GetLastError();
        // Get the specified init Function
        crtinitfunction = GetProcAddress(crthandle, _sInitFunctionName.Data());
        if ((error = GetLastError()) != NULL)
        {
            cerr << "Error in read init function " << error << endl;
            return;
        }
        functionInit = (OPTPPINITFCN) (crtinitfunction);
    }
#else
    void * crthandle;
    void* crtfunction;
    char * error;

    // Open up the specified library, or look in the global table if 
    // no library has been specified 
    crthandle = dlopen(_sLibrary.Data(), RTLD_LAZY);

    // if something went wrong, print out the error
    if (!crthandle)
    {
        cerr << dlerror() << endl;
        return;
    }
    dlerror();
    // Get the specified Function
    crtfunction = dlsym(crthandle, _sFunctionName.Data());
    if ((error = dlerror()) != NULL)
    {
        cerr << "Error in read function " << error << endl;
        return;
    }

    void* crtinitfunction;
    OPTPPINITFCN functionInit = init_eqm;

    if (_sInitFunctionName.Length() > 0)
    {
        dlerror();
        // Get the specified init Function
        crtinitfunction = dlsym(crthandle, _sInitFunctionName.Data());
        if ((error = dlerror()) != NULL)
        {
            cerr << "Error in read init function " << error << endl;
            return;
        }
        functionInit = (OPTPPINITFCN) (crtinitfunction);
    }
#endif
//      }

    Int_t ninp = _tds->getNAttributes();
    Int_t nout = 1;

    cout << "ninp[" << ninp << "]" << endl;

//       OPTPP::NLF2 *theUranieProblem = new OPTPP::NLF2(2, myRosen2, init_eqm);
//       OPTPP::NLF2 *theUranieProblem = new OPTPP::NLF2(2, myRosen2, _MyOptInitFunction);
//       OPTPP::NLF2 *theUranieProblem = new OPTPP::NLF2(2, _MyOptFunction, init_eqm);

    cout << "init " << endl;

    OPTPP::NLF2 *theUranieProblem = NULL;

    try
    {

        try
        {
            theUranieProblem = new OPTPP::NLF2(ninp, (USERFCN2) (crtfunction),
                    functionInit);
            //theUranieProblem = new OPTPP::NLF2(2, myRosen2, init_eqm);
        } catch (OPTPP::OptppExceptions& ue)
        {
            ue.print();
            exit(-1);
        }

        if (theUranieProblem == NULL)
            throw URANIE::Exceptions::UErrorExceptions(__FILE__, __LINE__,
                    Form(
                            "TOptimizerOpt::run EROOR: the pointer of the opt problem is null."));
    } catch (URANIE::Exceptions::UExceptions& ue)
    {
        ue.printMessage();
        if (ue.getMessageType() > URANIE::Exceptions::UExceptions::kWARNING)
            exit(-1);
    }

    cout << "end of init" << endl;
    if (1)
    {
        theUranieProblem->initFcn();
        theUranieProblem->printState(
                Form("%s Problem : Initial state ", _sname.Data()));
        theUranieProblem->eval();
        theUranieProblem->printState(Form("%s Problem : Fini ", _sname.Data()));
    }

    Double_t *valInp = new Double_t[ninp];
    Double_t *valOut = new Double_t[nout];

    NEWMAT::ColumnVector xCrt = theUranieProblem->getXc();

    valInp[0] = xCrt(1);
    valInp[1] = xCrt(2);

    Int_t ndim = ninp;
    cout << " *** ndim[" << ndim << "]" << endl;

    // M�thode d'optimisation: m�thode Newton avec r�gion de confiance 
    OPTPP::OptNIPS *OptTrustRegion = new OPTPP::OptNIPS(theUranieProblem,
            update_model);
    OptTrustRegion->setOutputFile(
            Form("_tst_opt++_%s_%d.out", _sname.Data(), ndim), 0);

    // Options Optimisation
    OptTrustRegion->setFcnTol(1.0e-06);
    OptTrustRegion->setMaxIter(150);
//     OptTrustRegion->setSearchStrategy(LineSearch);
//     OptTrustRegion->setMeritFcn(ArgaezTapia);

    OptTrustRegion->optimize();
    OptTrustRegion->printStatus(Form("Fin de convegrence"));

    //int ret_code = OptTrustRegion->getReturnCode();

    theUranieProblem->printState(
            Form("%s Problem : Terminal state ", _sname.Data()));
    
    delete[] valInp;
    delete[] valOut;
    delete OptTrustRegion;
    delete theUranieProblem;

}

// ----------------------------------------------------------------------
// Print Log
// ----------------------------------------------------------------------
void TOptimizerOpt::printLog(Option_t *option)
{
    cout << "TOptimizerOpt::printLog[" << option << "]" << endl;
}

} // Fin du namespace Optimizer
} // Fin du namespace URANIE

