//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TMULTIGENANALYTICALFUNCTION_H
#define CPPUNITEST_TMULTIGENANALYTICALFUNCTION_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TMultiGenAnalyticalFunction.h"
#include "TDataServer.h"
#include "TLauncher.h"
#include "TAttribute.h"
#include "UExceptions.h"

class TMultiGenAnalyticalFunctionTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TMultiGenAnalyticalFunctionTest);

    CPPUNIT_TEST (testTMultiGenAnalytiqueFunction);
    CPPUNIT_TEST (testTMultiGenAnalytiqueFunctionClone);
    CPPUNIT_TEST (testTMultiGenAnalyticalFunctionSetLog);
    CPPUNIT_TEST (throwExceptionTMultiGenAnalyticalFunctionConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTMultiGenAnalyticalFunctionConstructorFalseInput);
    CPPUNIT_TEST (throwExceptionTMultiGenAnalyticalFunctionConstructorFalseOutput);
    CPPUNIT_TEST (throwExceptionTMultiGenAnalyticalFunctionConstructorFalseCost);
    CPPUNIT_TEST (throwExceptionTMultiGenAnalyticalFunctionConstructorUnknowFunction);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TMultiGenAnalyticalFunctionTest();

    //! Default destructor
    virtual ~TMultiGenAnalyticalFunctionTest();

    void setUp();
    void tearDown();

    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    void testTMultiGenAnalytiqueFunction();
    void testTMultiGenAnalytiqueFunctionClone();
    void testTMultiGenAnalyticalFunctionSetLog();
    void throwExceptionTMultiGenAnalyticalFunctionConstructorTdsNull();
    void throwExceptionTMultiGenAnalyticalFunctionConstructorFalseInput();
    void throwExceptionTMultiGenAnalyticalFunctionConstructorFalseOutput();
    void throwExceptionTMultiGenAnalyticalFunctionConstructorFalseCost();
    void throwExceptionTMultiGenAnalyticalFunctionConstructorUnknowFunction();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds;
    TString _workingTestPath;
    void * _p2fRosenbrock;    ///< Pointer to the Rosenbrock Analytical function
    void * _p2fUnknown;          ///< Pointer to the unknown Analytical function
};

#endif  // CPPUNITEST_TMULTIGENANALYTICALFUNCTION_H
