//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTestTMultiGenCode.cxx
 \author Maud Salvoldi (OCC/Areva group)
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Optimizer::TestTMultiGenCode
 
 Cree le : jeu d�c 22 11:27:34 CET 2011
 
 */

#define DBL_PRECISION_COMPARE 1.e-12
#define EPSILON 0.01 

// Uranie
#include "TestTMultiGenCode.h"
#include "UDisableTest.h"

// ROOT
#include "TString.h"
#include "TROOT.h"
#include "TSystem.h" 
#include <iostream>
using namespace std;

TMultiGenCodeTest::TMultiGenCodeTest() :
        _tds(NULL), fOutput(NULL), mycode(NULL)
{

    _workingTestPath = gSystem->pwd();

}

TMultiGenCodeTest::~TMultiGenCodeTest()
{

}

void TMultiGenCodeTest::setUp()
{
    gSystem->cd(_workingTestPath);

    _tds = new URANIE::DataServer::TDataServer("testTMultiGenCode",
            "test of classes TMultiGenCode");
    URANIE::DataServer::TAttribute *x = new URANIE::DataServer::TAttribute("x",
            -1.5, 1.5);
    x->setDefaultValue(-1.2);
    x->setStepValue(0.1);
    x->setFileKey("toto.in", "a", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);

    URANIE::DataServer::TAttribute *y = new URANIE::DataServer::TAttribute("y",
            -1.5, 1.5);
    y->setDefaultValue(1.);
    y->setStepValue(0.1);
    y->setFileKey("toto.in", "b", "",
            URANIE::DataServer::TAttributeFileKey::kNewRow);

    _tds->addAttribute(x);
    _tds->addAttribute(y);
    _tds->getAttribute("x")->delShare();
    _tds->getAttribute("y")->delShare();
    
    fOutput = new URANIE::Launcher::TOutputFileDataServer(
            "_output_rosenbrock_with_rows_.dat");
    fOutput->addAttribute("fval");

    mycode = new URANIE::Launcher::TCode(_tds, "ls -al >> /dev/null");
    mycode->addOutputFile(fOutput);
}

void TMultiGenCodeTest::tearDown()
{
    delete mycode;
    mycode = NULL;

    delete fOutput;
    fOutput = NULL;

    delete _tds;
    _tds = NULL;

}
////////////////////////////////
///                          ///                            
///     test                 ///
///                          ///
////////////////////////////////
void TMultiGenCodeTest::testTMultiGenCodeConstructorClone()
{
    URANIE::DataServer::TAttribute *att = mycode->getAttribute(0, 0);
    _tds->addAttribute(att);

    URANIE::Optimizer::TMultiGenCode *topt =
            new URANIE::Optimizer::TMultiGenCode(_tds, mycode, "fval");
    topt->unsetLog();
    URANIE::Optimizer::TMultiGenCode *topt1 = topt->Clone();
    topt1->unsetLog();

    CPPUNIT_ASSERT_EQUAL(topt1->NDim(), topt->NDim());

    delete topt;
    topt = NULL;
    delete topt1;
    topt1 = NULL;

}

void TMultiGenCodeTest::testTMultiGenCodeConstructorNDim()
{
    URANIE::DataServer::TAttribute *att = mycode->getAttribute(0, 0);
    _tds->addAttribute(att);

    URANIE::Optimizer::TMultiGenCode *topt =
            new URANIE::Optimizer::TMultiGenCode(_tds, mycode, "fval");
    topt->unsetLog();        
    Int_t nint = topt->NDim();

    CPPUNIT_ASSERT_EQUAL(2, nint);

    delete topt;
    topt = NULL;
}

void TMultiGenCodeTest::testTMultiGenCodeSetLog()
{
    URANIE::DataServer::TAttribute *att = mycode->getAttribute(0, 0);
    _tds->addAttribute(att);

    URANIE::Optimizer::TMultiGenCode *topt =
            new URANIE::Optimizer::TMultiGenCode(_tds, mycode, "fval");
    topt->unsetLog();        
    CPPUNIT_ASSERT(!topt->getLog());
    
    topt->setLog();    
    CPPUNIT_ASSERT(topt->getLog());
    
    topt->changeLog();    
    CPPUNIT_ASSERT(!topt->getLog());
    
    topt->changeLog();    
    CPPUNIT_ASSERT(topt->getLog());

    delete topt;
    topt = NULL;
    
}

////////////////////////////////
///                          ///                            
///     throwException       ///
///                          ///
////////////////////////////////
void TMultiGenCodeTest::throwExceptionTMultiGenCodeConstructorTdsNull()
{
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenCode(NULL, mycode, "fval"),
            URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenCode(_tds, NULL, "fval"),
            URANIE::Exceptions::UErrorExceptions);
}

void TMultiGenCodeTest::throwExceptionTMultiGenCodeConstructorNoOutput()
{
    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenCode(_tds, mycode, "fval"),
            URANIE::Exceptions::UErrorExceptions);
}

void TMultiGenCodeTest::throwExceptionTMultiGenCodeConstructorFalseScost()
{
    URANIE::DataServer::TAttribute *att = mycode->getAttribute(0, 0);
    _tds->addAttribute(att);

    CPPUNIT_ASSERT_THROW(
            new URANIE::Optimizer::TMultiGenCode(_tds, mycode, "toto"),
            URANIE::Exceptions::UErrorExceptions);
}
