//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTObjective.cxx
 \author Maud Salvoldi (OCC/Areva group)
 \date   $Date$
 
 \brief  Fichier de test de la classe URANIE::Optimizer::TObjective
 
 Cree le : jeu d�c 22 11:27:34 CET 2011
 
 */

#define DBL_PRECISION_COMPARE 1.e-12
#define EPSILON 0.01 

// Uranie
#include "TestTObjective.h"
#include "TOutputFileKey.h"
#include "TOutputFileRow.h"
#include "TOutputFileDataServer.h"
#include "UDisableTest.h"

// ROOT
#include "TString.h"
#include "TROOT.h"
#include "TSystem.h" 
#include <iostream>
using namespace std;

TObjectiveTest::TObjectiveTest()
{

}

TObjectiveTest::~TObjectiveTest()
{

}

void TObjectiveTest::setUp()
{

}

void TObjectiveTest::tearDown()
{

}
////////////////////////////////
///                          ///                            
///     test                 ///
///                          ///
////////////////////////////////
void TObjectiveTest::testTObjectiveConstructorObjective()
{
    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective(
            "jjjj");
    tobj->unsetLog();

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, tobj->getObjective(),
            DBL_PRECISION_COMPARE);
    CPPUNIT_ASSERT_EQUAL(kTRUE, tobj->isActive());

    delete tobj;
    tobj = NULL;
}
void TObjectiveTest::testTObjectiveConstructorObjectiveL1()
{
    URANIE::Optimizer::TObjectiveL1 *tobj = new URANIE::Optimizer::TObjectiveL1(
            "jjjj");
    tobj->unsetLog();
    
    CPPUNIT_ASSERT_EQUAL(kTRUE, tobj->isActive());

    delete tobj;
    tobj = NULL;
}
void TObjectiveTest::testTObjectiveConstructorObjectiveUncertainty()
{
    URANIE::Optimizer::TObjectiveUncertainty *tobj =
            new URANIE::Optimizer::TObjectiveUncertainty("jjjj");
    tobj->unsetLog();
    
    CPPUNIT_ASSERT_EQUAL(kTRUE, tobj->isActive());

    delete tobj;
    tobj = NULL;
}

void TObjectiveTest::testTObjectivesetWeight()
{

    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective(
            "jjjj");
    tobj->unsetLog();
    tobj->setWeight(1.25);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.25, tobj->getWeight(),
            DBL_PRECISION_COMPARE);

    delete tobj;
    tobj = NULL;
}
void TObjectiveTest::testTObjectiveunActive()
{
    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective(
            "jjjj");
    tobj->unsetLog();
    tobj->setWeight(1.25);

    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.25, tobj->getWeight(),
            DBL_PRECISION_COMPARE);
    tobj->unActive();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, tobj->getWeight(), DBL_PRECISION_COMPARE);

    delete tobj;
    tobj = NULL;
}
void TObjectiveTest::testTObjectiveactive()
{
    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective(
            "jjjj");
    tobj->unsetLog();
    tobj->setWeight(1.25);
    tobj->unActive();

    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0, tobj->getWeight(), DBL_PRECISION_COMPARE);
    tobj->active();
    CPPUNIT_ASSERT_DOUBLES_EQUAL(1.25, tobj->getWeight(),
            DBL_PRECISION_COMPARE);

    delete tobj;
    tobj = NULL;
}

void TObjectiveTest::testTObjectiveGetObjective()
{
    // The output file of the code where values are stored in (key = value) format
    URANIE::Launcher::TOutputFileKey *fOutputFile = new URANIE::Launcher::TOutputFileKey("_output_rosenbrock_with_keys_.dat");
    fOutputFile->addAttribute("fval");
    

    URANIE::DataServer::TDataServer *tdsref = new URANIE::DataServer::TDataServer("TDSRef", "Objectives values");
    tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
    
    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective("obj1");
    tobj->unsetLog();
    tobj->setWeight(1.0);
    tobj->setYStar(tdsref, "fstar");
    tobj->setYHat(fOutputFile, "fval") ;
    
    Double_t dvalues[10];
    dvalues[0] = 6.301019000e+00;
    dvalues[1] = 7.039946000e+01;
    dvalues[2] = 2.608487000e+01;
    dvalues[3] = 4.678512000e+00;
    dvalues[4] = 2.201640000e+00;
    dvalues[5] = 2.162794000e+01;
    dvalues[6] = 1.334784000e+01;
    dvalues[7] = 0.0;//1.367355000e+00;
    dvalues[8] = 9.676354000e+00;
    dvalues[9] = 4.532999000e+01;
    
    
    Double_t res = tobj->getObjective(dvalues,10);
    

    Double_t ref = 1.367355000e+00*1.367355000e+0/10;
    CPPUNIT_ASSERT_DOUBLES_EQUAL(ref,res,DBL_PRECISION_COMPARE);
    
    
    // case nrow > nvalue (=10)
    res = tobj->getObjective(dvalues,12);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,res,DBL_PRECISION_COMPARE);
    
    delete tobj;
    //    fOutputFile->getAttribute(0)->delShare();
    delete fOutputFile;
    delete tdsref;
}

void TObjectiveTest::testTObjectiveUncertaintyGetObjective()
{
    // The output file of the code where values are stored in (key = value) format
    URANIE::Launcher::TOutputFileKey *fOutputFile = new URANIE::Launcher::TOutputFileKey("_output_rosenbrock_with_keys_.dat");
    fOutputFile->addAttribute("fval");
    

    URANIE::DataServer::TDataServer *tdsref = new URANIE::DataServer::TDataServer("TDSRef", "Objectives values");
    tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
    
    URANIE::Optimizer::TObjectiveUncertainty *tobju = new URANIE::Optimizer::TObjectiveUncertainty("obj1");
    tobju->unsetLog();
    tobju->setWeight(1.0);
    tobju->setYStar(tdsref, "fstar","w");
    tobju->setYHat(fOutputFile, "fval") ;
    
    Double_t dvalues[10];
    dvalues[0] = 6.301019000e+00;
    dvalues[1] = 7.039946000e+01;
    dvalues[2] = 2.608487000e+01;
    dvalues[3] = 4.678512000e+00;
    dvalues[4] = 2.201640000e+00;
    dvalues[5] = 2.162794000e+01;
    dvalues[6] = 1.334784000e+01;
    dvalues[7] = 0.0;//1.367355000e+00;
    dvalues[8] = 9.676354000e+00;
    dvalues[9] = 4.532999000e+01;
    
    
    Double_t res = tobju->getObjective(dvalues,10);
    

    Double_t ref = 1.367355000e+00*1.367355000e+0/0.15;
    CPPUNIT_ASSERT_DOUBLES_EQUAL(ref,res,DBL_PRECISION_COMPARE);
    
    // case nrow > nvalue (=10)
    res = tobju->getObjective(dvalues,12);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,res,DBL_PRECISION_COMPARE);
    
    delete tobju;
    //    fOutputFile->getAttribute(0)->delShare();
    delete fOutputFile;
    delete tdsref;
}

void TObjectiveTest::testTObjectiveL1GetObjective()
{
    // The output file of the code where values are stored in (key = value) format
    URANIE::Launcher::TOutputFileKey *fOutputFile = new URANIE::Launcher::TOutputFileKey("_output_rosenbrock_with_keys_.dat");
    fOutputFile->addAttribute("fval");
    

    URANIE::DataServer::TDataServer *tdsref = new URANIE::DataServer::TDataServer("TDSRef", "Objectives values");
    tdsref->fileDataRead("tds_rosenbrock_sampling_.dat");
    
    
    URANIE::Optimizer::TObjectiveL1 *tobjL1 = new URANIE::Optimizer::TObjectiveL1("obj1");
    tobjL1->unsetLog();
    tobjL1->setWeight(1.0);
    tobjL1->setYStar(tdsref, "fstar");
    tobjL1->setYHat(fOutputFile, "fval") ;
    
    Double_t dvalues[10];
    dvalues[0] = 6.301019000e+00;
    dvalues[1] = 7.039946000e+01;
    dvalues[2] = 2.608487000e+01;
    dvalues[3] = 4.678512000e+00;
    dvalues[4] = 2.201640000e+00;
    dvalues[5] = 2.162794000e+01;
    dvalues[6] = 1.334784000e+01;
    dvalues[7] = 0.0;//1.367355000e+00;
    dvalues[8] = 9.676354000e+00;
    dvalues[9] = 4.532999000e+01;
    
    
    Double_t res = tobjL1->getObjective(dvalues,10);
    

    Double_t ref = 1.367355000e+00;
    CPPUNIT_ASSERT_DOUBLES_EQUAL(ref,res,DBL_PRECISION_COMPARE);
    
    // case nrow > nvalue (=10)
    res = tobjL1->getObjective(dvalues,12);
    CPPUNIT_ASSERT_DOUBLES_EQUAL(0.0,res,DBL_PRECISION_COMPARE);
    
    delete tobjL1;
    ///    fOutputFile->getAttribute(0)->delShare();
    delete fOutputFile;
    delete tdsref;
}

void TObjectiveTest::testTObjectiveSetLog()
{
    URANIE::Optimizer::TObjective *tobj = new URANIE::Optimizer::TObjective(
            "jjjj");
    tobj->unsetLog();
    CPPUNIT_ASSERT(!tobj->getLog());
    
    tobj->setLog();    
    CPPUNIT_ASSERT(tobj->getLog());
    
    tobj->changeLog();    
    CPPUNIT_ASSERT(!tobj->getLog());
    
    tobj->changeLog();    
    CPPUNIT_ASSERT(tobj->getLog());
    
    
    delete tobj;
    tobj = NULL;
    
}










