//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   vizir.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2003-12-11.16.08.18

 \brief fonction interface pour vizir
 */

#ifndef VIZIR
#define VIZIR 1

#define OBSOLETE

#include "vzr_struct.h"

#ifdef WIN32
#include "libVizir_EXPORT.h"
#else
#define libVizir_EXPORT
#endif

typedef libVizir_EXPORT struct vizir vzr_t;
typedef libVizir_EXPORT struct vzrfitness vzrfitness_t;
typedef libVizir_EXPORT struct vzrfitnessbuild vzrfitnessbuild_t;
typedef libVizir_EXPORT struct vzrcrossover vzrcrossover_t;
typedef libVizir_EXPORT struct vzrcrossoverbuild vzrcrossoverbuild_t;
typedef libVizir_EXPORT struct vzrevent vzrevent_t;

/*******    DECLARATION DES FONCTIONS   *************/
#ifdef __cplusplus
extern "C"
{
#endif

/* init */
extern libVizir_EXPORT vzr_t *vizir1_vzr_multi(int, int);
extern libVizir_EXPORT vzr_t *vizir1_vzr_mono(int, double);
extern libVizir_EXPORT vzr_t *vizir1_vzr_monoConstraint(int, double, int);
extern libVizir_EXPORT vzr_t *vizir1_vzr_multiConstraint(int, int, int);
extern libVizir_EXPORT vzr_t *vizir1_vzr_init(int, int, int, int, double);

extern void vizir1_vzr_setLabel(vzr_t *, char *);
extern libVizir_EXPORT void vizir1_vzr_setSurvivalRate(vzr_t *, double);
extern libVizir_EXPORT void vizir1_vzr_setHomozygoteRate(vzr_t *, double);
extern libVizir_EXPORT void vizir1_vzr_setMutationRate(vzr_t *, double);
extern void vizir1_vzr_setParentsBias(vzr_t *, double);
extern void vizir1_vzr_setOutlierFactor(vzr_t *, double);

extern void vizir1_vzr_setOneGeneMutationRate(vzr_t *, double);
extern void vizir1_vzr_setCombinatorialSize(vzr_t *, int);

extern vzrfitnessbuild_t *vizir1_vzrfunstd_create(int (*)(vzrgene_t*, double*, void *),
        void *);

extern libVizir_EXPORT void vizir1_vzr_setFitness(vzr_t *, vzrfitnessbuild_t *);
extern void vizir1_vzr_setCrossover(vzr_t *, vzrcrossoverbuild_t *);

extern libVizir_EXPORT int vizir1_vzr_build(vzr_t *, int, int);

/* run */
extern libVizir_EXPORT void vizir1_vzr_run(vzr_t *);
extern void vizir1_vzr_start(vzr_t *);
extern void vizir1_vzr_stop(vzr_t *);
extern void vizir1_vzr_wait(vzr_t *);
extern int vizir1_vzr_running(vzr_t *);
extern libVizir_EXPORT void vizir1_vzr_reevaluate(vzr_t *);

/* result */
extern vzrgene_t **vizir1_vzr_genotype(vzr_t *, int);
extern libVizir_EXPORT vzrgene_t *vizir1_vzr_phenotype(vzr_t *, int);
extern libVizir_EXPORT double *vizir1_vzr_objective(vzr_t *, int);
extern double vizir1_vzr_rank(vzr_t *, int);
extern double vzr_constraintRank(vzr_t *, int);
extern double vzr_objectiveRank(vzr_t *, int);

extern int vizir1_vzr_birthCount(vzr_t *);
extern libVizir_EXPORT int vizir1_vzr_generationCount(vzr_t *);
extern int vizir1_vzr_survivorSize(vzr_t *);
extern int vizir1_vzr_populationSize(vzr_t *);
extern int vizir1_vzr_emigrationCount(vzr_t *);
extern int vizir1_vzr_immigrationCount(vzr_t *);

/* distribution */
extern int vzr_emigrate(vzr_t *, vzrgene_t **);
extern int vzr_immigrate(vzr_t *, vzrgene_t **);

extern void vzr_emptyStack(vzr_t *);
extern void vizir1_vzr_reset(vzr_t *);

/* destruction */
extern libVizir_EXPORT void vizir1_vzr_dealloc(vzr_t *);

#ifdef OBSOLETE
/* init */
extern vzr_t libVizir_EXPORT *vzr_mono_contraint(int, double, int);
extern vzr_t *vzr_multi_contraint(int, int, int);

extern void vzr_setTauxSurvie(vzr_t *, double);
extern void vzr_setHomozigotProb(vzr_t *, double);
extern void vzr_setMutationProb(vzr_t *, double);
extern void vzr_setNaissanceBiais(vzr_t *, double);
extern void vzr_setOutlierFacteur(vzr_t *, double);

extern void vzr_setOneGeneMutationProb(vzr_t *, double);
extern void vzr_setNombreGeneDiscret(vzr_t *, int);

extern vzrfitnessbuild_t *vzrfunstd_classe(int (*)(vzrgene_t*, double*, void *),
        void *);

extern void vzr_setFonctionEvaluation(vzr_t *, vzrfitnessbuild_t *);
extern void vzr_setOperateurCroisement(vzr_t *, vzrcrossoverbuild_t *);

extern int vzr_setTaille(vzr_t *, int, int);

/* run */
extern void vzr_evoluer(vzr_t *);
extern int vzr_isAlive(vzr_t *);

/* result */
extern vzrgene_t **vzr_getGenotype(vzr_t *, int);
extern vzrgene_t *vzr_getPhenotype(vzr_t *, int);
extern double *vzr_getCritere(vzr_t *, int);
extern double vzr_getRang(vzr_t *, int);
extern double vzr_getRangSoft(vzr_t *, int);
extern double vzr_getRangFun(vzr_t *, int);

extern int vzr_getNombreEvaluation(vzr_t *);
extern int vzr_getNombreGeneration(vzr_t *);
extern int vzr_getNombreSurvivant(vzr_t *);
extern int vzr_getTaillePopulation(vzr_t *);
extern int vzr_getNombreEmigre(vzr_t *);
extern int vzr_getNombreImigre(vzr_t *);

/* distribution */
extern int vzr_getMigrant(vzr_t *, vzrgene_t **);
extern int vzr_putMigrant(vzr_t *, vzrgene_t **);
#endif // OBSOLETE

#ifdef __cplusplus
} /* end of extern "C" */
#endif

#endif

