//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   vzr_multi.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2008-04-03.10.35.26

 \brief survie dans le cas multi contraint (version oep)
 */

#include "vzr_util.h"

/* etat de dominance */
#define EGAL 0
#define REF  1
#define TEST 2
#define DIFF 3

/** utilitaire **/
/* calcul de la dominance
 * cas multi critére
 * */
static short dominated_mc(struct vizir *viz, double *test, double *ref)
{
    short mark;
    int i;

    mark = EGAL;
    for (i = 0; i < viz->nbfun; i++)
    {
        if (ref[i] < test[i])
            mark |= REF;
        else if (test[i] < ref[i])
            mark |= TEST;
        if (mark == DIFF)
            break;
    }
    return mark;
}

/* calcul de la dominance
 * cas multi critére avec contrainte
 * */
static short dominated_mcc(struct vizir *viz, double *test, double *ref)
{
    short mark;
    int i;

    mark = EGAL;
    for (i = 0; i < viz->nbsoft; i++)
    {
        if (ref[i] < test[i])
            mark |= REF;
        else if (test[i] < ref[i])
            mark |= TEST;
        if (mark == DIFF)
            break;
    }
    if (mark == EGAL)
    {
        for (i = viz->nbsoft; i < viz->nbsoft + viz->nbfun; i++)
        {
            if (ref[i] < test[i])
                mark |= REF;
            else if (test[i] < ref[i])
                mark |= TEST;
            if (mark == DIFF)
                break;
        }
    }
    return mark;
}

/** fonction du protocole **/
static double rang_individu(void *truc, struct vizir *viz, struct vzrwork *wviz)
{
    int j, nbdom;
    double ret;
    short (*dominated)(struct vizir*, double*, double*);

    dominated = (short (*)(struct vizir*, double*, double*)) truc;

    nbdom = 0;
    for (j = 0; j < viz->N; j++)
        if ((*dominated)(viz, wviz->val, viz->pop_val[j]) == REF)
            nbdom++;
    ret = wviz->val[viz->rangdomi] = (double) nbdom;
    return (ret);
}

static void rang_population(void *truc, struct vizir *viz)
{
    int i, j;
    short (*dominated)(struct vizir*, double*, double*);

    dominated = (short (*)(struct vizir*, double*, double*)) truc;

    for (i = 0; i < viz->N; i++)
        viz->pop_val[i][viz->rangdomi] = 0;

    for (i = 1; i < viz->N; i++)
        for (j = 0; j < i; j++)
            switch ((*dominated)(viz, viz->pop_val[i], viz->pop_val[j]))
            {
            case TEST:
                viz->pop_val[j][viz->rangdomi] += 1;
                break;
            case REF:
                viz->pop_val[i][viz->rangdomi] += 1;
            case EGAL:
            case DIFF:
                break;
            }

}

static int elite(void *truc, struct vizir *viz)
{
    int i, ret;

    ret = 0;
    for (i = 0; i < viz->N; i++)
        if (viz->pop_val[i][viz->rangdomi] == 0.0)
            ret++;
    return ret;
}

/** definition de l'instance de protocole **/

struct vzrclasse *vizir1_multi(void)
{
    static struct vzrclasse _multi =
    { (void *) &dominated_mc, 1, &rang_individu, &rang_population, &elite };
    return &_multi;
}

struct vzrclasse *multi_contraint(void)
{
    static struct vzrclasse _multi =
    { (void *) &dominated_mcc, 1, &rang_individu, &rang_population, &elite };
    return &_multi;
}

