//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef ROOT_vizir
#define ROOT_vizir

#ifndef ROOT_TNamed
#include "TNamed.h"
#endif

#ifndef ROOT_TMethodCall
#include "TMethodCall.h"
#endif

#ifndef ROOT_TMatrixD
#include "TMatrixD.h"
#endif
#ifndef ROOT_TVectorD
#include "TVectorD.h"
#endif
#ifndef ROOT_TNtuple
#include "TNtuple.h"
#endif

#include "TGraph.h"

#include "Rtypes.h"

#include "vzr_alloc.h"
#include "TAttribute.h"
#include "TDataSpecification.h"
#include "TDataServer.h"
#include "TDSNtupleD.h"

#include <ctime>

#define MULTI   0 /* multi objectif */
#define MONO    1 /* mono objectif */
#define MONOC   2 /* mono objectif avec contraintes */
#define MULTIC  3

class VizirCommon: public TNamed
{
protected:
    URANIE::DataServer::TDataSpecification* _dataSpecification; // Specification of the parameters
    URANIE::DataServer::TDataSpecification* _coutSpecification; // Specification of costs
    Bool_t bImageZoneAdmissible;               //! Construction of the tuple
    URANIE::DataServer::TDSNtupleD* _ntImageZoneAdmissible; // Tree including the picture of the eligible zone
    Bool_t bZoneAdmissible;               //! Construction of the tuple
    URANIE::DataServer::TDSNtupleD* _ntZoneAdmissible; // Tree including the picture of the eligible zone

    TString _sdraw;              // Variables to visualize
    TString _sopt;               // Option of visualization
    TString _scut;               // Selection of visualization
public:
    int nmode;  //! Node of use
    int nbgen;  //! Number of genes
    int nbcout; //! Number of Cost Functions
    int nbcont; //! Number of Constraint Functions

    Double_t alpha; //! alpha
    Double_t phomo;  //! phomo
    Double_t pmut;  //! pmut
    Double_t bias;  //! bias
    Double_t prop;  //! prop
    Double_t _dprec;  //! Precision
    Double_t fitseuil;  //! fitseuil

    Int_t maxevalu; //! maxevalu
    Int_t itaille; //! itaille
    Int_t N; //! vv N
    Int_t consang; //! consang
    Int_t ipop;
    Int_t nbevalu;

    Int_t nbsurv;
    Int_t _nbelite;
    Int_t rgmax;
    Int_t _nIter;        ///< The number of iterations
    Int_t nadmis;

    Int_t nCrtInp;      ///< Usual meter of the number of uncertain parameters
    Int_t ncrtnbcout;      //! Usual number of added functions
    Int_t ncrtnbhardcont; //! Usual number of hard constraints
    TMatrixD descendant; //! Matrix of "descendants" with a size  2 * nbgen
    TMatrixD pop_val; //! Matrix of criteria (nbcout+nbcont) * Size of population.
    TMatrixD descendant_val; //! Temporary matrices of the "descendants" (nbcout+nbcont) * Size of population.
    TVectorD rangdomi;   //! Vector of ranks of dominance.
    double ***population;
    TNtuple* _datatree;             // Tree including data

    Double_t _xPointP;           //! X-axis of the point P
    Double_t _yPointP;           //! Y-axis of the point P

    Bool_t _blog;          /// Log Printing
    TMethodCall *fMethodCall; //Pointer to MethodCall in case of interpreted function
    TMethodCall *fDecodeCall; //Pointer to MethodCall in case of interpreted function
    TMethodCall *fCoutCall[10]; //Pointer to MethodCall in case of interpreted function
    TMethodCall *fHardContCall[10]; //Pointer to MethodCall in case of interpreted function
    void (*fFitness)(Int_t &npar, Double_t *descendant, Double_t &f,
            Double_t *val, void* info);
    void (*fDecode)(Int_t &npar, Double_t *descendant, Double_t *param);
    void (*fCout)(Double_t *gin, Double_t *f);
    void (*fHardCont)(Double_t *param, Int_t &ret);
public:
    VizirCommon();
    VizirCommon(int, int, int, int, double);
    VizirCommon(int, int, int, int, double, int (*)(double*, double*, void*),
            void*);
    virtual ~VizirCommon();

    // Les param�tres
    void addAttribute(URANIE::DataServer::TAttribute *x);

    // origine des query
    void setTauxSurvie(Double_t val);
    void setHomozigotProb(Double_t val);
    void setMutationProb(Double_t val);
    void setNaissanceBiais(Double_t val);
    void setOutlierFacteur(Double_t val);
    void setTaille(Int_t max_eval, Int_t taille_pop);

    void setTaille(Int_t taille_pop);
    void setSize(Int_t nSize)
    {
        setTaille(nSize);
    }
    void pheno(); //! Computes the phenotype og the used individual represented by \em child
    Int_t parents(); //! 

    //---------------------------------------------
    /** @name Manipulate the code
     **/
    //@{
    //! Runs the optimization phase
    void optimize(Option_t *option = "");
    //@}

    // 
    void initDefault();
    TMethodCall *GetMethodCall() const
    {
        return fMethodCall;
    }
    TMethodCall *GetDecodeCall() const
    {
        return fDecodeCall;
    }
    TMethodCall *GetMethodCallCout(Int_t i) const
    {
        return fCoutCall[i];
    }
    TMethodCall *GetHardContCall(Int_t i) const
    {
        return fHardContCall[i];
    }

    Int_t GetNbCout()
    {
        return nbcout;
    }
    Int_t GetNbHardCont()
    {
        return ncrtnbhardcont;
    }
    Int_t GetNbCrit()
    {
        return nbcout + nbcont;
    }
    Int_t GetNbGene()
    {
        return nbgen;
    }
    Int_t getNbGeneration()
    {
        return _nIter;
    }
    Int_t getNbZonePareto();
    Double_t *getZonePareto(TString sX);
    TNtuple* getTuple()
    {
        return _datatree;
    }

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    virtual void printLog(Option_t *option = "");
    //@}

    void SetDecode(void *fdecode);
    void SetFitness(void *fitness);
    //---------------------------------------------
    /** @name Adds the costs function
     **/
    //@{
    void addCost(void *fcout);
    //! Adds a function with multi-objective
    void addCost(void *fcout, const char *svarexp);
    //@}

    void addHardConstraint(void *fcout);

    void DecodeDefault(Double_t *descendant, Double_t *param);

    // Specific method.

    void setPointP(Double_t px, Double_t py);

    // Graphiques
    //---------------------------------------------
    /** @name Graphs
     **/
    //@{
    //! Draws the criteria defined by the indexex
    void drawCriteres(Int_t i, Int_t j);
    //! Draws the research area defined by the indexex
    /*!
     \param nx (Int_t) The index for the X-axis [0]
     \param ny (Int_t) The index for the Y-axis [1]
     \param nColor (Int_t)The color index [42]
     \exception When nx/ny is lesser than 0 and greater than the number of parameter
     */
    void drawZoneRecherche(Int_t nx = 0, Int_t ny = 1, Int_t nColor = 42);
    Int_t drawZoneAdmissible(Int_t nflag = 0);
    Int_t drawImageZoneAdmissible(Int_t nflag = 0);
    void NtDraw(TNtupleD *nt, TString xVar, TString yVar, TString sTitre,
            Option_t *option = "");
    void drawZonePareto(Int_t nflag = 0);
    void drawFrontPareto(Int_t nflag = 0);
    void drawLastGeneration(const char* varexp, Option_t* option = "");
    void setVarDraw(TString str, TString cut = "", TString opt = "")
    {
        _sdraw = str;
        _scut = cut;
        _sopt = opt;
    }
    //@}

    // Listing
    void Scan(const char *varexp = "", const char *selection = "",
            Option_t *option = "");
    void Draw(const char* varexp, const char* selection = "", Option_t* option =
            "");

private:
    void creation_aleatoire();
    double geneal(double valeur);
    int evaluation_initiale();
    void classe(Int_t n);
    void tridicho();
    void naissance();
    int evaluation();
    int rang(double *valcour, TMatrixD valcomp, Int_t n);
    int stock(double *valcour);

    void createTuple();
    void saveGeneration();

    ClassDef(VizirCommon, 1)
    //Definition of an variable

};

class VizirMulti: public VizirCommon
{
public:
    VizirMulti();
    VizirMulti(int gene, int cout);
    //! Creates a vizirMulti object with a TDataServer
    /*
     The parameter are defined in the TDataServer as kVariable.
     */
    VizirMulti(URANIE::DataServer::TDataServer *tds, int ncriteria);
    //! Creates a Vizir Multi Object with a DataServer and the size of the population
    VizirMulti(URANIE::DataServer::TDataServer *tds, int nSize, int ncriteria);

    VizirMulti(int gene, int cout, int (*fct)(double*, double*, void*),
            void*arg);ClassDef(VizirMulti, 1)
    //Definition of an multiobjective problem

};

class VizirMono: public VizirCommon
{
public:
    VizirMono();
    VizirMono(int gene);ClassDef(VizirMono, 1)
    //Definition of a monocriterion optimisation

};

R__EXTERN VizirCommon *gVizir;

#endif
