//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TFormEval.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-09-21.11.03.42

 \brief   fonction pour un problème form
 */

#include "TFormEval.h"

#include "TSorm.h"

ClassImp (URANIE::Reliability::TFormEval);

namespace URANIE
{
namespace Reliability
{

TFormEval::TFormEval(TSimpleTransform *obj, URANIE::Relauncher::TEval *cont)
{
    addEval(obj);
    addEval(cont);
    _valid = NULL;
    _fit = NULL;
}

TFormEval::~TFormEval()
{
}

void TFormEval::addConstraint(URANIE::DataServer::TAttribute *att,
        URANIE::Reoptimizer::TOptimFit *fit)
{
    _valid = att;
    _fit = fit;
}

void TFormEval::addObjective(URANIE::Reoptimizer::TOptimShare *opt)
{
    TSimpleTransform *obj;
    obj =
            (TSimpleTransform*) ((URANIE::Relauncher::TSubEval*) _eval->First())->getEval();

    opt->addConstraint(_valid, _fit);
    opt->addObjective(obj->getBeta());
}

void TFormEval::addLapse(TSorm*sorm)
{
    sorm->addLapse(_valid, _fit);
}

void TFormEval::freeze()
{
    int cont;
    TObjArray *in2;
    TObject *x1, *x2;
    TSimpleTransform *obj;
    obj =
            (TSimpleTransform*) ((URANIE::Relauncher::TSubEval*) _eval->First())->getEval();

    if (_freezed)
        return;
    TComposeEval::freeze();

    /* no extra input */
    in2 = obj->getInputList();
    TIter next1(&_input), next2(in2);
    cont = 0;
    while (!cont)
    {
        x1 = next1();
        x2 = next2();
        if (x1 != x2)
            cont = -1;
        if (!x1)
            cont = 1;
    }
    if (cont < 0)
    {
        std::cout << "Unexpected Input\n";
        throw new UnexpectedInput();
    }

}

}
} // namespace

