//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TSorm.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-07-09.13.54.55

 \brief SORM evaluation
 */

#ifndef __SORM__
#define __SORM__

//ROOT
#include "Rtypes.h"
#include "TNamed.h"

// Uranie
#include "Reliability.h"
#include "TInformMaster.h"

#include "TDataServer.h"
#include "TFormEval.h"
#include "TFitGetter.h"
#include "TOptimFit.h"
#include "ridder.h"

namespace URANIE
{
namespace Reliability
{

class TDemiAxeSorm
{
protected:
    struct ridder *_solv;
    double _xl, _fl, _xt, _ft;
    double *_ref;
    int _size, _state;

public:
    TDemiAxeSorm(Int_t size, Double_t *ref);
    ~TDemiAxeSorm();

    int next(double in, double *next);
    double courbure();
};

class TSorm: public URANIE::Relauncher::TInformMaster
{
protected:
    Int_t _size;
    URANIE::Reoptimizer::TFitGetter *_lapse;

    void create();
    void build();
    void fill(double id, vector<URANIE::DataServer::UEntry*> *in, 
        vector<URANIE::DataServer::UEntry*> *out,
        vector<URANIE::DataServer::UEntry*> *all, double correct);

    void getCurrentPattern(Int_t ipati, Double_t *res);

public:
    class PureEvalNeed
    {
    };

    TSorm(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TRun *run);
    TSorm(URANIE::DataServer::TDataServer *tds, TFormEval *fun);
    virtual ~TSorm();

    void addLapse(URANIE::DataServer::TAttribute *att,
            URANIE::Reoptimizer::TOptimFit *modif = NULL);

    virtual void solverLoop();

    ClassDef(URANIE::Reliability::TSorm, ID_RELIABILITY)

};

}
} //namespace
#endif
