#ifdef JIT
#include "TAttribute.h"
#include "TNormalDistribution.h"
#include "TLogNormalDistribution.h"
#include "TSimpleTransform.h"
#include "TCJitEval.h"
#include "TComposeEval.h"
#include "TSequentialRun.h"
#include "TThreadedRun.h"
#include "TDataServer.h"
#include "TOptimFit.h"
#include "TNloptCobyla.h"
#include "TNloptDirect.h"
#include "TVizirSimplex.h"
#include "TVizir2.h"
#include "TNlopt.h"
#include "TSorm.h"

#endif
/* possible solver
 * direct
 #define SOLV TNloptCobyla
 * lagragien direct
 #define SOLV TNloptBobyqa //doesn't work on this problem
 #define SOLV TNloptPraxis
 #define SOLV TNloptNelderMead
 #define SOLV TNloptSubplexe
 * */
#define SOLV TNloptCobyla

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;
using namespace URANIE::Reliability;

#define BREAK 2.2 // original value
//#define BREAK 1.5 // Cobyla praxis NM subplex trap 
//#define BREAK 1.4688 // pseudo equiv
//#define BREAK 1.4 // Bobyqa trap 

#define EPSI 1.0e-6

/********** User Function *****************/
static void fselect(double *in, double *res)
{
    double x, y, brk;

    x = in[0];
    y = in[1];
    brk = in[2];
    /* critere */
    res[0] = (x <= brk) ? y - x * x : y - 0.5 * (x - brk) - brk * brk;
}

/********** Optimization  *****************/
void test_TNloptSorm()
{
    /* inputs */
    TNormalDistribution x2("x2");
    TLogNormalDistribution x1("x1");
    x1.setUnderlyingNormalParameters(4., 1.);
    // trusting region patch 
    TAttribute seuil("seuil");
    /* outputs */
    TAttribute cont("cont");
    /* starting point */
    //double start[] = {0.0, 0.0};
    //double start[] = {-1.0, 1.0};
    //double start[] = {-3.0, 3.0};
    //double start[] = {-3.5, 1.5};
    double start[] =
    { 3.0, 1.0 };

    /* code */
    TSimpleTransform fobj;
    fobj.setParameters(2, &x1, &x2);

    TCJitEval fcont(&fselect);
    fcont.setInputs(3, &x1, &x2, &seuil);
    fcont.setOutputs(1, &cont);

    TFormEval code(&fobj, &fcont);
    TGreaterFit it(0.0);
    code.addConstraint(&cont, &it);
    code.addConstantValue(&seuil, BREAK);

    /* tds */
    TDataServer tds("toto", "tds for vizir test");

    /* run */
    TSequentialRun run(&code);
    run.startSlave();
    if (run.onMaster())
    {
        /* tds */
        code.addAllInputs(&tds);

        /* optimizer */
#ifdef SOLV
        SOLV solv;
        TNlopt nlo(&tds, &run, &solv);
#else
        TVizirSimplex solv;
        //TVizirSwarm solv;
        solv.setSize(32, 10000, 16);
        TVizir2 nlo(&tds, &run, &solv);
#endif
        code.addObjective(&nlo);

        /* resolution */
#ifdef SOLV
        nlo.setStartingPoint(2,start);
#endif
        nlo.solverLoop();
        run.stopSlave();
        tds.getTuple()->Scan("*");
    }

    /* run */
    TThreadedRun run1(&code, 4);
    run1.startSlave();
    if (run1.onMaster())
    {

        /*** SORM ***/
        TSorm sorm(&tds, &run1);

        sorm.solverLoop();

        /* resultats */
        run1.stopSlave();
        tds.getTuple()->Scan("*");
    }

}
