//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TNloptDirect.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-03-22.10.49.52

 \brief   algorithme local direct (sans gradient) de NLopt
 */

#include "TNloptDirect.h"

#include "nlopt.h"

ClassImp (URANIE::Reoptimizer::TNloptNelderMead);
ClassImp (URANIE::Reoptimizer::TNloptBobyqa);
ClassImp (URANIE::Reoptimizer::TNloptPraxis);
ClassImp (URANIE::Reoptimizer::TNloptSubplexe);

namespace URANIE
{
namespace Reoptimizer
{

//TNloptNelderMead
nlopt_opt TNloptNelderMead::build(int var, int ineq, int equ, double prec,
        nlopt_opt *subsolv)
{
    nlopt_opt ret;

    if (ineq || equ)
    {
        // contrainte, utilisation du lagragien
        *subsolv = solver(var);
        nlopt_set_ftol_abs(*subsolv, prec);
        ret = nlopt_create(NLOPT_AUGLAG, var);
        nlopt_set_local_optimizer(ret, *subsolv);
    }
    else
    {
        // pas de contrainte
        *subsolv = NULL;
        ret = solver(var);
    }
    nlopt_set_ftol_abs(ret, prec);

    return ret;
}

nlopt_opt TNloptNelderMead::solver(int var)
{
    return nlopt_create(NLOPT_LN_NELDERMEAD, var);
}

//TNloptBobyqa
nlopt_opt TNloptBobyqa::solver(int var)
{
    return nlopt_create(NLOPT_LN_BOBYQA, var);
}

//TNloptPraxis
nlopt_opt TNloptPraxis::solver(int var)
{
    return nlopt_create(NLOPT_LN_PRAXIS, var);
}

//TNloptSubplexe
nlopt_opt TNloptSubplexe::solver(int var)
{
    return nlopt_create(NLOPT_LN_SBPLX, var);
}

//TNloptSubplexe
nlopt_opt TNloptLBFGS::solver(int var)
{
    return nlopt_create(NLOPT_LD_LBFGS, var);
}

//TNloptSubplexe
nlopt_opt TNloptNewton::solver(int var)
{
    return nlopt_create(NLOPT_LD_TNEWTON_PRECOND_RESTART, var);
}

//TNloptSubplexe
nlopt_opt TNloptVariableMetric::solver(int var)
{
    return nlopt_create(NLOPT_LD_VAR2, var);
}

}
} // namespace

