//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.5 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TNlopt.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-03-20.16.04.27

 \brief interface algorithme optimisation de la bibliotheque NLOpt
 */

#ifndef __NLOPTGRADIENT__
#define __NLOPTGRADIENT__

#include <vector>
#include <map>

//ROOT
#include "Rtypes.h"

//Uranie
#include "Reoptimizer.h"
#include "TNloptShare.h"

#include "DataServer.h"
#include "TEval.h"
#include "TOptimFit.h"
#include "TFitGetter.h"


namespace URANIE
{
namespace Reoptimizer
{

class TNloptGradient: public TNloptShare
{
protected:
    //std::map<int, struct grad_work> _job;
    //int _coef;
    //std::vector<std::vector<URANIE::DataServer::UEntry *>> _codi, _codo, _codg;

    // solver
    void gradFromCode(double *code, double *solv);
    // abstract from TNloptShare
    void buildSolver(int siz, struct nlopt_work *);
    void cleanSolver(struct nlopt_work *);
    void put(struct nlopt_work *, Bool_t, const double *);
    virtual int get(long &);

public:
    class NeedAGradientAttribute
    {
    };

    TNloptGradient(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TRun *run,
            TNloptSolver *algo);
    TNloptGradient(URANIE::DataServer::TDataServer *tds, URANIE::Relauncher::TEval *fun,
            TNloptSolver *algo);
    virtual ~TNloptGradient();

    // initialisation
    virtual void addObjective(URANIE::DataServer::TAttribute *, 
            TOptimFit *f = NULL);
    void addObjective(URANIE::DataServer::TAttribute *, 
            URANIE::DataServer::TAttribute *, TOptimFit *f = NULL);
    virtual void addConstraint(URANIE::DataServer::TAttribute *, 
            TOptimFit *f = NULL);
    void addConstraint(URANIE::DataServer::TAttribute *, 
            URANIE::DataServer::TAttribute *, TOptimFit *f = NULL);
    void addEquality(URANIE::DataServer::TAttribute *, 
            URANIE::DataServer::TAttribute *, TOptimFit *f = NULL);
    // callback methode
    int critere(struct nlopt_work*, const double *ini, double *out,
            double *grad);
    int inequality(struct nlopt_work*, const double *in, double *out,
            double *grad);
    int equality(struct nlopt_work*, const double *in, double *out,
            double *grad);
    void run(struct nlopt_work *);

    ClassDef(URANIE::Reoptimizer::TNloptGradient, ID_REOPTIMIZER)
};


}
} //namespace
#endif

