//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizirIsland.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-02-10.11.12.36

 \brief   pseudo repartition en ilot homogene
 */

#include "TVizirIsland.h"

#include <stdio.h>

#include "TRun.h"

#include "vizir/vzragent_api.h"
#include "vizir/vzrcore_api.h"
#include "vizir/vzrtribe_api.h"
#include "vizir/vzr_macro.h"

ClassImp (URANIE::Reoptimizer::TVizirIsland);

namespace URANIE
{
namespace Reoptimizer
{

// TVizirIsland
TVizirIsland::TVizirIsland(URANIE::DataServer::TDataServer *tds,
        URANIE::Relauncher::TRun *Run, TVizirSolver *solv) :
        TVizirShare(tds, Run, solv)
{
    _ilot = 8;
}

TVizirIsland::TVizirIsland(URANIE::DataServer::TDataServer *tds,
        URANIE::Relauncher::TEval *fun, TVizirSolver *solv) :
        TVizirShare(tds, fun, solv)
{
    _ilot = 8;
}

/* solve */
void TVizirIsland::run(struct vzrtribe *all)
{
    int life, size, outsize, ret, dec;
    long i, j;

    //size = cpus;
    dec = _solv->isCoded();
    size = (_run->_ncpus < 2) ? 1 : _run->_ncpus -1 ;
    if ( size % all->size) // pas un multiple
        size = ((size / all->size) + 1) * all->size ; // arondi superieur
    outsize = _obj + _valid + _NoutDat + 1;

	struct vzrcore** core = new struct vzrcore*[size];
    double** in = new double*[size];
	for(j=0;j<size;j++) in[j] = new double[_NinDat+_Naux];
    double* phy = new double[_NinDat+_Naux];
    vector<vector<URANIE::DataServer::UEntry *>>  codin;
    codin.resize(size);
    allocInCodeEntries(size, codin);
	vector<vector<URANIE::DataServer::UEntry *>> codout;
	codout.resize(size);
	allocOutCodeEntries(size, codout);
    double* out = new double[outsize];
    vector<URANIE::DataServer::UEntry*> *dat;
    vector<vector<URANIE::DataServer::UEntry*>> alltrace;
    alltrace.resize(1);
    allocDataEntries(1, alltrace);
	
    for EACH(i, size)
    {
        core[i] = vzragent_core(all->ilot[i%all->size]);
        PRO_ASK(core[i], subscribe);
        PRO_CALL(core[i], get, in[i]);
        if (dec) decode(in[i], phy);
        else nodecode(in[i], phy);
        inputForCode(phy, &codin[i]);
        _run->put(i, &codin[i], &codout[i] );
    }

    life = size;
    while (life)
    {
        ret = _run->get(i);
        if (_tdstrc) fillTrace(&codin[i], &codout[i], &alltrace[0]);
        fillChoice(&codout[i], out);
        dat = fillData(&codin[i], &codout[i]);
        for (auto val : codout[i]) val->clearEntry();
        if (PRO_CALL(core[i], put, ret, in[i], out, dat)||
                PRO_CALL(core[i], get, in[i]))
        {
            life --;
            continue;
        }
        if (dec) decode(in[i], phy);
        else nodecode(in[i], phy);
        inputForCode(phy, &codin[i]);
        _run->put(i, &codin[i], &codout[i]);
        vzrtribe_migre(all, i%all->size);
    }

    ////for FROMEACH(j, 1, size) ret = _run->get(i);

    for EACH(i, size)
    {
        PRO_ASK(core[i], unsubscribe);
        PRO_DEALLOC(core[i]);
    }

	delete[] core;
	for EACH(j, size)
	{   delete[] in[j];
	    for (auto it : codin[j]) delete it ;
	    codin[j].clear();
	    for (auto it : codout[j]) delete it ;
	    codout[j].clear();
	}
	delete[] in;
	delete[] phy;
	//delete[] codin;
	//delete[] codout;
	delete[] out;
	
}

void TVizirIsland::solverLoop()
{
    int i, val;
    int size = (_ilot > _run->_ncpus) ? _run->_ncpus : _ilot ;

	struct vzragent** ag = new struct vzragent*[size];
	
    struct vzrtribe *all;

    build();
    _obj = _crit.GetEntries();
    _valid = _const.GetEntries();
    for (i = 0; i < size; i++)
        ag[i] = _solv->build(_NinDat+_Naux, _obj, _valid, _prec);
    all = vzrtribe_create(size, ag);
    run(all);
    val = 0;
    for (i = 0; i < size; i++)
        fill(ag[i], val);
    vzrtribe_dealloc(all);

    delete[] ag;
    fillConstant();
}

}
} //namespace
