//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizirSwarm.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-02-06.04.59.11

 \brief   particule swarm optimisation algorithm
 */

#include "TVizirSwarm.h"

#include <stdio.h>

#include "swarm/pso.h"
#include "vizir/vzr_macro.h"

ClassImp (URANIE::Reoptimizer::TVizirMovement);
ClassImp (URANIE::Reoptimizer::TVizirSwarm);

namespace URANIE
{
namespace Reoptimizer
{

// TVizirSwarm
/* constructeur destructeur */
TVizirSwarm::TVizirSwarm()
{
    /* specific defaults */
    defaultShare();
    _localpop = 32;
    _localstep = 3;
    _move = NULL;
}

TVizirSwarm::~TVizirSwarm()
{
}

/* parameter value */
void TVizirSwarm::setLocalSize(int pop, int step)
{
    _localpop = pop;
    _localstep = step;
}

/* initialisation */
struct vzragent * TVizirSwarm::build(int gene, int obj, int cont, double prec)
{
    struct psoswarm *pso;

    pso = pso_init(gene, obj, cont, prec);
    buildShare(&(pso->base));
    
    pso->move = (_move) ? _move->movement() : pso_movestd(gene, 0, 2);
    //pso->memg = psoglobalstd_create(_pop, _step);
    //pso->meml = psolocalstd_create(_localpop, _localstep);
    if (_doresume) {
        pso_resume(pso, _pop, _resume);
        vzrpop_set_again(&(pso->base.pop));
    }
    else pso_prepare(pso, _step, _pop, _eval);

    return vzragent_pso(pso);
}


}
} //namespace

