//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.2 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TVizirSwarm.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-02-06.04.59.03

 \brief particule swarm optimisation algorithm
 */

#ifndef __TVIZIRSWARM__
#define __TVIZIRSWARM__

//ROOT
#include "Rtypes.h"
#include "TObject.h"

//Uranie
#include "Reoptimizer.h"
#include "TVizirSolver.h"

struct psomovebuild;

namespace URANIE
{
namespace Reoptimizer
{

class TVizirMovement
{
public:
    virtual ~TVizirMovement() {}
    virtual struct psomovebuild *movement() = 0;

    ClassDef(URANIE::Reoptimizer::TVizirMovement, ID_REOPTIMIZER)
};

class TVizirSwarm: public TVizirSolverShare
{
protected:
    TVizirMovement *_move;
    int _localpop, _localstep;

public:
    TVizirSwarm();
    virtual ~TVizirSwarm();

    //set specific parameter
    //!  Set movement
    void setMovement(TVizirMovement *move)
    {
        _move = move;
    }
    void setLocalSize(int pop, int step = 3);

    // protocole
    virtual struct vzragent *build(int, int, int, double);

    ClassDef(URANIE::Reoptimizer::TVizirSwarm, ID_REOPTIMIZER)
};

}
} //namespace
#endif

