//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
// 
//    Object
//      two level MPI example
// 
//___________________________________________________________________

#ifdef JIT
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TAttribute.h"
#include "TMpiLn2Eval.h"
#include "TBiMpiRun.h"
#include "TDataServer.h"
#include "TOptimFit.h"
#include "TVizirGenetic.h"
#include "TVizir2.h"

#endif

using namespace URANIE::DataServer;
//using namespace URANIE::Relauncher;
using namespace URANIE::MpiRelauncher;
using namespace URANIE::Reoptimizer;

/* user defined */
#define TOLERANCE 0.0001
#define POP 256
#define EVALmax 50000

#define N 100000

/* solver */
void testVizirBiMpi()
{
    /* init */
    TAttribute pl("plus", 0.3, 3.), mn("moins", 0.3, 3.), ln("ln2_dist");

    TMpiLn2Eval code;
    code.addInput(&pl);
    code.addInput(&mn);
    code.addOutput(&ln);

    TBiMpiRun run(&code, 3);
    //run.calculSize(2);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tdsvzr("tdsvzr", "tds for vizir test");
        tdsvzr.addAttribute(&pl);
        tdsvzr.addAttribute(&mn);

        TVizirGenetic solv;
        solv.setSize(POP, EVALmax);
        TVizir2 vzr(&tdsvzr, &run, &solv);
        vzr.addObjective(&ln);
        vzr.setTolerance(TOLERANCE);

        /* resolution */
        vzr.solverLoop();

        /* sauvegarde */
        tdsvzr.exportData("_bi_mpi_.dat");
        run.stopSlave();
    }
}

