/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2015-2022 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzr_query.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2015-08-24
 
 \brief  genetic solver api
*/


#ifndef __VZR_QUERY__
#define __VZR_QUERY__

#include "genetic/vzr_struct.h"

#include "vizir/vzrfitness.h"

/* public */
/*******    DECLARATION DES FONCTIONS   *************/
#ifdef __cplusplus
extern "C" {
#endif

/* init */
VZRLIB_EXPORT vzr_t *vzr_multi(int, int);
VZRLIB_EXPORT vzr_t *vzr_mono(int, double);
VZRLIB_EXPORT vzr_t *vzr_monoConstraint(int, double, int);
VZRLIB_EXPORT vzr_t *vzr_multiConstraint(int, int, int);
VZRLIB_EXPORT vzr_t *vzr_init(int, int, int, double);

VZRLIB_EXPORT void vzr_setLabel(vzr_t *, char *);
//VZRLIB_EXPORT void vzr_setDominanceMode(vzr_t *, int);
VZRLIB_EXPORT void vzr_setSurvivalRate(vzr_t *, double);
VZRLIB_EXPORT void vzr_setHomozygoteRate(vzr_t *, double);
VZRLIB_EXPORT void vzr_setMutationRate(vzr_t *, double);
VZRLIB_EXPORT void vzr_setParentsBias(vzr_t *, double);

VZRLIB_EXPORT void vzr_setOneGeneMutationRate(vzr_t *, double);
VZRLIB_EXPORT void vzr_setCombinatorialSize(vzr_t *, int);

VZRLIB_EXPORT void vzr_setFitness(vzr_t *, vzrfitnessbuild_t *);
VZRLIB_EXPORT void vzr_setCrossover(vzr_t *, vzrcrossoverbuild_t *);

VZRLIB_EXPORT int vzr_prepare(vzr_t *, int, int, int);
VZRLIB_EXPORT int vzr_build(vzr_t *, int, int, int);
VZRLIB_EXPORT int vzr_resume(vzr_t *, int, char *);

/* run */
VZRLIB_EXPORT void vzr_run(vzr_t *);
VZRLIB_EXPORT void vzr_start(vzr_t *);
VZRLIB_EXPORT void vzr_stop(vzr_t *);
VZRLIB_EXPORT void vzr_wait(vzr_t *);
VZRLIB_EXPORT int  vzr_running(vzr_t *);
VZRLIB_EXPORT void vzr_reevaluate(vzr_t*);
void vzr_dump(vzr_t*, char*);

/* result */
VZRLIB_EXPORT vzrdiploid_t vzr_genotype(vzr_t *, int);
VZRLIB_EXPORT vzrgene_t *vzr_phenotype(vzr_t *, int);
VZRLIB_EXPORT double *vzr_objective(vzr_t *, int);
VZRLIB_EXPORT double vzr_rank(vzr_t *, int);

VZRLIB_EXPORT int vzr_birthCount(vzr_t *);
VZRLIB_EXPORT int vzr_generationCount(vzr_t *);
VZRLIB_EXPORT int vzr_survivorSize(vzr_t *);
VZRLIB_EXPORT int vzr_populationSize(vzr_t *);

/* distribution */
VZRLIB_EXPORT void vzr_emptyStack(vzr_t *);
VZRLIB_EXPORT void vzr_reset(vzr_t *);

/* destruction */
VZRLIB_EXPORT void vzr_dealloc(vzr_t *);

#ifdef __cplusplus
}
#endif

#endif
