/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2003-2022 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzr_struct.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2003-12-11
 
 \brief  structure pour vizir
*/

#ifndef __VZR_STRUCT__
#define __VZR_STRUCT__

/* predefinition */
#include "vizir/vzrbase_api.h"

struct vizir;
struct vzrwork;
//typedef struct vzrwork vzrwork_t;

//typedef vzrgene_t *vzrdiploid_t[2];
typedef struct vzrdiploid *vzrdiploid_t;

/* protocole */
struct vzrcrossover {
    void *data;
    void (*sample) (void*, vzrdiploid_t);
    void (*allele) (void*, vzrdiploid_t, vzrgene_t*);
    void (*mutation) (void*, vzrdiploid_t);
    void (*homozygote) (void*, vzrdiploid_t);
    void (*heterozygote) (void*, vzrdiploid_t);
    void (*dealloc) (struct vzrcrossover *);
};

struct vzrcrossoverbuild {
    void *data;
    struct vzrcrossover *(*create) (void *);
    void (*dealloc) (struct vzrcrossoverbuild *);
};

struct vzrmating {
    void *data;
    void (*couple) (void *, struct vizir*, struct vzrwork *);
    void (*dealloc) (struct vzrmating *);
};

struct vzrevent {
    void *data;
    void (*react) (void *, struct vizir*);
    void (*dealloc) (struct vzrevent *);
};

/* structure */
struct vzrcouple {
    int pa, ma;
};

struct vzrcoupling {
    enum popinfo genre;
    struct vzrcouple (*mate)(struct vzrpop*);
};

struct vizir {
    struct vzrbase base;
    /* parametre resolution*/
    int     genedisc, // nombre de gene discret
            consang; // taille risque consanguinite
    double  alpha, // taux de survie
            phomo, // taux d'homozigote cree
            pmut, // probabilite de mutation
            bias, // probabilite de biaisage des naissances (pere elite)
            pmut1g; // probabilite de mutation un gene
    /* operateur */
    struct vzrcrossoverbuild    *foper; // fabrique d'operateurs de croisement
    struct vzrmating    *eros; // selection parents
    struct vzrcoupling *mate;
    //struct vzrcouple (*mate)(void*, struct vzrpop *, int);
    struct vzrevent     *generation; // evenement nouvelle generation
};
    
struct vzrwork {
    struct vizir *self;
    /* working value */
    struct vzrdiploid pere, mere; //genotype de travail
    struct vzritem child;
    /* operateur */
    struct vzrcrossover   *oper; // operateurs de croisement
};

/* typedef */
typedef struct vizir vzr_t;
typedef struct vzrcrossover vzrcrossover_t;
typedef struct vzrcrossoverbuild vzrcrossoverbuild_t;
typedef struct vzrevent vzrevent_t;

#endif

