/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2022 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzrbase_api.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-12-02
 
 \brief  gestion des activités des solvers
*/

#include <pthread.h>
#include <sched.h>

#include "vizir/vzrbase_api.h"

#include "vizir/vzrpop_api.h"
#include "vizir/vzrfifo_api.h"
#include "vizir/vzrfitness.h"

void vzrbase_init(struct vzrbase *self)
{
    self->id = "";
    self->quiet = 0;
    self->fit = NULL;
    pthread_mutex_init(&(self->lock), NULL);
}

void vzrbase_build(struct vzrbase *self, int max)
{
    self->maxeval = max;
    self->slave = -1;
    self->eval = 0;
    self->stop = 0;
}

int vzrbase_first(struct vzrbase *self)
{
    int ret = 0;
    
    if (self->slave == -1) {
        //vzrpop_reset(&(self->pop));
        self->eval = 0;
        self->stop = 0;
        if (self->migr) vzrfifo_reset(self->migr);
        self->slave = 1;
        ret = 1;
    }
    else self->slave++;
    
    return ret;
}

void vzrbase_wait(struct vzrbase *self)
{
    int tag;
    pthread_mutex_lock(&(self->lock));
    tag = self->slave;
    pthread_mutex_unlock(&(self->lock));
    while (tag) {
        sched_yield();
        pthread_mutex_lock(&(self->lock));
        tag = self->slave;
        pthread_mutex_unlock(&(self->lock));
    }
}

int vzrbase_running(struct vzrbase *self)
{
    int ret;
    pthread_mutex_lock(&(self->lock));
    ret = self->slave ;
    //ret = (self->slave != 0)? 1 : 0;
    pthread_mutex_unlock(&(self->lock));
    return ret;
}
 
void vzrbase_stop(struct vzrbase *self)
{
    pthread_mutex_lock(&(self->lock));
    self->stop = 1;
    pthread_mutex_unlock(&(self->lock));
    //printf("%s  stopped\n", self->id);
}

void vzrbase_dump(struct vzrbase *self, char *nfile)
{
    vzrpop_dump(&(self->pop), nfile);
}

void vzrbase_dealloc(struct vzrbase *self)
{
    pthread_mutex_destroy(&(self->lock));
    if (self->fit) PRO_DEALLOC(self->fit);
}

