/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2022 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzrcore_api.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-06-04
 
 \brief  main loop pour un vzrcore
*/

#include "vzrcore_api.h"

#include "vzr_macro.h"

#include "vzrbase_api.h"
#include "vzrfitness.h"

void vzrcore_run(struct vzrcore *core)
{
    struct vzrbase *base = *((struct vzrbase **) core->data);
    struct vzrfitness *fun;
    VZRTAB(vzrgene_t, in, base->pop.gene);
    VZRTAB(double, out, base->pop.all);
    int good;

    /* alloc */
    fun = PRO_CREATE(base->fit);

    /* mainloop */
    PRO_ASK(core, subscribe);
    while (1)
    {
        if (PRO_CALL(core, get, in)) break;
        good = PRO_CALL(fun, compute, in, out);
        if (PRO_CALL(core, put, good, in, out, NULL)) break;
    }
    PRO_ASK(core, unsubscribe);

    /* nettoyage */
    PRO_DEALLOC(fun);
    PRO_DEALLOC(core);
    VZRTAB_OUT(out);
    VZRTAB_OUT(in);

}

static void *vzrcore_run_bg(void *info)
{
    struct vzrcore *self = (struct vzrcore *) info;

    vzrcore_run(self);
    return 0;
}

void vzrcore_start(struct vzrcore *self)
{
    pthread_t bg;
    pthread_attr_t attr;

    pthread_attr_init(&attr);
    pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
    pthread_create(&bg, &attr, vzrcore_run_bg, (void *)self);
    pthread_attr_destroy(&attr);
}

