/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2008-2022 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzrfun_std.c
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2008-03-19
 
 \brief  selection des individus
*/


#include <stdlib.h>

#include "vizir/vzrfun_std.h"

/* fonction d'evaluation std */
struct vzrfitnessstd {
    void *extra;
    int (*fun)(vzrgene_t*, double*, void*);
};

static int vzrfunstd_fun(void *info, vzrgene_t *in, double *out)
{
    struct vzrfitnessstd *std = (struct vzrfitnessstd *) info;

    return (*std->fun)(in, out, std->extra);
}

static void vzrfunstd_free(struct vzrfitness *fun)
{
    free(fun);
}

static struct vzrfitness *vzrfunstd_objet(void *info)
{
    struct vzrfitness *ret;

    ret = (struct vzrfitness *) malloc(sizeof(struct vzrfitness));
    ret->data = info;
    ret->compute = &vzrfunstd_fun;
    ret->dealloc = &vzrfunstd_free;

    return ret;
}

static void vzrfunstd_clean(struct vzrfitnessbuild *self)
{
    free(self->data);
    free(self);
}

struct vzrfitnessbuild *vzrfunstd_create(int (*fct)(vzrgene_t*, double*, void*), void *extra)
{
    struct vzrfitnessstd  *dt;
    struct vzrfitnessbuild *af;

    dt = malloc(sizeof(struct vzrfitnessstd));
    dt->extra = extra;
    dt->fun = fct;

    af = (struct vzrfitnessbuild*) malloc(sizeof(struct vzrfitnessbuild));
    af->data = (void *) dt;
    af->create = &vzrfunstd_objet;
    af->dealloc = &vzrfunstd_clean;

    return af;
}

