/* --- Ansi C --- */
//////////////////////////////////////////////////////////////////////
// Copyright (C) 2010-2022 CEA/DES
// 
// This program is free software: you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public License
// as published by the Free Software Foundation, either version 3 of
// the License, or any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program. 
// if not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////////////
/*!
 \file   vzrsort.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date   2010-03-09
 
 \brief  pareto rank order
*/

#ifndef __VZRSORT__
#define __VZRSORT__

#include "vzr_config.h"

/* predef */
struct vzrpop;
struct vzritem;

/* protocole */
struct vzrsort {
    void *data;
    void (*build)(void *, struct vzrpop *);
    int (*valid)(void*, struct vzritem *, struct vzrpop *);
    void (*rank)(void*, int, struct vzrpop*);
    void (*isol)(void*, int, struct vzrpop*);
    void (*survive)(void*, int, int, struct vzrpop*);
    void (*dealloc)(struct vzrsort*, struct vzrpop*OS);
};

/* public */

#ifdef __cplusplus
extern "C" {
#endif

VZRLIB_EXPORT struct vzrsort *create_moga(void);
VZRLIB_EXPORT struct vzrsort *create_moga2(void);
VZRLIB_EXPORT struct vzrsort *create_crowding(void);
VZRLIB_EXPORT struct vzrsort *create_neighbor_full(int prof);
VZRLIB_EXPORT struct vzrsort *create_neighbor(void);
VZRLIB_EXPORT struct vzrsort *create_pair_full(int prof, int seuil, int ratio);
VZRLIB_EXPORT struct vzrsort *create_pair(void);
VZRLIB_EXPORT struct vzrsort *create_strength(void);
VZRLIB_EXPORT struct vzrsort *create_spea_rand(void);
VZRLIB_EXPORT struct vzrsort *create_spea_crowd(void);
VZRLIB_EXPORT struct vzrsort *create_spea_full(int prof);
VZRLIB_EXPORT struct vzrsort *create_spea(void);
VZRLIB_EXPORT struct vzrsort *create_spea_pair_full(int prof, int seuil, int ratio);
VZRLIB_EXPORT struct vzrsort *create_spea_pair(void);

VZRLIB_EXPORT struct vzrsort *create_ibea_full(double k);
VZRLIB_EXPORT struct vzrsort *create_ibea(void);
VZRLIB_EXPORT struct vzrsort *create_ibea2(void);

VZRLIB_EXPORT struct vzrsort *create_knee_full(int prof, double taux);
VZRLIB_EXPORT struct vzrsort *create_knee(void);
VZRLIB_EXPORT struct vzrsort *create_elbow_full(int prof, double taux);
VZRLIB_EXPORT struct vzrsort *create_elbow(void);
VZRLIB_EXPORT int moeadd_pop_size(int obj, int cit1, int cut2);
VZRLIB_EXPORT struct vzrsort *create_moeadd_full(int cut1, int cut2, int vois);
VZRLIB_EXPORT struct vzrsort *create_moeadd(int cut1, int cut2);

VZRLIB_EXPORT struct vzrsort *create_modal_full(int vois);
VZRLIB_EXPORT struct vzrsort *create_modal(void);

#ifdef __cplusplus
}
#endif

#endif
