//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   nloptMPI.cxx
 \author Jean-Baptiste Blanchard <jean-baptiste.blanchard@©ea.fr>
 \date   lun nov 23 11:08:44 2009
 
 \brief  "Pseudo" Code \b created to play with hollowbar with reoptimizer using MPI.

 $Revision$
 $State$
 $Date$

 \par Revision
 \li JBB 2017-10-24
 Creation
 \endverbatim
 */

#ifndef __CLING__
#ifdef WIN32
#include <regex>
#endif
#include "TAttribute.h"
#include "TCodeEval.h"
#include "TFlatScript.h"
#include "TDataServer.h"
#include "TFlatResult.h"
#include "TKeyResult.h"
#include "TLineResult.h"
#include "TCJitEval.h"
#include "TComposeEval.h"
#include "TGradientEstimationRun.h"
#include "TMpiRun.h"
#include "TCJitEval.h"
#include "TCIntEval.h"
#include "TROOT.h"
#include "TNloptDirect.h"
#include "TNloptCobyla.h"
#include "TNlopt.h"
#include "TOptimFit.h"

// CHRONO
#include <chrono>
using std::chrono::high_resolution_clock;
using std::chrono::duration_cast;
using std::chrono::duration;
using std::chrono::milliseconds;

void usage(const char *scmd)
{
    printf("\n");
    printf("Usage: %s MMA | evalType Neval \n", scmd);
    printf("\n");
    printf("  Specific method to test standalone MPI distribution with reoptimizer. Very specific do not use it unless discussed with writters.\n");
    printf("\t It is used to check the consistency of MPI distributed optimisation using .\n");
    printf("\t   MMA          : MMA solver (Subplexe if this flag is not used). MMA could only be used with gradDec.\n");
    printf("\t   initInConst  : Use the TMpiRun and pass argc and argv to the constructor to init mpi.. \n");
    printf("\t   evalType     : The main evaluator for hollowbar ( 0 <-> CInt, 1 <-> CJit, 2 <-> Code with LineResult , 3 <-> Code with FlatResult , 4 <-> Code with KeyResult)\n");

}

void combineCost(double *param, double *res)
{
    double SectRef=0.55; // Reference Value used as a target for sect
    double DefRef=3; // Reference value used a target for deformation
    
    double sectDiff = (param[0]-SectRef)*(param[0]-SectRef); // simple distance for section
    double defDiff =  (param[1]-DefRef)*(param[1]-DefRef);  // simple distance for deformation
    res[0] = sectDiff + defDiff; // combination of both criteria with equal weight
}
#endif

void barAllCost(double *param, double *res)
{
  double x = param[0];
  double y = param[1];
  /* thickness constraint */
  res[0] = x-y; 
  /* section criterion */
  //res[1] = 10 * (x*x - y*y);
  res[1] = x*x - y*y;
  /* deformation criterion */
  res[2] = 1. / (  x*x*x*x - y*y*y*y);
}

#ifndef __CLING__
//int toto()//int main()//
int main(int argc, char **argv)//int GradientBarWithCodePlusCompo()
{    

    high_resolution_clock::time_point start, end;   
    start = high_resolution_clock::now();
    
    bool initInConst=false;
    bool MMA=false;
    int evalType=0; // 0 <-> CInt, 1 <-> CJit, 2 <-> LineResult , 3 <-> FlatResult , 4 <-> KeyResult
  
    if(argc>1)
    {
        for (int iarg=1; iarg<argc; iarg++)
        {
            // if( string(argv[iarg]) == "gradDec") gradDec=true;
            if( string(argv[iarg]) == "MMA") MMA=true;
            else if( string(argv[iarg]) == "initInConst") initInConst=true;
            else if( string(argv[iarg]) == "evalType") { evalType=atof(argv[iarg+1]); iarg++;}
            else
            {
                cout<<"c quoi ce merdier ? "<<string(argv[iarg])<<endl;
                usage("nloptmpi");
                return 0;
            }
        }
    }
    
    using namespace URANIE::DataServer;
    using namespace URANIE::Relauncher;
    using namespace URANIE::MpiRelauncher;
    using namespace URANIE::Reoptimizer;

    // variables
    TAttribute x("x", 0.0, 1.0),
        y("y", 0.0, 1.0),
        thick("thick"),
        sect("sect"),
        def("def"), 
        combine("combination");

    if(!initInConst)
        MPI_Init(&argc, &argv);

    TEval *code=NULL;
    TCodeResult *outputfile=NULL;
    TFlatScript inputfile("hollowbar_input.txt");
    if(evalType==0)
    {
        string this_file =  __FILE__;
#ifdef WIN32
        regex r("^\\\\[[:w:]]");
        if (regex_search(this_file, r))
            this_file.insert(0, 1, '\\');
#endif
        gROOT->LoadMacro(this_file.data());
        code = new TCIntEval("barAllCost");
        ((TCIntEval*)code)->addInput(&x);
        ((TCIntEval*)code)->addInput(&y);
        ((TCIntEval*)code)->addOutput(&thick);
        ((TCIntEval*)code)->addOutput(&sect);
        ((TCIntEval*)code)->addOutput(&def);
    }
    else if(evalType==1)
    {
        code=new TCJitEval(barAllCost);
        ((TCJitEval*)code)->addInput(&x);
        ((TCJitEval*)code)->addInput(&y);
        ((TCJitEval*)code)->addOutput(&thick);
        ((TCJitEval*)code)->addOutput(&sect);
        ((TCJitEval*)code)->addOutput(&def);
    }
    else if (evalType < 5)
    {
        TString cmd;
        if(evalType==2)
        {
            cmd="line";
            outputfile=new TLineResult("_hollowbar_output_line_.txt");
            ((TLineResult*)outputfile)->addOutput(&thick);
            ((TLineResult*)outputfile)->addOutput(&sect);
            ((TLineResult*)outputfile)->addOutput(&def);
        }
        else if(evalType==3)
        {
            cmd="flat";
            outputfile=new TFlatResult("_hollowbar_output_flat_.txt");
            ((TFlatResult*)outputfile)->addOutput(&thick);
            ((TFlatResult*)outputfile)->addOutput(&sect);
            ((TFlatResult*)outputfile)->addOutput(&def);
        }
        else if(evalType==4)
        {
            cmd="key";
            outputfile=new TKeyResult("_hollowbar_output_key_.txt");
            ((TKeyResult*)outputfile)->addOutput(&thick,"thick");
            ((TKeyResult*)outputfile)->addOutput(&sect,"sect");
            ((TKeyResult*)outputfile)->addOutput(&def,"def");
        }
        
        // Link to the evaluation code
        code = new TCodeEval("hollowbar "+cmd);
        inputfile.addInput(&x);
        inputfile.addInput(&y);
        ((TCodeEval*)code)->addInputFile(&inputfile);
        ((TCodeEval*)code)->addOutputFile(outputfile);
    }
    else
        return 0;
    
    // Dummy combination of the criteria
    TCJitEval pouet(combineCost);
    pouet.addInput(&sect);
    pouet.addInput(&def);
    pouet.addOutput(&combine);

    TComposeEval comp;
    comp.addEval(code);
    comp.addEval(&pouet);    

    // Create a runner
    TMpiRun *runner;

    if(!initInConst)
        runner = new TMpiRun(&comp); // Paralellised locally (shared memory technic)    
    else
    {
        runner = new TMpiRun(&comp, &argc, &argv); // Paralellised locally (shared memory technic)
        cout<<"You're using the new constructor "<<endl;
    }
     
    runner->startSlave();
    if(runner->onMaster())
    {
        // Create the TDS
        TDataServer tds("OptimDemo", "Param de l'opt vizir pour la barre");
        tds.addAttribute(&x);
        tds.addAttribute(&y);
        tds.keepFinalTuple(false);
        
        TNloptSolver *solv;        
        if(!MMA) solv=new TNloptSubplexe(); 
        else solv= new TNloptMMA(); 
        
        // Create the multi-objective constrained optimizer
        TNlopt opt(&tds, runner, solv);        
        // add the objective
        opt.addObjective(&combine);
        // Choose the limit for evaluation
        opt.setTolerance(0.000001); // Tolerance to be used (maybe too small)
        opt.setMaximumEval(10000);
        
        /* chosing starting point */
        double point[2] = {0.9 , 0.4};
        opt.setStartingPoint(2,point);

        // Perform the optimisation
        opt.solverLoop();
        
        // solution
        tds.exportData("_hollowBar_exported.dat");
        
        // Stop the slave processes
        runner->stopSlave();

        delete solv;
    }

    delete code;
    if( outputfile != NULL) delete outputfile;
    delete runner;
    
    end = high_resolution_clock::now();
    /* Getting number of milliseconds as a double. */
    duration<double, std::milli> ms_double = end - start;
    std::cout << "\n End of the code itself " << ms_double.count() << " ms" << std::endl;
    return 0;
}
#endif
