/* differents cas de saturation des contraintes
 #define ZZZ 0.4 // 3 saturées
 #define ZZZ 0.55 // 2 saturées
 #define ZZZ 0.65 // aucune saturées
 * */
#define ZZZ 0.55 // 2 saturées

/* possible solver
 * direct
 #define SOLV TNloptCobyla
 * lagragien direct
 #define SOLV TNloptBobyqa //doesn't work on this problem
 #define SOLV TNloptPraxis
 #define SOLV TNloptNelderMead
 #define SOLV TNloptSubplexe
 * derivative
 #define SOLV TNloptMMA
 #define SOLV TNloptSLSQP
 * lagragien derivative
 #define SOLV TNloptLBFGS
 #define SOLV TNloptNewton
 #define SOLV TNloptVariableMetric
 * */
#define SOLV TNloptVariableMetric

/*
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TDataServer.h"
#include "TCJitEval.h"
#include "TThreadedRun.h"
#include "TSequentialRun.h"
//#include "TMpiRun.h"
#include "TOptimFit.h"
#include "TNloptCobyla.h"
#include "TNloptDirect.h"
#include "TNlopt.h"
*/

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

/********** User Function *****************/
static int fselect(double *in, double *res, double *grad)
{
    double x, y, z;

    x = in[0];
    y = in[1];
    z = in[2];
    /* critere */
    // cumul des distances aux points (1,0.5,ZZZ) et (0.5,1,ZZZ)
    res[0] = (x - 1.) * (x - 1.) + (y - .5) * (y - 0.5)
            + 2 * (z - ZZZ) * (z - ZZZ) + (x - 0.5) * (x - .5)
            + (y - 1.) * (y - 1.);
    /* contraintes */
    res[1] = x * x + y * y + z * z; // distance au point (0,0,0)
    res[2] = (x - 1.5) * (x - 1.5) + (y - 0.5) * (y - 0.5) + z * z; // point (1.5,0.5,0)
    res[3] = (x - 0.5) * (x - 0.5) + (y - 1.5) * (y - 1.5) + z * z; // point (0.5,1.5,0)

    if (grad)
    {
        grad[0] = 2 * (x - 1.) + 2 * (x - .5);
        grad[1] = 2 * (y - .5) + 2 * (y - 1.);
        grad[2] = 4 * (z - ZZZ);

        grad[3] = 2 * x;
        grad[4] = 2 * y;
        grad[5] = 2 * z;

        grad[6] = 2 * (x - 1.5);
        grad[7] = 2 * (y - .5);
        grad[8] = 2 * z;

        grad[9] = 2 * (x - .5);
        grad[10] = 2 * (y - 1.5);
        grad[11] = 2 * z;
    }
    return 1;

}

/********** Optimization  *****************/
void test_TNloptMonoConstraint()
{
    /* inputs */
    TAttribute x1("x_{1}", -3.0, 3.0), x2("x_{2}", -3.0, 3.0), x3("x_{3}", -3.0,
            3.0);
    /* outputs */
    TAttribute cont1("c_{1}"), cont2("c_{2}"), cont3("c_{3}"), obj("fc1");
    /* starting point */
    double start[][3] =
    {
    { 1.5, 1.5, 1.5 },
    { -1.5, -1.5, -1.5 } };

    /* code */
    TCJitGradEval code(&fselect);
    code.setInputs(3, &x1, &x2, &x3);
    code.setOutputs(4, &obj, &cont1, &cont2, &cont3);

    /* run */
    TThreadedRun run(&code, 2);
    //TSequentialRun run(&code);
    //TMpiRun run(&code);
    run.startSlave();
    if (run.onMaster())
    {
        /* tds */
        TDataServer tds("toto", "tds for vizir test");
        // reverse output test
        tds.addAttribute(&x3);
        tds.addAttribute(&x2);
        tds.addAttribute(&x1);
        /* optimizer */
        SOLV solv;
        TGreaterFit it(1.0);
        TNlopt nlo(&tds, &code, &solv);
        nlo.addConstraint(&cont3, &it);
        nlo.addConstraint(&cont2, &it);
        nlo.addConstraint(&cont1, &it);
        nlo.addObjective(&obj);

        /* resolution */
        nlo.setStartingPoint(3,start[0]);
        nlo.setStartingPoint(3,start[1]);
        nlo.solverLoop();

        run.stopSlave();
        /* resultats */
        tds.getTuple()->Scan("*");
    }

}
