#ifdef JIT
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TDataServer.h"
#include "TCJitEval.h"
#include "TSequentialRun.h"
#include "TOptimFit.h"
#include "TNloptCobyla.h"
#include "TNloptDirect.h"
#include "TNlopt.h"

#endif /* JIT */

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

#define SIZE 10

/*** user defined ***/

/* possible solver
 * direct
 #define SOLV TNloptCobyla
 #define SOLV TNloptBobyqa
 #define SOLV TNloptPraxis
 #define SOLV TNloptNelderMead
 #define SOLV TNloptSubplexe
 #define SOLV TNloptMMA
 #define SOLV TNloptSLSQP
 #define SOLV TNloptLBFGS
 #define SOLV TNloptNewton
 #define SOLV TNloptVariableMetric
 */
#define SOLV TNloptMMA

int rosenbrock(Double_t *in, Double_t *res, Double_t *grad)
{
    double ret, next;
    int i;

    if (grad)
    {
        grad[0] = 2 * (in[0] - 1.);
        for (i = 1; i < SIZE; i++)
            grad[i] = 0.;
        for (i = 1; i < SIZE; i++)
        {
            grad[i - 1] += 4 * in[i - 1] * (in[i - 1] * in[i - 1] - in[i]);
            grad[i] += 2 * (in[i] - in[i - 1] * in[i - 1]);
        }
    }
    ret = (in[0] - 1.) * (in[0] - 1.);
    for (i = 1; i < SIZE; i++)
    {
        next = in[i - 1] * in[i - 1] - in[i];
        ret += next * next;
    }
    res[0] = ret;
    return 1;
}

void test_TNloptRosenbrock()
{

    TAttribute x0("x0", -4.0, 4.0);
    TAttribute x1("x1", -4.0, 4.0);
    TAttribute x2("x2", -4.0, 4.0);
    TAttribute x3("x3", -4.0, 4.0);
    TAttribute x4("x4", -4.0, 4.0);
    TAttribute x5("x5", -4.0, 4.0);
    TAttribute x6("x6", -4.0, 4.0);
    TAttribute x7("x7", -4.0, 4.0);
    TAttribute x8("x8", -4.0, 4.0);
    TAttribute x9("x9", -4.0, 4.0);
    TAttribute rose("rose");

    //double start[] = {-0.5, 2., 2., 2., 2., 2., 2., 2., 2., 2.};
    double start[] =
    { -0.5, -0.5, -0.5, -0.5, -0.5, -0.5, -0.5, -0.5, -0.5, -0.5, };

#ifdef JIT
    TCJitGradEval code(&rosenbrock);
#else
    TCJitGradEval code(&rosenbrock);
#endif
    code.setInputs(10, &x0, &x1, &x2, &x3, &x4, &x5, &x6, &x7, &x8, &x9);
    code.setOutputs(1, &rose);

    TDataServer tds("vizirDemo", "Param de l'opt vizir pour la barre");
    code.addAllInputs(&tds);

    SOLV solv;
    TNlopt opt(&tds, &code, &solv);
    opt.addObjective(&rose);
    opt.setTolerance(1.e-9);

    /* resolution */
    opt.setStartingPoint(10,start);
    opt.solverLoop();

    /* solution */
    tds.getTuple()->Scan("*");

}
