
//#define SWARM 

#ifdef JIT
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TDataServer.h"
#include "TCJitEval.h"
#include "TSequentialRun.h"
#include "TOptimFit.h"
#ifdef SWARM
#include "TVizirSwarm.h"
#else
#include "TVizirGenetic.h"
#endif
#include "TVizir2.h"

#endif /* JIT */

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

/* user defined */
//#define SLOW 500
#define TOLERANCE 0.001
#define NBmaxEVAL 50000
#define TAILLEPOP 400 
//#define TAILLEPOP 10 

/* constantes */
#define NIV 255.0 //niveau de gris
#define TAB 8 // nombre de régions
#define NBGENE 7 // nombre de paramètres

/* modèle */
static const char tgr[TAB * TAB] = {
    /*  0  1  2  3  4  5  6  7 */
    0, 1, 1, 1, 0, 0, 1, 1, // 0
    1, 0, 1, 1, 1, 0, 0, 0, // 1
    1, 1, 0, 0, 1, 0, 0, 0, // 2
    1, 1, 0, 0, 0, 1, 1, 0, // 3
    0, 1, 1, 0, 0, 1, 0, 1, // 4
    0, 0, 0, 1, 1, 0, 1, 0, // 5
    1, 0, 0, 1, 0, 1, 0, 1, // 6
    1, 0, 0, 0, 1, 0, 1, 0  // 7
};

static const char *tangram[TAB] = { 
    &tgr[0], &tgr[TAB], &tgr[2 * TAB], &tgr[3 * TAB], &tgr[4 * TAB],
    &tgr[5 * TAB], &tgr[6 * TAB], &tgr[7 * TAB] 
};

struct local {
    int size;
    const char **dat;
};

struct local tang = { TAB, tangram };

double contraste(struct local *self, double *sol)
{
    int i, j;
    double min, diff;

    min = NIV;
#ifdef SLOW
    for (int k=0; k<SLOW; k++)
#endif
    for (i = 0; i < self->size; i++)
    {
        for (j = i + 1; j < self->size; j++)
        {
            if (self->dat[i][j])
            {
                diff = fabs(sol[i] - sol[j]);
                if (diff < min)
                    min = diff;
            }
        }
    }
    return min;
}

/* wrapper */
void mydecode(int size, double *intern, double *user)
{
    int i;

    user[0] = NIV; /* bord blanc imposé */
    for (i = 1; i < size; i++)
        user[i] = intern[i - 1];
}

int myfitness(double *in, double *out, void *info)
{
    struct local *tan = (struct local *) info;
    double all[TAB]; // tan->size
    
    mydecode(tan->size, in, all);
    out[0] = contraste(tan, all);
    return 1;
}

/* solver */
void test_TVizirMono()
{
    /* init */
    TAttribute z1("zone1", 0., 255.), z2("zone2", 0., 255.), z3("zone3", 0.,
            255.), z4("zone4", 0., 255.), z5("zone5", 0., 255.), z6("zone6", 0.,
            255.), z7("zone7", 0., 255.), ct("contrast");

#ifdef JIT
    TCJitEval code(&myfitness, &tang);
#else
    TCIntEval code((void *) &myfitness, &tang);
#endif
    code.setInputs(NBGENE, &z1, &z2, &z3, &z4, &z5, &z6, &z7);
    code.setOutputs(1, &ct);

    TDataServer tdsvzr("tdsvzr", "tds for vizir test");
    code.addAllInputs(&tdsvzr);

    TMaximizeFit maximize;
#ifdef SWARM
    TVizirSwarm solv;
#else
    TVizirGenetic solv;
    solv.setSurvivalRate(0.5);
    solv.setSize(TAILLEPOP, NBmaxEVAL);
#endif
    TVizir2 vzr(&tdsvzr, &code, &solv);
    vzr.addObjective(&ct, &maximize);
    vzr.setTolerance(TOLERANCE);

    /* resolution */
    vzr.solverLoop();

    /* solution */
    tdsvzr.exportData("_int_.dat");
    tdsvzr.getTuple()->Scan("*");
    //tdsvzr.draw("-1.0*contrast:zone2");
    //tdsvzr.draw("generation","","");
    //tdsvzr.startViewer();
}
