#ifdef JIT
#include "TAttribute.h"
#include "TFlatScript.h"
#include "TKeyScript.h"
#include "TFlatResult.h"
#include "TCodeEval.h"
#include "TThreadedRun.h"
#include "TDataServer.h"
#include "TVizirSwarm.h"
#include "TVizir2.h"
#endif

/* user defined */

/* solutions
 *        0 *         0 *      0.05 * 47486.828 * 92534.018 *      63.1 *       990 *       820 *      1680 *      9855 *  7.819721 *        19 *
 *        0 *         0 *      0.05 * 47040.140 * 98023.885 *      63.1 *       990 *       820 *      1680 *      9855 *   7.81973 *        19 *
 *        0 *         0 * 0.0500000 * 38059.776 * 98523.004 *      63.1 *       990 * 819.99994 *      1680 * 9855.0002 *  7.819933 *        26 *

 */

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

void test_TVizirMonoFlowrate()
{
    // Define the DataServer

    // Add the study attributes ( min, max and nominal values)
    TAttribute rw("rw", 0.05, 0.15), r("r", 100.0, 50000.0), tu("tu", 63070.0,
            115600.0), tl("tl", 63.1, 116.0), hu("hu", 990.0, 1110.0), hl("hl",
            700.0, 820.0), l("l", 1120.0, 1680.0), kw("kw", 9855.0, 12045.0);
    TAttribute yh("yhat"), d("d");

    // Set the reference input file and the key for each input attributes
    //TKeyScript fin("flowrate_input_with_keys.in");
    //TKeyScript fin("flowrate.py");
    //fin.addVariables(8, &rw, "Rw", &r, "R", &tu, "Tu", &tl, "Tl",
    //        &hu, "Hu", &hl, "Hl", &l, "L", &kw, "Kw");
    TFlatScript fin("flowrate_input_with_values_rows.in");
    fin.setInputs(8, &rw, &r, &tu, &tl, &hu, &hl, &l, &kw);

    // The output file of the code
    //TFlatResult fout("_output_flowrate_withRow_.dat");
    TFlatResult fout("_output_flowrate_withRow_.dat");
    //TFlatResult fout("flowrate.res");
    // The attribute in the output file
    fout.setOutputs(2, &yh, &d);

    // Instanciation de mon code
    //TCodeEval mycode("flowrate -k >> /dev/null");
    TCodeEval mycode("flowrate -r >> /dev/null");
    //TCodeEval mycode("python flowrate.py > flowrate.res");
    mycode.setWorkingDir(
            gSystem->Getenv("HOME") + TString("/tmp/LanceurUranie/flowrate"));
    mycode.addInputFile(&fin);
    mycode.addOutputFile(&fout);

    //TSequentialRun run(&mycode);
    TThreadedRun run(&mycode, 14);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsFlowrate",
                "OATMinMax Design of experiments for Flowrate");
        tds.addAttribute(&rw);
        tds.addAttribute(&r);
        tds.addAttribute(&tu);
        tds.addAttribute(&tl);
        tds.addAttribute(&hu);
        tds.addAttribute(&hl);
        tds.addAttribute(&l);
        tds.addAttribute(&kw);

        TVizirSwarm solv;
        TVizir2 vzr(&tds, &run, &solv);
        vzr.addObjective(&yh);

        /* resolution */
        vzr.solverLoop();

        /* solution */
        //tds.getTuple()->Scan("*");
        tds.exportData("_flow_.dat");
        run.stopSlave();
    }

}

