using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

/* user defined */
#define TOLERANCE 0.001
#define NBmaxEVAL 20000
//#define NBmaxEVAL 202
#define TAILLEPOP 200 

/* solutions
 *        0 *         0 *      0.05 * 47486.828 * 92534.018 *      63.1 *       990 *       820 *      1680 *      9855 *  7.819721 *        19 *
 *        0 *         0 *      0.05 * 47040.140 * 98023.885 *      63.1 *       990 *       820 *      1680 *      9855 *   7.81973 *        19 *
 *        0 *         0 * 0.0500000 * 38059.776 * 98523.004 *      63.1 *       990 * 819.99994 *      1680 * 9855.0002 *  7.819933 *        26 *

 */

void test_TVizirMonoMpiFlowrate(void)
{
    // Define the DataServer

    // Add the study attributes ( min, max and nominal values)
    TAttribute rw("rw", 0.05, 0.15);
    TAttribute r("r", 100.0, 50000.0);
    TAttribute tu("tu", 63070.0, 115600.0);
    TAttribute tl("tl", 63.1, 116.0);
    TAttribute hu("hu", 990.0, 1110.0);
    TAttribute hl("hl", 700.0, 820.0);
    TAttribute l("l", 1120.0, 1680.0);
    TAttribute kw("kw", 9855.0, 12045.0);
    TAttribute yh("yhat");

    // Set the reference input file and the key for each input attributes
    //TKeyScript fin("flowrate_input_with_keys.in");
    TKeyScript fin("flowrate1.py");
    fin.setInputs(8, &rw, "Rw", &r, "R", &tu, "Tu", &tl, "Tl", &hu, "Hu", &hl,
            "Hl", &l, "L", &kw, "Kw");

    // The output file of the code
    //TFlatResult fout("_output_flowrate_withRow_.dat");
    TFlatResult fout("flowrate.res");
    // The attribute in the output file
    fout.addOutput(&yh);

    // Instanciation de mon code
    //TCodeEval mycode("flowrate -k >> /dev/null");
#ifdef PYTHON3
    TCodeEval mycode("python2 flowrate1.py > flowrate.res");
#else
    TCodeEval mycode("python flowrate1.py > flowrate.res");
#endif
    //mycode.setWorkingDir(
    //        gSystem->Getenv("HOME") + TString("/tmp/LanceurUranie/flowrate"));
    mycode.addInputFile(&fin);
    mycode.addOutputFile(&fout);

    TMpiRun run(&mycode);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsFlowrate",
                "OATMinMax Design of experiments for Flowrate");
        tds.addAttribute(&rw);
        tds.addAttribute(&r);
        tds.addAttribute(&tu);
        tds.addAttribute(&tl);
        tds.addAttribute(&hu);
        tds.addAttribute(&hl);
        tds.addAttribute(&l);
        tds.addAttribute(&kw);

        //TVizirGenetic solv;
        //TVizirSimplex solv;
        //solv.setSize(TAILLEPOP, NBmaxEVAL);
        TVizirSwarm solv;
        solv.setSize(TAILLEPOP, NBmaxEVAL, 16);
        TVizir2 vzr(&tds, &run, &solv);
        vzr.addObjective(&yh);
        vzr.setTolerance(TOLERANCE);

        /* resolution */
        vzr.solverLoop();
        run.stopSlave();

        /* solution */
        //tds.getTuple()->Scan("*");
        tds.exportData("_mpiflow_.dat");
    }

}

