
#ifdef JIT
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TAttribute.h"
#include "TTangramEval.h"
#include "TThreadedRun.h"
#include "TDataServer.h"
#include "TOptimFit.h"
#include "TVizirSimplex.h"
#include "TVizirGenetic.h"
#include "TVizirIsland.h"

#endif

using namespace URANIE::DataServer;
using namespace URANIE::Relauncher;
using namespace URANIE::Reoptimizer;

/* user defined */
#define TOLERANCE 0.001
#define NBmaxEVAL 2000
#define STEP 16 
#define STOP 128
//#define TAILLEPOP 10 

#define NBGENE 7 // nombre de paramètres

/* solver */
void test_TVizirThreaded()
{
    /* init */
    TAttribute z1("zone1", 0., 255.), z2("zone2", 0., 255.), z3("zone3", 0.,
            255.), z4("zone4", 0., 255.), z5("zone5", 0., 255.), z6("zone6", 0.,
            255.), z7("zone7", 0., 255.), ct("contrast");

    TTangramEval code;
    code.setInputs(NBGENE, &z1, &z2, &z3, &z4, &z5, &z6, &z7);
    code.addOutput(&ct);

    TThreadedRun run(&code, 14);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsvzr", "tds for vizir test");
        code.addAllInputs(&tds);
        TDataServer trc("tdstrc", "tds for vizir trace"); 

#ifdef JIT
        TVizirGenetic solv;
        if (solv.setResume(NBmaxEVAL, "genetic.dump"));
        else solv.setSize(STOP, NBmaxEVAL);
#else
        TVizirSimplex solv;
        if (solv.setResume(NBmaxEVAL, "swarm.dump"));
        else solv.setSize(STOP, NBmaxEVAL, STEP);
#endif
        //TVizirIsland vzr(&tds, &run, &solv);
        TVizir2 vzr(&tds, &run, &solv);
        vzr.setTolerance(TOLERANCE);
        TMaximizeFit maxim;
        vzr.addObjective(&ct, &maxim);
        vzr.setTrace(&trc);

        /* resolution */
        vzr.solverLoop();
        if (vzr.isConverged()) cout << "converged\n" ;
        else cout << "unconverged\n";

        //tds.getTuple()->Scan("*");
        tds.getNPatterns();
        //tds.setLog();
        tds.exportData("_thread_.dat");
        trc.exportData("_thread_trc_.dat");
        //vzr.save("_thread_.dat");
        run.stopSlave();
    }

}
