//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TClaytonCopula.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mer oct 17 18:08:40 CEST 2012
 
 \brief Interface of the class URANIE::Sampler::TClaytonCopula
 
 */
/**
 @class URANIE::Sampler::TClaytonCopula
 
 @brief Description of the class TClaytonCopula.
 
 The \b Clayton \b copula for bivariate is defined by the formula
 \f[
 C(u_1, u_2 ; \theta) \: = \: \left( u_1^{-\theta} + u_2^{-\theta} - 1 \right)^{-1/\theta}
 \f]
 with \f$\theta \in [0, \infty]\f$.
 */
#ifndef TCLAYTONCOPULA_H
#define TCLAYTONCOPULA_H

////// URANIE
#include "TArchimedianCopula.h"
#include "UExceptions.h"

namespace URANIE
{
namespace Sampler
{
class TClaytonCopula: public TArchimedianCopula
{
private:
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TClaytonCopula(URANIE::DataServer::TDataServer *tds, Double_t dparam,
            Int_t nCalcul);
    //! Default destructor
    virtual ~TClaytonCopula();
    //@}

    //---------------------------------------------
    /** @name Generation of the sample
     **/
    //@{
    //! Generates the matrix of U
    /*!
     \param option (Option_t *) The option to pass.
     */
    void generateUMatrix(Option_t *option);
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TClaytonCopula, ID_SAMPLER)
    // TClaytonCopula Class
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
