//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCopula.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   sat aug the 9th 10:05:13 CEST 2008
 
 \brief Interface of the class URANIE::Sampler::TCopula
 
 */
/**
 
 * @class URANIE::Sampler::TCopula
 *
 * @brief Description of the class TCopula.
 *
 * To be written by the developper.
 *
 */
#ifndef TCOPULA_H
#define TCOPULA_H

////// ROOT
#include "TMatrixD.h"

////// URANIE
#include "TSamplerStochastic.h"

#include <iostream>
using std::cout;
using std::endl;

namespace URANIE
{
namespace Sampler
{
class TCopula: public TSamplerStochastic
{
    // Attributes
public:
    TMatrixD _matUValue;                      ///< The matrix of U values       
private:
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Constructor with a TDataServer, the options and the size of the sample
    TCopula(URANIE::DataServer::TDataServer *tds, Option_t *option,
            Int_t nCalcul);

    //! Default destructor
    virtual ~TCopula();
    //@}

    //---------------------------------------------
    /** @name Generation of the sample
     **/
    //@{
    //! Generates the sample
    /*!
     \param option (Option_t *) The option to pass.
     */
    void generateSample(Option_t *option = "");
    //! Generates the matrix of U
    /*!
     \param option (Option_t *) The option to pass.
     */
    virtual void generateUMatrix(Option_t *option) = 0;
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TCopula, ID_SAMPLER)
    //Classe de
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
