//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TDistribution.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   Tue Mar  7 14:06:02 2006
 
 \brief  Creation of the abstract class TDistribution
 
 
 */
#ifndef TDISTRIBUTION_H
#define TDISTRIBUTION_H

#ifndef ROOT_TNtupleD
#include "TNtupleD.h"
#endif

#ifndef ROOT_TFile
#include "TFile.h"
#endif

#ifndef ROOT_TVectorD
#include "TVectorD.h"
#endif

#ifndef ROOT_TMatrixD
#include "TMatrixD.h"
#endif

#ifndef ROOT_TRandom3
#include "TRandom3.h"
#endif

#include "Sampler.h"

#include <iostream>
using std::cout;
using std::endl;

#include "Rtypes.h"

class TDistribution: public TObject
{
    // Associations
    // Attributes
protected:
    /*! \brief Generator of random number*/
    TRandom3* _rdm;
    /*! \brief Weight of the distribution*/
    Double_t _dprob;
    /*! \brief Sample size*/
    Int_t _taille;
    /*! \brief Tuple where simulated samples are stored */
    TNtupleD* _vect;
    /*! \brief Dimension of the distribution*/
    Int_t _dim;
    /*! \brief Mean value of the distribution*/
    TVectorD _vectMean;
    /*! \brief "_matEcartTypeCorr" represents the \f$\sigma_i\f$ along the diagonal and \f$\rho_{i,j}\f$ elsewhere. Be careful the matrix must be symmetric
     */
    TMatrixD _matEcartTypeCorr;
    // Operations
public:
    TDistribution();
    TDistribution(TVectorD M, TMatrixD A);
    void initialisation(TVectorD M, TMatrixD A);
    ~TDistribution();
    TNtupleD* getTuple()
    {
        return _vect;
    }
    //TFile* getFile() {return rootFile;}
    void setTaille(Int_t n);
    TRandom3* getRandom()
    {
        return _rdm;
    }
    Int_t getTaille()
    {
        return _taille;
    }
    Int_t getDim();
    TVectorD getMean();
    TVectorD getSigma();
    TMatrixD getMatEcartTypeCorrelation();
    TMatrixD getMatCorrelation();
    TMatrixD getMatCovariance();
    void setMean(TVectorD M);
    void setMatEcartTypeCorrelation(TMatrixD A);
    void setProb(Double_t proba)
    {
        _dprob = proba;
    }
    Double_t getProb()
    {
        return _dprob;
    }
    void simulation();
    void verification();ClassDef(TDistribution, ID_SAMPLER);
    // a abstract class of the distribution
};
#endif
