//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TEllipticalCopula.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   mer oct 17 17:36:39 CEST 2012
 
 \brief Interface of the class URANIE::Sampler::TEllipticalCopula
 
 */
/**
 @class URANIE::Sampler::TEllipticalCopula
 
 @brief Description of the class TEllipticalCopula.

 this familly of copula contains \b Gaussian and \b Student copulae.

 */
#ifndef TELLIPTICALCOPULA_H
#define TELLIPTICALCOPULA_H
#include "TCopula.h"

namespace URANIE
{
namespace Sampler
{
class TEllipticalCopula: public TCopula
{
    // Attributes
public:
    TMatrixD _matCorrelation;                  ///< The correlation matrix
private:
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TEllipticalCopula(URANIE::DataServer::TDataServer *tds, Option_t *option,
            Int_t nCalcul);
    //! Default destructor
    virtual ~TEllipticalCopula();
    //@}

    //---------------------------------------------
    /** @name Correlation matrix
     Methods to specify correlation between attributes
     **/
    //@{
    //! Defines a correlation between two attributes given by their indexes
    /*!
     \warning The indexes begin 0 to n-1.
     
     \param indx (Int_t) the index of the first attribute
     \param indy (Int_t) the index of the second attribute
     \param value (Double_t) the correlation factor between the two attributes
     */
    void setUserCorrelation(Int_t indx, Int_t indy, double value);

    //! Defines a correlation between two attributes given by their names
    /*!
     
     \param xname (TString) the first attribute
     \param yname (TString) the second attribute
     \param value (Double_t) the correlation factor between x and y
     */
    void setUserCorrelation(TString xname, TString yname, double value);

    //! Defines a correlation between two attributes
    /*!
     The correlation factor is define as the
     
     \param x (URANIE::DataServer::TAttribute *) the first attribute
     \param y (URANIE::DataServer::TAttribute *) the second attribute
     \param value (Double_t) the correlation factor between x and y
     */
    void setUserCorrelation(URANIE::DataServer::TAttribute *x,
            URANIE::DataServer::TAttribute *y, Double_t value);

    //! Retrieve the correlation matrix
    TMatrixD getCorrelationMatrix()
    {
        return _matCorrelation;
    }
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TEllipticalCopula, ID_SAMPLER)
    //  Elliptical Class
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
