//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef TMELANGE_H
#define TMELANGE_H

#ifndef ROOT_TNtupleD
#include "TNtupleD.h"
#endif
#ifndef ROOT_TFile
#include "TFile.h"
#endif
#ifndef ROOT_TRandom
#include "TRandom.h"
#endif
#ifndef ROOT_TArrayF
#include "TArrayF.h"
#endif

#include <iostream>
using std::cout;
using std::endl;

#include "Rtypes.h"
#include "TNormale.h"

class TList;
class TMelange
{
    // Associations
    // Attributes
private:
    TNormale* f;
    /*! \brief Normal law used for intermediary calculations*/
    TList* lLoi;
    /*! \brief Chained list used to go through the mixing*/
    Int_t _tailleMelange;
    Int_t _taille;
    /*! \brief Sample size to be simulated*/
    Int_t _dimMelange;
    TNtupleD* _nt;
    /*! \brief Tuple in which the sample is stocked*/
    TRandom* _rand;
    /*! \brief Generator of random number*/

    // Operations
public:
    TMelange(Int_t n);
    ~TMelange();
    void _printlog();
    void setTaille(Int_t n)
    {
        _taille = n;
    }
    Int_t getTaille()
    {
        return _taille;
    }
    Int_t getTailleMelange()
    {
        return _tailleMelange;
    }
    Double_t eval(TVectorD X);
    Double_t eval(Double_t x);
    Int_t getDimMelange()
    {
        return _dimMelange;
    }
    TNtupleD* getTuple()
    {
        return _nt;
    }
    TVectorD getSeuil();
    Double_t getProb(Int_t i);
    TVectorD getMean(Int_t i);
    TVectorD getMean();
    TVectorD getVariance();
    TVectorD getSigma();
    TMatrixD getMatEcartTypeCorrelation(Int_t i);
    TMatrixD getMatEcartTypeCorrelation();
    TVectorD getSigma(Int_t i);
    TMatrixD getMatCorrelation(Int_t i);
    TMatrixD getMatCovariance(Int_t i);
    TMatrixD getMatCovariance();
    TMatrixD getMatCorrelation();
    Double_t getObs();
    void simulationMelange();
    void addLoi(TVectorD moy, TMatrixD ecartTypeCorr, Double_t proba);
    void removeLoi(Int_t i);
    void addTuple(TNtupleD* t);
    TVectorD LoiDeZ(TVectorD X);
    TVectorD discrimination(TVectorD X);
    void reset();
    void remplacer(Int_t i, TVectorD M, TMatrixD A);
    void afficher();
};

#endif
