//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TSamplerStochastic.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   Wed Mar 8 09:16:13 2006
 
 \brief  Creation of the abstract class TSamplerStochastic
 
 
 */
#ifndef TSamplerStochastic_H
#define TSamplerStochastic_H

// Uranie
#include "TSampler.h"

namespace URANIE
{
namespace Sampler
{
class TSamplerStochastic: public TSampler
{

    // Attributes
protected:
    Int_t _nSeed;
    
    // Operations
protected:
    //---------------------------------------------
    /** @name State machine
     For the stochastic methods, two2 states can be created \em init and \em terminate
     which are executed just before the generation of the sample for the first one, and after the generation for the second one. They can be used for pre and post treatment purpose.
     **/
    //@{
    //! The preprocessing step
    virtual void init();
    //! The post-processing step
    virtual void terminate();
    //@}

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Constructor with a dataserver
    TSamplerStochastic(URANIE::DataServer::TDataServer *tds, Option_t *option,
            Int_t nCalcul);
    //! Default destructor
    virtual ~TSamplerStochastic();
    //@}

    //---------------------------------------------
    /** @name The Seed
     **/
    //@{
    //! Init the seed
    /*!
     If the value is zero, the seed is initialized with the hour in milliseconds
     
     \param ind (Int_t) the seed value
     */
    void setSeed(Int_t ind = 0);
    //! Returns the seed value
    Int_t getSeed()
    {
        return _nSeed;
    }
    //@}
    
    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    //! Prints the log
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sampler::TSamplerStochastic, ID_SAMPLER)
    //Abstract Class to generate random design.
};

} // Fin du namespace Sampler
} // Fin du namespace URANIE
#endif
// fin du fichier $RCSfile$.
