//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTNeuralGas.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 Created : 2011-12-15 13:51:45
 
 \brief  Test File of the class URANIE::UncertModeler::TNeuralGas

 */
#ifndef CPPUNITEST_TNEURALGAS_H
#define CPPUNITEST_TNEURALGAS_H

// CPPUnit
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TNeuralGas.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TNeuralGasTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TNeuralGasTest);
    CPPUNIT_TEST (testTMax);
    CPPUNIT_TEST (testSaveAll);
    CPPUNIT_TEST (testGenerateSampleWithLoop3);
    CPPUNIT_TEST (testGetSubSample);
    CPPUNIT_TEST (testGetSubSampleWithSaveAll);
    CPPUNIT_TEST (throwExceptionTMaxNegative);
    CPPUNIT_TEST (throwExceptionLoopOptionNotDigit);
    CPPUNIT_TEST (throwExceptionNGreaterThanNPat);
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TNeuralGasTest();
    //! Default destructor
    virtual ~TNeuralGasTest();
    //! Set Up Method
    void setUp();
    //! Tear Down method
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! get the Tmax parameter
    void testTMax();
    //! get the TVQ::bSaveAll parameter
    void testSaveAll();
    //! test generateSample with loop=3
    void testGenerateSampleWithLoop3();
    //! test the getSubSample method
    void testGetSubSample();
    //! test the getSubSample method with "saveall" option
    void testGetSubSampleWithSaveAll();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test of type throwException
     **/
    //@{
    //! Throw an exception when Tmax is not possitive
    void throwExceptionTMaxNegative();
    //! Throw an exception when not a digit in the loop option
    void throwExceptionLoopOptionNotDigit();
    //! Throw an exception when the nsize of subsample is greater than the size of the TDS
    void throwExceptionNGreaterThanNPat();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds1;

};

#endif  // CPPUNITEST_TNEURALGAS_H
