//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TOATSAMPLING_H
#define CPPUNITEST_TOATSAMPLING_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TOATSampling.h"
#include "UExceptions.h"
#include "TUniformDistribution.h"
#include "TNormalDistribution.h"

class TOATSamplingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TOATSamplingTest);
    CPPUNIT_TEST (testOATSampling);
    CPPUNIT_TEST (testOATSamplingWithStepValue);
    CPPUNIT_TEST (testTOATSamplingOptionFalse);
    CPPUNIT_TEST (testGenerateSampleSizeAttributModified);

    CPPUNIT_TEST (throwExceptionWhenNoDefaultValue);
    CPPUNIT_TEST (throwExceptionWhenNoBounds);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TOATSamplingTest();

    //! Default destructor
    virtual ~TOATSamplingTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of the OAT Sampling
    void testOATSampling();
    void testOATSamplingWithStepValue();
    void testTOATSamplingOptionFalse();
    void testGenerateSampleSizeAttributModified();
    //@}

    //---------------------------------------------
    /** @name Throw exceptions
     **/
    //@{ 
    //! Throw an exception when attributes are not properly set.
    /*!
     */
    void throwExceptionWhenNoDefaultValue();
    void throwExceptionWhenNoBounds();
    //@}

private:
    URANIE::DataServer::TUniformDistribution *att1;
    URANIE::DataServer::TUniformDistribution *att2;
    URANIE::DataServer::TUniformDistribution *att3;
    URANIE::DataServer::TNormalDistribution *attN; // unbounded normal distribution
    URANIE::DataServer::TUniformDistribution *attD; // attribute with no default value

};

#endif  // CPPUNITEST_TDATASERVER_H
