//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TPETRAS_H
#define CPPUNITEST_TPETRAS_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSampler.h"
#include "TPetras.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TPetrasTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TPetrasTest);

    CPPUNIT_TEST (testTPetrasConstructor);
    CPPUNIT_TEST (testTPetrasgenerateSample);

    CPPUNIT_TEST (throwExceptionTPetrasConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionTPetrasConstructorLevelNull);
    CPPUNIT_TEST (throwExceptionTPetrasConstructorLevelNoIntegerPositive);
    CPPUNIT_TEST (throwExceptionTPetrasConstructorLevelHigh);CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TPetrasTest();

    //! Default destructor
    virtual ~TPetrasTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    void testTPetrasConstructor();
    //! test the method generateSample()
    void testTPetrasgenerateSample();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test wich throw an exception
     **/
    //@{ 
    //! Verify the Exception throw if the Tds is Null
    void throwExceptionTPetrasConstructorTdsNull();
    //!Verify the Exception throw if the leval is null
    void throwExceptionTPetrasConstructorLevelNull();
    //!Verify the Exception throw if the level is not a positive integer
    void throwExceptionTPetrasConstructorLevelNoIntegerPositive();
    //! Verify the Exception throw if the level is too hight
    void throwExceptionTPetrasConstructorLevelHigh();
    //@}

private:
    URANIE::DataServer::TDataServer *_tds;
    URANIE::Sampler::TPetras *_sampler;

};

#endif  // CPPUNITEST_TPETRAS_H
