//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TQMC_H
#define CPPUNITEST_TQMC_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSampler.h"
#include "TQMC.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TQMCTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TQMCTest);
    CPPUNIT_TEST (testQMC);
    CPPUNIT_TEST (testQMCgenerateSampleOptionShift);
    CPPUNIT_TEST (throwExceptionQMCConstructorTdsNull);
    CPPUNIT_TEST (throwExceptionQMCHaltonUnbounded);
    CPPUNIT_TEST (throwExceptionQMCConstructorFalseOption);
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TQMCTest();

    //! Default destructor
    virtual ~TQMCTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of sampling with Sobol and Halton sequences
    void testQMC();
    //!
    void testQMCgenerateSampleOptionShift();

    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of throwException
     **/
    //@{
    void throwExceptionQMCConstructorTdsNull();
    void throwExceptionQMCHaltonUnbounded();
    void throwExceptionQMCConstructorFalseOption();

    //@}

private:
    URANIE::DataServer::TDataServer *_tds1;

};

#endif  // CPPUNITEST_TQMC_H
