//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TSAMPLING_H
#define CPPUNITEST_TSAMPLING_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSampler.h"
#include "TSampling.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TSamplingTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TSamplingTest);

    CPPUNIT_TEST (testSampling);
    CPPUNIT_TEST (testSamplingWithSeed);
    CPPUNIT_TEST (testgenerateSampleByDakota);
    CPPUNIT_TEST (testgenerateSampleByDakotaSeedNull);
    CPPUNIT_TEST (testTSamplingsetSaveRang);
    CPPUNIT_TEST (testTSamplingsetTypeSampling);
    CPPUNIT_TEST (testTSamplingsetUserCorrelation);
    CPPUNIT_TEST (testTSamplingConstrutorTypeFalse);
    CPPUNIT_TEST (testTSamplingNsEqual10);

    CPPUNIT_TEST (throwExceptionTSamplingConstrutorTdsNull);
    CPPUNIT_TEST (throwExceptionTSamplinggenerateSampleByDakota99991);
    CPPUNIT_TEST (throwExceptionTSamplinggenerateSampleByDakotaLawNoKnown);
    CPPUNIT_TEST (throwExceptionTSamplinggenerateSampleByDakotaMatCorrNegative);
    CPPUNIT_TEST (throwExceptionTSamplinggenerateSampleMatCorrNegative);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationSameIndex);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationIndexINegative);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationIndexJNegative);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationNameValueFalse);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationIndexValueFalse);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationAttributeValueFalse);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationIndexISup);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationIndexJSup);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationFalseNameI);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationFalseNameJ);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationFalseAttributI);
    CPPUNIT_TEST (throwExceptionTSamplingsetUserCorrelationFalseAttributJ);
    CPPUNIT_TEST (throwExceptionsetCorrelationMatrix);
    
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TSamplingTest();

    //! Default destructor
    virtual ~TSamplingTest();

    void setUp();
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test of the names
    void testSampling();
    //! Sampling Test with Seed
    void testSamplingWithSeed();
    //!
    void testgenerateSampleByDakota();
    //!
    void testgenerateSampleByDakotaSeedNull();
    //! 
    void testTSamplingsetSaveRang();
    //!
    void testTSamplingsetTypeSampling();
    //!
    void testTSamplingsetUserCorrelation();
    //!
    void testTSamplingConstrutorTypeFalse();
    //!
    void testTSamplingNsEqual10();

    //@}

    //---------------------------------------------
    /** @name throwException methods
     **/
    //@{
    //!
    void throwExceptionTSamplingConstrutorTdsNull();
    //!
    void throwExceptionTSamplinggenerateSampleByDakota99991();
    //! 
    void throwExceptionTSamplinggenerateSampleByDakotaLawNoKnown();
    //! 
    void throwExceptionTSamplinggenerateSampleByDakotaMatCorrNegative();
    //! 
    void throwExceptionTSamplinggenerateSampleMatCorrNegative();
    //! 
    void throwExceptionTSamplingsetUserCorrelationSameIndex();
    //! 
    void throwExceptionTSamplingsetUserCorrelationIndexINegative();
    //! 
    void throwExceptionTSamplingsetUserCorrelationIndexJNegative();
    //! 
    void throwExceptionTSamplingsetUserCorrelationNameValueFalse();
    //! 
    void throwExceptionTSamplingsetUserCorrelationIndexValueFalse();
    //! 
    void throwExceptionTSamplingsetUserCorrelationAttributeValueFalse();
    //! 
    void throwExceptionTSamplingsetUserCorrelationIndexISup();
    //! 
    void throwExceptionTSamplingsetUserCorrelationIndexJSup();
    //! 
    void throwExceptionTSamplingsetUserCorrelationFalseNameI();
    //! 
    void throwExceptionTSamplingsetUserCorrelationFalseNameJ();
    //! 
    void throwExceptionTSamplingsetUserCorrelationFalseAttributI();
    //! 
    void throwExceptionTSamplingsetUserCorrelationFalseAttributJ();
//!
    void throwExceptionsetCorrelationMatrix();
    
    //@}

private:
    URANIE::DataServer::TDataServer *_tds1;
    URANIE::DataServer::TDataServer *_tds;
    URANIE::Sampler::TSampling *_sampler;
    URANIE::Sampler::TSampling *_sampling;
    Int_t _nS;
};

#endif  // CPPUNITEST_TSAMPLING_H

