//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCMN.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   fri oct 10 10:02:29 CEST 2008

 \brief Interface of classe URANIE::Sensitivity::TCMN

 */
/**

 * @class URANIE::Sensitivity::TCMN
 *
 * @brief Description of the class TCMN.
 *
 * To be written by the developper.
 *
 */
#ifndef TCMN_H
#define TCMN_H
#include "TSensitivityTest.h"

namespace URANIE {
namespace Sensitivity {
class TCMN: public TSensitivityTest {
public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TCMN();
    //! Default constructor
    TCMN(URANIE::DataServer::TDataServer *tds, const char *varexpinput,
         const char *varexpoutput, Option_t * option = "");
    //! Default destructor
    virtual ~TCMN();
    //@}

    //---------------------------------------------
    /** @name Generation of the sample
     **/
    //@{
    virtual void generateSample(Option_t * option = "");
    //@}

    //---------------------------------------------
    /** @name Compute the indexes
     **/
    //@{
    void computeIndexes(Option_t * option = "");
    void evaluateIndexes(Option_t * option = "");
    Double_t getPValue()
    {
        return _dpvalue;
    }
    //@}

    //---------------------------------------------
    /** @name Printing Log
     **/
    //@{
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::Sensitivity::TCMN, ID_SENSITIVITY)

    //---------------------------------------------
    /** @name Private and Public attributes
     *
     **/
    //@{
private:
    Int_t _nBins;             //! The number of bins
    Double_t _dpvalue;        //! The pvalue for a variable
    //@}

};

} // End Of namespace Sensitivity
} // End Of namespace URANIE
#endif
