//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef CPPUNITEST_TMORRIS_H
#define CPPUNITEST_TMORRIS_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "TSensitivity.h"
#include "TScreeming.h"
#include "TMorris.h"
#include "TDataServer.h"
#include "UExceptions.h"

class TMorrisTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TMorrisTest);
 
    CPPUNIT_TEST (throwExceptionTMorrisConstructorTdsNull);
#ifdef ROOT5 // ROOT6 obsolete 
    CPPUNIT_TEST (throwExceptionTMorrisConstructorFalseFunction);
#endif
    CPPUNIT_TEST (throwExceptionTMorrisConstructorFalseCode);
    CPPUNIT_TEST (throwExceptionTMorrisConstructorFalseInput);
    CPPUNIT_TEST (throwExceptionTMorrisConstructorFalseOutput);
    CPPUNIT_TEST (throwExceptionTMorrisConstructorFalseNReplique);
    CPPUNIT_TEST (throwExceptionTMorrisConstructorFalseLevel);
    CPPUNIT_TEST (throwExceptionTMorrisConstructorFalseLevelandDelta);
    CPPUNIT_TEST (throwExceptionTMorrisConstructorFalseDelta);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputFunction);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputCode);
    CPPUNIT_TEST (throwExceptionConstructorTooLongOutputRun);
    CPPUNIT_TEST (throwExceptionTMorrisUnboundedInfiniteLaws);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedNormal,
        URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedLogNormal,
        URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedExponential,
        URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedCauchy,
        URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedGumbel,
        URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedWeibull,
        URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedGenPareto,
        URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedGamma,
        URANIE::Exceptions::UErrorExceptions);
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedInvGamma,
        URANIE::Exceptions::UErrorExceptions);
#ifdef WITH_MATHMORE
    CPPUNIT_TEST_EXCEPTION(throwExceptionConstructorUnboundedStudent,
        URANIE::Exceptions::UErrorExceptions);
#endif
    CPPUNIT_TEST (testTMorrissetLevel);
#ifdef WITH_LEVELE
    CPPUNIT_TEST (testLevelEMorris);
    CPPUNIT_TEST (testLevelEMorrisOutputRestrVarexp);
    CPPUNIT_TEST (testLevelEMorrisOutputRestrOption);
    CPPUNIT_TEST (testLevelEMorrisOutputRestrBoth);
#endif
    CPPUNIT_TEST (throwExceptionTMorrissetLevel);
    CPPUNIT_TEST (testTMorrissetDelta);
    CPPUNIT_TEST (throwExceptionTMorrisetDelta);
    CPPUNIT_TEST (testTMorrisgetReplica);

    CPPUNIT_TEST (testgFunction6Optimized);
    CPPUNIT_TEST (testFonctionAffine);
    CPPUNIT_TEST (testFonctionAffine2);
    CPPUNIT_TEST (testIshigamiMorrisWithDrawings);
    CPPUNIT_TEST (testIshigamiMorrisComputeIndexesWithTLauncherFunction);
    CPPUNIT_TEST (testIshigamiMorrisComputeIndexesGenerateSampleWithTLauncherFunction);
    CPPUNIT_TEST (testIshigamiMorrisComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction);
    CPPUNIT_TEST (testFlowrateMorrisComputeIndexesWithTLauncherCode);
    CPPUNIT_TEST (testFlowrateMorrisComputeIndexesWithTCodeEval);
    CPPUNIT_TEST (testFlowrateMorrisComputeIndexesWithThreadedandNot);
    CPPUNIT_TEST (testFlowrateMorrisComputeIndexesGenerateSampleWithTLauncherCode);
    CPPUNIT_TEST (testFlowrateMorrisComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode);
#ifdef WITH_PYTHON
    CPPUNIT_TEST (testSensitivityMorrisFunctionFlowrateWithPython);
#endif

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TMorrisTest();

    //! Default destructor
    virtual ~TMorrisTest();

    void setUp();
    void setUp1();
    void setUp2();
    void tearDown();
    //@}

    //---------------------------------------------

    /** @name Validation of the sample values for Ishigami
     **/
    //@{
    void validateValuesIshigami();
    //@}

    /** @name Validation of the sample values for Flowrate
     **/
    //@{
    void validateValuesFlowrate();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{ 
    //! Test the Morris method for the Ishigami function
    void throwExceptionTMorrisConstructorTdsNull();
    void throwExceptionTMorrisConstructorFalseFunction();
    void throwExceptionTMorrisConstructorFalseCode();
    void throwExceptionTMorrisConstructorFalseInput();
    void throwExceptionTMorrisConstructorFalseOutput();
    void throwExceptionTMorrisConstructorFalseNReplique();
    void throwExceptionTMorrisConstructorFalseLevel();
    void throwExceptionTMorrisConstructorFalseLevelandDelta();
    void throwExceptionTMorrisConstructorFalseDelta();
    void throwExceptionConstructorTooLongOutputFunction();
    void throwExceptionConstructorTooLongOutputCode();
    void throwExceptionConstructorTooLongOutputRun();
    void throwExceptionTMorrisUnboundedInfiniteLaws();
    void throwExceptionConstructorUnboundedNormal();
    void throwExceptionConstructorUnboundedLogNormal();
    void throwExceptionConstructorUnboundedExponential();
    void throwExceptionConstructorUnboundedCauchy();
    void throwExceptionConstructorUnboundedGumbel();
    void throwExceptionConstructorUnboundedWeibull();
    void throwExceptionConstructorUnboundedGenPareto();
    void throwExceptionConstructorUnboundedGamma();
    void throwExceptionConstructorUnboundedInvGamma();
#ifdef WITH_MATHMORE
    void throwExceptionConstructorUnboundedStudent();
#endif
    void testTMorrissetLevel();    
    void throwExceptionTMorrissetLevel();
    void testTMorrissetDelta();
    void throwExceptionTMorrisetDelta();
    void testTMorrisgetReplica();

    //! Test the morris method on a filled TDs with th gFunction
    void testgFunction6Optimized();

    //! Test the morris method on a dummy affine function
    void testFonctionAffine();

    //! Test the morris method on a dummy affine function
    void testFonctionAffine2();
    
#ifdef WITH_LEVELE
    //! Test of the Morris for both SRC and SRRC using LevelE
    void testLevelEMorris();
    //! Test the Sobol method for the Levele function restraining output vector in constructor
    void testLevelEMorrisOutputRestrVarexp();
    //! Test the Sobol method for the Levele function restraining output vector in option
    void testLevelEMorrisOutputRestrOption();
    //! Test the Sobol method for the Levele function restraining output vector in both
    void testLevelEMorrisOutputRestrBoth();
    //! Check the results of the levelE sobol estrimation
    void checkLevelEResults(TTree *ntresu, vector<int> *elmts = NULL);
#endif

    void testIshigamiMorrisWithDrawings();
    //! Test the ability to launch computeIndexes alone with a TLauncherFunction
    void testIshigamiMorrisComputeIndexesWithTLauncherFunction();
    //! Test the ability to launch computeIndexes after generateSample with a TLauncherFunction
    void testIshigamiMorrisComputeIndexesGenerateSampleWithTLauncherFunction();
    //! Test the ability to launch computeIndexes after generateSample a code creation with a TLauncherFunction
    void testIshigamiMorrisComputeIndexesGenerateSampleCreateLauncherWithTLauncherFunction();
    //! Test the ability to launch computeIndexes alone with a TLauncher
    void testFlowrateMorrisComputeIndexesWithTLauncherCode();
    //! Test the ability to launch computeIndexes alone with a TLauncher2
    void testFlowrateMorrisComputeIndexesWithTCodeEval();
    //! Test the ability to launch computeIndexes alone in parallel
    void testFlowrateMorrisComputeIndexesWithThreadedandNot();
    //! Test the ability to launch computeIndexes after generateSample with a TLauncher
    void testFlowrateMorrisComputeIndexesGenerateSampleWithTLauncherCode();
    //! Test the ability to launch computeIndexes after generateSample a code creation with a TLauncher
    void testFlowrateMorrisComputeIndexesGenerateSampleCreateLauncherWithTLauncherCode();

#ifdef WITH_PYTHON
    void testSensitivityMorrisFunctionFlowrateWithPython();
#endif
    //@}

private:
    URANIE::DataServer::TDataServer *_tds1, *_tds, *_tds2;
    URANIE::Sensitivity::TMorris * _tmorris;
    URANIE::Launcher::TCode *_mycode;
    void * _p2fIshigami;
    void * _p2fBadPrototypeFunction;
    URANIE::Launcher::TOutputFile *_fout;
};

#endif  // CPPUNITEST_TMORRIS_H
