#!/usr/bin/env python

import ROOT
from ROOT.URANIE import DataServer, Launcher, Sensitivity

ROOT.gROOT.LoadMacro("UserFunctions.C")

# The reference input file
sJDDReference = "flowrate_input_with_keys.in"

tds = DataServer.TDataServer("tdsflowrate", "DataBase flowrate")
tds.addAttribute( DataServer.TUniformDistribution("rw", 0.05, 0.15))
tds.addAttribute( DataServer.TUniformDistribution("r", 100.0, 50000.0))
tds.addAttribute( DataServer.TUniformDistribution("tu", 63070.0, 115600.0))
tds.addAttribute( DataServer.TUniformDistribution("tl", 63.1, 116.0))
tds.addAttribute( DataServer.TUniformDistribution("hu", 990.0, 1110.0))
tds.addAttribute( DataServer.TUniformDistribution("hl", 700.0, 820.0))
tds.addAttribute( DataServer.TUniformDistribution("l", 1120.0, 1680.0))
tds.addAttribute( DataServer.TUniformDistribution("kw", 9855.0, 12045.0))

# Set the reference input file and the key for each input attributes
tds.getAttribute("rw").setFileKey(sJDDReference, "Rw")
tds.getAttribute("r").setFileKey(sJDDReference, "R")
tds.getAttribute("tu").setFileKey(sJDDReference, "Tu")
tds.getAttribute("tl").setFileKey(sJDDReference, "Tl")
tds.getAttribute("hu").setFileKey(sJDDReference, "Hu")
tds.getAttribute("hl").setFileKey(sJDDReference, "Hl")
tds.getAttribute("l").setFileKey(sJDDReference, "L")
tds.getAttribute("kw").setFileKey(sJDDReference, "Kw")

nreplica = 3
nlevel = 10

scmo = Sensitivity.TMorris(tds, "flowrateModel", nreplica, nlevel)
scmo.setSeed(123)
scmo.generateSample()

fichier = open("sensitivityMorrisFunctionFlowrateData.txt", "w")
fichier.write(str(scmo.getLevel())+"\n")
fichier.write(str(scmo.getDelta())+"\n")
fichier.write(str(scmo.getNReplica())+"\n")
fichier.close()

