#include <iostream>
using namespace TMath;
using namespace URANIE::DataServer;
using namespace URANIE::Sensitivity;

void waterborehole(double *x, Double_t *y)
{
    Double_t drw = x[0], dr = x[1];
    Double_t dtu = x[2], dtl = x[3];
    Double_t dhu = x[4], dhl = x[5];
    Double_t dl = x[6], dkw = x[7];

    Double_t dnum = 2.0 * TMath::Pi() * dtu * (dhu - dhl);
    Double_t dlnronrw = TMath::Log(dr / drw);
    Double_t dden =
            dlnronrw
                    * (1.0 + (2.0 * dl * dtu) / (dlnronrw * drw * drw * dkw)
                            + dtu / dtl);

    y[0] = dnum / dden;
}

void testRBDFlowrate()
{
    URANIE::DataServer::TDataServer *tds = new TDataServer("tdsflowreate",
            "DataBase flowreate");
    tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("rw", 0.05, 0.15));
    tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("r", 100.0, 50000.0));
    tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("tu", 63070.0,
                    115600.0));
    tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("tl", 63.1, 116.0));
    tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("hu", 990.0, 1110.0));
    tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("hl", 700.0, 820.0));
    tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("l", 1120.0, 1680.0));
    tds->addAttribute(
            new URANIE::DataServer::TUniformDistribution("kw", 9855.0,
                    12045.0));

    Int_t ns = 10000;
    URANIE::Sensitivity::TRBD * trbd = new URANIE::Sensitivity::TRBD(tds,
            waterborehole, "", "waterborehole", ns);
    trbd->setDrawProgressBar(kFALSE);

    trbd->computeIndexes();

    Double_t sobol;
    TTree *tree = trbd->getResultTuple();
    tree->SetBranchAddress("Sobol", &sobol);

    Int_t npar = (Int_t)(tree->GetEntries());

    ofstream sortie;
    sortie.open("testRBDFlowrate.out", ios::out);
    sortie.precision(12);
    sortie.setf(ios::scientific);
    sortie << npar << "\n";
    for (Int_t i = 0; i < npar; i++)
    {
        tree->GetEntry(i);
        sortie << sobol << "\n";
    }
    sortie.close();

}
