//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 @file   TTestCramerVonMises.h
 @author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 * @date   Tue Jan  6 11:53:53 2009
 
 @brief  Interface for the Cramer-von Mises test
 
 */

#ifndef TTESTCRAMERVONMISES_H
#define TTESTCRAMERVONMISES_H

// ROOT

// Uranie
#include "TTester.h"

namespace URANIE
{
namespace UncertModeler
{
class TTestCramerVonMises: public TTester
{
    // Attributes

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TTestCramerVonMises(URANIE::DataServer::TDataServer *tds,
            const char *varexpinput, Option_t * option = "");
    //! Default constructor
    virtual ~TTestCramerVonMises();
    //@}

    //---------------------------------------------
    /** @name Computes the score
     **/
    //@{
    //! The true computation with a vector of information and return the current value
    Double_t computeTheCurrentScore(Int_t i, Double_t dzi);
    //! Compute the modified value of the statistic
    void computeModifiedScore(
            URANIE::DataServer::TStochasticAttribute::ELawType tlaw);
    //! Return the name of the statistic
    const char * getTitleStatistic()
    {
        return "W^{2}";
    }
    //@}

    ClassDef(URANIE::UncertModeler::TTestCramerVonMises, ID_UNCERTMODELER)
    // Test the class
};
} // Fin du namespace UncertModeler
} // Fin du namespace URANIE
#endif
