//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 @file   TUncertModeler.cxx
 @author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 @date   Tue Jan  6 10:26:57 2009
 
 @brief  Implement the TUncertModeler class
 
 */

// Uranie
#include "TUncertModeler.h"
#include "UExceptions.h"

ClassImp (URANIE::UncertModeler::TUncertModeler)

namespace URANIE
{
namespace UncertModeler
{

// -----------------------------------------------------------------------
//  Constructors
// -----------------------------------------------------------------------
TUncertModeler::TUncertModeler(URANIE::DataServer::TDataServer *tds,
        const char* stest, const char *varexpinput, Option_t * option) :
        _tds(tds), _blog(kFALSE)

{
    if (_blog)
    {
        cout << "construit TUncertModeler tds[" << _tds->GetName()
                << "] Option [" << option << "]" << endl;
        cout << "test[" << stest << "]" << endl;
        cout << "varexpinput[" << varexpinput << "]" << endl;
    }

    try
    {
        if (_tds->getNPatterns() == 0)
            throw URANIE::Exceptions::UWarningExceptions(__FILE__, __LINE__,
                    Form(
                            "TUncertModeler::constructor Method\n The TTree [%s] is empty",
                            _tds->GetName()));

    } catch (URANIE::Exceptions::UWarningExceptions& ue)
    {
        ue.printMessage();
        throw ue;
    }
}
// -----------------------------------------------------------------------
//  Destructor
// -----------------------------------------------------------------------
TUncertModeler::~TUncertModeler()
{
    cout << "destroy TUncertModeler" << endl;

}

// -----------------------------------------------------------------------
//  Log
// -----------------------------------------------------------------------
void TUncertModeler::printLog(Option_t *option)
{
    cout << "************** TUncertModeler::printLog[" << option << "] Name["
            << GetName() << "] Title[" << GetTitle() << "] ID[" << getID()
            << "]" << endl;
    cout << " **   TDataServer Log (unactive)" << endl;
//     _tds->printLog();
    cout << " **   End of TDataServer Log" << endl;
    cout << "******** TUncertModeler::End of printLog[]" << endl;
}

} // Fin du namespace UncertModeler
} // Fin du namespace URANIE

