//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 @file   TUncertModeler.h
 @author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 @date   Tue Jan  6 10:21:14 2009
 
 @brief  Interface with statistical test
 
 */

#ifndef TUNCERTMODELER_H
#define TUNCERTMODELER_H

// ROOT
#include "TNamed.h"

// Uranie
#include "UncertModeler.h"
#include "TDataServer.h"

namespace URANIE
{
namespace UncertModeler
{
class TUncertModeler: public TNamed
{
    // Attributes

protected:
    Char_t _sType[12];
    Char_t _sMethod[12];
    Float_t _pValue;

public:
    URANIE::DataServer::TDataServer *_tds;               ///< Pointer to a TDS
    Bool_t _blog;                                   ///< Boolean to edit the log

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TUncertModeler(URANIE::DataServer::TDataServer *tds, const char *stest,
            const char *varexpinput, Option_t * option = "");
    //! Default constructor
    virtual ~TUncertModeler();
    //@}

    //---------------------------------------------
    /** @name Setting and Getting attributs
     **/
    //@{
    Int_t getID()
    {
        return ID_UNCERTMODELER;
    }
    //@}

    //---------------------------------------------
    /** @name Log Printing 
     **/
    //@{
    void setLog()
    {
        _blog = kTRUE;
    }
    void unsetLog()
    {
        _blog = kFALSE;
    }
    void changeLog()
    {
        _blog = _blog ? kFALSE : kTRUE;
    }
    Bool_t getLog()
    {
        return _blog;
    }
    virtual void printLog(Option_t *option = "");
    //@}

    ClassDef(URANIE::UncertModeler::TUncertModeler, ID_UNCERTMODELER)
    // UncertModeler classe
};
} // Fin du namespace UncertModeler
} // Fin du namespace URANIE
#endif
