//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestMain.cxx
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   jeu jan 24 15:56:01 2008
 
 $Revision$
 $Date$
 
 \brief Main program for unitary tests on the uncertmodeler library.
 
 \par Modifications
 \li M. Salvoldi 18/01/2012
 - add a new class of unit tests (TUncertModelerTest)
 
 */
#include "cppunit/CompilerOutputter.h"
#include "cppunit/TestResult.h"
#include "cppunit/TestResultCollector.h"
#include "cppunit/TextTestProgressListener.h"
#include "cppunit/BriefTestProgressListener.h"
#include "cppunit/extensions/TestFactoryRegistry.h"
#include "cppunit/TestRunner.h"
#include "cppunit/XmlOutputter.h"

#include <stdexcept>

#include <iostream>
#include <fstream>

//uranie
#include "UVersion.h"
// Liste des Tests Unitaires d'uranie
#include "TestTTestKolmogorovSmirnov.h"
#include "TestTTestAndersonDarling.h"
#include "TestTTestCramerVonMises.h"
#include "TestTCirce.h"
#include "TestTUncertModeler.h"
#include "uranie_commands.h"

CPPUNIT_TEST_SUITE_REGISTRATION (TTestKolmogorovSmirnovTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TTestAndersonDarlingTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TTestCramerVonMisesTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TCirceTest);
CPPUNIT_TEST_SUITE_REGISTRATION (TUncertModelerTest);

bool uranie_test_suite(const char *module, const char *xml, const char *xsl);

int main()
{

    bool wasSuccessful;
#ifdef WIN32
    wasSuccessful = uranie_test_suite("UncertModeler",
                                      "testUncertModelerResults.xml", "..\\..\\..\\..\\testReport.xsl");
#else
    wasSuccessful = uranie_test_suite("UncertModeler",
                                      "testUncertModelerResults.xml", "../../../../testReport.xsl");
#endif
    ////////////////
    // Clean
    ////////////////

    UranieTests::removeFile( gSystem->pwd(), "basic.root");
    UranieTests::removeFile( gSystem->pwd(), "bliss.root");
    UranieTests::removeFile( gSystem->pwd(), "_dataserver_.root");
    UranieTests::removeFile( gSystem->pwd(), "jdd_circe_summerschool2006_dataserver.root");

    gDirectory->ls();

    // ---  Return error code 1 if the one of test failed.
    return wasSuccessful ? 0 : 1;
}
