//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTCirce.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test File of the class URANIE::DataServer::TCirce
 
 Created: 2011-12-12 09:44:32
 
 */
#ifndef CPPUNITEST_TESTTCIRCE_H
#define CPPUNITEST_TESTTCIRCE_H

#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

// Uranie
#include "../TCirce.h"

/*!
 \class TCirceTest
 \brief Unit Tests for the TCirce classe

 This unitary tests are based on the JDD of circe given in the 2006 SummerSchool

 The Circe JDD is transformed to TDataServer ASCII File called \em "jdd_circe_summerschool2006_dataserver.dat"
 
 \section JDD_CIRCE_SUMMERSCHOOL JDD Circe
 * NOMBRE DE REPONSES
 150
 * NOMBRE TOTAL DE PARAMETRES
 2
 * NOMBRE DE PARAMETRES CONSIDERES
 2
 * SEUIL DE CONVERGENCE
 1.D-5
 * NUMERO DES PARAMETRES CONSIDERES
 1 2 
 * PRISE EN COMPTE DES INCERTITUDES EXPERIMENTALES (Y/N)
 N
 * NOMBRE DE BLOCS
 2
 * NOMBRE DE PARAMETRES PAR BLOC
 1 1 
 * MATRICE CPRIME INITIALE
 *1.382900 0. 0.
 *0.798822 0.
 *0.206729
 * BIAIS INITIAL
 *1.290436 1.535354 1.124778 -1.761406
 * NOMBRE DE MATRICES GENEREES ALEATOIREMENT
 *5
 * NOMBRE DE REPLIQUES
 *50
 *************************************************************************
 ECART   CODE    EXPERIEN 0.00  1     0.853828      0.720995
 0.0
 DERIVEE W.R.T.  A1       1.28069465184215
 DERIVEE W.R.T.  A2       0.426960643634018
 
 ECART   CODE    EXPERIEN 0.00  1     1.420676      1.467705
 0.0
 DERIVEE W.R.T.  A1       2.130798384266
 DERIVEE W.R.T.  A2       0.71055392085385
 
 ECART   CODE    EXPERIEN 0.00  1     1.986837      1.277730
 0.0
 DERIVEE W.R.T.  A1       2.97966447669473
 DERIVEE W.R.T.  A2       0.99400960229471
 
 ECART   CODE    EXPERIEN 0.00  1     2.552036      1.991193
 0.0
 DERIVEE W.R.T.  A1       3.82680010293528
 DERIVEE W.R.T.  A2       1.27727279795579
 
 ECART   CODE    EXPERIEN 0.00  1     3.116001      2.036849
 0.0
 DERIVEE W.R.T.  A1       4.67171366530476
 DERIVEE W.R.T.  A2       1.56028865511034
 
 ECART   CODE    EXPERIEN 0.00  1     3.678459      3.445518
 0.0
 DERIVEE W.R.T.  A1       5.51391514353132
 DERIVEE W.R.T.  A2       1.84300236892767
 
 ECART   CODE    EXPERIEN 0.00  1     4.239138      4.735902
 0.0
 DERIVEE W.R.T.  A1       6.35291644246116
 DERIVEE W.R.T.  A2       2.12535919308588
 
 ECART   CODE    EXPERIEN 0.00  1     4.797768      3.381548
 0.0
 DERIVEE W.R.T.  A1       7.18823173830835
 DERIVEE W.R.T.  A2       2.40730445037326
 
 ECART   CODE    EXPERIEN 0.00  1     5.354081      5.383797
 0.0
 DERIVEE W.R.T.  A1       8.0193778231856
 DERIVEE W.R.T.  A2       2.68878354327628
 
 ECART   CODE    EXPERIEN 0.00  1     5.907808      5.001590
 0.0
 DERIVEE W.R.T.  A1       8.84587444765514
 DERIVEE W.R.T.  A2       2.96974196455219
 
 ECART   CODE    EXPERIEN 0.00  1     6.458685      5.330333
 0.0
 DERIVEE W.R.T.  A1       9.66724466104056
 DERIVEE W.R.T.  A2       3.2501253077841
 
 ECART   CODE    EXPERIEN 0.00  1     7.006447      7.952286
 0.0
 DERIVEE W.R.T.  A1       10.4830151492423
 DERIVEE W.R.T.  A2       3.52987927791659
 
 ECART   CODE    EXPERIEN 0.00  1     7.550833      4.561176
 0.0
 DERIVEE W.R.T.  A1       11.29271656980094
 DERIVEE W.R.T.  A2       3.80894970176967
 
 ECART   CODE    EXPERIEN 0.00  1     8.091583      7.968353
 0.0
 DERIVEE W.R.T.  A1       12.09588388395529
 DERIVEE W.R.T.  A2       4.08728253852922
 
 ECART   CODE    EXPERIEN 0.00  1     8.628440      8.644601
 0.0
 DERIVEE W.R.T.  A1       12.89205668544376
 DERIVEE W.R.T.  A2       4.36482389021177
 
 ECART   CODE    EXPERIEN 0.00  1     9.161150      7.772117
 0.0
 DERIVEE W.R.T.  A1       13.68077952580078
 DERIVEE W.R.T.  A2       4.64152001210159
 
 ECART   CODE    EXPERIEN 0.00  1     9.689460     11.946291
 0.0
 DERIVEE W.R.T.  A1       14.46160223590208
 DERIVEE W.R.T.  A2       4.91731732315816
 
 ECART   CODE    EXPERIEN 0.00  1    10.213121      9.110840
 0.0
 DERIVEE W.R.T.  A1       15.2340802435161
 DERIVEE W.R.T.  A2       5.19216241639188
 
 ECART   CODE    EXPERIEN 0.00  1    10.731888      9.179312
 0.0
 DERIVEE W.R.T.  A1       15.99777488662129
 DERIVEE W.R.T.  A2       5.46600206920615
 
 ECART   CODE    EXPERIEN 0.00  1    11.245518     12.044400
 0.0
 DERIVEE W.R.T.  A1       16.75225372225293
 DERIVEE W.R.T.  A2       5.73878325370367
 
 ECART   CODE    EXPERIEN 0.00  1    11.753772      9.955391
 0.0
 DERIVEE W.R.T.  A1       17.49709083064608
 DERIVEE W.R.T.  A2       6.01045314695507
 
 ECART   CODE    EXPERIEN 0.00  1    12.256413      8.516376
 0.0
 DERIVEE W.R.T.  A1       18.23186711444503
 DERIVEE W.R.T.  A2       6.28095914122786
 
 ECART   CODE    EXPERIEN 0.00  1    12.753210     11.832538
 0.0
 DERIVEE W.R.T.  A1       18.95617059275352
 DERIVEE W.R.T.  A2       6.55024885417371
 
 ECART   CODE    EXPERIEN 0.00  1    13.243933     15.764511
 0.0
 DERIVEE W.R.T.  A1       19.66959668980388
 DERIVEE W.R.T.  A2       6.81827013897204
 
 ECART   CODE    EXPERIEN 0.00  1    13.728360     13.636206
 0.0
 DERIVEE W.R.T.  A1       20.37174851802735
 DERIVEE W.R.T.  A2       7.08497109442804
 
 ECART   CODE    EXPERIEN 0.00  1    14.206269     15.828666
 0.0
 DERIVEE W.R.T.  A1       21.06223715531225
 DERIVEE W.R.T.  A2       7.35030007502315
 
 ECART   CODE    EXPERIEN 0.00  1    14.677444     15.371526
 0.0
 DERIVEE W.R.T.  A1       21.74068191624102
 DERIVEE W.R.T.  A2       7.61420570091594
 
 ECART   CODE    EXPERIEN 0.00  1    15.141674     17.624294
 0.0
 DERIVEE W.R.T.  A1       22.40671061710195
 DERIVEE W.R.T.  A2       7.87663686789166
 
 ECART   CODE    EXPERIEN 0.00  1    15.598751     16.365027
 0.0
 DERIVEE W.R.T.  A1       23.05995983447602
 DERIVEE W.R.T.  A2       8.13754275725832
 
 ECART   CODE    EXPERIEN 0.00  1    16.048474     13.622278
 0.0
 DERIVEE W.R.T.  A1       23.70007515720437
 DERIVEE W.R.T.  A2       8.39687284568752
 
 ECART   CODE    EXPERIEN 0.00  1    16.490644     16.654472
 0.0
 DERIVEE W.R.T.  A1       24.32671143154693
 DERIVEE W.R.T.  A2       8.6545769149981
 
 ECART   CODE    EXPERIEN 0.00  1    16.925069     18.445503
 0.0
 DERIVEE W.R.T.  A1       24.93953299934804
 DERIVEE W.R.T.  A2       8.91060506188065
 
 ECART   CODE    EXPERIEN 0.00  1    17.351561     21.030571
 0.0
 DERIVEE W.R.T.  A1       25.53821392903017
 DERIVEE W.R.T.  A2       9.16490770756109
 
 ECART   CODE    EXPERIEN 0.00  1    17.769937     17.357715
 0.0
 DERIVEE W.R.T.  A1       26.12243823924255
 DERIVEE W.R.T.  A2       9.41743560740141
 
 ECART   CODE    EXPERIEN 0.00  1    18.180020     11.956423
 0.0
 DERIVEE W.R.T.  A1       26.69190011499693
 DERIVEE W.R.T.  A2       9.66813986043565
 
 ECART   CODE    EXPERIEN 0.00  1    18.581638     19.157803
 0.0
 DERIVEE W.R.T.  A1       27.24630411612868
 DERIVEE W.R.T.  A2       9.91697191883941
 
 ECART   CODE    EXPERIEN 0.00  1    18.974624     16.126637
 0.0
 DERIVEE W.R.T.  A1       27.78536537792731
 DERIVEE W.R.T.  A2       10.16388359733087
 
 ECART   CODE    EXPERIEN 0.00  1    19.358818     18.922050
 0.0
 DERIVEE W.R.T.  A1       28.30880980378635
 DERIVEE W.R.T.  A2       10.40882708250174
 
 ECART   CODE    EXPERIEN 0.00  1    19.734065     20.848071
 0.0
 DERIVEE W.R.T.  A1       28.81637424972888
 DERIVEE W.R.T.  A2       10.65175494207607
 
 ECART   CODE    EXPERIEN 0.00  1    20.100213     18.048485
 0.0
 DERIVEE W.R.T.  A1       29.30780670067119
 DERIVEE W.R.T.  A2       10.89262013409535
 
 ECART   CODE    EXPERIEN 0.00  1    20.457121      8.858695
 0.0
 DERIVEE W.R.T.  A1       29.78286643829326
 DERIVEE W.R.T.  A2       11.13137601602795
 
 ECART   CODE    EXPERIEN 0.00  1    20.804650     17.677597
 0.0
 DERIVEE W.R.T.  A1       30.24132420039149
 DERIVEE W.R.T.  A2       11.36797635380128
 
 ECART   CODE    EXPERIEN 0.00  1    21.142669     22.920734
 0.0
 DERIVEE W.R.T.  A1       30.6829623315954
 DERIVEE W.R.T.  A2       11.60237533075481
 
 ECART   CODE    EXPERIEN 0.00  1    21.471051     18.370473
 0.0
 DERIVEE W.R.T.  A1       31.10757492533667
 DERIVEE W.R.T.  A2       11.83452755651226
 
 ECART   CODE    EXPERIEN 0.00  1    21.789678     16.656787
 0.0
 DERIVEE W.R.T.  A1       31.51496795696608
 DERIVEE W.R.T.  A2       12.0643880757713
 
 ECART   CODE    EXPERIEN 0.00  1    22.098436     19.602911
 0.0
 DERIVEE W.R.T.  A1       31.90495940791976
 DERIVEE W.R.T.  A2       12.29191237700892
 
 ECART   CODE    EXPERIEN 0.00  1    22.397218     27.669603
 0.0
 DERIVEE W.R.T.  A1       32.27737938084425
 DERIVEE W.R.T.  A2       12.5170564011009
 
 ECART   CODE    EXPERIEN 0.00  1    22.685923     22.298174
 0.0
 DERIVEE W.R.T.  A1       32.63207020559582
 DERIVEE W.R.T.  A2       12.73977654985366
 
 ECART   CODE    EXPERIEN 0.00  1    22.964458     21.384184
 0.0
 DERIVEE W.R.T.  A1       32.96888653603695
 DERIVEE W.R.T.  A2       12.96002969444693
 
 ECART   CODE    EXPERIEN 0.00  1    23.232734     30.519337
 0.0
 DERIVEE W.R.T.  A1       33.28769543756008
 DERIVEE W.R.T.  A2       13.17777318378539
 
 ECART   CODE    EXPERIEN 0.00  1    23.490671     15.001831
 0.0
 DERIVEE W.R.T.  A1       33.58837646527553
 DERIVEE W.R.T.  A2       13.39296485275793
 
 ECART   CODE    EXPERIEN 0.00  1    23.738192     25.366767
 0.0
 DERIVEE W.R.T.  A1       33.87082173280778
 DERIVEE W.R.T.  A2       13.60556303040269
 
 ECART   CODE    EXPERIEN 0.00  1    23.975231     23.949371
 0.0
 DERIVEE W.R.T.  A1       34.13493597165184
 DERIVEE W.R.T.  A2       13.81552654797654
 
 ECART   CODE    EXPERIEN 0.00  1    24.201726     13.391711
 0.0
 DERIVEE W.R.T.  A1       34.3806365810481
 DERIVEE W.R.T.  A2       14.02281474692716
 
 ECART   CODE    EXPERIEN 0.00  1    24.417621     21.553269
 0.0
 DERIVEE W.R.T.  A1       34.6078536683418
 DERIVEE W.R.T.  A2       14.22738748676646
 
 ECART   CODE    EXPERIEN 0.00  1    24.622868     26.531200
 0.0
 DERIVEE W.R.T.  A1       34.81653007980055
 DERIVEE W.R.T.  A2       14.42920515284357
 
 ECART   CODE    EXPERIEN 0.00  1    24.817425     20.392323
 0.0
 DERIVEE W.R.T.  A1       35.00662142187031
 DERIVEE W.R.T.  A2       14.62822866401608
 
 ECART   CODE    EXPERIEN 0.00  1    25.001258     32.189940
 0.0
 DERIVEE W.R.T.  A1       35.17809607285803
 DERIVEE W.R.T.  A2       14.82441948021793
 
 ECART   CODE    EXPERIEN 0.00  1    25.174337     20.032718
 0.0
 DERIVEE W.R.T.  A1       35.33093518503614
 DERIVEE W.R.T.  A2       15.0177396099225
 
 ECART   CODE    EXPERIEN 0.00  1    25.336642     26.195740
 0.0
 DERIVEE W.R.T.  A1       35.46513267717175
 DERIVEE W.R.T.  A2       15.20815161749959
 
 ECART   CODE    EXPERIEN 0.00  1    25.488157     30.414972
 0.0
 DERIVEE W.R.T.  A1       35.58069521749041
 DERIVEE W.R.T.  A2       15.39561863046458
 
 ECART   CODE    EXPERIEN 0.00  1    25.628873     29.119988
 0.0
 DERIVEE W.R.T.  A1       35.67764219709215
 DERIVEE W.R.T.  A2       15.58010434661869
 
 ECART   CODE    EXPERIEN 0.00  1    25.758789     32.045293
 0.0
 DERIVEE W.R.T.  A1       35.75600569384415
 DERIVEE W.R.T.  A2       15.76157304107873
 
 ECART   CODE    EXPERIEN 0.00  1    25.877910     21.195366
 0.0
 DERIVEE W.R.T.  A1       35.81583042678243
 DERIVEE W.R.T.  A2       15.93998957319507
 
 ECART   CODE    EXPERIEN 0.00  1    25.986247     28.350611
 0.0
 DERIVEE W.R.T.  A1       35.85717370106172
 DERIVEE W.R.T.  A2       16.1153193933565
 
 ECART   CODE    EXPERIEN 0.00  1    26.083817     38.814487
 0.0
 DERIVEE W.R.T.  A1       35.88010534350011
 DERIVEE W.R.T.  A2       16.28752854968057
 
 ECART   CODE    EXPERIEN 0.00  1    26.170646     31.542012
 0.0
 DERIVEE W.R.T.  A1       35.88470762877235
 DERIVEE W.R.T.  A2       16.4565836945883
 
 ECART   CODE    EXPERIEN 0.00  1    26.246764     26.275016
 0.0
 DERIVEE W.R.T.  A1       35.87107519631279
 DERIVEE W.R.T.  A2       16.62245209126172
 
 ECART   CODE    EXPERIEN 0.00  1    26.312208     38.313217
 0.0
 DERIVEE W.R.T.  A1       35.83931495799592
 DERIVEE W.R.T.  A2       16.78510161998328
 
 ECART   CODE    EXPERIEN 0.00  1    26.367023     28.568492
 0.0
 DERIVEE W.R.T.  A1       35.78954599666999
 DERIVEE W.R.T.  A2       16.94450078435567
 
 ECART   CODE    EXPERIEN 0.00  1    26.411259     31.091609
 0.0
 DERIVEE W.R.T.  A1       35.72189945562559
 DERIVEE W.R.T.  A2       17.10061871740096
 
 ECART   CODE    EXPERIEN 0.00  1    26.444972     16.263460
 0.0
 DERIVEE W.R.T.  A1       35.63651841908857
 DERIVEE W.R.T.  A2       17.25342518753787
 
 ECART   CODE    EXPERIEN 0.00  1    26.468224     21.124831
 0.0
 DERIVEE W.R.T.  A1       35.53355778383328
 DERIVEE W.R.T.  A2       17.40289060443601
 
 ECART   CODE    EXPERIEN 0.00  1    26.481085     33.708891
 0.0
 DERIVEE W.R.T.  A1       35.41318412201886
 DERIVEE W.R.T.  A2       17.54898602474587
 
 ECART   CODE    EXPERIEN 0.00  1    26.483629     24.250273
 0.0
 DERIVEE W.R.T.  A1       35.27557553535836
 DERIVEE W.R.T.  A2       17.69168315770361
 
 ECART   CODE    EXPERIEN 0.00  1    26.475938     35.046525
 0.0
 DERIVEE W.R.T.  A1       35.12092150073671
 DERIVEE W.R.T.  A2       17.83095437060949
 
 ECART   CODE    EXPERIEN 0.00  1    26.458098     24.052985
 0.0
 DERIVEE W.R.T.  A1       34.94942270740037
 DERIVEE W.R.T.  A2       17.96677269417873
 
 ECART   CODE    EXPERIEN 0.00  1    26.430201     34.135678
 0.0
 DERIVEE W.R.T.  A1       34.76129088584793
 DERIVEE W.R.T.  A2       18.09911182776408
 
 ECART   CODE    EXPERIEN 0.00  1    26.392347     27.700029
 0.0
 DERIVEE W.R.T.  A1       34.55674862855703
 DERIVEE W.R.T.  A2       18.22794614444878
 
 ECART   CODE    EXPERIEN 0.00  1    26.344640     32.737134
 0.0
 DERIVEE W.R.T.  A1       34.33602920268978
 DERIVEE W.R.T.  A2       18.35325069600917
 
 ECART   CODE    EXPERIEN 0.00  1    26.287189     28.450629
 0.0
 DERIVEE W.R.T.  A1       34.09937635492414
 DERIVEE W.R.T.  A2       18.47500121774569
 
 ECART   CODE    EXPERIEN 0.00  1    26.220109     23.829647
 0.0
 DERIVEE W.R.T.  A1       33.84704410856569
 DERIVEE W.R.T.  A2       18.59317413318178
 
 ECART   CODE    EXPERIEN 0.00  1    26.143522     27.528282
 0.0
 DERIVEE W.R.T.  A1       33.5792965530994
 DERIVEE W.R.T.  A2       18.70774655862926
 
 ECART   CODE    EXPERIEN 0.00  1    26.057552     36.671849
 0.0
 DERIVEE W.R.T.  A1       33.29640762634709
 DERIVEE W.R.T.  A2       18.81869630761972
 
 ECART   CODE    EXPERIEN 0.00  1    25.962331     26.930743
 0.0
 DERIVEE W.R.T.  A1       32.99866088940201
 DERIVEE W.R.T.  A2       18.92600189520079
 
 ECART   CODE    EXPERIEN 0.00  1    25.857996     32.444448
 0.0
 DERIVEE W.R.T.  A1       32.68634929451718
 DERIVEE W.R.T.  A2       19.02964254209664
 
 ECART   CODE    EXPERIEN 0.00  1    25.744687     28.236267
 0.0
 DERIVEE W.R.T.  A1       32.3597749461298
 DERIVEE W.R.T.  A2       19.1295981787318
 
 ECART   CODE    EXPERIEN 0.00  1    25.622549     22.031751
 0.0
 DERIVEE W.R.T.  A1       32.01924885520915
 DERIVEE W.R.T.  A2       19.22584944911754
 
 ECART   CODE    EXPERIEN 0.00  1    25.491734     21.495018
 0.0
 DERIVEE W.R.T.  A1       31.66509068712056
 DERIVEE W.R.T.  A2       19.31837771460008
 
 ECART   CODE    EXPERIEN 0.00  1    25.352397     21.132186
 0.0
 DERIVEE W.R.T.  A1       31.29762850320268
 DERIVEE W.R.T.  A2       19.40716505746987
 
 ECART   CODE    EXPERIEN 0.00  1    25.204696     19.217357
 0.0
 DERIVEE W.R.T.  A1       30.91719849626069
 DERIVEE W.R.T.  A2       19.49219428443131
 
 ECART   CODE    EXPERIEN 0.00  1    25.048797     26.490158
 0.0
 DERIVEE W.R.T.  A1       30.52414472018196
 DERIVEE W.R.T.  A2       19.57344892993214
 
 ECART   CODE    EXPERIEN 0.00  1    24.884866     21.316733
 0.0
 DERIVEE W.R.T.  A1       30.11881881388582
 DERIVEE W.R.T.  A2       19.65091325935192
 
 ECART   CODE    EXPERIEN 0.00  1    24.713076     25.592418
 0.0
 DERIVEE W.R.T.  A1       29.70157971982285
 DERIVEE W.R.T.  A2       19.72457227204899
 
 ECART   CODE    EXPERIEN 0.00  1    24.533603     19.577027
 0.0
 DERIVEE W.R.T.  A1       29.27279339724373
 DERIVEE W.R.T.  A2       19.79441170426527
 
 ECART   CODE    EXPERIEN 0.00  1    24.346625     34.922599
 0.0
 DERIVEE W.R.T.  A1       28.83283253046127
 DERIVEE W.R.T.  A2       19.86041803188838
 
 ECART   CODE    EXPERIEN 0.00  1    24.152327     35.735641
 0.0
 DERIVEE W.R.T.  A1       28.38207623233321
 DERIVEE W.R.T.  A2       19.92257847307052
 
 ECART   CODE    EXPERIEN 0.00  1    23.950895     19.302306
 0.0
 DERIVEE W.R.T.  A1       27.92090974319701
 DERIVEE W.R.T.  A2       19.98088099070364
 
 ECART   CODE    EXPERIEN 0.00  1    23.742519     22.246825
 0.0
 DERIVEE W.R.T.  A1       27.44972412549127
 DERIVEE W.R.T.  A2       20.03531429475035
 
 ECART   CODE    EXPERIEN 0.00  1    23.527392     27.026746
 0.0
 DERIVEE W.R.T.  A1       26.96891595430164
 DERIVEE W.R.T.  A2       20.08586784443022
 
 ECART   CODE    EXPERIEN 0.00  1    23.305709     22.656735
 0.0
 DERIVEE W.R.T.  A1       26.47888700407213
 DERIVEE W.R.T.  A2       20.1325318502609
 
 ECART   CODE    EXPERIEN 0.00  1    23.077671     15.588268
 0.0
 DERIVEE W.R.T.  A1       25.98004393172577
 DERIVEE W.R.T.  A2       20.1752972759539
 
 ECART   CODE    EXPERIEN 0.00  1    22.843477     26.051802
 0.0
 DERIVEE W.R.T.  A1       25.47279795644106
 DERIVEE W.R.T.  A2       20.21415584016436
 
 ECART   CODE    EXPERIEN 0.00  1    22.603332     26.182246
 0.0
 DERIVEE W.R.T.  A1       24.95756453633325
 DERIVEE W.R.T.  A2       20.2491000180947
 
 ECART   CODE    EXPERIEN 0.00  1    22.357443     38.381341
 0.0
 DERIVEE W.R.T.  A1       24.43476304229199
 DERIVEE W.R.T.  A2       20.28012304295182
 
 ECART   CODE    EXPERIEN 0.00  1    22.106018     22.768312
 0.0
 DERIVEE W.R.T.  A1       23.90481642922852
 DERIVEE W.R.T.  A2       20.30721890725739
 
 ECART   CODE    EXPERIEN 0.00  1    21.849267     26.030200
 0.0
 DERIVEE W.R.T.  A1       23.36815090498805
 DERIVEE W.R.T.  A2       20.33038236401122
 
 ECART   CODE    EXPERIEN 0.00  1    21.587402     15.635641
 0.0
 DERIVEE W.R.T.  A1       22.82519559718429
 DERIVEE W.R.T.  A2       20.34960892770733
 
 ECART   CODE    EXPERIEN 0.00  1    21.320639     18.753400
 0.0
 DERIVEE W.R.T.  A1       22.27638221821499
 DERIVEE W.R.T.  A2       20.36489487520249
 
 ECART   CODE    EXPERIEN 0.00  1    21.049191     20.947923
 0.0
 DERIVEE W.R.T.  A1       21.72214472871805
 DERIVEE W.R.T.  A2       20.37623724643724
 
 ECART   CODE    EXPERIEN 0.00  1    20.773276     24.642890
 0.0
 DERIVEE W.R.T.  A1       21.16291899972978
 DERIVEE W.R.T.  A2       20.3836338450091
 
 ECART   CODE    EXPERIEN 0.00  1    20.493113     14.828323
 0.0
 DERIVEE W.R.T.  A1       20.59914247380697
 DERIVEE W.R.T.  A2       20.38708323859784
 
 ECART   CODE    EXPERIEN 0.00  1    20.208919     17.455233
 0.0
 DERIVEE W.R.T.  A1       20.03125382537558
 DERIVEE W.R.T.  A2       20.3865847592429
 
 ECART   CODE    EXPERIEN 0.00  1    19.920916     16.168686
 0.0
 DERIVEE W.R.T.  A1       19.45969262056937
 DERIVEE W.R.T.  A2       20.38213850347272
 
 ECART   CODE    EXPERIEN 0.00  1    19.629322     15.843812
 0.0
 DERIVEE W.R.T.  A1       18.88489897682189
 DERIVEE W.R.T.  A2       20.37374533228603
 
 ECART   CODE    EXPERIEN 0.00  1    19.334360     18.983964
 0.0
 DERIVEE W.R.T.  A1       18.30731322247558
 DERIVEE W.R.T.  A2       20.36140687098514
 
 ECART   CODE    EXPERIEN 0.00  1    19.036251     20.541221
 0.0
 DERIVEE W.R.T.  A1       17.72737555667158
 DERIVEE W.R.T.  A2       20.34512550886119
 
 ECART   CODE    EXPERIEN 0.00  1    18.735215     14.704089
 0.0
 DERIVEE W.R.T.  A1       17.14552570978318
 DERIVEE W.R.T.  A2       20.32490439873151
 
 ECART   CODE    EXPERIEN 0.00  1    18.431475     27.724483
 0.0
 DERIVEE W.R.T.  A1       16.56220260465613
 DERIVEE W.R.T.  A2       20.30074745632903
 
 ECART   CODE    EXPERIEN 0.00  1    18.125252     20.854847
 0.0
 DERIVEE W.R.T.  A1       15.97784401891744
 DERIVEE W.R.T.  A2       20.27265935954408
 
 ECART   CODE    EXPERIEN 0.00  1    17.816766     15.890789
 0.0
 DERIVEE W.R.T.  A1       15.39288624861407
 DERIVEE W.R.T.  A2       20.2406455475185
 
 ECART   CODE    EXPERIEN 0.00  1    17.506238     16.750030
 0.0
 DERIVEE W.R.T.  A1       14.80776377344123
 DERIVEE W.R.T.  A2       20.20471221959236
 
 ECART   CODE    EXPERIEN 0.00  1    17.193888     10.107342
 0.0
 DERIVEE W.R.T.  A1       14.22290892381922
 DERIVEE W.R.T.  A2       20.16486633410352
 
 ECART   CODE    EXPERIEN 0.00  1    16.879934     15.763192
 0.0
 DERIVEE W.R.T.  A1       13.6387515500759
 DERIVEE W.R.T.  A2       20.12111560704017
 
 ECART   CODE    EXPERIEN 0.00  1    16.564594     27.922521
 0.0
 DERIVEE W.R.T.  A1       13.05571869399018
 DERIVEE W.R.T.  A2       20.07346851054663
 
 ECART   CODE    EXPERIEN 0.00  1    16.248084     14.705787
 0.0
 DERIVEE W.R.T.  A1       12.4742342629501
 DERIVEE W.R.T.  A2       20.02193427128278
 
 ECART   CODE    EXPERIEN 0.00  1    15.930621      9.417969
 0.0
 DERIVEE W.R.T.  A1       11.89471870697695
 DERIVEE W.R.T.  A2       19.96652286863736
 
 ECART   CODE    EXPERIEN 0.00  1    15.612417     13.446139
 0.0
 DERIVEE W.R.T.  A1       11.31758869886463
 DERIVEE W.R.T.  A2       19.9072450327955
 
 ECART   CODE    EXPERIEN 0.00  1    15.293685     13.532392
 0.0
 DERIVEE W.R.T.  A1       10.74325681768058
 DERIVEE W.R.T.  A2       19.84411224266086
 
 ECART   CODE    EXPERIEN 0.00  1    14.974634     20.769703
 0.0
 DERIVEE W.R.T.  A1       10.17213123587262
 DERIVEE W.R.T.  A2       19.77713672363276
 
 ECART   CODE    EXPERIEN 0.00  1    14.655473     17.227635
 0.0
 DERIVEE W.R.T.  A1       9.6046154102225
 DERIVEE W.R.T.  A2       19.70633144523886
 
 ECART   CODE    EXPERIEN 0.00  1    14.336409     12.008431
 0.0
 DERIVEE W.R.T.  A1       9.04110777688405
 DERIVEE W.R.T.  A2       19.63171011862357
 
 ECART   CODE    EXPERIEN 0.00  1    14.017644     17.118440
 0.0
 DERIVEE W.R.T.  A1       8.48200145074117
 DERIVEE W.R.T.  A2       19.55328719389303
 
 ECART   CODE    EXPERIEN 0.00  1    13.699381     14.424335
 0.0
 DERIVEE W.R.T.  A1       7.92768392931629
 DERIVEE W.R.T.  A2       19.47107785731685
 
 ECART   CODE    EXPERIEN 0.00  1    13.381817     14.105580
 0.0
 DERIVEE W.R.T.  A1       7.37853680145766
 DERIVEE W.R.T.  A2       19.38509802838736
 
 ECART   CODE    EXPERIEN 0.00  1    13.065150      8.072573
 0.0
 DERIVEE W.R.T.  A1       6.83493546102893
 DERIVEE W.R.T.  A2       19.29536435673693
 
 ECART   CODE    EXPERIEN 0.00  1    12.749572     10.533065
 0.0
 DERIVEE W.R.T.  A1       6.29724882582122
 DERIVEE W.R.T.  A2       19.20189421891373
 
 ECART   CODE    EXPERIEN 0.00  1    12.435272     11.022880
 0.0
 DERIVEE W.R.T.  A1       5.76583906190352
 DERIVEE W.R.T.  A2       19.10470571501693
 
 ECART   CODE    EXPERIEN 0.00  1    12.122439     12.865744
 0.0
 DERIVEE W.R.T.  A1       5.24106131362276
 DERIVEE W.R.T.  A2       19.00381766519164
 
 ECART   CODE    EXPERIEN 0.00  1    11.811257     10.789248
 0.0
 DERIVEE W.R.T.  A1       4.72326343946107
 DERIVEE W.R.T.  A2       18.89924960598447
 
 ECART   CODE    EXPERIEN 0.00  1    11.501904      5.134292
 0.0
 DERIVEE W.R.T.  A1       4.21278575395215
 DERIVEE W.R.T.  A2       18.79102178656041
 
 ECART   CODE    EXPERIEN 0.00  1    11.194558     11.284947
 0.0
 DERIVEE W.R.T.  A1       3.70996077585491
 DERIVEE W.R.T.  A2       18.67915516478159
 
 ECART   CODE    EXPERIEN 0.00  1    10.889392     10.426379
 0.0
 DERIVEE W.R.T.  A1       3.21511298277689
 DERIVEE W.R.T.  A2       18.56367140314894
 
 ECART   CODE    EXPERIEN 0.00  1    10.586576      8.555081
 0.0
 DERIVEE W.R.T.  A1       2.72855857243528
 DERIVEE W.R.T.  A2       18.44459286460729
 
 ECART   CODE    EXPERIEN 0.00  1    10.286274     10.517881
 0.0
 DERIVEE W.R.T.  A1       2.25060523073802
 DERIVEE W.R.T.  A2       18.32194260821494
 
 ECART   CODE    EXPERIEN 0.00  1     9.988648     12.657006
 0.0
 DERIVEE W.R.T.  A1       1.78155190686199
 DERIVEE W.R.T.  A2       18.19574438467829
 
 ECART   CODE    EXPERIEN 0.00  1     9.693856      9.822359
 0.0
 DERIVEE W.R.T.  A1       1.3216885954999
 DERIVEE W.R.T.  A2       18.06602263175268
 
 ECART   CODE    EXPERIEN 0.00  1     9.402049      9.082523
 0.0
 DERIVEE W.R.T.  A1       0.871296126442228
 DERIVEE W.R.T.  A2       17.9328024695101
 
 ECART   CODE    EXPERIEN 0.00  1     9.113378     12.983850
 0.0
 DERIVEE W.R.T.  A1       0.430645961653806
 DERIVEE W.R.T.  A2       17.7961096954748
 
 ECART   CODE    EXPERIEN 0.00  1     8.827985      6.205202
 0.0
 DERIVEE W.R.T.  A1       -2.897217962760751E-15
 DERIVEE W.R.T.  A2       17.65597077962768
 
 
 */

class TCirceTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TCirceTest);
    CPPUNIT_TEST (testGetNTDS);
    CPPUNIT_TEST (testGetNPatterns);
    CPPUNIT_TEST (testGetNParameters);
    CPPUNIT_TEST (testGetTolerance);
    CPPUNIT_TEST (testGetNCMatrix);
    CPPUNIT_TEST (testSetBVectorInitial);
    CPPUNIT_TEST (testSetCMatrixInitial);
    CPPUNIT_TEST (testGetSeed);
    CPPUNIT_TEST (testGetLikelihood);
    CPPUNIT_TEST (testGetInputFile);
    CPPUNIT_TEST (testEstimate);
    CPPUNIT_TEST (testEstimateConfidenceIntervals1);
    CPPUNIT_TEST (testEstimateWithUncertainties);
    CPPUNIT_TEST (throwExceptionsConstructor);
    CPPUNIT_TEST (throwExceptionsAddData);CPPUNIT_TEST_SUITE_END();

public:
    /*! Constructs a TCirceTest object.
     */
    TCirceTest();

    /// Destructor.
    virtual ~TCirceTest();

    void setUp();
    void tearDown();
    void removeFile(const char* folder, const char* filename);
    
    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! get the number of TDS stored
    void testGetNTDS();
    //! get the number of patterns
    void testGetNPatterns();
    //! get the number of parameters
    void testGetNParameters();
    //! get the tolerance parameter
    void testGetTolerance();
    //! get the number of random matrix C to generate
    void testGetNCMatrix();
    //! test the setBVectorInitial method
    void testSetBVectorInitial();
    //! test the setCMatrixInitial method
    void testSetCMatrixInitial();
    //! get the seed
    void testGetSeed();
    //! get the Likelihood
    void testGetLikelihood();
    //! get the input file
    void testGetInputFile();
    //! uses the estimate method
    void testEstimate();
    //! compute the confidence inetravls useCase 1
    void testEstimateConfidenceIntervals1();
    //! uses the estimate method with uncertainties
    void testEstimateWithUncertainties();
    //@}

    //---------------------------------------------
    /** @name Throw Exceptions
     The list of throw exceptions
     **/
    //@{
    //! get the number of TDS stored
    void throwExceptionsConstructor();
    // addData with different number of parameters
    void throwExceptionsAddData();

    //@}

private:
    URANIE::DataServer::TDataServer *tds1, *tds2;
};

#endif  // CPPUNITEST_TATTRIBUTE_H
