//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision$
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TestTTestCramerVonMises.h
 \author Fabrice Gaudier <fabrice.gaudier@cea.fr>
 \date   $Date$
 
 \brief  Test File of the class URANIE::DataServer::TTestCramerVonMises
 
 Created: wen nov 21 07:52:44 2007
 \par Modifications
 \li M. Salvoldi 18/01/2012
 - add 8 news unit tests  
 */
#ifndef CPPUNITEST_TTESTTTESTCRAMERVONMISES_H
#define CPPUNITEST_TTESTTTESTCRAMERVONMISES_H

// CPPUNIT
#include <cppunit/extensions/HelperMacros.h>
//#include <cppunit/portability/CppUnitDeque.h>
#include <cppunit/Test.h>
#include <cppunit/TestFailure.h>
#include <cppunit/TestResultCollector.h>

//ROOT

//URANIE
#include "TTestCramerVonMises.h"
#include "TDataServer.h"
#include "TAttribute.h"
#include "UExceptions.h"

/*! \class TTestCramerVonMisesTest
 * \brief Unit tests for TTestCramerVonMises.
 */
class TTestCramerVonMisesTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TTestCramerVonMisesTest);

    CPPUNIT_TEST (testW2blissCase3);
    CPPUNIT_TEST (testW2blissCase0);
    CPPUNIT_TEST (testUniformValues);
    CPPUNIT_TEST (throwExceptionTestCramerVonMisesConstructorTdsNullEmpty);
    CPPUNIT_TEST (throwExceptionTestCramerVonMisesConstructorMoreAttribut);
    CPPUNIT_TEST (throwExceptionTestCramerVonMisesConstructorFalseAttribut);
    CPPUNIT_TEST (throwExceptionTestCramerVonMisescomputeScoreNoLaw);
    CPPUNIT_TEST (throwExceptionTestCramerVonMisescomputeScoreBadLaw);
    CPPUNIT_TEST (throwExceptionTestCramerVonMisescomputeScoreNormalBadDefined);
    CPPUNIT_TEST (throwExceptionTestCramerVonMisescomputeScoreUniformBadDefined);
    CPPUNIT_TEST (throwExceptionTestCramerVonMisescomputeScoreLogNormalBadDefined);

    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor, Destructor, methods with the setUp and tearDown
     **/
    //@{
    //! Constructs a TTestCramerVonMisesTest object.
    TTestCramerVonMisesTest();

    //! Destructor.
    virtual ~TTestCramerVonMisesTest();

    //! the setUp method
    void setUp();

    //! the tearDown method
    void tearDown();
    //@}

    //---------------------------------------------
    /** @name Unit Tests
     The list of unit test
     **/
    //@{
    //! Computes the W^2 statistic on the bliss dataset
    void testW2blissCase3();
    //! Computes the W^2 statistic on the bliss dataset when specified mean and standard deviation for the normal law
    void testW2blissCase0();
    //@}
    //! Test uniform law
    void testUniformValues();
    //---------------------------------------------
    /** @name Throw Exceptions
     The list of throw exceptions
     **/
    //@{
    //! Verfiy that the exception is thrown if the tds is NULL or Empty
    void throwExceptionTestCramerVonMisesConstructorTdsNullEmpty();
    //! Verfiy that the exception is thrown if more than 2 attributs are used
    void throwExceptionTestCramerVonMisesConstructorMoreAttribut();
    //! Verfiy that the exception is thrown if the inputattribut doens't exist in the tds
    void throwExceptionTestCramerVonMisesConstructorFalseAttribut();
    //! Verfiy that the exception is thrown if the tds doesn't contained law
    void throwExceptionTestCramerVonMisescomputeScoreNoLaw();
    //! Verfiy that the exception is thrown if a law no accepted is used 
    void throwExceptionTestCramerVonMisescomputeScoreBadLaw();
    //! Verfiy that the exception is thrown if the normal isn't well defined
    void throwExceptionTestCramerVonMisescomputeScoreNormalBadDefined();
    //! Verfiy that the exception is thrown if the uniform isn't well defined
    void throwExceptionTestCramerVonMisescomputeScoreUniformBadDefined();
    //! Verfiy that the exception is thrown if the lognormal isn't well defined
    void throwExceptionTestCramerVonMisescomputeScoreLogNormalBadDefined();
    //@}

private:
    URANIE::DataServer::TDataServer *_tdsBliss;
    URANIE::UncertModeler::TTestCramerVonMises *_tcm;
};

#endif  // CPPUNITEST_TATTRIBUTE_H
