//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
  \file   TBaseEval.cxx
  \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
  \date    2011-12-13.12.25.48

  \brief   implementation partage au TEval
*/

#include "TBaseEval.h"
#include "TComposeEval.h"
#include "TCodeEval.h"
#include "TCJitEval.h"

#ifdef WITH_PYTHON
#include "TPythonEval.h"
#endif

ClassImp (URANIE::Relauncher::TBaseEval);

namespace URANIE
{
namespace Relauncher
{

/* TBaseEval */
TBaseEval::TBaseEval()
{
    _freezed = 0;
 }

void TBaseEval::freeze()
{
    _freezed = 1;
}

TEval *TBaseEval::rebuild(int rank, Bool_t chef)
{
    UNUSED(rank,chef);
    return this;
}
Bool_t TBaseEval::unbuild()
{
    return kFALSE;
}

void TBaseEval::clean()
{
    _freezed = 0;
    _input.Clear();
    _output.Clear();
}

void TBaseEval::cleanOutputs()
{
    _freezed = 0;
    _output.Clear();
}

void TBaseEval::addAllInputs(URANIE::DataServer::TDataServer *tds)
{
    URANIE::DataServer::TAttribute *att;
    TIter next(getInputList());

    while ((att = (URANIE::DataServer::TAttribute *) next()))
    {
        tds->addAttribute(att);
    }
}

TObjArray *TBaseEval::getInputList()
{
    freeze();
    return &_input;
}
  
TObjArray *TBaseEval::getOutputList()
{
    freeze();
    return &_output;
}

int TBaseEval::getOutDimension(vector<URANIE::DataServer::UEntry*> *vec)
{
    int ret=1+vec->size();
    for(vector<URANIE::DataServer::UEntry*>::iterator it=vec->begin(); it!=vec->end(); it++)
	ret+=( ((*it)->tag==URANIE::DataServer::UEntry::Vector) ? (*it)->getDim() : 1 );
    
    return ret;
}


bool TBaseEval::containsStringType()
{
    URANIE::DataServer::TAttribute *att;
    bool resultat=false;
    
    TIter inatt( &_input );
    while( (att = (URANIE::DataServer::TAttribute*) inatt()) )
        if( att->getDataType() == URANIE::DataServer::TAttribute::kString )
            resultat=true;

    TIter outatt( &_output );
    while( (att = (URANIE::DataServer::TAttribute*) outatt()) )
        if( att->getDataType() == URANIE::DataServer::TAttribute::kString )
            resultat=true;

    return resultat;
}


void TBaseEval::checkAttributeType( URANIE::DataServer::TAttribute *att )
{

    if( att->getDataType() == URANIE::DataServer::TAttribute::kVector || att->getDataType() == URANIE::DataServer::TAttribute::kString )
    {
	
#ifdef WITH_PYTHON
	if( this->IsA() != TPythonEval::Class() && this->IsA() != TCJitEval::Class() && this->IsA() != TCodeEval::Class() )
	    throw TEval::CanNotDealWithNotDouble(this->GetName());
#else
	if( this->IsA() != TCJitEval::Class() && this->IsA() != TCodeEval::Class() )
	    throw TEval::CanNotDealWithNotDouble(this->GetName());
#endif
    }

}


}
}
