//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TBaseEval.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-12-13.12.25.42

 \brief implementation partage au TEval
 */

#ifndef __TBaseEval__
#define __TBaseEval__

//ROOT
#include "Rtypes.h"
#include "TObjArray.h"

// Uranie
#include "Relauncher.h"
#include "TEval.h"
#include "TDataServer.h"

namespace URANIE
{
namespace Relauncher
{

class TBaseEval: public TEval
{
protected:
    int _freezed;
    TObjArray _input, _output;
    
public:
    TBaseEval();

    virtual void addAllInputs(URANIE::DataServer::TDataServer *);
    virtual void freeze();
    virtual TEval *rebuild(int rank, Bool_t chef = kTRUE);
    virtual Bool_t unbuild();

    virtual TObjArray *getInputList();
    virtual TObjArray *getOutputList();

    virtual void checkAttributeType( URANIE::DataServer::TAttribute *att );
    
    virtual int getOutDimension(vector<URANIE::DataServer::UEntry*>*);

    virtual bool isThreadSafe() {return true;}
    virtual bool containsStringType();

    virtual void clean();
    virtual void cleanOutputs();
    
    ClassDef(URANIE::Relauncher::TBaseEval, ID_RELAUNCHER)
};

}
} // namespace
#endif
