//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TCJitEval.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-09-22.08.51.02

 \brief   version compilée de macros
 */

#include "TCJitEval.h"

ClassImp (URANIE::Relauncher::TCJitEval);

namespace URANIE
{
namespace Relauncher
{

// TCJitEval
TCJitEval::TCJitEval(int (*fun)(double*, double*, void*), void *info):
    TSimpleEval()
{
    _typ = ferm; _p.ferm = fun; _extra = info;
}

TCJitEval::TCJitEval(int (*fun)(double *, double *)):
    TSimpleEval()
{
    _typ = func; _p.func = fun; _extra = NULL;
}

TCJitEval::TCJitEval(void (*fun)(double *, double *)):
    TSimpleEval()
{
    _typ = proc; _p.proc = fun; _extra = NULL;
}

TCJitEval::TCJitEval(int (*fun)(vector<URANIE::DataServer::UEntry*> *, vector<URANIE::DataServer::UEntry*>*)):
    TSimpleEval()
{
    _typ = hard; _p.hard = fun; _extra = NULL;
}

TCJitEval::TCJitEval(std::function<int(double*, double*)> *foncteur) :
    TSimpleEval()
{
    _typ = fonc; _p.fonc = foncteur; _extra = NULL;
}
  
TCJitEval::TCJitEval(std::function<int(vector<URANIE::DataServer::UEntry*>*,vector<URANIE::DataServer::UEntry*>*)> *hardfonc):
    TSimpleEval()
{
    _typ = harf; _p.harf = hardfonc; _extra = NULL;
}

  
TCJitEval::~TCJitEval()
{
}

int TCJitEval::eval(vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out, int ncalc)
{
    UNUSED(ncalc);
    double *inp = new double[in->size()], *outp = new double[out->size()];
    if(_typ!=hard && _typ!=harf)
        for(unsigned int ival=0; ival<in->size(); ival++)
            inp[ival] = *in->at(ival)->val.dou;
    
    int ret=0;
    switch (_typ)
    {
    case ferm :
        ret =  _p.ferm(inp, outp, _extra);
        break;
    case func :
        ret = _p.func(inp, outp);
        break;
    case proc :
        _p.proc(inp, outp);
        ret = 1;
        break;
    case hard :
        ret = _p.hard(in, out);
        break;
    case fonc :
        ret = (*_p.fonc)(inp, outp);
        break;
    case harf :
        ret = (*_p.harf)(in, out);
        break;
    }
    
    if(_typ!=hard && _typ!=harf)
        for(unsigned int ival=0; ival<out->size(); ival++)
            out->at(ival)->assignEntry( (void*)(&outp[ival] ) );

    delete [] outp;
    delete [] inp;
    
    return ((ret!=0) ? getOutDimension(out) : ret);
}


}
} //namespace
