//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TFlatScript.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-09-19.15.00.57

 \brief entree sur une ligne
 */

#ifndef __TFLATSCRIPT__
#define __TFLATSCRIPT__

#include <fstream>
//ROOT
#include "TString.h"
#include "TObjArray.h"

//URANIE
#include "Relauncher.h"
#include "TAttribute.h"
#include "TCodeEval.h"

namespace URANIE
{
namespace Relauncher
{

class TShareScript: public TCodeScript
{
protected:
    TString _path;
    TString _svecbeg, _svecend, _svecdel, _sstrbeg, _sstrend; 
    TString _format;

public:
    TShareScript(const char *path);
    virtual ~TShareScript();

    void setVectorProperties( TString beg, TString delim, TString end);
    void setStringProperties( TString beg, TString end);
    void setFormat(TString form) {_format=form;}
    
    ClassDef(URANIE::Relauncher::TShareScript, ID_RELAUNCHER)
};

class TVoidScript: public TShareScript
{
protected:
    TObjArray _vars;
    
public:
    TVoidScript(const char *path);
    virtual ~TVoidScript();

    void addInput(URANIE::DataServer::TAttribute *);
    void setInputs(Int_t size, URANIE::DataServer::TAttribute *, ...);

    virtual void build(TString &dir, TObjArray *in);

    ClassDef(URANIE::Relauncher::TVoidScript, ID_RELAUNCHER)
};

class TNoParser : public TObject
{
public:
    URANIE::DataServer::TAttribute *_att;
    int _index;
    
    TNoParser(URANIE::DataServer::TAttribute *att);
    virtual ~TNoParser();
    
    void build(int id);
};

class TFlatScript: public TVoidScript
{
public:
    TFlatScript(const char *path);
    virtual ~TFlatScript();

    virtual void put(TString &ref, TString &tmp, Double_t *in);
    virtual void put(TString &ref, TString &tmp, vector<URANIE::DataServer::UEntry*> *in);

    ClassDef(URANIE::Relauncher::TFlatScript, ID_RELAUNCHER)
};

}
} //namespace

#endif
