//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.5 $
// $State$
//////////////////////////////////////////////////////////////
/*!
  \file   TInformMaster.cxx
  \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
  \date    2012-07-24.14.59.29

  \brief   master qui complete un tds existant
*/

#include "TInformMaster.h"

#include "TLeaf.h"

ClassImp (URANIE::Relauncher::TInformMaster);

namespace URANIE
{
namespace Relauncher
{

TInformMaster::TInformMaster(URANIE::DataServer::TDataServer *tds, TRun *run) :
    TMaster(tds, run)
{
    getSample();
}

TInformMaster::TInformMaster(URANIE::DataServer::TDataServer *tds, TEval *fun) :
    TMaster(tds, fun)
{
    getSample();
}

TInformMaster::TInformMaster(URANIE::DataServer::TDataServer *tds
			     , TStandardEval *fun, const char *in, const char *out) : 
    TMaster(tds, fun, in, out)
{
    getSample();
}

TInformMaster::~TInformMaster(void)
{
    _inleaves.RemoveAll();
    if (gROOT->Get(_samplename.c_str())) {
        delete _sampl;
        _sampl = NULL;
    }

}

/* constructor helpers */
void TInformMaster::getSample(void)
{
    //URANIE::Relauncher::TInputVar *var;
    URANIE::DataServer::TAttribute *att;
    TList *tmplist;

    _sampl = _tds->getTuple();
    _tds->setTuple((URANIE::DataServer::TDSNtupleD *) NULL);
    _samplename = Form("%s_InformMaster", _sampl->GetName());
    _sampl->SetName(_samplename.c_str());

    tmplist = _tds->getListOfAttributes("*");
    TIter next(tmplist);
    while ((att = (URANIE::DataServer::TAttribute *) next())) {
        _inleaves.Add(_sampl->GetLeaf(att->GetName()));
    }
    
    delete tmplist;

}

/* run utilities */
void TInformMaster::build(void)
{
    fillDS();
    _tds->createTuple();

}

void TInformMaster::allocSamplEntries(vector<URANIE::DataServer::UEntry*> &vec)
{
    // iterator
    vec.push_back( new URANIE::DataServer::UEntry(URANIE::DataServer::UEntry::Double, "error__n__iter__"));
    // other tds fields
    TIter nextLeaf(&_inleaves);
    while (TLeaf *leaf = (TLeaf *) nextLeaf())
    {
        if( string(leaf->GetTypeName()) == "string" )
        {
            vec.push_back( new URANIE::DataServer::UEntry(URANIE::DataServer::UEntry::String, (string(leaf->GetName())+"_buffer") ) );
        }
        else if( string(leaf->GetTypeName()) == "vector<double>" )
        {
            vec.push_back( new URANIE::DataServer::UEntry(URANIE::DataServer::UEntry::Vector, (string(leaf->GetName())+"_buffer") ) );
        }
        else
        {
            vec.push_back( new URANIE::DataServer::UEntry(URANIE::DataServer::UEntry::Double, (string(leaf->GetName())+"_buffer") ) );
        }
    }

}


// ================================================================
//                    Entry part for vectors
// ================================================================
void TInformMaster::getCurrentPattern(Int_t ipat, vector<URANIE::DataServer::UEntry*> *samp, 
        vector<URANIE::DataServer::UEntry*> *res)
{
    int i;
    auto tmp = new vector<URANIE::DataServer::UEntry*>(samp->begin()+1, samp->end());

    // import data from sampl
    _sampl->GetEntry(ipat);

    i = 0;
    TIter nextLeaf(&_inleaves);
    while (TLeaf *leaf = (TLeaf *) nextLeaf()) {
        tmp->at(i++)->assignEntry((void*)(leaf->GetValuePointer()));
    }
    // sampling data -> code
    *res->at(0)->val.dou = ipat;
    inputForCode(tmp, new vector<URANIE::DataServer::UEntry*>(res->begin()+1,res->end()) );

    delete tmp;
}

void TInformMaster::putCurrentPattern(vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out, vector<URANIE::DataServer::UEntry*> *evt)
{

    allFromCode(new vector<URANIE::DataServer::UEntry*>(in->begin()+1, in->end()), new vector<URANIE::DataServer::UEntry*>(out->begin(), out->end()),
		new vector<URANIE::DataServer::UEntry*>(evt->begin()+1, evt->end()));
    
    evt->at(0)->copyEntry( in->at(0) );
    _tds->getTuple()->Fill( evt );
}

void TInformMaster::putCurrentAsError(Int_t ipat, vector<URANIE::DataServer::UEntry*> *samp,
        URANIE::DataServer::TDataServer *tds)
{
    int i;
    
    *samp->at(0)->val.dou = ipat;
    // import data from sampl
    _sampl->GetEntry(ipat);

    i = 1;
    TIter nextLeaf(&_inleaves);
    while (TLeaf *leaf = (TLeaf *) nextLeaf()) {
        samp->at(i++)->assignEntry((void*)(leaf->GetValuePointer()));
    }
    // sampling data -> code

    tds->getTuple()->Fill( samp);
}

}
} // namespace
