//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.3 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TKeyResult.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-06-07.16.29.51

 \brief gestion format key valeur des fichiers résultats
 */

#ifndef __TKEYRESULT__
#define __TKEYRESULT__

//ROOT
#include "TString.h"
#include "TList.h"
#include "TObjArray.h"
#include "TRegexp.h"

//Uranie
#include "Relauncher.h"
#include "TAttribute.h"
#include "TCodeEval.h"

namespace URANIE
{
namespace Relauncher
{

class TKeyFinder: public TNamed
{
protected:
    static TRegexp _value;
    TRegexp _re_full, _re_equ, _re_fin, _re_equ_empty, _re_fin_empty;
    TString _svecbeg, _svecend, _svecdel, _sstrbeg, _sstrend;
    
public:
    URANIE::DataServer::TAttribute *_att;

    TKeyFinder(const char *name, URANIE::DataServer::TAttribute *att);
    virtual ~TKeyFinder();

    Bool_t match(TString &);
    double get(TString &);
    void get(TString &, URANIE::DataServer::UEntry *);
    TString getTStr(TString &line);

    void setVectorProperties( TString beg, TString delim, TString end);
    TString getVectorProperties(){ return _svecbeg+"/"+_svecdel+"/"+_svecend; }
    void setStringProperties( TString beg, TString end);
};

class TKeyResult: public TCodeResult
{
protected:
    TString _path;
    TObjArray _vars;
    bool _bnoVectorSpecificities;
    
public:
    TKeyResult(const char *);
    virtual ~TKeyResult(void);

    void addOutput(URANIE::DataServer::TAttribute *at, const char *key);
    void setOutputs(int, URANIE::DataServer::TAttribute *at, const char *key, ...);
    void build(TObjArray *);
    int get(TString &, Double_t*);
    int get(TString &work, vector<URANIE::DataServer::UEntry*> *sout);

    void clearFile(TString tmpdir);
    void setVectorProperties( TString beg, TString delim, TString end);
    void setStringProperties( TString beg, TString end);

    ClassDef(URANIE::Relauncher::TKeyResult, ID_RELAUNCHER)

};

}
} //namespace

#endif
