//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.5 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TMaster.h
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2011-03-30.10.54.14

 \brief superviseur de calculs parametriques
 */

#ifndef __TMaster__
#define __TMaster__

#include <vector>
//ROOT
#include "Rtypes.h"
#include "TList.h"

// Uranie
#include "Relauncher.h"
#include "TDataServer.h"
#include "TRun.h"
#include "TStandardEval.h"
#include "TMasterVar.h"

namespace URANIE
{
namespace Relauncher
{

class TMaster
{
protected:
    URANIE::DataServer::TDataServer *_tds;
    TRun *_run, *_localrun;
    TEval *_fun;
    TList _param, _sortie;
    int _NinDat, _NinCod, _Naux, _NoutCod, _NoutDat;

    // constructor helper
    void match_input();
    // subclasse helper
    void fillDS();
    
    void allocInCodeEntries(int siz, vector<vector<URANIE::DataServer::UEntry*>> &, bool withid=false);
    void allocOutCodeEntries(int siz, vector<vector<URANIE::DataServer::UEntry*>> &);
    void allocGradCodeEntries(int siz, vector<vector<URANIE::DataServer::UEntry*>> &);
    void allocDataEntries(int siz, vector<vector<URANIE::DataServer::UEntry*>> &);
    void allocDataEntries(vector<URANIE::DataServer::UEntry*> &);
 
    void inputForCode(const double*, double*);

    void inputForCode(const double*, vector<URANIE::DataServer::UEntry *>*);
    void outputFromCode(vector<URANIE::DataServer::UEntry *>*, double*);

    //  Entry part for vectors
    void inputForCode(const vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*);
    void allFromCode(vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*, vector<URANIE::DataServer::UEntry*>*);

public:
    TMaster(URANIE::DataServer::TDataServer *, TRun *);
    TMaster(URANIE::DataServer::TDataServer *, TEval *);
    TMaster(URANIE::DataServer::TDataServer *, TStandardEval *, const char*, const char*);
    virtual ~TMaster();

    // gestion des parametres
    void addConstantValue(URANIE::DataServer::TAttribute *att, double, bool KeepIt=false);
    void addConstantValue(URANIE::DataServer::TAttribute *att, URANIE::DataServer::UEntry*, bool KeepIt=false);
    void addTemporary(URANIE::DataServer::TAttribute *att);
    void fillConstant();
    URANIE::DataServer::TDataServer *getTDS() {return _tds;}
    
    TRun* getRunner() { return _run; }
    
    // protocole
    virtual void solverLoop() = 0;

    void clean();

    ClassDef(URANIE::Relauncher::TMaster, ID_RELAUNCHER)
};

}
}

#endif
