//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.4 $
// $State$
//////////////////////////////////////////////////////////////
/*!
  \file   TSequentialRun.cxx
  \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
  \date    2009-04-01.13.25.35

  \brief   main_loop de resolution
*/

#include "TSequentialRun.h"

ClassImp (URANIE::Relauncher::TSequentialRun);

namespace URANIE
{
namespace Relauncher
{

TSequentialRun::TSequentialRun(TEval *fun) :
    TSimpleRun(fun)
{
    _ncpus = 1;
    _workfun = NULL;
}

TSequentialRun::~TSequentialRun(void)
{
    stopSlave();
}

void TSequentialRun::startSlave(void)
{
    _workfun = _fun->rebuild(0);
}

void TSequentialRun::stopSlave(void)
{
    if (!_workfun)
        return;

    if (_workfun->unbuild())
        delete _workfun;
    _workfun = NULL;

//    while (! _qin.empty()) {
//        //pa = q.front();
//        _qn.pop();
//    }
}

int TSequentialRun::get(long &id)
{
    struct element tuple;
    int res=-1;

    tuple = _qin.front();
    _qin.pop();
    res = _workfun->eval(tuple.inv, tuple.outv, tuple.calcnb);
    id = tuple.id;    
    return res;
}

}
} //namespace

