//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or any 
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////
// $Id$
// $Author$
// $Date$
// $Revision 1.4 $
// $State$
//////////////////////////////////////////////////////////////
/*!
 \file   TShareRun.cxx
 \author Gilles ARNAUD <Gilles.Arnaud@cea.fr>
 \date    2012-07-18.09.03.50

 \brief   TRun base class
 */

#include "TShareRun.h"

ClassImp (URANIE::Relauncher::TShareRun);
ClassImp (URANIE::Relauncher::TSimpleRun);
ClassImp (URANIE::Relauncher::TDecoratorRun);

namespace URANIE
{
namespace Relauncher
{

/* TShareRun */
TShareRun::TShareRun(TEval *fun) :
        _fun(fun)
{
    try{
        _fun->freeze();
    }catch(URANIE::Exceptions::UErrorExceptions &ue)
    {
        throw ue;
    }
}

TShareRun::~TShareRun(void)
{
}

/* TSimpleRun */
TSimpleRun::TSimpleRun(TEval *fun) :
        TShareRun(fun)
{
}

TSimpleRun::~TSimpleRun(void)
{
}

Bool_t TSimpleRun::onMaster(void)
{
    return kTRUE;
}

void TSimpleRun::put(long i, vector<URANIE::DataServer::UEntry*> *in, vector<URANIE::DataServer::UEntry*> *out, int icalc)
{
    struct element tupl =
        { i, icalc, in, out};

    _qin.push(tupl);
}

Bool_t TSimpleRun::readyToGet()
{
    //return kTRUE;
    return kFALSE;
}

/* TDecoratorRun */
TDecoratorRun::TDecoratorRun(TRun *run) :
        _subrun(run)
{
}

TDecoratorRun::~TDecoratorRun(void)
{
}

TEval *TDecoratorRun::getEval()
{
    return _subrun->getEval();
}

void TDecoratorRun::startSlave(void)
{
    _subrun->startSlave();
}

Bool_t TDecoratorRun::onMaster(void)
{
    return _subrun->onMaster();
}

void TDecoratorRun::stopSlave(void)
{
    _subrun->stopSlave();
}

}
} //namespace

