//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////

#ifndef CPPUNITEST_TRELAUNCHER_H
#define CPPUNITEST_TRELAUNCHER_H

#include <cppunit/extensions/HelperMacros.h>

// Uranie
#include "TDataServer.h"
#include "TAttribute.h"

class TLauncher2Test: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE (TLauncher2Test);

    //Composition
    CPPUNIT_TEST (testComposeScalaire);
#ifdef WITH_PYTHON	
    CPPUNIT_TEST (testComposeMultiKey);
    CPPUNIT_TEST (testComposeMultiKeyWithEmpty);
    CPPUNIT_TEST (testComposeMultiLine);
    CPPUNIT_TEST (testComposeMultiLineWithEmpty);
    CPPUNIT_TEST (testComposeMultiFlat);
    CPPUNIT_TEST (testComposeMultiFlatWithEmpty);
    CPPUNIT_TEST (testComposeMultiFlatMultiLine);
#endif

    CPPUNIT_TEST (testFlowrateSequential);
    CPPUNIT_TEST (testFlowrateSequentialFailure);
    CPPUNIT_TEST (testFlowrateThreaded);
    CPPUNIT_TEST (testFlowrateThreadedFailure);

    CPPUNIT_TEST (testFlowrateSequentialAllFolders);
    CPPUNIT_TEST (testFlowrateThreadedAllFolders);
    
#ifdef WITH_MPI
    //ROOT6 
    CPPUNIT_TEST( testFlowrateMPI );
    CPPUNIT_TEST( testFlowrateMPIFailure );
    CPPUNIT_TEST( testFlowrateMPIAllFolders ); 
#endif

    CPPUNIT_TEST (testIshigamiSequential);
    CPPUNIT_TEST (testIshigamiShort);
    CPPUNIT_TEST (testIshigamiSo);  
    CPPUNIT_TEST (testIshigami2So);
    CPPUNIT_TEST (testIshigami3So);
    CPPUNIT_TEST (testIshigami4lambda);
    
#ifdef WITH_MPI
    CPPUNIT_TEST (testIshigamiMPI);
#endif
#ifdef WITH_PCL
    CPPUNIT_TEST (testIshigamiPclInt);
    //CPPUNIT_TEST (testIshigamiPclJit);
#endif

#ifdef WITH_PYTHON
    //ROOT6 
    CPPUNIT_TEST (testIshigamiSequentialPython);
    CPPUNIT_TEST (testReadmtSequentialPython);
    CPPUNIT_TEST (testmtSequentialPython);    
	
#ifdef WITH_MPI
    //ROOT6 
    CPPUNIT_TEST (testIshigamiMPIPython); 
#endif

    CPPUNIT_TEST (testBidonSequentialKey);
    CPPUNIT_TEST (testBidonSequentialKeyWithEmpty);
    CPPUNIT_TEST (testBidonSequentialLine);
    CPPUNIT_TEST (testBidonSequentialLineWithEmpty);
    CPPUNIT_TEST (testBidonSequentialFlat);
    CPPUNIT_TEST (testBidonSequentialFlatWithEmpty);
    CPPUNIT_TEST (testBidonSequentialFlatMultiLine);
    CPPUNIT_TEST (testBidonThreadedKey);
    CPPUNIT_TEST (testBidonThreadedKeyWithEmpty);
    CPPUNIT_TEST (testBidonThreadedLine);
    CPPUNIT_TEST (testBidonThreadedLineWithEmpty);
    CPPUNIT_TEST (testBidonThreadedFlat);
    CPPUNIT_TEST (testBidonThreadedFlatWithEmpty);
    CPPUNIT_TEST (testBidonThreadedFlatMultiLine);

#endif	
	
    CPPUNIT_TEST (testThreadedWithDataServer);
    CPPUNIT_TEST (testThreadedWithDataServerFailing);
    
#ifdef WITH_MPI
#ifdef WITH_PYTHON
    //ROOT6 
    CPPUNIT_TEST( testBidonMPIKey );
    CPPUNIT_TEST( testBidonMPIKeyWithEmpty );
    CPPUNIT_TEST( testBidonMPILine );  
    CPPUNIT_TEST( testBidonMPILineWithEmpty );
    CPPUNIT_TEST( testBidonMPIFlat );
    CPPUNIT_TEST( testBidonMPIFlatWithEmpty );
    CPPUNIT_TEST( testBidonMPIFlatMultiLine );
#endif
#endif

    //Multitype As output
    CPPUNIT_TEST( testMultiTypeOutputFileFlat);
    CPPUNIT_TEST( testMultiTypeOutputFileFlat2);
    CPPUNIT_TEST( testMultiTypeOutputFileFlatMultiLine);
    CPPUNIT_TEST( testMultiTypeOutputFileKey);
    CPPUNIT_TEST( testMultiTypeOutputFileKeyCondensate);
    CPPUNIT_TEST( testMultiTypeOutputFileLine);
    CPPUNIT_TEST( testMultiTypeOutputFileLineThreaded);

    //Mulititype As input
    CPPUNIT_TEST( testMultiTypeAsInputKey);
    CPPUNIT_TEST( testMultiTypeAsInputFlat);
    CPPUNIT_TEST( testMultiTypeAsInputFlat2);
    CPPUNIT_TEST( testMultiTypeAsInputLine);
    CPPUNIT_TEST( testMultiTypeAsInputLineThreaded);
    CPPUNIT_TEST( testMultiTypeAsInputCJit);
    CPPUNIT_TEST( testLambdaMultiTypeAsInputCJit);
    CPPUNIT_TEST( testMultiTypeAsInputCJitWithConstantVec);
    CPPUNIT_TEST( testMultiTypeAsInputCJitThreaded);

    //Mulititype composition
    CPPUNIT_TEST( testMultiTypeOutputFileKeyCompoKey);
    CPPUNIT_TEST( testMultiTypeOutputFileLineCompoLine);
    CPPUNIT_TEST( testMultiTypeOutputFileFlatCompoFlat);
    CPPUNIT_TEST( testMultiTypeOutputFileFlat2CompoFlat2);
    CPPUNIT_TEST( testMultiTypeOutputFileFlatMultiLineCompoFlat);
    CPPUNIT_TEST( testMultiTypeOutputFileKeyCompoKeyThreaded);

    CPPUNIT_TEST( testFillMultiTypeFormat);
    CPPUNIT_TEST( testFillMultiTypeFormatScientific);

    //Exception control
    CPPUNIT_TEST( testThrowExceptionTypeForTCIntEval);
    CPPUNIT_TEST( testThrowExceptionPythonEvalThreaded);
    CPPUNIT_TEST( testThrowExceptionConstantIssues);
    CPPUNIT_TEST( testThrowExceptionStringInMpi);
    
    CPPUNIT_TEST_SUITE_END();

public:
    //---------------------------------------------
    /** @name Constructor and Destructor
     **/
    //@{
    //! Default constructor
    TLauncher2Test();

    //! Default destructor
    virtual ~TLauncher2Test();
    //@}

    //---------------------------------------------
    /** @name SetUp and internals methods
     **/
    //@{
    void setUp();
    void tearDown();
    //! the clean tempory directories
    void cleanWorkingDirectories(const char *option = "without");
    //@}

    //---------------------------------------------
    /** @name Unit Tests
	The list of unit test
    **/
    //@{ 
    //! Test chaining for a scalar code
    void testComposeScalaire();

    //! Test chaining for a multi code
    void testComposeMulti(int outType, int readempty=0);
    void testComposeMultiKey();
    void testComposeMultiKeyWithEmpty();
    void testComposeMultiLine();
    void testComposeMultiLineWithEmpty();
    void testComposeMultiFlat();
    void testComposeMultiFlatWithEmpty();
    void testComposeMultiFlatMultiLine();
    
    //! Test Flowrate in sequential mdoe
    void testFlowrateSequential();
    void testFlowrateSequentialFailure();

    //! Test Flowrate in multithread mode
    void testFlowrateThreaded();
    void testFlowrateThreadedFailure();

    //! Test Flowrate in sequential mdoe with all folders option
    void testFlowrateSequentialAllFolders();

    //! Test Flowrate in multithread mode with all folders option
    void testFlowrateThreadedAllFolders();

#ifdef WITH_MPI 
    //! Test Flowrate in MPI mode
    void testFlowrateMPI();
    void testFlowrateMPIFailure();
    void testFlowrateMPIAllFolders();
#endif

    //! Test Ishigami in sequential mode
    void testIshigamiSequential();
    void testIshigamiShort();

    //! Test Ishigami in multithread mode
    void testIshigamiSo();
    void testIshigami2So();
    void testIshigami3So();
    void testIshigami4lambda();

#ifdef WITH_MPI
    //! Test Ishigami in MPI mode
    void testIshigamiMPI();
#endif
#ifdef WITH_PCL
    //! Test Ishigami in Couroutine mode
    void testIshigamiPcl(TString &macro);
    void testIshigamiPclInt();
    void testIshigamiPclJit();
#endif

#ifdef WITH_PYTHON
    //! Test Ishigami in sequential mode in Python
    void testIshigamiSequentialPython();
    //! Test read multitype as input in TPythonEval
    void testReadmtSequentialPython();
    //! Test dump multitype as output TPythonEval
    void testmtSequentialPython();
    
#ifdef WITH_MPI
    //! Test Ishigami in MPI mode in Python
    void testIshigamiMPIPython();
#endif
#endif

    //! Test Bidon in sequential mode
    void testBidonSequential(int outType, int empty=0);
    void testBidonSequentialKey();
    void testBidonSequentialKeyWithEmpty();
    void testBidonSequentialLine();
    void testBidonSequentialLineWithEmpty();
    void testBidonSequentialFlat();
    void testBidonSequentialFlatWithEmpty();
    void testBidonSequentialFlatMultiLine();

    //! Test Bidon in threaded mode
    void testBidonThreaded(int outType, int empty=0);
    void testBidonThreadedKey();
    void testBidonThreadedKeyWithEmpty();
    void testBidonThreadedLine();
    void testBidonThreadedLineWithEmpty();
    void testBidonThreadedFlat();
    void testBidonThreadedFlatWithEmpty();
    void testBidonThreadedFlatMultiLine();


    void testThreadedWithDataServer();
    void testThreadedWithDataServerFailing();

#ifdef WITH_MPI
#ifdef WITH_PYTHON
    //! Test Bidon in MPI mode
    void testBidonMPI(string outType, string readempty="0");
    void testBidonMPIKey();
    void testBidonMPIKeyWithEmpty();
    void testBidonMPILine();
    void testBidonMPILineWithEmpty();
    void testBidonMPIFlat();
    void testBidonMPIFlatWithEmpty();
    void testBidonMPIFlatMultiLine();
#endif
#endif
    //@}

    /** @name Multitype method in output
     **/
    //@{ 
    void setUpMultiType();
    //! multitype with DataServer output file
    void testMultiTypeOutputFileFlat();
    //! multitype with Flat output file
    void testMultiTypeOutputFileFlat2();
    //! multitype with Row output file
    void testMultiTypeOutputFileFlatMultiLine();
    //! multitype with key output file
    void testMultiTypeOutputFileKey();
    //! multitype with key on a single line output file
    void testMultiTypeOutputFileKeyCondensate();
    //! multitype with  output file line
    void testMultiTypeOutputFileLine();
    //! multitype with Row output file Threaded
    void testMultiTypeOutputFileLineThreaded();
    //@}

    
    /** @name Multitype method in input
     **/
    //@{ 
    void setUpMultiTypeInput();
    //! multitype as input with key 
    void testMultiTypeAsInputKey();
    //! multitype as input with flat 
    void testMultiTypeAsInputFlat();
    //! multitype as input with flat 
    void testMultiTypeAsInputFlat2();
    //! multitype as input with line 
    void testMultiTypeAsInputLine();
    //! multitype as input with line Threaded 
    void testMultiTypeAsInputLineThreaded();
    //! multitype as input with CJit function
    void testMultiTypeAsInputCJit();
    //! lambda multitype as input with CJit function
    void testLambdaMultiTypeAsInputCJit();
    //! multitype as input with CJit function and constant vector
    void testMultiTypeAsInputCJitWithConstantVec();
    //! multitype as input with CJit function Threaded
    void testMultiTypeAsInputCJitThreaded();

    void testFillMultiTypeFormat();
    void testFillMultiTypeFormatScientific();
    
    //@}


    /** @name Multitype composition 
     **/
    //@{ 
    void setUpMultiTypeCompo();
    //! multitype with key and key on a single line output file
    void testMultiTypeOutputFileKeyCompoKey();
    //! multitype with line and line on a single line output file
    void testMultiTypeOutputFileLineCompoLine();
    //! multitype with flat and flat on a single line output file
    void testMultiTypeOutputFileFlatCompoFlat();
    //! multitype with flat and flat on a single line output file
    void testMultiTypeOutputFileFlat2CompoFlat2();
    //! multitype with flat and flat on a multi line output file
    void testMultiTypeOutputFileFlatMultiLineCompoFlat();
    //! multitype with key and key on a single line output file
    void testMultiTypeOutputFileKeyCompoKeyThreaded();
    //@}

    
    /** @name Validation methods
     **/
    //@{ 
    //! Test the output values for flowrate
    void validateValuesFlowrate();
    void validateValuesFlowrateFailure();

    //! Test the output values for Ishigami
    void validateValuesIshigami(URANIE::DataServer::TDataServer *tds=NULL);
    void validateValuesIshigami3(URANIE::DataServer::TDataServer *tds=NULL);
    void validateValuesIshigamiErr(URANIE::DataServer::TDataServer *tds);

    //! Test the output values for bidon
    void validateValuesBidon(int readempty=0);

    //! Test the output values for threaded
    void validateValuesThreadable(bool hasfile);

    //! Test the output values for ReadMultiType
    void validateValuesReadmt(URANIE::DataServer::TDataServer *tds, bool nov2=false);

    //! Test the output of multiType
    void validateOutputMultiTypeResults(URANIE::DataServer::TDataServer *tds);
    //@}

    /** @name Exceptions
     **/
    //@{
    //! Check that including string is not allowed in TCIntEval
    void testThrowExceptionTypeForTCIntEval();
    //! Check that including string is not allowed in TCJitGradEval
    void testThrowExceptionTypeForTCJitGradEval();
    //! Shout if string provided to Mpi
    void testThrowExceptionStringInMpi();
    //! Shout if attribute exists in tds and is added constant
    void testThrowExceptionConstantIssues();
    //! Shout if python Given to threadedrun
    void testThrowExceptionPythonEvalThreaded();
    //@}
    
private:
    TString _workingTestPath;
    Bool_t _bLog;

    void *_p2fPrecodeComposeScalaire;
    void *_p2fPostcodeComposeScalaire;
    void *_p2fPrecodeComposeMulti;
    void *_p2fPostcodeComposeMulti;
    void *_p2fIshigami;
    void *_p2fIshigami2;

    URANIE::DataServer::TDataServer *_tdsFlowrate, *_errFlowrate;
    URANIE::DataServer::TAttribute *_yhat, *_d;

    URANIE::DataServer::TDataServer *_tdsIshigami;
    URANIE::DataServer::TAttribute *_ct;

    URANIE::DataServer::TDataServer *_tdsBidon, *_errBidon;
    URANIE::DataServer::TAttribute *_x, *_diff;

    URANIE::DataServer::TDataServer *_tds1;
    URANIE::DataServer::TAttribute *_w1, *_w2, *_v1, *_v2, *_f1, *_thev1, *_thev2;
};

#endif
