
#ifdef JIT
#include "TUniformDistribution.h"
#include "TFlagScript.h"
#include "TFlatResult.h"
#include "TCodeEval.h"
#include "TThreadedRun.h"
#include "TDataServer.h"
#include "TSampling.h"
#include "TLauncher2.h"
#endif

using namespace URANIE::DataServer;
using namespace URANIE::Sampler;
using namespace URANIE::Relauncher;

void test_TLauncherBidon()
{

    // Add the study attributes ( min, max )
    TUniformDistribution delai("delai", 0.0, 1.0), pente("pente", 0.8, 1.2);
    TAttribute x("x"), diff("diff");

    // Set the reference input file and the key for each input attributes
    TFlagScript fin("bidon.py");
    fin.addInput(&delai, "START");
    fin.addInput(&pente, "PENTE");

    // The output file of the code
    TFlatResult fout("bidon.res");
    fout.addOutput(&x);
    fout.addOutput(&diff);

    // Instanciation de mon code
#ifdef PYTHON3
    TCodeMulti mycode("python2 bidon.py > bidon.res");
#else
    TCodeMulti mycode("python bidon.py > bidon.res");
#endif
    mycode.setWorkingDir(gSystem->Getenv("HOME") + TString("/tmp/"));
    mycode.addInputFile(&fin);
    mycode.addOutputFile(&fout);

    //TSequentialRun run(&mycode);
    TThreadedRun run(&mycode, 2);
    //TMpiRun run(mycode);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsBidon", "Design of experiments for Bidon");
        tds.addAttribute(&delai);
        tds.addAttribute(&pente);

        TSampling sampling(&tds, "lhs", 50);
        sampling.generateSample();

        TLauncher2 lanceur(&tds, &run);

        /* resolution */
        lanceur.solverLoop();

        /* solution */
        tds.getTuple()->Scan("*");
        tds.exportData("_bidon_.dat");
        run.stopSlave();
    }

}
