using namespace URANIE::DataServer;
using namespace URANIE::Sampler;
using namespace URANIE::Relauncher;

int precode(Double_t *in, Double_t *out)
{
    out[0] = (in[0] - in[1]);
    return 1;
}

int postcode(Double_t *in, Double_t *out)
{
    out[0] = 2.0 * in[0];
    return 1;
}

void test_TLauncherComposeScalaire()
{

    // Define the DataServer

    // Add the study attributes ( min, max and nominal values)
    TUniformDistribution rw("rw", 0.05, 0.15), r("r", 100.0, 50000.0), tu("tu",
            63070.0, 115600.0), tl("tl", 63.1, 116.0), hu("hu", 990.0, 1110.0),
            hl("hl", 700.0, 820.0), l("l", 1120.0, 1680.0), kw("kw", 9855.0,
                    12045.0);

    TAttribute delai("delai"), diff("diff"), yhat("yhat"), d("d");

    // pretraitement
    TCJitEval pre(&precode);
    pre.setInputs(2, &hu, &hl);
    pre.addOutput(&delai);

    // traitement code
    // Set the reference input file and the key for each input attributes
    TKeyScript fin("flowrate_input_with_keys.in");
    fin.setInputs(8, &rw, "Rw", &r, "R", &tu, "Tu", &tl, "Tl", &hu, "Hu", &hl,
            "Hl", &l, "L", &kw, "Kw");

    // The output file of the code
    TFlatResult fout("_output_flowrate_withRow_.dat");
    fout.setOutputs(2, &yhat, &d);

    // Instanciation de mon code
    TCodeEval mycode("flowrate -s -k");
    //     mycode.setWorkingDir(gSystem->Getenv("HOME") + TString("/tmp/"));
    mycode.addInputFile(&fin);
    mycode.addOutputFile(&fout);

    // postraitement
    TCJitEval post(&postcode);
    post.setInputs(1, &d);
    post.setOutputs(1, &diff);

    // composition
    TComposeEval code;
    code.setEvals(3, &pre, &mycode, &post);

    //TSequentialRun run(&code);
    TSocketRun run(&code);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tdsBidon", "Design of experiments for Bidon");
        tds.addAttribute(&rw);
        tds.addAttribute(&r);
        tds.addAttribute(&tu);
        tds.addAttribute(&tl);
        tds.addAttribute(&hu);
        tds.addAttribute(&hl);
        tds.addAttribute(&l);
        tds.addAttribute(&kw);

        TSampling sampling(&tds, "lhs", 50);
        sampling.generateSample();

        TLauncher2 lanceur(&tds, &run);

        /* resolution */
        lanceur.solverLoop();

        /* solution */
        //        tds.getTuple()->Scan("*");
        tds.exportData("_compose_m_.dat");
        tds.draw("d:diff");

        run.stopSlave();
    }

}
