#ifdef JIT
#include "TUniformDistribution.h"
#include "TCJitEval.h"
#include "TSampling.h"
#include "TLauncher2.h"
#include "TCanvas.h"
#endif

using namespace URANIE::DataServer;
using namespace URANIE::Sampler;
using namespace URANIE::Relauncher;

/* user defined */
#define TAILLEPOP 250 
//#define TAILLEPOP 10 

#define NBGENE 3 // nombre de paramètres

#define PI 3.1415926535

void Ishigami(Double_t *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    Double_t x1 = paramt[0];
    Double_t x2 = paramt[1];
    Double_t x3 = paramt[2];

    res[0] = sin(x1);
    res[0] += a * sin(x2) * sin(x2);
    res[0] += b * x3 * x3 * x3 * x3 * sin(x1);

    //std::cout << x1 << " " << x2 << " " << x3 << " " << res[0] << std::endl;
}

void Ishigami2(Double_t *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    res[0] = sin(paramt[0]);
    res[0] += a * sin(paramt[1]) * sin(paramt[1]);
    res[0] += b * paramt[2] * paramt[2] * paramt[2] * paramt[2]
            * sin(paramt[0]);
    res[1] = 5.0 * paramt[0];
}

void test_TLauncherIshigami()
{
    /* init */
    TUniformDistribution    x1("x_{1}", -1.0 * PI, PI),
                            x2("x_{2}", -1.0 * PI, PI),
                            x3("x_{3}", -1.0 * PI, PI);
    //TAttribute ct("Ishigami");

    TCJitEval code(Ishigami);
    //code.addInput(&x1);
    //code.addInput(&x2);
    //code.addInput(&x3);
    //code.addOutput(&ct);

    TDataServer tds("tds", "tds for vizir test");
    tds.addAttribute(&x3);
    tds.addAttribute(&x1);
    tds.addAttribute(&x2);

    TSampling sampl(&tds, "lhs", TAILLEPOP);
    sampl.generateSample();

    TLauncher2 launch(&tds, &code, "x_{1}:x_{2}:x_{3}", "Ishigami");

    /* resolution */
    launch.solverLoop();

    // Visualisation
    TCanvas *canvas = new TCanvas("c1", "c1", 15, 36, 713, 934);
    canvas->Clear();
    canvas->Divide(1, 3);
    canvas->cd(1);
    tds.Draw("Ishigami:x_{1}");
    //   saveCanvas(canvas, "usermanual_launcher_ishigami_y_x1");
    canvas->cd(2);
    tds.Draw("Ishigami:x_{2}");
    //   saveCanvas(canvas, "usermanual_launcher_ishigami_y_x2");
    canvas->cd(3);
    tds.Draw("Ishigami:x_{3}");
    //tds.getTuple()->Scan("*");
    //tds.exportData("_thread_.dat");

}

