#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "TAttribute.h"
#include "TUniformDistribution.h"
#include "TCJitEval.h"
#include "TThreadedRun.h"
#include "TDataServer.h"
#include "TSampling.h"
//#include "TIter.h"
#include "TLauncher2.h"
#include "TCanvas.h"

using namespace URANIE::DataServer;
using namespace URANIE::Sampler;
using namespace URANIE::Relauncher;

#define PI 3.1415926535

void Ishigami(Double_t *paramt, Double_t *res)
{
    Double_t a = 7.0;
    Double_t b = 0.1;

    Double_t x1 = paramt[0];
    Double_t x2 = paramt[1];
    Double_t x3 = paramt[2];

    res[0] = sin(x1);
    res[0] += a * sin(x2) * sin(x2);
    res[0] += b * x3 * x3 * x3 * x3 * sin(x1);

    //std::cout << x1 << " " << x2 << " " << x3 << " " << res[0] << std::endl;
}
void test_TLauncherIshigamiSo()
{
    /* init */
    TUniformDistribution x1("x_{1}", -1.0 * PI, PI), x2("x_{2}", -1.0 * PI, PI),
            x3("x_{3}", -1.0 * PI, PI);

    TAttribute ct("Ishigami");

    TCJitEval code(Ishigami);
    code.setInputs(3, &x1, &x2, &x3);
    code.setOutputs(1, &ct);

    TThreadedRun run(&code, 14);
    run.startSlave();
    if (run.onMaster())
    {
        TDataServer tds("tds", "tds for vizir test");
        TIter next(code.getInputList());
        TAttribute *i;
        while ((i = (TAttribute *) next()))
            tds.addAttribute(i);

        TSampling sampl(&tds, "lhs", 5000);
        sampl.generateSample();

        TLauncher2 launch(&tds, &run);

        /* resolution */
        launch.solverLoop();

        //tds.getTuple()->Scan("*");
        tds.exportData("_thread_.dat");
        // Visualisation
        TCanvas *canvas = new TCanvas("c1", "c1", 15, 36, 713, 934);
        canvas->Clear();
        canvas->Divide(1, 3);
        canvas->cd(1);
        tds.Draw("Ishigami:x_{1}");
        //   saveCanvas(canvas, "usermanual_launcher_ishigami_y_x1");
        canvas->cd(2);
        tds.Draw("Ishigami:x_{2}");
        //   saveCanvas(canvas, "usermanual_launcher_ishigami_y_x2");
        canvas->cd(3);
        tds.Draw("Ishigami:x_{3}");
        //tds.getTuple()->Scan("*");
        //tds.exportData("_thread_.dat");

        run.stopSlave();
    }

}
