#include <cppunit/Test.h>
#include <cppunit/portability/Stream.h>
#include "ResourceListener.h"
#include "ResourceModel.h"
#include <stdio.h>
#include "stdlib.h"

ResourceListener::ResourceListener( ResourceModel *model,
                                  bool text )
    : m_model( model )
    , m_text( text )
{
}


ResourceListener::~ResourceListener()
{
}


void ResourceListener::startTestRun( CPPUNIT_NS::Test *test, 
                               CPPUNIT_NS::TestResult *eventManager )
{
  m_model->setExpectedTestCount( test->countTestCases() *2 );
}


void ResourceListener::endTestRun( CPPUNIT_NS::Test *test, 
                             CPPUNIT_NS::TestResult *eventManager )
{
  if ( m_text )
    printStatistics();
}


void ResourceListener::startTest( CPPUNIT_NS::Test *test )
{
  m_model->enterTest( test, false );
}

void ResourceListener::endTest( CPPUNIT_NS::Test *test )
{
  m_model->exitTest( test, false );
}


void ResourceListener::startSuite( CPPUNIT_NS::Test *suite )
{
  m_model->enterTest( suite, true );
}


void ResourceListener::endSuite( CPPUNIT_NS::Test *suite )
{
  m_model->exitTest( suite, true );
}


void ResourceListener::printStatistics() const
{
  printTest( 0, "" );
  CPPUNIT_NS::stdCOut()  <<  "\n";
  CPPUNIT_NS::stdCOut()  <<  "Total elapsed time: ";
  printTime( m_model->totalElapsedTime() );
  CPPUNIT_NS::stdCOut()  <<  ", average test case time: ";
  printTime( m_model->averageTestCaseTime() );
  CPPUNIT_NS::stdCOut()  << "\n";
}


void ResourceListener::printTest( int testIndex,
                            const std::string &indentString ) const
{
  std::string indent = indentString;
  const int indentLength = 3;

  printTestIndent( indentString, indentLength );
  double time= m_model->testTimeFor( testIndex ) ;
  double memory=m_model->testMemoryFor( testIndex );
  printTimeMemory( time, memory );
  CPPUNIT_NS::stdCOut()  <<  m_model->testPathFor( testIndex ).getChildTest()->getName();
  CPPUNIT_NS::stdCOut()  <<  "\n";

  if ( m_model->childCountFor( testIndex ) == 0 )
    indent+= std::string( indentLength, ' ' );
  else
    indent+= "|" + std::string( indentLength -1, ' ' );

  for ( int index =0; index < m_model->childCountFor( testIndex ); ++index )
    printTest( m_model->childAtFor( testIndex, index ), indent );
}


void ResourceListener::printTestIndent( const std::string &indent,
                                  const int indentLength ) const
{
  if ( indent.empty() )
    return;

  CPPUNIT_NS::stdCOut() << "   ";
  CPPUNIT_NS::stdCOut() << indent.substr( 0, indent.length() - indentLength ) ;
  CPPUNIT_NS::stdCOut() << "+"  <<  std::string( indentLength -1, '-' );
}


void ResourceListener::printTime( double time ) const
{
  CPPUNIT_NS::stdCOut() <<  '('  <<  ResourceModel::timeStringFor( time )  <<  " ms) ";
}

void ResourceListener::printTimeMemory( double time, double memory ) const
{
  CPPUNIT_NS::stdCOut() <<  '('  <<  ResourceModel::timeStringFor( time )  <<  " ms and " << memory << " kB) ";
}

void ResourceListener::printMemory( double memory ) const
{
  CPPUNIT_NS::stdCOut() <<  '('  <<  memory  <<  " kB) ";
}
