#include "ResourceModel.h"
#include <cppunit/config/SourcePrefix.h>

#include <stdio.h>
#include "stdlib.h"
#include "string.h"

ResourceModel::ResourceModel()
    : m_testCaseCount( 0 )
    , m_totalTestCaseTime( 0 )
{
}


ResourceModel::~ResourceModel()
{
}


void ResourceModel::setExpectedTestCount( int count )
{
    m_tests.reserve( count );
}


void ResourceModel::enterTest( CPPUNIT_NS::Test *test,
                         bool isSuite )
{
    m_currentPath.add( test );

    int testIndex = m_tests.size();
    if ( !m_testIndexes.empty() )
        m_tests[ m_testIndexes.top() ].m_childIndexes.push_back( testIndex );
    m_testIndexes.push( testIndex );
    m_testToIndexes.insert( TestToIndexes::value_type( test, testIndex ) );

    TestInfo info;
    info.m_timer.start();
    info.m_memory.start();
    info.m_path = m_currentPath;
    info.m_isSuite = isSuite;
    m_tests.push_back( info );

    if ( !isSuite )
        ++m_testCaseCount;
}


void ResourceModel::exitTest( CPPUNIT_NS::Test *test,
                        bool isSuite )
{
    m_tests[ m_testIndexes.top() ].m_timer.finish();
    m_tests[ m_testIndexes.top() ].m_memory.finish();
    if ( !isSuite )
        m_totalTestCaseTime += m_tests.back().m_timer.elapsedTime();

    m_currentPath.up();
    m_testIndexes.pop();
}


double ResourceModel::totalElapsedTime() const
{
    return m_tests[0].m_timer.elapsedTime();
}


double ResourceModel::averageTestCaseTime() const
{
    double average = 0;
    if ( m_testCaseCount > 0 )
        average = m_totalTestCaseTime / m_testCaseCount;
    return average;
}


double ResourceModel::testTimeFor( int testIndex ) const
{
    return m_tests[ testIndex ].m_timer.elapsedTime();
}

double ResourceModel::testMemoryFor( int testIndex ) const
{
    return m_tests[ testIndex ].m_memory.elapsedMemory();
}

std::string ResourceModel::timeStringFor( double time )
{
    char buffer[320];
    const char *format;
    if ( time < 1 )
        format = "%2.3f";
    else if ( time < 10 )
        format = "%3.2f";
    else if (time < 100 )
        format = "%4.1f";
    else
        format = "%6f";

    ::sprintf( buffer, format, time );

    return buffer;
}


bool ResourceModel::isSuite( int testIndex ) const
{
    return m_tests[ testIndex ].m_isSuite;
}


const CPPUNIT_NS::TestPath &ResourceModel::testPathFor( int testIndex ) const
{
    return m_tests[ testIndex ].m_path;
}


int ResourceModel::indexOf( CPPUNIT_NS::Test *test ) const
{
    TestToIndexes::const_iterator itFound = m_testToIndexes.find( test );
    if ( itFound != m_testToIndexes.end() )
        return itFound->second;
    return -1;
}


int ResourceModel::childCountFor( int testIndex ) const
{
    return m_tests[ testIndex ].m_childIndexes.size();
}


int ResourceModel::childAtFor( int testIndex, 
                          int chidIndex ) const
{
    return m_tests[ testIndex ].m_childIndexes[ chidIndex ];
}
