//////////////////////////////////////////////////////////////
// Copyright (C) 2013-2022 CEA/DES
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation, either version 3 of the License, or any
// later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//////////////////////////////////////////////////////////////
#ifndef TestXmlOutputterHook_H
#define TestXmlOutputterHook_H

#include "cppunit/XmlOutputterHook.h"
#include "cppunit/tools/XmlElement.h"
#include "cppunit/tools/XmlDocument.h"
#include "cppunit/tools/StringTools.h"

#include <iostream>

// ROOT
#include "TDatime.h"
#include "TROOT.h"

/// XML output hook to add test timing and test hierarchy timing.
class TestXmlOutputterHook: public CPPUNIT_NS::XmlOutputterHook
{
public:
    /*! Constructs a TestXmlOutputterHook object.
     */
    TestXmlOutputterHook(const std::string projectName,
            const std::string author);

    /// Destructor.
    virtual ~TestXmlOutputterHook();

    //! Set the name and the version of the module
    void setModule(const std::string moduleName, const std::string versionName);

    void beginDocument(CPPUNIT_NS::XmlDocument* document)
    {
        if (!document)
            return;

        // dump current time
        //std::string szDate          = CppUnit::StringTools::toString( (int)time(0) );

        CppUnit::XmlElement* metaEl = new CppUnit::XmlElement("SuiteInfo", "");

        metaEl->addElement(new CppUnit::XmlElement("Project", m_projectName));
        metaEl->addElement(new CppUnit::XmlElement("Module", m_moduleName));
        metaEl->addElement(new CppUnit::XmlElement("Version", m_versionName));
        metaEl->addElement(new CppUnit::XmlElement("Author", m_author));
        metaEl->addElement(
                new CppUnit::XmlElement("Date", m_beginTime->AsString()));
        metaEl->addElement(
                new CppUnit::XmlElement("ROOT", gROOT->GetVersion()));

        document->rootElement().addElement(metaEl);

    }

//     void successfulTestAdded(CPPUNIT_NS::XmlDocument* document,
//             CPPUNIT_NS::XmlElement* testElement, CPPUNIT_NS::Test* test)
//     {
//         if (!document)
//             return;

//         // dump current time
//         //    std::cout << "successful test[" << test->getName() <<"] Child["<<
//         //  test->getChildTestCount ()  << "]" << std::endl;

//         TString sss = TString(test->getName());

// //     CppUnit::XmlElement* metaEl = new CppUnit::XmlElement("Link", "");
// //     metaEl->addElement( new CppUnit::XmlElement("file", Form("../htmldoc/%s", sss.Data()) ) );
// //     document->rootElement().addElement(metaEl);
//     }

    void endDocument(CPPUNIT_NS::XmlDocument* document)
    {
        if (!document)
            return;

        // dump current time
        TDatime *theTime = new TDatime();

        CppUnit::XmlElement* metaEl = new CppUnit::XmlElement("TimeInfo", "");
        metaEl->addElement(new CppUnit::XmlElement("Begin", Form("%04d-%02d-%02dT%02d:%02d:%02d",
							       m_beginTime->GetYear(), m_beginTime->GetMonth(), m_beginTime->GetDay(),
							       m_beginTime->GetHour(), m_beginTime->GetMinute(), m_beginTime->GetSecond()
								 )));
        
        metaEl->addElement(new CppUnit::XmlElement("End", Form("%04d-%02d-%02dT%02d:%02d:%02d",
							       theTime->GetYear(), theTime->GetMonth(), theTime->GetDay(),
							       theTime->GetHour(), theTime->GetMinute(), theTime->GetSecond()
							       )));
		
        document->rootElement().addElement(metaEl);
		
       delete theTime;
    }
private:
    std::string m_projectName;
    std::string m_moduleName;
    std::string m_versionName;
    std::string m_author;
    TDatime *m_beginTime;
	
};

#endif  // TestXmlOutputterHook_H
