///////////////////////////////////////////////////////////////////////////////
//$<AM-V1.0>
//
//$Type
//       DEF
//
//$Projet
//       Marmottes
//
//$Application
//       Club
//
//$Nom
//>       ChaineSimple.cpp
//
//$Resume
//       fichier d'implantation de la classe ChaineSimple
//
//$Description
//       Module de dfinition de la classe
//
//$Contenu
//>       inline short int & EntierCourt()          
//>       inline short int & NombreTotalCellules()  
//>       inline short int & NombreCellulesLibres() 
//>       inline short int & PremiereCelluleLibre() 
//>       inline short int & CellulesBlocCourant()  
//>       inline short int & CellulesBlocPrecedent()
//>       inline char *MemoireUtilisateur()         
//>       static int CreeTampon()                   
//>       static int CelluleReservable()            
//>       static void ReserveChaine()               
//>       static void RestitueChaine()              
//>       class ChaineSimple                        
//>         alloueChaine()                          
//>         etendChaine()                           
//>         ChaineSimple()                          
//>         ~ChaineSimple()                         
//>         reinitialise()                          
//>         formate()                               
//>         vFormate()                              
//>         modifieCaractere()                      
//>         elimineBlancsFinaux()                   
//>         elimineBlancsInitiaux()                 
//>         operator +=()                           
//>         operator()()                            
//
//$Historique
//       $Log: ChaineSimple.cpp,v $
//       Revision 1.20  2003/07/17 12:38:06  club
//       Mise a jour suite aux patchs du 17/07/003
//
//       Revision 1.19  2003/02/03 09:05:10  club
//       DM-ID 17 Changement de l'extension du fichier dans le bandeau
//
//       Revision 1.18  2002/12/10 08:51:42  club
//       DM-ID 12 Modification portant sur l'initialisation des pointeurset des variables.
//
//       Revision 1.17  2000/10/24 08:33:29  club
//       modification des include standard du C : utilisation de cNAME au lieu de
//       NAME.h (pour permettre la compilation avec le Sun Workshop 5.0)
//
//       Revision 1.16  2000/08/30 14:20:44  club
//       Ajout du '\0'  la fin de la ChaineSimple construite  partir d'un string STL.
//
//       Revision 1.15  2000/08/25 07:43:13  club
//       Ajout du constructeur de ChaineSimple  partir d'un string.
//
//       Revision 1.14  2000/07/03 08:57:18  club
//       Ajout du copyright CNES
//
//       Revision 1.13.2.2  2000/03/30 14:46:46  club
//       ajout du copyright CNES
//
//       Revision 1.13.2.1  2000/03/30 11:23:14  club
//       limination d'une portion de code optionnelle posant
//       des problmes de droits de distribution
//
//       Revision 1.13  1999/03/15 13:15:41  club
//       modification lors de la rinitialisation d'une chane simple, si la chane
//       n'existe pas elle est alloue automatiquement
//
//       Revision 1.12  1998/07/27 10:13:40  club
//       passage  une configuration de type GNU (fichier ClubConfig.h gnr)
//
//       Revision 1.11  1998/06/24 08:43:39  luc
//       correction d'un crasement mmoire lors des librations de chanes
//
//       Revision 1.10  1998/06/21 15:53:34  luc
//       modification du format de l'en-tte
//       limination de la chane statique RCS
//
//       Revision 1.9  1998/04/26 13:02:19  luc
//       inversion de la convention de nommage des attributs
//
//       Revision 1.8  1998/01/22 08:20:14  luc
//       initialisation d'une variable pour supprimer un avertissement
//
//       Revision 1.7  1997/10/14 16:44:42  luc
//       acclration de la recherche des cellules libres contigus
//       passage par un typedef pour faciliter le travail du compilateur SUN
//
//       Revision 1.6  1997/08/21 16:48:10  luc
//       ajout d'un en-tte de fichier
//       prise en compte des '\0' pouvant tre contenus dans une chane
//       dont on donne explicitement une longueur plus grande
//       constructeur ChaineSimple::ChaineSimple (const char *chaine, int longueur)
//
//       Revision 1.5  1997/04/28 14:18:18  luc
//       correction d'une longueur de copie
//       
//       Revision 1.4  1997/04/27 17:15:04  luc
//       allocation du tampon d'criture de ChaineSimple::vFormate
//       uniquement au premier appel
//       
//       Revision 1.3  1997/03/25 14:45:44  luc
//       ajout d'une mthode vFormate prenant un argument de type va_list
//       changement des rgles de nommage
//       
//       Revision 1.2  1997/03/19 15:13:44  luc
//       ajout d'un niveau intermdiaire pour la gestion mmoire
//       entre les chanes de caractres et les oprateurs new/delete
//       
//       Revision 1.1  1997/03/16 18:09:59  luc
//       Initial revision
//
//$Version
//       $Id: ChaineSimple.cpp,v 1.20 2003/07/17 12:38:06 club Exp $
//
//$Auteur
//       L. Maisonobe CNES
//       O. Queyrut   CSSI
//       Copyright (C) 2000 CNES
//
//$<>
///////////////////////////////////////////////////////////////////////////////

#include "club/ChaineSimple.h"

// gestion de la mmoire par allocation de plusieurs tampons
// et distribution de cellules de taille fixe dans ces tampons
static int    TailleTable       = 0;
static char **TableTampons      = 0;

// avec 250 cellules de 16 octets plus une cellule pour la gestion,
// les tampons minimaux occuperont 4000 octets, mme en comptant large
// les oprateurs new et les ventuels malloc sous-jacents peuvent ajouter
// leur mmoire de gestion tout en restant dans une page de 4096 octets.
const  int    TailleCellule     = 16;
const  int    NombreCellulesMin = 250;

// les tampons sont des zones contigus de cellules toutes de mme
// taille, la premire cellule est rserve  la gestion du tampon
// elle contient le nombre total de cellules (sans compter la
// premire) et le nombre de cellules libres ; lorsque de la mmoire
// est attribue  l'utilisateur, le nombre de cellules concernes et
// le nombre de cellules du bloc prcdent sont stocks dans deux
// entiers courts au dbut de la premire cellule, on attribue donc
// toujours une taille de la forme k * tc - 2 * s, o tc est la taille
// d'une cellule et s la taille d'un entier court
inline short int & EntierCourt (int i, int decalage)
{ return *((short int *) (TableTampons [i] + decalage)); }


// informations globales du tampon d'index i
inline short int & NombreTotalCellules (int i)
{ return EntierCourt (i, 0); }

inline short int & NombreCellulesLibres (int i)
{ return EntierCourt (i, sizeof(short int)); }

inline short int & PremiereCelluleLibre (int i)
{ return EntierCourt (i, 2 * sizeof(short int)); }


// information pour le bloc commenant  la cellule c du tampon i
// (attention, les nombres de cellules sont positifs si les blocs
//  contiennent des donnes, et ngatifs si les blocs sont libres)
inline short int & CellulesBlocCourant (int i, int c)
{ return EntierCourt (i, (c + 1) * TailleCellule); }

inline short int & CellulesBlocPrecedent (int i, int c)
{ return EntierCourt (i, (c + 1) * TailleCellule + sizeof(short int)); }

inline char *MemoireUtilisateur (int i, int c)
{ return TableTampons [i] + (c + 1) * TailleCellule + 2 * sizeof(short int); }


static int CreeTampon (int nombreCellules)
{ // cration d'un tampon contenant au moins un nombre de cellules donn
  if (nombreCellules < NombreCellulesMin)
    nombreCellules = NombreCellulesMin;

  int i, index = -1;

  // recherche d'une entre disponible dans la table
  for (i = 0; (i < TailleTable) && (index < 0); i++)
    if (TableTampons [i] == 0)
      index = i;

  if (index < 0)
  { // la table est pleine (ou inexistante), il faut l'tendre
    typedef char *CharPTR;
    int    nouvelleTaille = TailleTable ? (TailleTable << 1) : 8;
    char **nouvelleTable  = new CharPTR [nouvelleTaille];

    for (i = 0; i < TailleTable; i++)
      nouvelleTable [i] = TableTampons [i];
    for (i = TailleTable; i < nouvelleTaille; i++)
      nouvelleTable [i] = 0;

    // mmorisation de la premire entre libre
    index = TailleTable;

    // mise en place de la nouvelle table
    if (TailleTable)
      delete [] TableTampons;
    TailleTable  = nouvelleTaille;
    TableTampons = nouvelleTable;

  }

  // allocation du tampon
  TableTampons [index] = new char [(1 + nombreCellules) * TailleCellule];

  // initialisation du tampon
  NombreTotalCellules   (index)    = nombreCellules;
  NombreCellulesLibres  (index)    = nombreCellules;
  PremiereCelluleLibre  (index)    = 0;
  CellulesBlocCourant   (index, 0) = -nombreCellules;
  CellulesBlocPrecedent (index, 0) = 0;

  return index;

}

static int CelluleReservable (int index, int nombreCellules)
{ // recherche d'une zone de cellules libres contigus
  int cellule        = PremiereCelluleLibre (index);
  int cellulesLibres = NombreCellulesLibres (index);

  while (nombreCellules <= cellulesLibres)
  { // test de la cellule courante
    int reservees = CellulesBlocCourant (index, cellule);

    if (reservees > 0)
    { // il y a des cellules rserves ici, on les vite
      cellule += reservees;
    }
    else
    { // il y a des cellules libres
      if (nombreCellules > -reservees)
      { // le trou est trop petit
        cellule        += -reservees;
        cellulesLibres +=  reservees;
      }
      else
        return cellule;
    }

  }

  // il n'y a pas de zone utilisable
  return -1;

}

static void ReserveChaine (int tailleRequise,
                           int *ptrTailleReservee, char **ptrChaine)
{ // rservation d'espace dans les tampons pour stocker une chane
  if (tailleRequise < 0)
    tailleRequise = 0;

  int nombreCellules = (tailleRequise + 2 * sizeof(short int)
                      + TailleCellule - 1) / TailleCellule;

  // recherche de cellules libres contigus dans les tampons
  int cellule = -1, index = -1;
  for (int i = 0; (i < TailleTable) && (index < 0); i++)
  { // parcours de la table

    if (TableTampons [i] != 0)
    { // peut-on rserver une zone dans ce tampon ?
      cellule = CelluleReservable (i, nombreCellules);
      if (cellule >= 0)
        index = i; // le couple (i, cellule) convient, on arrte la recherche
    }

  }

  if (index < 0)
  { // aucun tampon ne disposait des cellules ncessaires : on en cre un
    index   = CreeTampon (nombreCellules);
    cellule = PremiereCelluleLibre (index);
  }

  // rservation des cellules
  int reste = (-CellulesBlocCourant (index, cellule)) - nombreCellules;
  if (reste > 0)
  { // on ne prend qu'une partie d'un bloc libre
    if (cellule + nombreCellules + reste < NombreTotalCellules (index))
      CellulesBlocPrecedent (index, cellule + nombreCellules + reste) = -reste;
    CellulesBlocCourant   (index, cellule + nombreCellules) = -reste;
  }

  if (cellule + nombreCellules < NombreTotalCellules (index))
    CellulesBlocPrecedent (index, cellule + nombreCellules) = nombreCellules;
  CellulesBlocCourant (index, cellule) = nombreCellules;

  // mise  jour du tampon
  NombreCellulesLibres (index) -= nombreCellules;
  if (cellule == PremiereCelluleLibre (index))
    PremiereCelluleLibre (index) = CelluleReservable (index, 1);

  // affectation des variables de l'appelant
  *ptrTailleReservee = nombreCellules * TailleCellule - 2 * sizeof(short int);
  *ptrChaine         = MemoireUtilisateur (index, cellule);

}

static void RestitueChaine (char *chaine)
{ // restitution de mmoire
  // cette fonction ne marche QUE si la chane provient d'un ReserveChaine

  // recherche du tampon contenant la chane
  int index = 0;
  while ((TableTampons [index] == 0)
         ||
         (MemoireUtilisateur (index, 0) > chaine)
         ||
         (MemoireUtilisateur (index,
                              NombreTotalCellules (index) - 1) < chaine))
    index++;

  // rcupration des lments de gestion du tampon
  int nombreTotal    = NombreTotalCellules (index);
  int cellule        = (chaine - TableTampons [index]) / TailleCellule - 1;
  int nombreRestitue = CellulesBlocCourant (index, cellule);

  if (NombreCellulesLibres (index) + nombreRestitue == nombreTotal)
  { // on doit librer la totalit du tampon : on le rend au systme
    delete [] TableTampons [index];
    TableTampons [index] = 0;
  }
  else
  { // il reste des chanes alloues dans le tampon : on le met  jour
    int tailleBloc = nombreRestitue;

    if (cellule + nombreRestitue < nombreTotal)
    { // analyse du bloc suivant le bloc restitu
      int cellulesSuivant = CellulesBlocCourant (index,
                                                 cellule + nombreRestitue);

      // si le bloc suivant est un trou, on l'incorpore au bloc courant
      if (cellulesSuivant < 0)
        tailleBloc += -cellulesSuivant;

    }

    // analyse du bloc prcdant le bloc restitu
    int cellulesPrecedant = CellulesBlocPrecedent (index, cellule);

    // si le bloc prcdant est un trou, on l'incorpore au bloc courant
    if (cellulesPrecedant < 0)
    { tailleBloc += -cellulesPrecedant;
      cellule    +=  cellulesPrecedant;
    }

    // transformation du bloc allou en bloc libre
    CellulesBlocCourant (index, cellule) = -tailleBloc;
    if (cellule + tailleBloc < nombreTotal)
      CellulesBlocPrecedent (index, cellule + tailleBloc) = -tailleBloc;

    // mise  jour des compteurs globaux du tampon
    NombreCellulesLibres (index) += nombreRestitue;
    short int premiere = PremiereCelluleLibre (index);
    if ((premiere < 0) || (premiere > cellule))
      PremiereCelluleLibre (index) = cellule;

  }

}

void ChaineSimple::alloueChaine (int taille)
{ // allocation d'une chane d'au moins taille caractres
  ReserveChaine (taille, &taille_, &chaine_);

  // mise en place d'un chien de garde pour la mthode modifieCaractere 
  chaine_ [taille_ - 1] = '\0';

  // initialisation de la chane
  chaine_ [0]           = '\0';
  longueur_             = 0;

}

void ChaineSimple::etendChaine (int taille)
{ // extension du tableau de mmorisation d'une chane

  // prparation d'un nouveau tableau
  int   nouvelleTaille = 0;
  char *nouvelleChaine = NULL;
  ReserveChaine (taille, &nouvelleTaille, &nouvelleChaine);

  // mise en place d'un chien de garde pour la mthode modifieCaractere 
  nouvelleChaine [nouvelleTaille - 1] = '\0';

  // copie de l'ancienne chane
  (void) memcpy ((void *) nouvelleChaine, (void *) chaine_, 1 + longueur_);

  // limination du tableau prcdent
  RestitueChaine (chaine_);

  // remplacement
  taille_ = nouvelleTaille;
  chaine_ = nouvelleChaine;

}

ChaineSimple::ChaineSimple (const char *chaine)
{ // cration  partir d'une chane constante
  if (chaine)
  { // il y a bien une chane
    int longueur = strlen (chaine);
    alloueChaine (1 + longueur);
    longueur_    = longueur;
    (void) memcpy ((void *) chaine_, (void *) chaine, 1 + longueur_);
  }
  else
    alloueChaine (-1);

}

ChaineSimple::ChaineSimple (const char *chaine, int longueur)
{ // cration  partir d'un tronon (ventuellement vide) de chane constante
  alloueChaine (1 + longueur);

  if (chaine)
  { // il y a bien une chane
    if (longueur)
      (void) memcpy ((void *) chaine_, (void *) chaine, longueur);

    // recherche d'un ventuel '\0' avant la fin de la chane
    longueur_ = 0;
    while ((longueur_ < longueur) && (chaine_ [longueur_] != '\0'))
      longueur_++;

  }
  else
    longueur_ = 0;

  chaine_ [longueur_] = '\0';

}

ChaineSimple::ChaineSimple (char caractere, int repetitions)
{ // cration par duplication d'un caractre
  alloueChaine (1 + repetitions);

  longueur_ = repetitions;

  char *ptr = chaine_;
  while (repetitions-- > 0)
    *ptr++ = caractere;

  chaine_ [longueur_] = '\0';

}

ChaineSimple::ChaineSimple (const ChaineSimple& c)
{ alloueChaine (1 + c.longueur_);
  longueur_ = c.longueur_;
  (void) strcpy (chaine_, c.chaine_);
}

ChaineSimple::ChaineSimple (const string& c)
{ alloueChaine (1 + c.size ());
  longueur_ = c.size ();
  for (int i=0 ; i<longueur_ ; i++)
    chaine_[i] = c[i];
  chaine_ [longueur_] = '\0';
}

ChaineSimple::~ChaineSimple ()
{ // nettoyage de l'instance
  RestitueChaine (chaine_);
}

void ChaineSimple::reinitialise (const ChaineSimple& c)
{ // raffectation  partir d'une autre instance

  if (this != &c)     // protection contre x.reinitialise (x)
  { // on doit copier tous les caractres stocks au cas o
    // l'utilisateur ait insr un '\0' en plein milieu de sa chane
    // avec modifieCaractere (i, '\0') et qu'il dsire malgr tout
    // pouvoir reconstruire la chane complte initiale

    // vrification de la place ncessaire
    if (taille_ < c.taille_)
    {
      if( !chaine_ )
        alloueChaine( c.taille_ );
      else
        etendChaine (c.taille_);
    }

    // copie
    (void) memcpy ((void *) chaine_, (void *) c.chaine_, c.taille_);
    longueur_ = c.longueur_;

  }

}

void ChaineSimple::reinitialise (const char *chaine)
{ // raffectation  partir d'une chane brute
  if (chaine)
  { // taille ncessaire en comptant le '\0' final
    int tailleNecessaire = 1 + strlen (chaine);
  
    // vrification de la place ncessaire

    if ((taille_ < tailleNecessaire) || (!chaine_)){
      if( !chaine_ )
        alloueChaine( tailleNecessaire );
      else
        etendChaine (tailleNecessaire);
    }    

    // copie
    (void) memcpy ((void *) chaine_, (void *) chaine, tailleNecessaire);
    longueur_ = tailleNecessaire - 1;

  }
  else
  { // il n'y a pas de chane brute
    chaine_ [0] = '\0';
    longueur_   = 0;
  }
}

void ChaineSimple::reinitialise (const char *chaine, int longueur)
{ // raffectation  partir d'un tronon (ventuellement vide) de chane

  // vrification de la place ncessaire
  int tailleNecessaire = longueur + 1;
  if (taille_ < tailleNecessaire)
    if( !chaine_ )
      alloueChaine( tailleNecessaire );
    else
      etendChaine (tailleNecessaire);

  if (chaine)
  { // il y a bien une chane
    if (longueur)
      (void) memcpy ((void *) chaine_, (void *) chaine, longueur);
  }
  else
    longueur = 0;

  chaine_ [longueur] = '\0';
  longueur_ = strlen (chaine_); // indispensable s'il y a des '\0' inclus

}

void ChaineSimple::reinitialise (char caractere, int repetitions)
{ // raffectation par duplication d'un caractre

  // vrification de la place ncessaire
  int tailleNecessaire = repetitions + 1;
  if (taille_ < tailleNecessaire)
    if( !chaine_ )
      alloueChaine( tailleNecessaire );
    else
      etendChaine (tailleNecessaire);

  longueur_ = repetitions;

  char *ptr = chaine_;
  while (repetitions-- > 0)
    *ptr++ = caractere;

  chaine_ [longueur_] = '\0';

}

void ChaineSimple::formate (const char *f ...)
{ // crit dans l'instance  l'aide d'un format C
  va_list    ap;

  // initialisation du traitement des arguments variables
  va_start(ap, f);

  vFormate (f, ap);

  // fin de rcupration des arguments d'appel
  va_end(ap);

}

void ChaineSimple::vFormate (const char *f, va_list ap)
{ // crit dans l'instance  l'aide d'un format C

  // criture dans un tampon de taille suppose suffisante
  static char *tampon = 0;
  if (tampon == 0)
    tampon = new char [4096];
  (void) vsprintf (tampon, f, ap);

  // recopie du tampon dans l'instance
  *this = tampon;

}

void ChaineSimple::modifieCaractere (int i, char c)
{ // modification du caractre d'index i

  // on autorise la modification dans toute la chane stocke,
  // y compris au del de la fin courante de la chane
  if ((i < 0) || (i >= taille_ - 1))
    return;

  // modification
  chaine_ [i] = c;

  // mise  jour de la longueur
  if (c == '\0')
    longueur_ = i;
  else if (i == longueur_)
    longueur_ += strlen (chaine_ + i);

}

void ChaineSimple::elimineBlancsFinaux ()
{ // limine les blancs finaux

  // blancs finaux
  for (char *mobile = chaine_ + longueur_ - 1; isspace(*mobile); mobile--)
  { // limination du dernier caractre
    *mobile = '\0';
    longueur_--;
  }

}

void ChaineSimple::elimineBlancsInitiaux ()
{ // limine les blancs initiaux

  // blancs initiaux
  if (isspace(chaine_ [0]))
  { // comptage des blancs  liminer
    char *src = chaine_ + 1;
    while (isspace(*src))
      src++;

    // dcalage de la fin de chane
    char *dst = chaine_;
    while (*src)
      *dst++ = *src++;
    *dst = '\0';

    // correction de la longueur
    longueur_ = dst - chaine_;

  }

}

ChaineSimple& ChaineSimple::operator += (const ChaineSimple& c)
{ // ajout d'une instance  la fin d'une autre

  // vrification de la place ncessaire
  int tailleNecessaire = longueur_ + c.longueur_ + 1;
  if (taille_ < tailleNecessaire)
    etendChaine (tailleNecessaire);

  // copie
  (void) memcpy ((void *) (chaine_ + longueur_), (void *) c.chaine_,
                 1 + c.longueur_);
  longueur_ += c.longueur_;

  return *this;

}

ChaineSimple& ChaineSimple::operator += (const char *chaine)
{ // ajout d'une chane  la fin de l'instance

  if (chaine)
  { // il y a une chane brute
    int l = strlen (chaine);

    // vrification de la place ncessaire
    int tailleNecessaire = longueur_ + l + 1;
    if (taille_ < tailleNecessaire)
      etendChaine (tailleNecessaire);

    // copie
    (void) memcpy ((void *) (chaine_ + longueur_), (void *) chaine, 1 + l);
    longueur_ += l;

  }

  return *this;

}

ChaineSimple& ChaineSimple::operator += (char c)
{ // ajout d'un caractre  la fin de l'instance

    // vrification de la place ncessaire
    int tailleNecessaire = longueur_ + 2;
    if (taille_ < tailleNecessaire)
      etendChaine (tailleNecessaire);

    // copie
    chaine_ [longueur_++] = c;
    chaine_ [longueur_]   = '\0';

  return *this;

}

ChaineSimple ChaineSimple::operator () (const char *debut,
                                        const char *fin) const
{ // extraction d'une sous-chane

  // limitation des pointeurs
  if (debut < chaine_)
    debut = chaine_;
  if (fin > chaine_ + longueur_ - 1)
    fin = chaine_ + longueur_ - 1;

  // cration d'une nouvelle instance
  int longueur = fin + 1 - debut;
  ChaineSimple sousChaine (1 + longueur);

  // copie de la sous-chane
  (void) memcpy ((void *) sousChaine.chaine_, (void *) debut, longueur);
  sousChaine.chaine_ [longueur] = '\0';
  sousChaine.longueur_ = longueur;

  return sousChaine;

}

ChaineSimple ChaineSimple::operator () (int debut, int fin) const
{ // extraction d'une sous-chane

  // limitation des pointeurs
  if (debut < 0)
    debut = 0;
  if (fin > longueur_ - 1)
    fin = longueur_ - 1;

  // cration d'une nouvelle instance
  int longueur = fin + 1 - debut;
  ChaineSimple sousChaine (1 + longueur);

  // copie de la sous-chane
  (void) memcpy ((void *) sousChaine.chaine_, (void *) (chaine_ + debut),
                 longueur);
  sousChaine.chaine_ [longueur] = '\0';
  sousChaine.longueur_ = longueur;

  return sousChaine;

}

int ChaineSimple::operator == (const char *chaine) const
{ return strcmp (chaine_, chaine) == 0; }

int ChaineSimple::operator == (const ChaineSimple& chaine) const
{ return strcmp (chaine_, chaine.chaine_) == 0; }

int ChaineSimple::operator != (const char *chaine) const
{ return strcmp (chaine_, chaine); }

int ChaineSimple::operator != (const ChaineSimple& chaine) const
{ return strcmp (chaine_, chaine.chaine_); }
